@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -74,13 +74,15 @@ public final class FreeCol {
     public static final String  META_SERVER_ADDRESS = "meta.freecol.org";
     public static final int     META_SERVER_PORT = 3540;
     public static final int     DEFAULT_PORT = 3541;
+    public static final int     DEFAULT_TIMEOUT = 60; // 1 minute
+    public static final int     TIMEOUT_MIN = 10; // 10s
 
     public static final String CLIENT_THREAD = "FreeColClient:";
     public static final String SERVER_THREAD = "FreeColServer:";
     public static final String METASERVER_THREAD = "FreeColMetaServer:";
     public static final String DEFAULT_TC = "freecol";
 
-    private static final String FREECOL_VERSION = "0.10.0";
+    private static final String FREECOL_VERSION = "0.10.5";
     private static String FREECOL_REVISION;
 
     private static final String MIN_JDK_VERSION = "1.5";
@@ -100,15 +102,14 @@ public final class FreeCol {
     private static boolean standAloneServer = false;
     private static boolean publicServer = true;
 
-    private static final int DEBUG_OFF = 0;
-    private static final int DEBUG_LIMITED = 1;
-    private static final int DEBUG_FULL = 2;
+    public static final int DEBUG_OFF = 0;
+    public static final int DEBUG_LIMITED = 1;
+    public static final int DEBUG_FULL = 2;
+    public static final int DEBUG_FULL_COMMS = 3;
     private static int debugLevel = DEBUG_OFF;
     private static int debugRunTurns = -1;
     private static String debugRunSave = null;
 
-    private static boolean usesExperimentalAI = false;
-
     private static String fontName = null;
 
     private static int serverPort = DEFAULT_PORT;
@@ -146,6 +147,8 @@ public final class FreeCol {
 
     private static long freeColSeed = 0L;
 
+    private static int freeColTimeout = -1;
+
 
     private FreeCol() {
         // Hide constructor
@@ -174,9 +177,6 @@ public final class FreeCol {
         createAndSetDirectories();
         initLogging();
 
-        if (freeColSeed == 0L) freeColSeed = new SecureRandom().nextLong();
-        logger.info("Using seed: " + freeColSeed);
-
         Locale locale = getLocale();
         Locale.setDefault(locale);
         Messages.setMessageBundle(locale);
@@ -238,42 +238,42 @@ public final class FreeCol {
     public static Locale getLocale() {
         XMLInputFactory xif = XMLInputFactory.newInstance();
         XMLStreamReader in = null;
-        try {
-            in = xif.createXMLStreamReader(new FileInputStream(getClientOptionsFile()), "UTF-8");
-            in.nextTag();
-            /**
-             * The following code was contributed by armcode to fix
-             * bug #[ 2045521 ] "Exception in Freecol.log on starting
-             * game". I was never able to reproduce the bug, but the
-             * patch did no harm either.
-             */
-            for(int eventid = in.getEventType();eventid != XMLEvent.END_DOCUMENT; eventid = in.getEventType()) {
-
-                //TODO: Is checking for XMLEvent.ATTRIBUTE needed?
-                if(eventid == XMLEvent.START_ELEMENT) {
-                    if (ClientOptions.LANGUAGE.equals(in.getAttributeValue(null, "id"))) {
-                        return LanguageOption.getLocale(in.getAttributeValue(null, "value"));
-                    }
-                }
-                in.nextTag();
-            }
-            //We don't have a language option in our file, it is either not there or the file is corrupt
-            logger.log(Level.WARNING, "Language setting not found in client options file.  Using default.");
-            return Locale.getDefault();
-
-        } catch (Exception e) {
-            logger.log(Level.WARNING, "Exception while loading options.", e);
-            return Locale.getDefault();
-        } finally {
+        File options = getClientOptionsFile();
+        if (options.canRead()) {
             try {
-                if (in != null) {
-                    in.close();
+                in = xif.createXMLStreamReader(new FileInputStream(options), "UTF-8");
+                in.nextTag();
+                /**
+                 * The following code was contributed by armcode to fix
+                 * bug #[ 2045521 ] "Exception in Freecol.log on starting
+                 * game". I was never able to reproduce the bug, but the
+                 * patch did no harm either.
+                 */
+                for(int eventid = in.getEventType();eventid != XMLEvent.END_DOCUMENT; eventid = in.getEventType()) {
+
+                    //TODO: Is checking for XMLEvent.ATTRIBUTE needed?
+                    if (eventid == XMLEvent.START_ELEMENT) {
+                        if (ClientOptions.LANGUAGE.equals(in.getAttributeValue(null, "id"))) {
+                            return LanguageOption.getLocale(in.getAttributeValue(null, "value"));
+                        }
+                    }
+                    in.nextTag();
                 }
+                //We don't have a language option in our file, it is either not there or the file is corrupt
+                logger.log(Level.WARNING, "Language setting not found in client options file.  Using default.");
             } catch (Exception e) {
-                logger.log(Level.WARNING, "Exception while closing stream.", e);
-                return Locale.getDefault();
+                logger.log(Level.WARNING, "Exception while loading options.", e);
+            } finally {
+                try {
+                    if (in != null) {
+                        in.close();
+                    }
+                } catch (Exception e) {
+                    logger.log(Level.WARNING, "Exception while closing stream.", e);
+                }
             }
         }
+        return Locale.getDefault();
     }
 
     /**
@@ -618,10 +618,6 @@ public final class FreeCol {
         options.addOption(OptionBuilder.withLongOpt("check-savegame")
                           .withDescription(Messages.message("cli.check-savegame"))
                           .create());
-        // TODO: remove option when AI is no longer experimental
-        options.addOption(OptionBuilder.withLongOpt("experimentalAI")
-                          .withDescription(Messages.message("cli.experimentalAI"))
-                          .create());
         options.addOption(OptionBuilder.withLongOpt("font")
                           .withDescription(Messages.message("cli.font"))
                           .withArgName(Messages.message("cli.arg.font"))
@@ -632,6 +628,11 @@ public final class FreeCol {
                           .withArgName(Messages.message("cli.arg.seed"))
                           .hasArg()
                           .create());
+        options.addOption(OptionBuilder.withLongOpt("timeout")
+                          .withDescription(Messages.message("cli.timeout"))
+                          .withArgName(Messages.message("cli.arg.timeout"))
+                          .hasArg()
+                          .create());
 
         try {
             // parse the command line arguments
@@ -732,7 +733,7 @@ public final class FreeCol {
                 try {
                     debugLevel = Integer.parseInt(line.getOptionValue("debug"));
                     debugLevel = Math.min(Math.max(debugLevel, DEBUG_OFF),
-                                          DEBUG_FULL);
+                                          DEBUG_FULL_COMMS);
                 } catch (NumberFormatException e) {
                     debugLevel = DEBUG_FULL;
                 }
@@ -781,9 +782,6 @@ public final class FreeCol {
             if (line.hasOption("server-name")) {
                 serverName = line.getOptionValue("server-name");
             }
-            if (line.hasOption("experimentalAI")) {
-                usesExperimentalAI = true;
-            }
             if (line.hasOption("font")) {
                 fontName = line.getOptionValue("font");
             }
@@ -795,6 +793,20 @@ public final class FreeCol {
                     System.err.println("Ignoring bad seed: " + seedStr);
                 }
             }
+            if (line.hasOption("timeout")) {
+                String timeoutStr = line.getOptionValue("timeout");
+                int result;
+                try {
+                    result = Integer.parseInt(timeoutStr);
+                } catch (NumberFormatException nfe) {
+                    result = -1;
+                }
+                if (result < TIMEOUT_MIN) {
+                    System.err.println("Ignoring bad timeout: " + timeoutStr);
+                } else {
+                    freeColTimeout = result;
+                }
+            }
         } catch (ParseException e) {
             System.err.println("\n" + e.getMessage() + "\n");
             printUsage();
@@ -828,6 +840,16 @@ public final class FreeCol {
         return FREECOL_REVISION;
     }
 
+
+    /**
+     * Returns the name of the log file.
+     *
+     * @return a <code>String</code> value
+     */
+    public static String getLogFile() {
+        return logFile;
+    }
+
     /**
      * Checks if the program is in "Debug mode".
      * @return <code>true</code> if the program is in debug
@@ -838,14 +860,12 @@ public final class FreeCol {
     }
 
     /**
-     * Checks if the program is in the full debug mode, rather than
-     * the limited version which does not initialize a player colony.
+     * Gets the debug level.
      *
-     * @return <code>true</code> if the program is in the higher level
-     *       mode and <code>false</code> otherwise.
+     * @return The debug level.
      */
-    public static boolean isInFullDebugMode() {
-        return debugLevel > DEBUG_LIMITED;
+    public static int getDebugLevel() {
+        return debugLevel;
     }
 
     /**
@@ -892,21 +912,16 @@ public final class FreeCol {
         return debugRunSave;
     }
 
-    /**
-     * Checks if the program is in "Experimental AI mode".
-     * @return <code>true</code> if the program is in Experimental AI
-     *       mode and <code>false</code> otherwise.
-     */
-    public static boolean usesExperimentalAI() {
-        return usesExperimentalAI;
-    }
-
     /**
      * Gets the seed for the PRNG.
      *
      * @return The seed.
      */
     public static long getFreeColSeed() {
+        if (freeColSeed == 0L) {
+            freeColSeed = new SecureRandom().nextLong();
+            logger.info("Using seed: " + freeColSeed);
+        }
         return freeColSeed;
     }
 
@@ -914,7 +929,23 @@ public final class FreeCol {
      * Increments the seed for the PRNG.
      */
     public static void incrementFreeColSeed() {
-        freeColSeed++;
+        freeColSeed = getFreeColSeed() + 1;
+        logger.info("Reseeded with: " + freeColSeed);
+    }
+
+    /**
+     * Gets the timeout.
+     * Use the command line specified one if any, otherwise default
+     * to `infinite' in single player and the DEFAULT_TIMEOUT for
+     * multiplayer.
+     *
+     * @param singleplayer True if this is a single player game.
+     * @return A suitable timeout value.
+     */
+    public static int getFreeColTimeout(boolean single) {
+        return (freeColTimeout >= TIMEOUT_MIN) ? freeColTimeout
+            : (single) ? Integer.MAX_VALUE
+            : DEFAULT_TIMEOUT;
     }
 
     private static void startServer() {
