@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -25,16 +25,14 @@ import java.io.FileInputStream;
 import java.io.FileNotFoundException;
 import java.io.InputStream;
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.Comparator;
-import java.util.HashMap;
 import java.util.List;
-import java.util.Map;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
 import net.sf.freecol.FreeCol;
 import net.sf.freecol.common.io.FreeColModFile;
-import net.sf.freecol.common.io.FreeColModFile.ModInfo;
 import net.sf.freecol.common.io.Mods;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.Europe;
@@ -44,15 +42,12 @@ import net.sf.freecol.common.model.ModelMessage;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.option.BooleanOption;
+import net.sf.freecol.common.option.IntegerOption;
 import net.sf.freecol.common.option.ListOption;
-import net.sf.freecol.common.option.ListOptionSelector;
-import net.sf.freecol.common.option.Option;
 import net.sf.freecol.common.option.OptionGroup;
 import net.sf.freecol.common.option.SelectOption;
 
 import javax.xml.stream.XMLInputFactory;
-import javax.xml.stream.XMLStreamConstants;
-import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 
 
@@ -118,7 +113,7 @@ public class ClientOptions extends OptionGroup {
      * Used by GUI, the number will be displayed when a group of goods are
      * higher than this number.
      *
-     * @see net.sf.freecol.client.gui.GUI
+     * @see net.sf.freecol.client.gui.MapViewer
      */
     public static final String MIN_NUMBER_FOR_DISPLAYING_GOODS_COUNT = "model.option.guiMinNumberToDisplayGoodsCount";
 
@@ -126,7 +121,7 @@ public class ClientOptions extends OptionGroup {
      * Used by GUI, this is the most repetitions drawn of a goods image for a
      * single goods grouping.
      *
-     * @see net.sf.freecol.client.gui.GUI
+     * @see net.sf.freecol.client.gui.MapViewer
      */
     public static final String MAX_NUMBER_OF_GOODS_IMAGES = "model.option.guiMaxNumberOfGoodsImages";
 
@@ -152,6 +147,16 @@ public class ClientOptions extends OptionGroup {
      */
     public static final String UNIT_LAST_MOVE_DELAY = "model.option.unitLastMoveDelay";
 
+    /**
+     * Pixmap setting to work around Java 2D graphics bug.
+     */
+    public static final String USE_PIXMAPS = "model.option.usePixmaps";
+
+    /**
+     * Whether to remember the positions of various panels.
+     */
+    public static final String REMEMBER_PANEL_POSITIONS = "model.option.rememberPanelPositions";
+
     /**
      * Whether to display borders by default or not.
      */
@@ -179,7 +184,7 @@ public class ClientOptions extends OptionGroup {
      * Used by GUI, this defines the grouping of ModelMessages. Possible values
      * include nothing, type and source.
      *
-     * @see net.sf.freecol.client.gui.GUI
+     * @see net.sf.freecol.client.gui.MapViewer
      * @see net.sf.freecol.common.model.ModelMessage
      */
     public static final String MESSAGES_GROUP_BY = "model.option.guiMessagesGroupBy";
@@ -194,10 +199,12 @@ public class ClientOptions extends OptionGroup {
 
     public static final String AUDIO_VOLUME = "model.option.audioVolume";
 
+    public static final String AUDIO_ALERTS = "model.option.audioAlerts";
+
     /**
      * Used by GUI, this defines whether SoL messages will be displayed.
      *
-     * @see net.sf.freecol.client.gui.GUI
+     * @see net.sf.freecol.client.gui.MapViewer
      */
     public static final String SHOW_COLONY_WARNINGS = "model.option.guiShowColonyWarnings";
 
@@ -330,6 +337,11 @@ public class ClientOptions extends OptionGroup {
      */
     public static final String AUTO_END_TURN = "model.option.autoEndTurn";
 
+    /**
+     * Show EndTurnDialog.
+     */
+    public static final String SHOW_END_TURN_DIALOG = "model.option.showEndTurnDialog";
+
     /**
      * The type of labour report to display.
      */
@@ -337,6 +349,13 @@ public class ClientOptions extends OptionGroup {
     public static final int LABOUR_REPORT_CLASSIC = 0;
     public static final int LABOUR_REPORT_COMPACT = 1;
 
+    /**
+     * Option for selecting the compact colony report.
+     */
+    public static final String COLONY_REPORT = "model.option.colonyReport";
+    public static final int COLONY_REPORT_CLASSIC = 0;
+    public static final int COLONY_REPORT_COMPACT = 1;
+
     /**
      * The Indian demand action.
      */
@@ -361,6 +380,13 @@ public class ClientOptions extends OptionGroup {
     public static final int COLONY_LABELS_CLASSIC = 1;
     public static final int COLONY_LABELS_MODERN = 2;
 
+    /**
+     * Style of map controls.
+     */
+    public static final String MAP_CONTROLS = "model.option.mapControls";
+    public static final String MAP_CONTROLS_CORNERS = "CornerMapControls";
+    public static final String MAP_CONTROLS_CLASSIC = "ClassicMapControls";
+
 
     /**
      * Comparators for sorting colonies.
@@ -370,11 +396,9 @@ public class ClientOptions extends OptionGroup {
             if (s1.getEstablished().getNumber() > 0
                 && s2.getEstablished().getNumber() > 0) {
                 return s1.getEstablished().getNumber() - s2.getEstablished().getNumber();
-            } else {
-                // TODO: remove 0.9.x compatibility code
-                // ID should indicate age
+            } else { // @compat 0.9.x
                 return s1.getIntegerID().compareTo(s2.getIntegerID());
-            }
+            } // end compatibility code
         }
     };
 
@@ -479,40 +503,9 @@ public class ClientOptions extends OptionGroup {
     /**
      * Adds the options to this <code>GameOptions</code>.
      */
-    protected void addDefaultOptions() {
-        load(new File(new File(FreeCol.getDataDirectory(), "base"),
-                      "client-options.xml"), false);
-
-        final OptionGroup modsGroup = new OptionGroup("clientOptions.mods");
-        final ListOptionSelector<ModInfo> selector = new ListOptionSelector<ModInfo>() {
-            private Map<String, ModInfo> mods = null;
-            private void init() {
-                if (mods == null) {
-                    List<FreeColModFile> fcmfs = Mods.getAllMods();
-                    mods = new HashMap<String, ModInfo>();
-                    for (FreeColModFile f : fcmfs) {
-                        ModInfo modInfo = f.getModInfo();
-                        mods.put(modInfo.getId(), modInfo);
-                    }
-                }
-            }
-            public String getId(ModInfo t) {
-                return t.getId();
-            }
-            public ModInfo getObject(String id) {
-                init();
-                return mods.get(id);
-            }
-            public List<ModInfo> getOptions() {
-                init();
-                return new ArrayList<ModInfo>(mods.values());
-            }
-            public String toString(ModInfo t) {
-                return t.getName();
-            }
-        };
-        new ListOption<ModInfo>(selector, USER_MODS, modsGroup);
-        add(modsGroup);
+    private void addDefaultOptions() {
+        loadOptions(new File(new File(FreeCol.getDataDirectory(), "base"),
+                "client-options.xml"));
     }
 
     /**
@@ -520,17 +513,20 @@ public class ClientOptions extends OptionGroup {
      *
      * @return A list of active mods.
      */
+    @SuppressWarnings("unchecked")
     public List<FreeColModFile> getActiveMods() {
-        final List<FreeColModFile> fcmfs = Mods.getAllMods();
+        final Collection<FreeColModFile> fcmfs = Mods.getAllMods();
         List<FreeColModFile> active = new ArrayList<FreeColModFile>();
-        ListOption<?> options = (ListOption<?>) getOption(ClientOptions.USER_MODS);
-        for (Object o : options.getValue()) {
-            ModInfo modInfo = (ModInfo) o;
-            if (modInfo == null) continue;
-            for (FreeColModFile f : fcmfs) {
-                if (modInfo.getId().equals(f.getModInfo().getId())) {
-                    active.add(f);
-                    break;
+        ListOption<FreeColModFile> options = (ListOption<FreeColModFile>) getOption(ClientOptions.USER_MODS);
+        if (options != null) {
+            for (FreeColModFile modInfo : options.getOptionValues()) {
+                if (modInfo != null) {
+                    for (FreeColModFile f : fcmfs) {
+                        if (modInfo.getId().equals(f.getId())) {
+                            active.add(f);
+                            break;
+                        }
+                    }
                 }
             }
         }
@@ -538,99 +534,80 @@ public class ClientOptions extends OptionGroup {
     }
 
     /**
-     * Reads the options from the given file.
+     * Loads the options from the given file.
      *
-     * @param loadFile The <code>File</code> to read the
-     *            options from.
+     * @param optionsFile The <code>File</code> to read the options from.
      */
-    public void load(File loadFile) {
-        load(loadFile, true);
-    }
-
-    /**
-     * Reads the options from the given file.
-     *
-     * @param loadFile The <code>File</code> to read the
-     *            options from.
-     * @param update a <code>boolean</code> value
-     */
-    public void load(File loadFile, boolean update) {
-        if (loadFile == null || !loadFile.exists()) {
-            logger.warning("Could not find the client options file.");
-            return;
-        }
-        logger.finest((update ? "Updating" : "Loading") + " client options from " + loadFile.getPath());
+    public void loadOptions(File optionsFile) {
         try {
-            load(new BufferedInputStream(new FileInputStream(loadFile)), update);
-        } catch(FileNotFoundException e) {
-            logger.warning("Could not find the client options file.");
+            loadOptions(new BufferedInputStream(new FileInputStream(optionsFile)));
+        } catch (FileNotFoundException e) {
+            logger.warning("Could not find the client options file: "
+                + optionsFile.getPath());
         }
     }
 
     /**
-     * Reads the options from the given file.
+     * Loads the options from the given stream.
      *
-     * @param in <code>InputStream</code> from XML resource
-     * @param update a <code>boolean</code> value
+     * @param in The <code>InputStream</code> to read the options from.
      */
-    public void load(InputStream in, boolean update) {
+    public void loadOptions(InputStream in) {
+        if (in == null) return;
         try {
-            XMLStreamReader xsr = XMLInputFactory.newInstance().createXMLStreamReader(in, "UTF-8");
+            XMLStreamReader xsr = XMLInputFactory.newInstance()
+                .createXMLStreamReader(in, "UTF-8");
             xsr.nextTag();
-            if (update) {
-                logger.finest("Updating " + getId());
-                updateFromXML(xsr);
-            } else {
-                logger.finest("Loading " + getId());
-                readFromXML(xsr);
-            }
+            readFromXML(xsr);
         } catch (Exception e) {
-            logger.log(Level.WARNING, "Exception while loading options.", e);
+            logger.log(Level.WARNING, "Exception when loading options.", e);
         } finally {
             try {
-                if (in != null) {
-                    in.close();
-                }
+                in.close();
             } catch (Exception e) {
-                logger.log(Level.WARNING, "Exception while closing stream.", e);
+                logger.log(Level.WARNING, "Exception when closing stream.", e);
             }
         }
     }
 
     /**
-     * Initialize this object from an XML-representation of this object.
-     * @param in The input stream with the XML.
-     * @throws XMLStreamException if a problem was encountered
-     *      during parsing.
+     * Updates the options from the given file.
+     *
+     * @param optionsFile The <code>File</code> to read the options from.
      */
-    private void updateFromXML(XMLStreamReader in) throws XMLStreamException {
-        while (in.nextTag() != XMLStreamConstants.END_ELEMENT) {
-            if (in.getLocalName().equals(OptionGroup.getXMLElementTagName())) {
-                updateFromXML(in);
-            } else {
-                String idStr = in.getAttributeValue(null, ID_ATTRIBUTE_TAG);
-                if (idStr == null) {
-                    idStr = in.getLocalName();
-                }
-                Option o = getOption(idStr);
+    public void updateOptions(File optionsFile) {
+        try {
+            updateOptions(new BufferedInputStream(new FileInputStream(optionsFile)));
+        } catch (FileNotFoundException e) {
+            logger.warning("Could not find the client options file: "
+                + optionsFile.getPath());
+        }
+    }
 
-                if (o != null) {
-                    o.readFromXML(in);
-                } else {
-                    // Normal only if this option is from an old save game:
-                    logger.info("Option \"" + idStr + "\" (" + in.getLocalName() + ") could not be found.");
-
-                    // Ignore the option:
-                    final String ignoredTag = in.getLocalName();
-                    while (in.nextTag() != XMLStreamConstants.END_ELEMENT
-                           || !in.getLocalName().equals(ignoredTag)) {}
-                }
+    /**
+     * Updates the options from the given stream.
+     *
+     * @param in The <code>InputStream</code> to read the options from.
+     */
+    public void updateOptions(InputStream in) {
+        try {
+            XMLStreamReader xsr = XMLInputFactory.newInstance()
+                .createXMLStreamReader(in, "UTF-8");
+            xsr.nextTag();
+            readFromXML(xsr);
+        } catch (Exception e) {
+            logger.log(Level.WARNING, "Exception when loading options.", e);
+        } finally {
+            try {
+                if (in != null) in.close();
+            } catch (Exception e) {
+                logger.log(Level.WARNING, "Exception when closing stream.", e);
             }
         }
-        // DONE BY while-loop: in.nextTag();
     }
 
 
+
     /**
      * Return the client's preferred tile text type.
      *
@@ -640,6 +617,18 @@ public class ClientOptions extends OptionGroup {
         return getInteger(DISPLAY_TILE_TEXT);
     }
 
+    /**
+     * Gets a fresh list of sorted colonies for a player.
+     * Helper function for a common idiom.
+     *
+     * @param p The <code>Player</code> to get the colony list for.
+     * @return A fresh list of colonies sorted according to the
+     *    current comparator.
+     */
+    public List<Colony> getSortedColonies(Player p) {
+        return p.getSortedColonies(getColonyComparator());
+    }
+
     /**
      * Return the client's preferred comparator for colonies.
      *
@@ -698,6 +687,45 @@ public class ClientOptions extends OptionGroup {
         return (BooleanOption) getOption(message.getMessageType().getOptionName());
     }
 
+    /**
+     * Perform backward compatibility fixups on new client options as
+     * they are introduced.  Annotate with introduction version so we
+     * can clean these up as they become standard.
+     */
+    public void fixClientOptions() {
+        // @compat 0.9.x
+        addBooleanOption("model.option.guiShowDemands",
+            "clientOptions.messages", true);
+        addBooleanOption("model.option.guiShowGifts",
+            "clientOptions.messages", true);
+        addBooleanOption("model.option.guiShowGoodsMovement",
+            "clientOptions.messages", true);
+        // @compat 0.10.1
+        addIntegerOption(COLONY_REPORT,
+            "clientOptions.messages", 0);
+        addBooleanOption(USE_PIXMAPS,
+            "clientOptions.gui", true);
+    }
+
+    private void addBooleanOption(String id, String gr, boolean val) {
+        if (getOption(id) == null) {
+            BooleanOption op = new BooleanOption(id);
+            op.setGroup(gr);
+            op.setValue(val);
+            add(op);
+        }
+    }
+
+    private void addIntegerOption(String id, String gr, int val) {
+        if (getOption(id) == null) {
+            IntegerOption op = new IntegerOption(id);
+            op.setGroup(gr);
+            op.setValue(val);
+            add(op);
+        }
+    }
+
+
     protected boolean isCorrectTagName(String tagName) {
         return getXMLElementTagName().equals(tagName);
     }
