@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -24,6 +24,7 @@ package net.sf.freecol.client.control;
 import java.io.File;
 import java.io.FileNotFoundException;
 import java.io.IOException;
+import java.io.InputStream;
 import java.io.PrintWriter;
 import java.io.StringWriter;
 import java.net.ConnectException;
@@ -39,20 +40,22 @@ import javax.xml.stream.XMLStreamWriter;
 import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.ClientOptions;
 import net.sf.freecol.client.FreeColClient;
-import net.sf.freecol.client.gui.Canvas;
+import net.sf.freecol.client.gui.GUI;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.client.gui.panel.LoadingSavegameDialog;
 import net.sf.freecol.client.networking.Client;
 import net.sf.freecol.common.FreeColException;
 import net.sf.freecol.common.ServerInfo;
+import net.sf.freecol.common.io.FreeColModFile;
 import net.sf.freecol.common.io.FreeColSavegameFile;
 import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.NationOptions.Advantages;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Specification;
 import net.sf.freecol.common.model.Tile;
+import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.networking.Connection;
-import net.sf.freecol.common.networking.Message;
+import net.sf.freecol.common.networking.DOMMessage;
 import net.sf.freecol.common.networking.NoRouteToServerException;
 import net.sf.freecol.common.option.OptionGroup;
 import net.sf.freecol.common.resources.ResourceManager;
@@ -64,22 +67,25 @@ import org.w3c.dom.NodeList;
 
 
 /**
-* The controller responsible for starting a server and
-* connecting to it. {@link PreGameInputHandler} will be set
-* as the input handler when a successful login has been completed,
-*/
+ * The controller responsible for starting a server and
+ * connecting to it. {@link PreGameInputHandler} will be set
+ * as the input handler when a successful login has been completed,
+ */
 public final class ConnectController {
 
     private static final Logger logger = Logger.getLogger(ConnectController.class.getName());
 
     private final FreeColClient freeColClient;
 
+    private GUI gui;
+
     /**
      * Creates a new <code>ConnectController</code>.
      * @param freeColClient The main controller.
      */
-    public ConnectController(FreeColClient freeColClient) {
+    public ConnectController(FreeColClient freeColClient, GUI gui) {
         this.freeColClient = freeColClient;
+        this.gui = gui;
     }
 
 
@@ -101,7 +107,7 @@ public final class ConnectController {
 
         if (freeColClient.getFreeColServer() != null &&
             freeColClient.getFreeColServer().getServer().getPort() == port) {
-            if (freeColClient.getCanvas().showConfirmDialog("stopServer.text",
+            if (gui.showConfirmDialog("stopServer.text",
                                                             "stopServer.yes",
                                                             "stopServer.no")) {
                 freeColClient.getFreeColServer().getController().shutdown();
@@ -114,10 +120,10 @@ public final class ConnectController {
             FreeColServer freeColServer = new FreeColServer(specification, publicServer, false, port, null, advantages);
             freeColClient.setFreeColServer(freeColServer);
         } catch (NoRouteToServerException e) {
-            freeColClient.getCanvas().errorMessage("server.noRouteToServer");
+            gui.errorMessage("server.noRouteToServer");
             return;
         } catch (IOException e) {
-            freeColClient.getCanvas().errorMessage("server.couldNotStart");
+            gui.errorMessage("server.couldNotStart");
             return;
         }
 
@@ -145,7 +151,7 @@ public final class ConnectController {
 
         if (freeColClient.getFreeColServer() != null
             && freeColClient.getFreeColServer().getServer().getPort() == port) {
-            if (freeColClient.getCanvas().showConfirmDialog("stopServer.text",
+            if (gui.showConfirmDialog("stopServer.text",
                                                             "stopServer.yes",
                                                             "stopServer.no")) {
                 freeColClient.getFreeColServer().getController().shutdown();
@@ -154,6 +160,8 @@ public final class ConnectController {
             }
         }
 
+        loadModFragments(specification);
+
         try {
             FreeColServer freeColServer = new FreeColServer(specification, false, true, port, null, advantages);
             if (freeColClient.getClientOptions().getBoolean(ClientOptions.AUTOSAVE_DELETE)) {
@@ -164,7 +172,7 @@ public final class ConnectController {
             logger.warning("Illegal state: An exception occured that can only appear in public multiplayer games.");
             return;
         } catch (IOException e) {
-            freeColClient.getCanvas().errorMessage("server.couldNotStart");
+            gui.errorMessage("server.couldNotStart");
             return;
         }
 
@@ -172,7 +180,7 @@ public final class ConnectController {
 
         if (login(username, "127.0.0.1", port)) {
             freeColClient.getPreGameController().setReady(true);
-            freeColClient.getCanvas().showStartGamePanel(freeColClient.getGame(), freeColClient.getMyPlayer(),
+            gui.showStartGamePanel(freeColClient.getGame(), freeColClient.getMyPlayer(),
                                                          true);
 
         }
@@ -187,7 +195,6 @@ public final class ConnectController {
     * @param port The port to use when connecting to the host.
     */
     public void joinMultiplayerGame(String username, String host, int port) {
-        final Canvas canvas = freeColClient.getCanvas();
         freeColClient.setMapEditor(false);
 
         if (freeColClient.isLoggedIn()) {
@@ -196,7 +203,7 @@ public final class ConnectController {
 
         List<String> vacantPlayers = getVacantPlayers(host, port);
         if (vacantPlayers != null) {
-            String choice = canvas.showSimpleChoiceDialog(null,
+            String choice = gui.showSimpleChoiceDialog(null,
                                                           "connectController.choicePlayer",
                                                           "cancel",
                                                           vacantPlayers);
@@ -209,7 +216,7 @@ public final class ConnectController {
 
         freeColClient.setSingleplayer(false);
         if (login(username, host, port) && !freeColClient.isInGame()) {
-            canvas.showStartGamePanel(freeColClient.getGame(), freeColClient.getMyPlayer(), false);
+            gui.showStartGamePanel(freeColClient.getGame(), freeColClient.getMyPlayer(), false);
         }
     }
 
@@ -224,7 +231,6 @@ public final class ConnectController {
      */
     public boolean login(String username, String host, int port) {
         Client client = freeColClient.getClient();
-        Canvas canvas = freeColClient.getCanvas();
         freeColClient.setMapEditor(false);
 
         if (client != null) {
@@ -232,12 +238,14 @@ public final class ConnectController {
         }
 
         try {
-            client = new Client(host, port, freeColClient.getPreGameInputHandler());
+            client = new Client(host, port,
+                                freeColClient.getPreGameInputHandler(),
+                                FreeCol.CLIENT_THREAD + username);
         } catch (ConnectException e) {
-            canvas.errorMessage("server.couldNotConnect");
+            gui.errorMessage("server.couldNotConnect");
             return false;
         } catch (IOException e) {
-            canvas.errorMessage("server.couldNotConnect");
+            gui.errorMessage("server.couldNotConnect");
             return false;
         }
 
@@ -284,18 +292,24 @@ public final class ConnectController {
                         freeColClient.getInGameController()
                             .setCurrentPlayer(thisPlayer);
                         if (activeUnitId != null) {
-                            freeColClient.setActiveUnit(activeUnitId);
+                            Unit active = (Unit) freeColClient.getGame().getFreeColGameObject(activeUnitId);
+                            if (active != null) {
+                                active.getOwner().resetIterators();
+                                active.getOwner().setNextActiveUnit(active);
+                                gui.setActiveUnit(active);
+                            }
+                            
                         } else {
-                            freeColClient.getGUI().setSelectedTile(entryTile, false);
+                            gui.setSelectedTile(entryTile, false);
                         }
                     } else {
-                        freeColClient.getGUI().setSelectedTile(entryTile, false);
+                        gui.setSelectedTile(entryTile, false);
                     }
-                    freeColClient.getGUI().setSelectedTile(thisPlayer
+                    gui.setSelectedTile(thisPlayer
                         .getEntryLocation().getTile(), false);
                 }
             } else if (in.getLocalName().equals("error")) {
-                canvas.errorMessage(in.getAttributeValue(null, "messageID"), in.getAttributeValue(null, "message"));
+                gui.errorMessage(in.getAttributeValue(null, "messageID"), in.getAttributeValue(null, "message"));
 
                 c.endTransmission(in);
                 return false;
@@ -308,7 +322,7 @@ public final class ConnectController {
             StringWriter sw = new StringWriter();
             e.printStackTrace(new PrintWriter(sw));
             logger.warning(sw.toString());
-            canvas.errorMessage(null, "Could not send XML to the server.");
+            gui.errorMessage(null, "Could not send XML to the server.");
             try {
                 c.endTransmission(in);
             } catch (IOException ie) {
@@ -330,7 +344,7 @@ public final class ConnectController {
         final String host = freeColClient.getClient().getHost();
         final int port = freeColClient.getClient().getPort();
 
-        freeColClient.getCanvas().removeInGameComponents();
+        gui.removeInGameComponents();
         logout(true);
         login(username, host, port);
         freeColClient.getInGameController().nextModelMessage();
@@ -342,7 +356,7 @@ public final class ConnectController {
     * and loads the game.
     */
     public void loadGame() {
-        File file = freeColClient.getCanvas().showLoadDialog(FreeCol.getSaveDirectory());
+        File file = gui.showLoadDialog(FreeCol.getSaveDirectory());
         if (file != null) {
             //FreeCol.setSaveDirectory(file.getParentFile());
             loadGame(file);
@@ -351,22 +365,22 @@ public final class ConnectController {
 
     /**
      * Loads a game from the given file.
+     *
      * @param file The <code>File</code>.
      */
     public void loadGame(File file) {
-        final Canvas canvas = freeColClient.getCanvas();
         final File theFile = file;
 
         freeColClient.setMapEditor(false);
 
         class ErrorJob implements Runnable {
-            private final  String  message;
+            private final String message;
             ErrorJob( String message ) {
                 this.message = message;
             }
             public void run() {
-                canvas.closeMenus();
-                canvas.errorMessage( message );
+                gui.closeMenus();
+                gui.errorMessage( message );
             }
         }
 
@@ -389,18 +403,20 @@ public final class ConnectController {
                 defaultPublicServer = false;
             }
             xs.close();
+
+            // Reload the client options saved with this game.
             try {
-                freeColClient.getClientOptions().load(fis.getInputStream(FreeColSavegameFile.CLIENT_OPTIONS), true);
-                freeColClient.getActionManager().update();
-            } catch(IOException e) {
-                // old savegame format, we don't care
-                logger.info("No saved client options available.");
+                ClientOptions options = freeColClient.getClientOptions();
+                options.updateOptions(fis.getInputStream(FreeColSavegameFile.CLIENT_OPTIONS));
+                options.fixClientOptions();
+            } catch (FileNotFoundException e) {
+                // no client options, we don't care
             }
             final int sgo = freeColClient.getClientOptions().getInteger(ClientOptions.SHOW_SAVEGAME_SETTINGS);
             if (sgo == ClientOptions.SHOW_SAVEGAME_SETTINGS_ALWAYS
                     || !defaultSingleplayer && sgo == ClientOptions.SHOW_SAVEGAME_SETTINGS_MULTIPLAYER) {
-                if (canvas.showLoadingSavegameDialog(defaultPublicServer, defaultSingleplayer)) {
-                    LoadingSavegameDialog lsd = canvas.getLoadingSavegameDialog();
+                if (gui.showLoadingSavegameDialog(defaultPublicServer, defaultSingleplayer)) {
+                    LoadingSavegameDialog lsd = gui.getLoadingSavegameDialog();
                     singleplayer = lsd.isSingleplayer();
                     name = lsd.getName();
                     port = lsd.getPort();
@@ -434,14 +450,14 @@ public final class ConnectController {
         }
 
         if (freeColClient.getFreeColServer() != null && freeColClient.getFreeColServer().getServer().getPort() == port) {
-            if (freeColClient.getCanvas().showConfirmDialog("stopServer.text", "stopServer.yes", "stopServer.no")) {
+            if (gui.showConfirmDialog("stopServer.text", "stopServer.yes", "stopServer.no")) {
                 freeColClient.getFreeColServer().getController().shutdown();
             } else {
                 return;
             }
         }
 
-        canvas.showStatusPanel(Messages.message("status.loadingGame"));
+        gui.showStatusPanel(Messages.message("status.loadingGame"));
 
         Runnable loadGameJob = new Runnable() {
             public void run() {
@@ -457,38 +473,38 @@ public final class ConnectController {
                         public void run() {
                             ResourceManager.setScenarioMapping(savegame.getResourceMapping());
                             login(username, "127.0.0.1", FreeCol.getDefaultPort());
-                            canvas.closeStatusPanel();
+                            gui.closeStatusPanel();
                         }
                     } );
                 } catch (NoRouteToServerException e) {
                     SwingUtilities.invokeLater(new Runnable() {
                         public void run() {
-                            freeColClient.getCanvas().closeMainPanel();
-                            freeColClient.getCanvas().showMainPanel();
+                            gui.closeMainPanel();
+                            gui.showMainPanel();
                         }
                     });
                     SwingUtilities.invokeLater( new ErrorJob("server.noRouteToServer") );
                 } catch (FileNotFoundException e) {
                     SwingUtilities.invokeLater(new Runnable() {
                         public void run() {
-                            freeColClient.getCanvas().closeMainPanel();
-                            freeColClient.getCanvas().showMainPanel();
+                            gui.closeMainPanel();
+                            gui.showMainPanel();
                         }
                     });
                     SwingUtilities.invokeLater( new ErrorJob("fileNotFound") );
                 } catch (IOException e) {
                     SwingUtilities.invokeLater(new Runnable() {
                         public void run() {
-                            freeColClient.getCanvas().closeMainPanel();
-                            freeColClient.getCanvas().showMainPanel();
+                            gui.closeMainPanel();
+                            gui.showMainPanel();
                         }
                     });
                     SwingUtilities.invokeLater( new ErrorJob("server.couldNotStart") );
                 } catch (FreeColException e) {
                     SwingUtilities.invokeLater(new Runnable() {
                         public void run() {
-                            freeColClient.getCanvas().closeMainPanel();
-                            freeColClient.getCanvas().showMainPanel();
+                            gui.closeMainPanel();
+                            gui.showMainPanel();
                         }
                     });
                     SwingUtilities.invokeLater( new ErrorJob(e.getMessage()) );
@@ -507,7 +523,7 @@ public final class ConnectController {
     */
     public void logout(boolean notifyServer) {
         if (notifyServer) {
-            Element logoutMessage = Message.createNewRootElement("logout");
+            Element logoutMessage = DOMMessage.createNewRootElement("logout");
             logoutMessage.setAttribute("reason", "User has quit the client.");
 
             freeColClient.getClient().sendAndWait(logoutMessage);
@@ -598,9 +614,9 @@ public final class ConnectController {
         }
 
         ArrayList<String> items = new ArrayList<String>();
-        Element element = Message.createNewRootElement("getVacantPlayers");
+        Element element = DOMMessage.createNewRootElement("getVacantPlayers");
         try {
-            Element reply = mc.ask(element);
+            Element reply = mc.askDumping(element);
             if (reply == null) {
                 logger.warning("The server did not return a list.");
                 return null;
@@ -633,23 +649,21 @@ public final class ConnectController {
     * @return A list of {@link ServerInfo} objects.
     */
     public ArrayList<ServerInfo> getServerList() {
-        Canvas canvas = freeColClient.getCanvas();
-
         Connection mc;
         try {
             mc = new Connection(FreeCol.META_SERVER_ADDRESS, FreeCol.META_SERVER_PORT, null, FreeCol.CLIENT_THREAD);
         } catch (IOException e) {
             logger.warning("Could not connect to meta-server.");
-            canvas.errorMessage("metaServer.couldNotConnect");
+            gui.errorMessage("metaServer.couldNotConnect");
             return null;
         }
 
         try {
-            Element gslElement = Message.createNewRootElement("getServerList");
-            Element reply = mc.ask(gslElement);
+            Element gslElement = DOMMessage.createNewRootElement("getServerList");
+            Element reply = mc.askDumping(gslElement);
             if (reply == null) {
                 logger.warning("The meta-server did not return a list.");
-                canvas.errorMessage("metaServer.communicationError");
+                gui.errorMessage("metaServer.communicationError");
                 return null;
             } else {
                 ArrayList<ServerInfo> items = new ArrayList<ServerInfo>();
@@ -661,7 +675,7 @@ public final class ConnectController {
             }
         } catch (IOException e) {
             logger.warning("Network error while communicating with the meta-server.");
-            canvas.errorMessage("metaServer.communicationError");
+            gui.errorMessage("metaServer.communicationError");
             return null;
         } finally {
             try {
@@ -672,4 +686,31 @@ public final class ConnectController {
             }
         }
     }
+
+    private void loadModFragments(Specification specification) {
+        boolean loadedMod = false;
+        for (FreeColModFile f : freeColClient.getClientOptions()
+                 .getActiveMods()) {
+            InputStream sis = null;
+            try {
+                sis = f.getSpecificationInputStream();
+            } catch (IOException ioe) {
+                logger.warning("IO error in mod fragment " + f.getId()
+                    + ": " + ioe.getMessage());
+            }
+            if (sis != null) {
+                try {
+                    specification.loadFragment(sis);
+                    loadedMod = true;
+                    logger.info("Loaded mod fragment " + f.getId());
+                } catch (RuntimeException rte) {
+                    logger.warning("Parse error in mod fragment " + f.getId()
+                        + ": " + rte.getMessage());
+                }
+            }
+        }
+        if (loadedMod) { // Update actions in case new ones loaded.
+            freeColClient.getActionManager().update();
+        }
+    }
 }
