@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -34,11 +34,9 @@ import javax.swing.SwingUtilities;
 
 import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.ClientOptions;
+import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.control.MapEditorController;
-import net.sf.freecol.client.gui.panel.EditSettlementDialog;
 import net.sf.freecol.client.gui.panel.MapEditorTransformPanel.TileTypeTransform;
-import net.sf.freecol.client.gui.panel.RiverStylePanel;
-import net.sf.freecol.common.model.IndianSettlement;
 import net.sf.freecol.common.model.Map;
 import net.sf.freecol.common.model.Map.Direction;
 import net.sf.freecol.common.model.Tile;
@@ -55,7 +53,7 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
 
     private final Canvas canvas;
 
-    private final GUI gui;
+    private final MapViewer mapViewer;
 
     private ScrollThread scrollThread;
 
@@ -66,16 +64,22 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
     private Point oldPoint;
     private Point startPoint;
 
+    private FreeColClient freeColClient;
+
+    private GUI gui;
+
     /**
      * The constructor to use.
      *
      * @param canvas The component this object gets created for.
-     * @param g The GUI that holds information such as screen resolution.
+     * @param gui The GUI that holds information such as screen resolution.
      */
-    public CanvasMapEditorMouseListener(Canvas canvas, GUI g) {
+    public CanvasMapEditorMouseListener(FreeColClient freeColClient, GUI gui, Canvas canvas) {
+        this.freeColClient = freeColClient;
+        this.gui = gui;
         this.canvas = canvas;
-        gui = g;
-        scrollThread = null;
+        this.mapViewer = gui.getMapViewer();
+        this.scrollThread = null;
     }
 
 
@@ -85,8 +89,8 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
      */
     private Map getMap() {
         Map map = null;
-        if (canvas.getClient().getGame() != null)
-            map = canvas.getClient().getGame().getMap();
+        if (freeColClient.getGame() != null)
+            map = freeColClient.getGame().getMap();
         return map;
     }
 
@@ -101,7 +105,7 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
         }
         try {
             if (e.getClickCount() > 1) {
-                Tile tile = gui.convertToMapTile(e.getX(), e.getY());
+                Tile tile = mapViewer.convertToMapTile(e.getX(), e.getY());
                 canvas.showColonyPanel(tile);
             } else {
                 canvas.requestFocus();
@@ -144,11 +148,11 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
         }
         try {
             if (e.getButton() == MouseEvent.BUTTON3 || e.isPopupTrigger()) {
-                Tile tile = gui.convertToMapTile(e.getX(), e.getY());
+                Tile tile = mapViewer.convertToMapTile(e.getX(), e.getY());
                 if (tile != null) {
                     if (tile.hasRiver()) {
                         TileImprovement river = tile.getRiver();
-                        int style = canvas.showFreeColDialog(new RiverStylePanel(canvas));
+                        int style = canvas.showRiverStyleDialog();
                         if (style == -1) {
                             // user canceled
                         } else if (style == 0) {
@@ -160,7 +164,7 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
                         }
                     }
                     if (tile.getIndianSettlement() != null) {
-                        canvas.showFreeColDialog(new EditSettlementDialog(canvas, tile.getIndianSettlement()));
+                        canvas.showEditSettlementDialog(tile.getIndianSettlement());
                     }
                 } else {
                     gui.setSelectedTile(null, true);
@@ -187,7 +191,8 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
         }
         JComponent component = (JComponent)e.getSource();
 
-        MapEditorController controller = canvas.getClient().getMapEditorController();
+        MapEditorController controller = freeColClient
+            .getMapEditorController();
         boolean isTransformActive = controller.getMapTransform() != null;
 
         if(startPoint == null){
@@ -198,11 +203,11 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
         }
         drawBox(component, startPoint, oldPoint);
         if (gui.getFocus() != null) {
-            Tile start = gui.convertToMapTile(startPoint.x, startPoint.y);
+            Tile start = mapViewer.convertToMapTile(startPoint.x, startPoint.y);
             Tile end = start;
             //Optimization, only check if the points are different
             if(startPoint.x != oldPoint.x || startPoint.y != oldPoint.y){
-            	end = gui.convertToMapTile(oldPoint.x, oldPoint.y);
+            	end = mapViewer.convertToMapTile(oldPoint.x, oldPoint.y);
             }
 
             // no option selected, just center map
@@ -248,7 +253,7 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
                     }
                 }
             }
-            canvas.refresh();
+            gui.refresh();
             canvas.requestFocus();
         }
     }
@@ -264,7 +269,8 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
         }
 
         if (e.getComponent().isEnabled() &&
-            canvas.getClient().getClientOptions().getBoolean(ClientOptions.AUTO_SCROLL)) {
+            freeColClient.getClientOptions()
+            .getBoolean(ClientOptions.AUTO_SCROLL)) {
             auto_scroll(e.getX(), e.getY());
         } else if (scrollThread != null) {
             scrollThread.stopScrolling();
@@ -287,13 +293,14 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
         oldPoint = e.getPoint();
         drawBox(component, startPoint, oldPoint);
         if (e.getComponent().isEnabled() &&
-            canvas.getClient().getClientOptions().getBoolean(ClientOptions.MAP_SCROLL_ON_DRAG)) {
+            freeColClient.getClientOptions()
+            .getBoolean(ClientOptions.MAP_SCROLL_ON_DRAG)) {
             drag_scroll(e.getX(), e.getY());
         } else if (scrollThread != null) {
             scrollThread.stopScrolling();
             scrollThread = null;
         }
-        canvas.refresh();
+        gui.refresh();
     }
 
     private void drawBox(JComponent component, Point startPoint, Point endPoint) {
@@ -305,7 +312,7 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
         }
 
         // only bother to draw if a transformation is active
-        MapEditorController controller = canvas.getClient().getMapEditorController();
+        MapEditorController controller = freeColClient.getMapEditorController();
         if(controller.getMapTransform() == null){
         	return;
         }
@@ -343,22 +350,22 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
         if ((x < scrollspace) && (y < scrollspace)) {
             // Upper-Left
             direction = Direction.NW;
-        } else if ((x >= gui.getWidth() - scrollspace) && (y < scrollspace)) {
+        } else if ((x >= mapViewer.getWidth() - scrollspace) && (y < scrollspace)) {
             // Upper-Right
             direction = Direction.NE;
-        } else if ((x >= gui.getWidth() - scrollspace) && (y >= gui.getHeight() - scrollspace)) {
+        } else if ((x >= mapViewer.getWidth() - scrollspace) && (y >= mapViewer.getHeight() - scrollspace)) {
             // Bottom-Right
             direction = Direction.SE;
-        } else if ((x < scrollspace) && (y >= gui.getHeight() - scrollspace)) {
+        } else if ((x < scrollspace) && (y >= mapViewer.getHeight() - scrollspace)) {
             // Bottom-Left
             direction = Direction.SW;
         } else if (y < scrollspace) {
             // Top
             direction = Direction.N;
-        } else if (x >= gui.getWidth() - scrollspace) {
+        } else if (x >= mapViewer.getWidth() - scrollspace) {
             // Right
             direction = Direction.E;
-        } else if (y >= gui.getHeight() - scrollspace) {
+        } else if (y >= mapViewer.getHeight() - scrollspace) {
             // Bottom
             direction = Direction.S;
         } else if (x < scrollspace) {
@@ -378,7 +385,7 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
             scrollThread.setDirection(direction);
         } else {
             // start scrolling in a direction
-            scrollThread = new ScrollThread(gui);
+            scrollThread = new ScrollThread(mapViewer);
             scrollThread.setDirection(direction);
             scrollThread.start();
         }
@@ -390,7 +397,7 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
      */
     private class ScrollThread extends Thread {
 
-        private final GUI gui;
+        private final MapViewer mapViewer;
 
         private Direction direction;
 
@@ -400,12 +407,12 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
         /**
          * The constructor to use.
          *
-         * @param g The GUI that holds information such as screen resolution.
+         * @param mapViewer The GUI that holds information such as screen resolution.
          */
-        public ScrollThread(GUI g) {
+        public ScrollThread(MapViewer mapViewer) {
             super(FreeCol.CLIENT_THREAD+"Mouse scroller");
-            gui = g;
-            cont = true;
+            this.mapViewer = mapViewer;
+            this.cont = true;
         }
 
         /**
@@ -440,7 +447,7 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
                             public void run() {
                                 try {
                                     int x, y;
-                                    Tile t = gui.getFocus();
+                                    Tile t = mapViewer.getFocus();
                                     if (t == null) {
                                         return;
                                     }
@@ -450,53 +457,53 @@ public final class CanvasMapEditorMouseListener implements MouseListener, MouseM
                                         return;
                                     }
 
-                                    if (gui.isMapNearTop(t.getY()) && gui.isMapNearTop(gui.getFocus().getY())) {
-                                        if (t.getY() > gui.getFocus().getY()) {
+                                    if (mapViewer.isMapNearTop(t.getY()) && mapViewer.isMapNearTop(mapViewer.getFocus().getY())) {
+                                        if (t.getY() > mapViewer.getFocus().getY()) {
                                             y = t.getY();
                                             do {
                                                 y += 2;
-                                            } while (gui.isMapNearTop(y));
+                                            } while (mapViewer.isMapNearTop(y));
                                         } else {
-                                            y = gui.getFocus().getY();
+                                            y = mapViewer.getFocus().getY();
                                         }
-                                    } else if (gui.isMapNearBottom(t.getY()) && gui.isMapNearBottom(gui.getFocus().getY())) {
-                                        if (t.getY() < gui.getFocus().getY()) {
+                                    } else if (mapViewer.isMapNearBottom(t.getY()) && mapViewer.isMapNearBottom(mapViewer.getFocus().getY())) {
+                                        if (t.getY() < mapViewer.getFocus().getY()) {
                                             y = t.getY();
                                             do {
                                                 y -= 2;
-                                            } while (gui.isMapNearBottom(y));
+                                            } while (mapViewer.isMapNearBottom(y));
                                         } else {
-                                            y = gui.getFocus().getY();
+                                            y = mapViewer.getFocus().getY();
                                         }
                                     } else {
                                         y = t.getY();
                                     }
 
-                                    if (gui.isMapNearLeft(t.getX(), t.getY())
-                                        && gui.isMapNearLeft(gui.getFocus().getX(), gui.getFocus().getY())) {
-                                        if (t.getX() > gui.getFocus().getX()) {
+                                    if (mapViewer.isMapNearLeft(t.getX(), t.getY())
+                                        && mapViewer.isMapNearLeft(mapViewer.getFocus().getX(), mapViewer.getFocus().getY())) {
+                                        if (t.getX() > mapViewer.getFocus().getX()) {
                                             x = t.getX();
                                             do {
                                                 x++;
-                                            } while (gui.isMapNearLeft(x, y));
+                                            } while (mapViewer.isMapNearLeft(x, y));
                                         } else {
-                                            x = gui.getFocus().getX();
+                                            x = mapViewer.getFocus().getX();
                                         }
-                                    } else if (gui.isMapNearRight(t.getX(), t.getY())
-                                               && gui.isMapNearRight(gui.getFocus().getX(), gui.getFocus().getY())) {
-                                        if (t.getX() < gui.getFocus().getX()) {
+                                    } else if (mapViewer.isMapNearRight(t.getX(), t.getY())
+                                               && mapViewer.isMapNearRight(mapViewer.getFocus().getX(), mapViewer.getFocus().getY())) {
+                                        if (t.getX() < mapViewer.getFocus().getX()) {
                                             x = t.getX();
                                             do {
                                                 x--;
-                                            } while (gui.isMapNearRight(x, y));
+                                            } while (mapViewer.isMapNearRight(x, y));
                                         } else {
-                                            x = gui.getFocus().getX();
+                                            x = mapViewer.getFocus().getX();
                                         }
                                     } else {
                                         x = t.getX();
                                     }
 
-                                    gui.setFocus(getMap().getTile(x,y));
+                                    mapViewer.setFocus(getMap().getTile(x,y));
                                 } catch (Exception e) {
                                     logger.log(Level.WARNING, "Exception while scrolling!", e);
                                 }
