@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -26,7 +26,6 @@ import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.util.ArrayList;
 import java.util.Collections;
-import java.util.Iterator;
 import java.util.List;
 import java.util.logging.Logger;
 
@@ -41,11 +40,11 @@ import net.sf.freecol.client.gui.action.UnloadAction;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.client.gui.panel.ChoiceItem;
 import net.sf.freecol.client.gui.panel.ReportPanel;
-import net.sf.freecol.client.gui.panel.TilePanel;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.CombatModel.CombatOdds;
 import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.Goods;
+import net.sf.freecol.common.model.GoodsContainer;
 import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.IndianSettlement;
 import net.sf.freecol.common.model.LostCityRumour.RumourType;
@@ -57,10 +56,8 @@ import net.sf.freecol.common.model.Tension;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.TradeRoute;
 import net.sf.freecol.common.model.Unit;
-import net.sf.freecol.common.model.Unit.MoveType;
-import net.sf.freecol.common.model.Unit.UnitState;
 import net.sf.freecol.common.model.UnitType;
-import net.sf.freecol.common.networking.Message;
+import net.sf.freecol.common.networking.DOMMessage;
 import net.sf.freecol.server.ai.AIColony;
 import net.sf.freecol.server.ai.AIGoods;
 import net.sf.freecol.server.ai.AIUnit;
@@ -83,30 +80,28 @@ public final class TilePopup extends JPopupMenu {
     public static final int UNIT_LINES_IN_FIRST_MENU = 9;
     public static final int UNIT_LINES_IN_OTHER_MENUS = 19;
 
-    private final Canvas canvas;
     private final FreeColClient freeColClient;
     private final GUI gui;
+    private final MapViewer mapViewer;
 
     private boolean hasAnItem = false;
 
     /**
      * The constructor that will insert the MenuItems.
-     *
+     * @param freeColClient The main controller object for the client.
      * @param tile The <code>Tile</code> to create a popup for.
      *       The popup menu also appears near this <code>Tile</code>.
-     * @param freeColClient The main controller object for the client.
-     * @param canvas The component containing the map.
+     * @param gui The GUI frontend.
      * @param gui An object with methods used for making the popup.
      */
-    public TilePopup(final Tile tile, final FreeColClient freeColClient,
-                     final Canvas canvas, final GUI gui) {
+    public TilePopup(final FreeColClient freeColClient, final GUI gui, final Tile tile) {
         super(Messages.message(StringTemplate.template("tile")
                                .addAmount("%x%", tile.getX())
                                .addAmount("%y%", tile.getY())));
 
-        this.canvas = canvas;
         this.freeColClient = freeColClient;
         this.gui = gui;
+        this.mapViewer = gui.getMapViewer();
 
         final Player player = freeColClient.getMyPlayer();
         final Unit activeUnit = gui.getActiveUnit();
@@ -115,7 +110,7 @@ public final class TilePopup extends JPopupMenu {
             JMenuItem gotoMenuItem = null;
             if (activeUnit.isOffensiveUnit() &&
                 unitTile.isAdjacent(tile) &&
-                activeUnit.getMoveType(tile) == MoveType.ATTACK) {
+                activeUnit.getMoveType(tile).isAttack()) {
                 CombatOdds combatOdds = activeUnit.getGame().getCombatModel()
                     .calculateCombatOdds(activeUnit, tile.getDefendingUnit(activeUnit));
 
@@ -129,7 +124,7 @@ public final class TilePopup extends JPopupMenu {
                 }
                 gotoMenuItem = new JMenuItem(Messages.message(StringTemplate.
                          template("attackTileOdds").addName("%chance%", victoryPercent)));
-            } else if (activeUnit.getSimpleMoveType(unitTile, tile, false).isLegal()) {
+            } else if (activeUnit.getSimpleMoveType(unitTile, tile).isLegal()) {
                 //final Image gotoImage = (Image) UIManager.get("cursor.go.image");
                 //JMenuItem gotoMenuItem = new JMenuItem(Messages.message("gotoThisTile"), new ImageIcon(gotoImage));
                 gotoMenuItem = new JMenuItem(Messages.message("gotoThisTile"));
@@ -137,7 +132,7 @@ public final class TilePopup extends JPopupMenu {
             if (gotoMenuItem != null) {
                 gotoMenuItem.addActionListener(new ActionListener() {
                         public void actionPerformed(ActionEvent event) {
-                            if (freeColClient.getGame().getCurrentPlayer() != player) {
+                            if (!freeColClient.currentPlayerIsMyPlayer()) {
                             	return;
                             }
                             Tile currTile = activeUnit.getTile();
@@ -151,7 +146,7 @@ public final class TilePopup extends JPopupMenu {
 
                             //if unit did not move, we should show the goto path
                             if(activeUnit.getTile() == currTile){
-                            	gui.updateGotoPathForActiveUnit();
+                            	mapViewer.updateGotoPathForActiveUnit();
                             }
                         }
                     });
@@ -166,12 +161,11 @@ public final class TilePopup extends JPopupMenu {
                     = new JMenuItem(Messages.message(StringTemplate.template("gotoEurope")));
                 europeMenuItem.addActionListener(new ActionListener() {
                         public void actionPerformed(ActionEvent event) {
-                            if (freeColClient.getGame().getCurrentPlayer()
-                                != player) {
+                            if (!freeColClient.currentPlayerIsMyPlayer()) {
                                 return;
                             }
                             freeColClient.getInGameController()
-                                .moveToEurope(activeUnit);
+                                .moveTo(activeUnit, player.getEurope());
                         }
                     });
                 add(europeMenuItem);
@@ -199,7 +193,7 @@ public final class TilePopup extends JPopupMenu {
         int maxUnits = UNIT_LINES_IN_FIRST_MENU;
         Container currentMenu = this;
         boolean moreUnits = false;
-        List<Unit> units = new ArrayList<Unit>(tile.getUnitList());
+        List<Unit> units = tile.getUnitList();
         Collections.sort(units, ReportPanel.unitTypeComparator);
         for (final Unit currentUnit : units) {
 
@@ -225,7 +219,7 @@ public final class TilePopup extends JPopupMenu {
             activateAllItem.addActionListener(new ActionListener() {
                     public void actionPerformed(ActionEvent event) {
                         Unit lastUnit = null;
-                        for (Unit unit: new ArrayList<Unit>(tile.getUnitList())) {
+                        for (Unit unit : tile.getUnitList()) {
                             freeColClient.getInGameController().clearOrders(unit);
                             lastUnit = unit;
                         }
@@ -270,7 +264,7 @@ public final class TilePopup extends JPopupMenu {
                         JMenuItem menuItem = new JMenuItem(currentUnit.toString());
                         menuItem.addActionListener(new ActionListener() {
                                 public void actionPerformed(ActionEvent event) {
-                                    canvas.showInformationMessage(au.getMission().toString());
+                                    gui.showInformationMessage(au.getMission().toString());
                                 }
                             });
                         transportLists.add(menuItem);
@@ -307,24 +301,21 @@ public final class TilePopup extends JPopupMenu {
                             .getAIMain().getAIColony(serverTile.getColony());
                         StringBuilder info = new StringBuilder(ac.getColonyPlan().toString());
                         info.append("\n\nTILE IMPROVEMENTS:\n");
-                        Iterator<TileImprovementPlan> tipIt = ac.getTileImprovementPlanIterator();
-                        while (tipIt.hasNext()) {
-                            info.append(tipIt.next().toString());
+                        for (TileImprovementPlan tip : ac.getTileImprovementPlans()) {
+                            info.append(tip.toString());
                             info.append("\n");
                         }
                         info.append("\n\nWISHES:\n");
-                        Iterator<Wish> wishIterator = ac.getWishIterator();
-                        while (wishIterator.hasNext()) {
-                            info.append(wishIterator.next().toString());
+                        for (Wish w : ac.getWishes()) {
+                            info.append(w.toString());
                             info.append("\n");
                         }
                         info.append("\n\nEXPORT GOODS:\n");
-                        Iterator<AIGoods> goodsIterator = ac.getAIGoodsIterator();
-                        while (goodsIterator.hasNext()) {
-                            info.append(goodsIterator.next().toString());
+                        for (AIGoods aig : ac.getAIGoods()) {
+                            info.append(aig.toString());
                             info.append("\n");
                         }
-                        canvas.showInformationMessage(info.toString());
+                        gui.showInformationMessage(info.toString());
                     }
                 });
                 add(displayColonyPlan);
@@ -336,7 +327,7 @@ public final class TilePopup extends JPopupMenu {
                         public void actionPerformed(ActionEvent event) {
                             final IndianSettlement sis = (IndianSettlement)
                                 serverGame.getFreeColGameObject(is.getId());
-                            canvas.showInformationMessage(
+                            gui.showInformationMessage(
                                 debugSummarizeSettlement(serverGame, sis));
                         }
                     });
@@ -367,15 +358,43 @@ public final class TilePopup extends JPopupMenu {
                 });
             add(addu);
 
+            if (!tile.isEmpty()) {
+                final List<Unit> tileUnits = tile.getUnitList();
+                JMenuItem adda = new JMenuItem("Reset moves");
+                adda.setOpaque(false);
+                adda.addActionListener(new ActionListener() {
+                        public void actionPerformed(ActionEvent event) {
+                            debugResetUnitsMoves(serverGame, tileUnits);
+                        }
+                    });
+                add(adda);
+            }
+
+            for (Unit u : tile.getUnitList()) {
+                if (u.getSpaceLeft() > 0) {
+                    final Unit unit = u;
+                    JMenuItem addg = new JMenuItem("Add goods");
+                    addg.setOpaque(false);
+                    addg.addActionListener(new ActionListener() {
+                            public void actionPerformed(ActionEvent event) {
+                                debugAddGoodsToUnit(serverGame, unit);
+                            }
+                        });
+                    add(addg);
+                    break;
+                }
+            }
+
             JMenuItem dumpItem = new JMenuItem("Dump tile");
             dumpItem.setOpaque(false);
             dumpItem.addActionListener(new ActionListener() {
                     public void actionPerformed(ActionEvent event) {
-                        System.err.println("\nClient side:\n");
+                        System.err.println("\nClient side:");
                         tile.dumpObject();
-                        System.err.println("\nServer side:\n");
+                        System.err.println("\n\nServer side:");
                         serverGame.getFreeColGameObject(tile.getId())
                             .dumpObject();
+                        System.err.println("\n");
                     }
                 });
             add(dumpItem);
@@ -464,7 +483,7 @@ public final class TilePopup extends JPopupMenu {
 
         if (hasGoods) {
             JMenuItem dumpItem = new JMenuItem(Messages.message("dumpCargo"));
-            dumpItem.setAction(new UnloadAction(freeColClient, unit));
+            dumpItem.setAction(new UnloadAction(freeColClient, gui, unit));
             menu.add(dumpItem);
             lineCount++;
         }
@@ -481,7 +500,7 @@ public final class TilePopup extends JPopupMenu {
         JMenuItem menuItem = new JMenuItem(Messages.message(name));
         menuItem.addActionListener(new ActionListener() {
                 public void actionPerformed(ActionEvent event) {
-                    canvas.showColonyPanel(colony);
+                    gui.showColonyPanel(colony);
                 }
             });
 
@@ -511,7 +530,7 @@ public final class TilePopup extends JPopupMenu {
         JMenuItem menuItem = new JMenuItem(Messages.message(name));
         menuItem.addActionListener(new ActionListener() {
                 public void actionPerformed(ActionEvent event) {
-                    canvas.showIndianSettlementPanel(settlement);
+                    gui.showIndianSettlementPanel(settlement);
                 }
             });
         add(menuItem);
@@ -527,7 +546,7 @@ public final class TilePopup extends JPopupMenu {
         JMenuItem menuItem = new JMenuItem(Messages.message(tile.getNameKey()));
         menuItem.addActionListener(new ActionListener() {
                 public void actionPerformed(ActionEvent event) {
-                    canvas.showPanel(new TilePanel(canvas, tile));
+                    gui.showTilePanel(tile);
                 }
             });
 
@@ -564,8 +583,7 @@ public final class TilePopup extends JPopupMenu {
             rumours.add(new ChoiceItem<RumourType>(rumour.toString(), rumour));
         }
 
-        RumourType rumourChoice = freeColClient.getCanvas()
-            .showChoiceDialog(null, "Select Lost City Rumour", "Cancel",
+        RumourType rumourChoice = gui.showChoiceDialog(null, "Select Lost City Rumour", "Cancel",
                               rumours);
         tile.getTileItemContainer().getLostCityRumour().setType(rumourChoice);
         final Tile serverTile = (Tile) serverGame
@@ -587,8 +605,7 @@ public final class TilePopup extends JPopupMenu {
             uts.add(new ChoiceItem<UnitType>(Messages.message(t.toString()
                                                               + ".name"), t));
         }
-        UnitType unitChoice = freeColClient.getCanvas()
-            .showChoiceDialog(null, "Select Unit Type", "Cancel", uts);
+        UnitType unitChoice = gui.showChoiceDialog(null, "Select Unit Type", "Cancel", uts);
         if (unitChoice == null) return;
 
         Player player = freeColClient.getMyPlayer();
@@ -596,17 +613,86 @@ public final class TilePopup extends JPopupMenu {
             .getFreeColGameObject(player.getId());
         Tile serverTile = (Tile) serverGame
             .getFreeColGameObject(tile.getId());
-        ServerUnit serverUnit
-            = new ServerUnit(serverGame, serverTile, serverPlayer,
-                             unitChoice, UnitState.ACTIVE,
-                             unitChoice.getDefaultEquipment());
+        Unit carrier = null;
+        if (!serverTile.isLand() && !unitChoice.isNaval()) {
+            for (Unit u : serverTile.getUnitList()) {
+                if (u.isNaval()
+                    && u.getSpaceLeft() >= unitChoice.getSpaceTaken()) {
+                    carrier = u;
+                    break;
+                }
+            }
+        }
+        ServerUnit serverUnit = new ServerUnit(serverGame,
+            (carrier != null) ? carrier : serverTile,
+            serverPlayer, unitChoice);
         serverUnit.setMovesLeft(serverUnit.getInitialMovesLeft());
-        Unit unit = new Unit(freeColClient.getGame(),
-                serverUnit.toXMLElement(Message.createNewDocument()));
-        tile.add(unit);
+        Game game = freeColClient.getGame();
+        Unit unit = new Unit(game,
+            serverUnit.toXMLElement(DOMMessage.createNewDocument()));
+        if (carrier == null) {
+            tile.add(unit);
+        } else {
+            ((Unit)game.getFreeColGameObject(carrier.getId())).add(unit);
+        }
         gui.setActiveUnit(unit);
         player.invalidateCanSeeTiles();
-        canvas.refresh();
+        gui.refresh();
+    }
+
+    /**
+     * Debug action to reset the moves left of the units on a tile.
+     *
+     * @param serverGame The server <code>Game</code> containing the tile.
+     * @param units The <code>Unit</code>s to reactivate.
+     */
+    private void debugResetUnitsMoves(final Game serverGame, List<Unit> units) {
+        boolean first = true;
+        for (Unit u : units) {
+            Unit su = (Unit)serverGame.getFreeColGameObject(u.getId());
+            u.setMovesLeft(u.getInitialMovesLeft());
+            su.setMovesLeft(su.getInitialMovesLeft());
+            if (first) {
+                gui.setActiveUnit(u);
+                first = false;
+            }
+        }
+        gui.refresh();
+    }
+
+    /**
+     * Debug action to add goods to a unit.
+     *
+     * @param serverGame The server <code>Game</code> containing the tile.
+     * @param tile The <code>Unit</code> to add to.
+     */
+    private void debugAddGoodsToUnit(final Game serverGame, Unit unit) {
+        Specification spec = serverGame.getSpecification();
+        List<ChoiceItem<GoodsType>> gtl
+            = new ArrayList<ChoiceItem<GoodsType>>();
+        for (GoodsType t : spec.getGoodsTypeList()) {
+            if (t.isFoodType() && t != spec.getPrimaryFoodType()) continue;
+            gtl.add(new ChoiceItem<GoodsType>(Messages.message(t.toString() + ".name"),
+                                              t));
+        }
+        GoodsType goodsType = gui.showChoiceDialog(null, "Select Goods Type", "Cancel", gtl);
+        if (goodsType == null) return;
+        String amount = gui.showInputDialog(null,
+            StringTemplate.name("Select Goods Amount"), "20",
+            "ok", "cancel", true);
+        if (amount == null) return;
+        int a;
+        try {
+            a = Integer.parseInt(amount);
+        } catch (NumberFormatException nfe) {
+            return;
+        }
+        GoodsType sGoodsType = spec.getGoodsType(goodsType.getId());
+        GoodsContainer ugc = unit.getGoodsContainer();
+        GoodsContainer sgc = (GoodsContainer) serverGame
+            .getFreeColGameObject(ugc.getId());
+        ugc.setAmount(goodsType, a);
+        sgc.setAmount(sGoodsType, a);
     }
 
     /**
