@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -21,9 +21,8 @@ package net.sf.freecol.client.gui.action;
 
 import java.awt.event.ActionEvent;
 
-
 import net.sf.freecol.client.FreeColClient;
-import net.sf.freecol.client.gui.panel.ReportExplorationPanel;
+import net.sf.freecol.client.gui.GUI;
 
 /**
  * An action for displaying an Exploration Report.
@@ -35,9 +34,10 @@ public class ReportExplorationAction extends FreeColAction {
     /**
      * Creates this action.
      * @param freeColClient The main controller object for the client.
+     * @param gui 
      */
-    ReportExplorationAction(FreeColClient freeColClient) {
-        super(freeColClient, id);
+    ReportExplorationAction(FreeColClient freeColClient, GUI gui) {
+        super(freeColClient, gui, id);
     }
 
     /**
@@ -45,6 +45,6 @@ public class ReportExplorationAction extends FreeColAction {
      * @param e The <code>ActionEvent</code>.
      */
     public void actionPerformed(ActionEvent e) {
-        freeColClient.getCanvas().showPanel(new ReportExplorationPanel(freeColClient.getCanvas()));
+        gui.showReportExplorationPanel();
     }
 }
