@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -25,6 +25,7 @@ import java.awt.event.ActionEvent;
 
 
 import net.sf.freecol.client.FreeColClient;
+import net.sf.freecol.client.gui.GUI;
 
 
 /**
@@ -37,9 +38,10 @@ public class ReportTurnAction extends FreeColAction {
     /**
      * Creates this action.
      * @param freeColClient The main controller object for the client.
+     * @param gui 
      */
-    ReportTurnAction(FreeColClient freeColClient) {
-        super(freeColClient, id);
+    ReportTurnAction(FreeColClient freeColClient, GUI gui) {
+        super(freeColClient, gui,id);
     }
 
     /**
@@ -47,6 +49,6 @@ public class ReportTurnAction extends FreeColAction {
      * @param e The <code>ActionEvent</code>.
      */
     public void actionPerformed(ActionEvent e) {
-        freeColClient.getInGameController().displayModelMessages(true);
+        freeColClient.getInGameController().displayModelMessages(true, true);
     }
 }
