@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -22,8 +22,7 @@ package net.sf.freecol.client.gui.action;
 import java.awt.event.ActionEvent;
 
 import net.sf.freecol.client.FreeColClient;
-import net.sf.freecol.client.gui.Canvas;
-import net.sf.freecol.client.gui.panel.TradeRouteDialog;
+import net.sf.freecol.client.gui.GUI;
 
 /**
  * An action for editing trade routes.
@@ -36,9 +35,10 @@ public class TradeRouteAction extends MapboardAction {
     /**
      * Creates this action.
      * @param freeColClient The main controller object for the client.
+     * @param gui 
      */
-    TradeRouteAction(FreeColClient freeColClient) {
-        super(freeColClient, id);
+    TradeRouteAction(FreeColClient freeColClient, GUI gui) {
+        super(freeColClient, gui, id);
     }
 
     /**
@@ -46,7 +46,6 @@ public class TradeRouteAction extends MapboardAction {
      * @param e The <code>ActionEvent</code>.
      */
     public void actionPerformed(ActionEvent e) {
-        Canvas canvas = getFreeColClient().getCanvas();
-        canvas.showFreeColDialog(new TradeRouteDialog(canvas, null));
+        gui.showTradeRouteDialog(null, null);
     }
 }
