@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -43,15 +43,45 @@ import net.sf.freecol.common.model.StringTemplate;
 import net.sf.freecol.common.model.StringTemplate.TemplateType;
 import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.model.UnitType;
+import net.sf.freecol.common.option.Option;
 
 /**
- * Represents a collection of messages in a particular locale. <p/>
+ * <p>Represents a collection of messages in a particular locale.</p>
  *
- * This class is NOT thread-safe. (CO: I cannot find any place that really has a
- * problem) <p/>
+ * <p>The individual messages are read from property files in the
+ * <code>data/strings</code> directory. The property files are called
+ * "FreeColMessages[_LANGUAGE[_COUNTRY[_VARIANT]]].properties", where
+ * LANGUAGE should be an ISO 639-2 or ISO 639-3 language code, COUNTRY
+ * should be an ISO 3166-2 country code, and VARIANT is an arbitrary
+ * string. The encoding of the property files is UTF-8. Since the Java
+ * Properties class is unable to handle UTF-8 directly, this class
+ * uses its own implementation.</p>
+ *
+ * <p>The individual messages may include variables, which must be
+ * delimited by percent characters (e.g. "%nation%"), and will be
+ * replaced when the message is formatted. Furthermore, the messages
+ * may include choice formats consisting of a tag followed by a colon
+ * (":"), a selector and one or several choices separated from the
+ * selector and each other by pipe characters ("|"). The entire choice
+ * format must be enclosed in double brackets ("{{" and "}}",
+ * respectively).</p>
+ *
+ * <p>Each choice must consist of a key and a value separated by an
+ * equals character ("="), unless it is a variable, in which case the
+ * variable must resolve to another choice format. The selector may
+ * also be a variable. If the selector is omitted, then one of the
+ * choices should use the key "default". Choice formats may be
+ * nested.</p>
+ *
+ * <pre>
+ *   key1=%colony% tuottaa tuotetta {{tag:acc|%goods%}}.
+ *   key2={{plural:%amount%|one=ruoka|other=ruokaa|default={{tag:|acc=viljaa|default=Vilja}}}}
+ *   key3={{tag:|acc=viljaa|default={{plural:%amount%|one=ruoka|other=ruokaa|default=Ruoka}}}}
+ * </pre>
+ *
+ * <p>This class is NOT thread-safe. (CO: I cannot find any place that
+ * really has a problem)</p>
  *
- * Messages are put in the file "FreeColMessages.properties". This file is
- * presently located in the same directory as the source file of this class.
  */
 public class Messages {
 
@@ -63,14 +93,40 @@ public class Messages {
 
     public static final String FILE_SUFFIX = ".properties";
 
+    private static final String[] DESCRIPTION_KEYS = new String[] {
+        ".description", ".shortDescription", ".name"
+    };
+
+
     private static Map<String, String> messageBundle =
         new HashMap<String, String>();
 
-    private static Number grammaticalNumber = NumberRules.OTHER_NUMBER_RULE;
+    /**
+     * A map with Selector values and the tag keys used in choice
+     * formats.
+     */
+    private static Map<String, Selector> tagMap =
+        new HashMap<String, Selector>();
 
 
+
+    /**
+     * Returns the Selector with the given tag.
+     *
+     * @param tag a <code>String</code> value
+     * @return a <code>Selector</code> value
+     */
+    private static Selector getSelector(String tag) {
+        return tagMap.get(tag.toLowerCase(Locale.US));
+    }
+
+    /**
+     * Set the grammatical number rule.
+     *
+     * @param number a <code>Number</code> value
+     */
     public static void setGrammaticalNumber(Number number) {
-        grammaticalNumber = number;
+        tagMap.put("plural", number);
     }
 
     /**
@@ -121,14 +177,11 @@ public class Messages {
                 }
             } else {
                 logger.warning("Could not find string directory: "
-                               + stringDirectory.getName());
+                               + stringDirectory.getPath());
             }
         }
 
-        Number number = NumberRules.getNumberForLanguage(language);
-        if (number != null) {
-            grammaticalNumber = number;
-        }
+        setGrammaticalNumber(NumberRules.getNumberForLanguage(language));
 
         for (String fileName : filenames) {
             File resourceFile = new File(getI18nDirectory(), fileName);
@@ -145,10 +198,10 @@ public class Messages {
                     InputStream is = fcmf.getInputStream(fileName);
                     loadResources(is);
                     logger.finest("Loaded message bundle " + fileName + " from "
-                                  + fcmf.getModInfo().getName() + ".");
+                                  + fcmf.getId() + ".");
                 } catch (IOException e) {
                     logger.fine("No message bundle " + fileName + " in "
-                                + fcmf.getModInfo().getName() + ".");
+                                + fcmf.getId() + ".");
                 }
             }
         }
@@ -244,13 +297,17 @@ public class Messages {
                         continue;
                     } else {
                         selector = message(replacement);
-                        if ("plural".equalsIgnoreCase(tag)) {
-                            selector = grammaticalNumber.getKey(selector);
+                        Selector taggedSelector = getSelector(tag);
+                        if (taggedSelector != null) {
+                            selector = taggedSelector.getKey(selector, input);
                         }
                     }
                 }
-            } else if ("plural".equalsIgnoreCase(tag)) {
-                selector = grammaticalNumber.getKey(selector);
+            } else {
+                Selector taggedSelector = getSelector(tag);
+                if (taggedSelector != null) {
+                    selector = taggedSelector.getKey(selector, input);
+                }
             }
             int keyIndex = input.indexOf(selector, pipeIndex + 1);
             if (keyIndex < 0 || keyIndex > closeChoice) {
@@ -287,8 +344,24 @@ public class Messages {
                     otherKey = getChoice(messageBundle.get(otherKey), selector);
                     result.append(otherKey);
                 } else {
-                    logger.warning("Unknown key or untagged choice: " + otherKey);
-                    continue;
+                    logger.warning("Unknown key or untagged choice: '" + otherKey
+                                   + "', selector was '" + selector
+                                   + "', trying 'default' instead");
+                    int defaultStart = otherKey.indexOf("default=");
+                    if (defaultStart >= 0) {
+                        defaultStart += 8;
+                        int defaultEnd = otherKey.indexOf('|', defaultStart);
+                        String defaultChoice;
+                        if (defaultEnd < 0) {
+                            defaultChoice = otherKey.substring(defaultStart);
+                        } else {
+                            defaultChoice = otherKey.substring(defaultStart, defaultEnd);
+                        }
+                        result.append(defaultChoice);
+                    } else {
+                        logger.warning("No default choice found.");
+                        continue;
+                    }
                 }
             } else {
                 int start = keyIndex + selector.length() + 1;
@@ -388,7 +461,8 @@ public class Messages {
         String result = "";
         switch (template.getTemplateType()) {
         case LABEL:
-            if (template.getReplacements() == null) {
+            if (template.getReplacements() == null
+                || template.getReplacements().isEmpty()) {
                 return message(template.getId());
             } else {
                 for (StringTemplate other : template.getReplacements()) {
@@ -397,7 +471,7 @@ public class Messages {
                 if (result.length() > template.getId().length()) {
                     return result.substring(template.getId().length());
                 } else {
-                    logger.warning("Incorrect use of template with id " + template.getId());
+                    logger.warning("incorrect use of template " + template.toString());
                     return result;
                 }
             }
@@ -471,6 +545,25 @@ public class Messages {
     }
 
 
+    public static String getName(Option object) {
+        return message(object.getId() + ".name");
+    }
+
+    public static String getDescription(Option object) {
+        for (String suffix : DESCRIPTION_KEYS) {
+            String key = object.getId() + suffix;
+            if (containsKey(key)) {
+                return message(key);
+            }
+        }
+        return object.getId();
+    }
+
+    public static String getShortDescription(Option object) {
+        return message(object.getId() + ".shortDescription");
+    }
+
+
     /**
      * Returns the name of a unit in a human readable format. The
      * label consists of up to three items: If the unit has a role
@@ -521,15 +614,16 @@ public class Messages {
     }
 
 
-     /**
+    /**
      * Returns the name of a unit in a human readable format. The return value
      * can be used when communicating with the user.
      *
      * @param someType an <code>UnitType</code> value
      * @param someRole a <code>Role</code> value
+     * @param count an <code>int</code> value
      * @return The given unit type as a String
      */
-    public static String getLabel(UnitType someType, Unit.Role someRole) {
+    public static String getLabel(UnitType someType, Unit.Role someRole, int count) {
         String key = someRole.toString().toLowerCase();
         if (someRole == Unit.Role.DEFAULT) {
             key = "name";
@@ -539,6 +633,7 @@ public class Messages {
             return message(messageID);
         } else {
             return message(StringTemplate.template("model.unit." + key + ".name")
+                           .addAmount("%number%", count)
                            .addName("%unit%", someType));
         }
     }
@@ -706,5 +801,26 @@ public class Messages {
         }
     }
 
+    /**
+     * Breaks a line between two words. The breaking point
+     * is as close to the center as possible.
+     *
+     * @param string The line for which we should determine a
+     *               breaking point.
+     * @return The best breaking point or <code>-1</code> if there
+     *         are none.
+     */
+     public static int getBreakingPoint(String string) {
+         int center = string.length() / 2;
+         for (int offset = 0; offset < center; offset++) {
+             if (string.charAt(center + offset) == ' ') {
+                 return center + offset;
+             } else if (string.charAt(center - offset) == ' ') {
+                 return center - offset;
+             }
+         }
+         return -1;
+     }
+
 
 }
