@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -25,13 +25,12 @@ import java.awt.datatransfer.DataFlavor;
 import java.awt.datatransfer.Transferable;
 import java.awt.datatransfer.UnsupportedFlavorException;
 import java.awt.event.ActionEvent;
-import java.awt.event.ItemEvent;
 import java.awt.event.ActionListener;
+import java.awt.event.ItemEvent;
 import java.awt.event.ItemListener;
 import java.awt.event.MouseAdapter;
 import java.awt.event.MouseEvent;
 import java.beans.PropertyChangeEvent;
-
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -48,8 +47,8 @@ import javax.swing.ImageIcon;
 import javax.swing.JButton;
 import javax.swing.JCheckBox;
 import javax.swing.JComponent;
-import javax.swing.JList;
 import javax.swing.JLabel;
+import javax.swing.JList;
 import javax.swing.JPanel;
 import javax.swing.JScrollPane;
 import javax.swing.KeyStroke;
@@ -60,11 +59,12 @@ import javax.swing.SwingConstants;
 import javax.swing.TransferHandler;
 import javax.swing.plaf.PanelUI;
 
+import net.miginfocom.swing.MigLayout;
+import net.sf.freecol.client.FreeColClient;
+import net.sf.freecol.client.gui.GUI;
 import net.sf.freecol.client.gui.i18n.Messages;
-import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.plaf.FreeColComboBoxRenderer;
 import net.sf.freecol.client.gui.plaf.FreeColSelectedPanelUI;
-
 import net.sf.freecol.common.model.Ability;
 import net.sf.freecol.common.model.AbstractGoods;
 import net.sf.freecol.common.model.BuildableType;
@@ -79,8 +79,6 @@ import net.sf.freecol.common.model.UnitType;
 import net.sf.freecol.common.resources.ResourceManager;
 import net.sf.freecol.common.util.Utils;
 
-import net.miginfocom.swing.MigLayout;
-
 
 public class BuildQueuePanel extends FreeColPanel implements ActionListener, ItemListener {
 
@@ -114,16 +112,16 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
      */
     private List<UnitType> buildableUnits = new ArrayList<UnitType>();
 
-    public BuildQueuePanel(Colony colony, Canvas parent) {
+    public BuildQueuePanel(FreeColClient freeColClient, GUI gui, Colony colony) {
 
-        super(parent, new MigLayout("wrap 3", "[260:][390:, fill][260:]", "[][][300:400:][]"));
+        super(freeColClient, gui, new MigLayout("wrap 3", "[260:][390:, fill][260:]", "[][][300:400:][]"));
         this.colony = colony;
         this.unitCount = colony.getUnitCount();
-        featureContainer = new FeatureContainer(getSpecification());
+        featureContainer = new FeatureContainer();
 
         for (UnitType unitType : getSpecification().getUnitTypeList()) {
             if (!(unitType.getGoodsRequired().isEmpty()
-                  || unitType.hasAbility("model.ability.bornInColony"))) {
+                  || unitType.hasAbility(Ability.BORN_IN_COLONY))) {
                 // can be built
                 buildableUnits.add(unitType);
             }
@@ -211,7 +209,7 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
         buyBuilding.setActionCommand(BUY);
         buyBuilding.addActionListener(this);
 
-        constructionPanel = new ConstructionPanel(getCanvas(), colony, false);
+        constructionPanel = new ConstructionPanel(freeColClient, gui, colony, false);
         constructionPanel.setOpaque(false);
         StringTemplate buildingNothing = StringTemplate.template("colonyPanel.currentlyBuilding")
             .add("%buildable%", "nothing");
@@ -257,10 +255,10 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
             }
 
             if (!(colony.getFeatureContainer()
-                  .hasAbility("model.ability.build", unitType, getGame().getTurn())
-                  || featureContainer.hasAbility("model.ability.build", unitType))) {
+                  .hasAbility(Ability.BUILD, unitType, getGame().getTurn())
+                  || featureContainer.hasAbility(Ability.BUILD, unitType))) {
                 boolean builderFound = false;
-                for (Ability ability : getSpecification().getAbilities("model.ability.build")) {
+                for (Ability ability : getSpecification().getAbilities(Ability.BUILD)) {
                     if (ability.appliesTo(unitType)
                         && ability.getValue()
                         && ability.getSource() != null
@@ -375,7 +373,7 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
 
     private void updateAllLists() {
         DefaultListModel current = (DefaultListModel) buildQueueList.getModel();
-        featureContainer = new FeatureContainer(getSpecification());
+        featureContainer = new FeatureContainer();
         for (Object type: current.toArray()) {
             if (getMinimumIndex((BuildableType) type) >= 0) {
                 featureContainer.add(((BuildableType) type).getFeatureContainer());
@@ -450,7 +448,7 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
             } else {
                 for (int index = 0; index < buildQueue.getSize(); index++) {
                     if (((BuildableType) buildQueue.getElementAt(index))
-                        .hasAbility("model.ability.build", buildableType)) {
+                        .hasAbility(Ability.BUILD, buildableType)) {
                         return index + 1;
                     }
                 }
@@ -499,7 +497,7 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
                     continue;
                 }
 
-                if (toBuild.hasAbility("model.ability.build", buildableType)) {
+                if (toBuild.hasAbility(Ability.BUILD, buildableType)) {
                     return buildQueueLastPos;
                 }
             }
@@ -544,7 +542,7 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
                 }
 
                 // Don't go past a unit this building can build.
-                if (buildableType.hasAbility("model.ability.build", toBuild)) {
+                if (buildableType.hasAbility(Ability.BUILD, toBuild)) {
                     return index;
                 }
             }
@@ -566,7 +564,7 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
             String command = event.getActionCommand();
             List<BuildableType> buildables = getBuildableTypes(buildQueueList);
             if (!buildables.isEmpty() && lockReasons.get(buildables.get(0)) != null) {
-                getCanvas().showInformationMessage(buildables.get(0),
+                getGUI().showInformationMessage(buildables.get(0),
                                                    StringTemplate.template("colonyPanel.unbuildable")
                                                    .addName("%colony%", colony.getName())
                                                    .add("%object%", buildables.get(0).getNameKey()));
@@ -581,7 +579,7 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
                 logger.warning("Unsupported command " + command);
             }
         }
-        getCanvas().remove(this);
+        getGUI().removeFromCanvas(this);
     }
 
     public void itemStateChanged(ItemEvent event) {
@@ -1000,13 +998,7 @@ public class BuildQueuePanel extends FreeColPanel implements ActionListener, Ite
                 if ((e.getButton() == MouseEvent.BUTTON3 || e.isPopupTrigger())) {
                     int index = source.locationToIndex(e.getPoint());
                     BuildableType type = (BuildableType) source.getModel().getElementAt(index);
-                    if (type instanceof BuildingType) {
-                        getCanvas().showPanel(new ColopediaPanel(getCanvas(),
-                                                                 ColopediaPanel.PanelType.BUILDINGS, type));
-                    } else if (type instanceof UnitType) {
-                        getCanvas().showPanel(new ColopediaPanel(getCanvas(),
-                                                                 ColopediaPanel.PanelType.UNITS, type));
-                    }
+                    getGUI().showColopediaPanel(type.getId());
                 } else if ((e.getClickCount() > 1 && !e.isConsumed())) {
                         DefaultListModel model = (DefaultListModel) buildQueueList.getModel();
                         if (source.getSelectedIndex() == -1) {
