@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -25,6 +25,7 @@ import java.awt.Container;
 import java.awt.FlowLayout;
 import java.awt.Graphics;
 import java.awt.Graphics2D;
+import java.awt.GridLayout;
 import java.awt.Insets;
 import java.awt.Point;
 import java.awt.Rectangle;
@@ -40,6 +41,7 @@ import java.beans.PropertyChangeListener;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
+import java.util.Locale;
 import java.util.logging.Logger;
 
 import javax.swing.BorderFactory;
@@ -60,10 +62,9 @@ import javax.swing.SwingUtilities;
 import javax.swing.border.BevelBorder;
 
 import net.miginfocom.swing.MigLayout;
-
 import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.ClientOptions;
-import net.sf.freecol.client.gui.Canvas;
+import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.gui.GUI;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.common.model.AbstractGoods;
@@ -76,22 +77,28 @@ import net.sf.freecol.common.model.FreeColGameObject;
 import net.sf.freecol.common.model.Goods;
 import net.sf.freecol.common.model.GoodsContainer;
 import net.sf.freecol.common.model.GoodsType;
-import net.sf.freecol.common.model.IndianSettlement;
 import net.sf.freecol.common.model.Map.Direction;
 import net.sf.freecol.common.model.ModelMessage;
 import net.sf.freecol.common.model.Player;
-import net.sf.freecol.common.model.Settlement;
+import net.sf.freecol.common.model.Player.NoClaimReason;
+import net.sf.freecol.common.model.ProductionInfo;
 import net.sf.freecol.common.model.Specification;
 import net.sf.freecol.common.model.StringTemplate;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.TileType;
 import net.sf.freecol.common.model.TradeRoute;
 import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.UnitLocation.NoAddReason;
+import net.sf.freecol.common.model.UnitType;
 
 
 /**
  * This is a panel for the Colony display. It shows the units that are working
  * in the colony, the buildings and much more.
+ *
+ * Beware that in debug mode, this might be a server-side version of the colony
+ * which is why we need to call getColony().getSpecification() to get the
+ * spec that corresponds to the good types in this colony.
  */
 public final class ColonyPanel extends FreeColPanel
     implements ActionListener, PropertyChangeListener {
@@ -156,13 +163,15 @@ public final class ColonyPanel extends FreeColPanel
         ? new JButton("Set Goods") : null;
 
 
+
     /**
      * The constructor for the panel.
+     * @param freeColClient 
      *
      * @param parent The parent of this panel
      */
-    public ColonyPanel(final Canvas parent, Colony colony) {
-        super(parent);
+    public ColonyPanel(FreeColClient freeColClient, GUI gui, Colony colony) {
+        super(freeColClient, gui);
 
         setFocusCycleRoot(true);
 
@@ -184,7 +193,7 @@ public final class ColonyPanel extends FreeColPanel
 
         netProductionPanel.setOpaque(false);
 
-        constructionPanel = new ConstructionPanel(parent, colony, true);
+        constructionPanel = new ConstructionPanel(freeColClient, gui, colony, true);
         constructionPanel.setOpaque(true);
 
         outsideColonyPanel = new OutsideColonyPanel();
@@ -193,15 +202,15 @@ public final class ColonyPanel extends FreeColPanel
 
         warehousePanel = new WarehousePanel(this);
 
-        tilePanel = new TilePanel(this);
+        tilePanel = new TilePanel(freeColClient, gui, this);
 
         buildingsPanel = new BuildingsPanel(this);
 
-        cargoPanel = new ColonyCargoPanel(parent);
+        cargoPanel = new ColonyCargoPanel(freeColClient, gui);
         cargoPanel.setParentPanel(this);
 
-        defaultTransferHandler = new DefaultTransferHandler(parent, this);
-        pressListener = new DragListener(this);
+        defaultTransferHandler = new DefaultTransferHandler(freeColClient, gui, this);
+        pressListener = new DragListener(getFreeColClient(), gui, this);
         releaseListener = new DropListener();
 
         JScrollPane outsideColonyScroll = new JScrollPane(outsideColonyPanel,
@@ -221,15 +230,13 @@ public final class ColonyPanel extends FreeColPanel
                                                   ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
         JScrollPane buildingsScroll = new JScrollPane(buildingsPanel,
                                                       ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
-                                                      ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
+                                                      ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
         buildingsScroll.getVerticalScrollBar().setUnitIncrement( 16 );
 
         // Make the colony label
         nameBox = new JComboBox();
         nameBox.setFont(smallHeaderFont);
-        List<Colony> settlements = colony.getOwner().getColonies();
-        sortColonies(settlements);
-        for (Colony aColony : settlements) {
+        for (Colony aColony : getSortedColonies()) {
             nameBox.addItem(aColony);
         }
         nameBox.setSelectedItem(colony);
@@ -239,6 +246,14 @@ public final class ColonyPanel extends FreeColPanel
                 }
             });
 
+        // the following actions are pre-defined
+        nameBox.getInputMap().put(KeyStroke.getKeyStroke("LEFT"), "selectPrevious2");
+        nameBox.getInputMap().put(KeyStroke.getKeyStroke("RIGHT"), "selectNext2");
+        InputMap nameInputMap = new ComponentInputMap(nameBox);
+        nameInputMap.put(KeyStroke.getKeyStroke("LEFT"), "selectPrevious2");
+        nameInputMap.put(KeyStroke.getKeyStroke("RIGHT"), "selectNext2");
+        SwingUtilities.replaceUIInputMap(nameBox, JComponent.WHEN_IN_FOCUSED_WINDOW, nameInputMap);
+
         buildingsScroll.setAutoscrolls(true);
         buildingsScroll.getViewport().setOpaque(false);
         buildingsPanel.setOpaque(false);
@@ -380,10 +395,10 @@ public final class ColonyPanel extends FreeColPanel
         colony.invalidateCache();
         netProductionPanel.removeAll();
 
-        for (GoodsType goodsType : getSpecification().getGoodsTypeList()) {
+        for (GoodsType goodsType : getColony().getSpecification().getGoodsTypeList()) {
             int amount = colony.getAdjustedNetProductionOf(goodsType);
             if (amount != 0) {
-                netProductionPanel.add(new ProductionLabel(goodsType, amount, getCanvas()));
+                netProductionPanel.add(new ProductionLabel(getFreeColClient(), getGUI(), goodsType, amount));
             }
         }
 
@@ -394,11 +409,6 @@ public final class ColonyPanel extends FreeColPanel
         Collections.sort(buildings);
     }
 
-    private void sortColonies(List<Colony> colonies) {
-        Collections.sort(colonies, getClient().getClientOptions()
-                         .getColonyComparator());
-    }
-
     /**
      * Analyzes an event and calls the right external methods to take
      * care of the user's request.
@@ -406,7 +416,6 @@ public final class ColonyPanel extends FreeColPanel
      * @param event The incoming action event
      */
     public void actionPerformed(ActionEvent event) {
-        Canvas canvas = getCanvas();
         Colony colony = getColony();
         String command = event.getActionCommand();
         if (OK.equals(command)) {
@@ -418,20 +427,19 @@ public final class ColonyPanel extends FreeColPanel
                     unload();
                     break;
                 case WAREHOUSE:
-                    if (canvas.showFreeColDialog(new WarehouseDialog(canvas,
-                                                                     colony))) {
+                    if (getGUI().showWarehouseDialog(colony)) {
                         updateWarehousePanel();
                     }
                     break;
                 case BUILDQUEUE:
-                    canvas.showSubPanel(new BuildQueuePanel(colony, canvas));
+                    getGUI().showBuildQueuePanel(colony);
                     updateConstructionPanel();
                     break;
                 case FILL:
                     fill();
                     break;
                 case SETGOODS:
-                    debugSetGoods(canvas, colony);
+                    debugSetGoods(colony);
                     break;
                 default:
                     logger.warning("Invalid action");
@@ -449,7 +457,7 @@ public final class ColonyPanel extends FreeColPanel
      * @param canvas The <code>Canvas</code> to use.
      * @param colony The <code>Colony</code> to set goods amounts in.
      */
-    private void debugSetGoods(Canvas canvas, Colony colony) {
+    private void debugSetGoods(Colony colony) {
         Specification spec = getSpecification();
         List<ChoiceItem<GoodsType>> gtl
             = new ArrayList<ChoiceItem<GoodsType>>();
@@ -458,10 +466,10 @@ public final class ColonyPanel extends FreeColPanel
             gtl.add(new ChoiceItem<GoodsType>(Messages.message(t.toString() + ".name"),
                                               t));
         }
-        GoodsType goodsType = canvas.showChoiceDialog(null, "Select Goods Type",
+        GoodsType goodsType = getGUI().showChoiceDialog(null, "Select Goods Type",
                                                       "Cancel", gtl);
         if (goodsType == null) return;
-        String amount = canvas.showInputDialog(null,
+        String amount = getGUI().showInputDialog(null,
                 StringTemplate.name("Select Goods Amount"),
                 Integer.toString(colony.getGoodsCount(goodsType)),
                 "ok", "cancel", true);
@@ -472,11 +480,11 @@ public final class ColonyPanel extends FreeColPanel
         } catch (NumberFormatException nfe) {
             return;
         }
-        GoodsType sGoodsType = getClient().getFreeColServer()
+        GoodsType sGoodsType = getFreeColClient().getFreeColServer()
             .getSpecification().getGoodsType(goodsType.getId());
         GoodsContainer cgc = colony.getGoodsContainer();
-        GoodsContainer sgc = (GoodsContainer) getClient().getFreeColServer()
-            .getGame().getFreeColGameObject(cgc.getId());
+        GoodsContainer sgc = (GoodsContainer) getFreeColClient()
+            .getFreeColServer().getGame().getFreeColGameObject(cgc.getId());
         cgc.setAmount(goodsType, a);
         sgc.setAmount(sGoodsType, a);
         updateConstructionPanel();
@@ -493,7 +501,7 @@ public final class ColonyPanel extends FreeColPanel
             for (Goods goods : unit.getGoodsContainer().getGoods()) {
                 getController().unloadCargo(goods, false);
             }
-            for (Unit u : new ArrayList<Unit>(unit.getUnitList())) {
+            for (Unit u : unit.getUnitList()) {
                 getController().leaveShip(u);
             }
             cargoPanel.initialize();
@@ -667,26 +675,25 @@ public final class ColonyPanel extends FreeColPanel
      * Closes the <code>ColonyPanel</code>.
      */
     public void closeColonyPanel() {
-        Canvas canvas = getCanvas();
         if (getColony().getUnitCount() == 0) {
-            if (canvas.showConfirmDialog("abandonColony.text",
+            if (getGUI().showConfirmDialog("abandonColony.text",
                                          "abandonColony.yes",
                                          "abandonColony.no")) {
-                canvas.remove(this);
+                getGUI().removeFromCanvas(this);
                 getController().abandonColony(getColony());
             }
         } else {
             BuildableType buildable = getColony().getCurrentlyBuilding();
             if (buildable != null
                 && buildable.getPopulationRequired() > getColony().getUnitCount()
-                && !canvas.showConfirmDialog(null, StringTemplate.template("colonyPanel.reducePopulation")
+                && !getGUI().showConfirmDialog(null, StringTemplate.template("colonyPanel.reducePopulation")
                                              .addName("%colony%", getColony().getName())
                                              .addAmount("%number%", buildable.getPopulationRequired())
                                              .add("%buildable%", buildable.getNameKey()),
                                              "ok", "cancel")) {
                 return;
             }
-            canvas.remove(this);
+            getGUI().removeFromCanvas(this);
 
             // remove property listeners
             removePropertyChangeListeners();
@@ -700,16 +707,16 @@ public final class ColonyPanel extends FreeColPanel
             cargoPanel.setCarrier(null);
             outsideColonyPanel.cleanup();
 
-            if (getGame().getCurrentPlayer() == getMyPlayer()) {
+            if (getFreeColClient().currentPlayerIsMyPlayer()) {
                 getController().nextModelMessage();
-                Unit activeUnit = canvas.getGUI().getActiveUnit();
+                Unit activeUnit = getGUI().getActiveUnit();
                 if (activeUnit == null || activeUnit.getTile() == null || activeUnit.getMovesLeft() <= 0
                     || (!(activeUnit.getLocation() instanceof Tile) && !(activeUnit.isOnCarrier()))) {
-                    canvas.getGUI().setActiveUnit(null);
+                    getGUI().setActiveUnit(null);
                     getController().nextActiveUnit();
                 }
             }
-            getClient().getGUI().restartBlinking();
+            getGUI().getMapViewer().restartBlinking();
         }
     }
 
@@ -761,17 +768,17 @@ public final class ColonyPanel extends FreeColPanel
         } else if (ColonyChangeEvent.BONUS_CHANGE.toString().equals(property)) {
             ModelMessage msg = getColony().checkForGovMgtChangeMessage();
             if (msg != null) {
-                getCanvas().showInformationMessage(msg);
+                getGUI().showInformationMessage(msg);
             }
             populationPanel.update();
         } else if (ColonyChangeEvent.UNIT_TYPE_CHANGE.toString().equals(property)) {
             FreeColGameObject object = (FreeColGameObject) event.getSource();
-            String oldType = (String) event.getOldValue();
-            String newType = (String) event.getNewValue();
-            getCanvas().showInformationMessage(object,
-                                               StringTemplate.template("model.colony.unitChange")
-                                               .addName("%oldType%", oldType)
-                                               .addName("%newType%", newType));
+            UnitType oldType = (UnitType) event.getOldValue();
+            UnitType newType = (UnitType) event.getNewValue();
+            getGUI().showInformationMessage(object,
+                StringTemplate.template("model.colony.unitChange")
+                    .add("%oldType%", oldType.getNameKey())
+                    .add("%newType%", newType.getNameKey()));
             updateTilePanel();
         } else if (ColonyTile.UNIT_CHANGE.toString().equals(property)) {
             // Note: ColonyTile.UNIT_CHANGE.equals(Building.UNIT_CHANGE)
@@ -799,9 +806,9 @@ public final class ColonyPanel extends FreeColPanel
      */
     public final class ColonyCargoPanel extends CargoPanel {
 
-        public ColonyCargoPanel(Canvas canvas) {
-            super(canvas, true);
-            //setLayout(new MigLayout("wrap 4, fill"));
+        public ColonyCargoPanel(FreeColClient freeColClient, GUI gui) {
+            super(freeColClient, gui, true);
+            setLayout(new MigLayout("wrap 6, fill, insets 0"));
         }
 
         @Override
@@ -831,7 +838,7 @@ public final class ColonyPanel extends FreeColPanel
          * @param colonyPanel The panel that holds this BuildingsPanel.
          */
         public BuildingsPanel(ColonyPanel colonyPanel) {
-            setLayout(new MigLayout("fill, wrap 4, gap 0:10:10:push"));
+            setLayout(new MigLayout("fill, wrap 4, insets 0, gap 0:10:10:push"));
             this.colonyPanel = colonyPanel;
         }
 
@@ -853,7 +860,7 @@ public final class ColonyPanel extends FreeColPanel
 
             MouseAdapter mouseAdapter = new MouseAdapter() {
                     public void mousePressed(MouseEvent e) {
-                        getCanvas().showSubPanel(new BuildQueuePanel(getColony(), getCanvas()));
+                        getGUI().showBuildQueuePanel(getColony());
                     }
                 };
             ASingleBuildingPanel aSingleBuildingPanel;
@@ -898,7 +905,7 @@ public final class ColonyPanel extends FreeColPanel
              * @param building The building to display information from.
              */
             public ASingleBuildingPanel(Building building) {
-                super(building, getCanvas());
+                super(getFreeColClient(), building, getGUI());
             }
 
             public void autoscroll(Point p) {
@@ -944,25 +951,36 @@ public final class ColonyPanel extends FreeColPanel
              * @return The component argument.
              */
             public Component add(Component comp, boolean editState) {
-
+                Container oldParent = comp.getParent();
                 if (editState) {
                     if (comp instanceof UnitLabel) {
-                        Unit unit = ((UnitLabel) comp).getUnit();
-
-                        if (getBuilding().canAdd(unit)) {
-                            //oldParent.remove(comp);
-                            getController().work(unit, getBuilding());
+                        if (tryWork(((UnitLabel) comp).getUnit())) {
+                            oldParent.remove(comp);
                         } else {
                             return null;
                         }
                     } else {
-                        logger.warning("An invalid component got dropped on this BuildingsPanel.");
+                        logger.warning("An invalid component was dropped"
+                            + " on this ASingleBuildingPanel.");
                         return null;
                     }
                 }
                 initialize();
                 return null;
             }
+
+            private boolean tryWork(Unit unit) {
+                Building building = getBuilding();
+                NoAddReason reason = building.getNoAddReason(unit);
+                if (reason != NoAddReason.NONE) {
+                    getGUI().errorMessage("noAddReason."
+                        + reason.toString().toLowerCase(Locale.US));
+                    return false;
+                }
+
+                getController().work(unit, building);
+                return true;
+            }
         }
     }
 
@@ -993,7 +1011,7 @@ public final class ColonyPanel extends FreeColPanel
         }
 
         public JToolTip createToolTip() {
-            return new RebelToolTip(getColony(), getCanvas());
+            return new RebelToolTip(getFreeColClient(), getGUI(), getColony());
         }
 
         public void update() {
@@ -1036,7 +1054,7 @@ public final class ColonyPanel extends FreeColPanel
         private Colony colony;
 
         public OutsideColonyPanel() {
-            super(new MigLayout("wrap 4, fill"));
+            super(new MigLayout("wrap 4, fill, insets 0"));
             setBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(),
                                                        Messages.message("outsideColony")));
         }
@@ -1063,7 +1081,7 @@ public final class ColonyPanel extends FreeColPanel
                     continue;
                 }
 
-                UnitLabel unitLabel = new UnitLabel(unit, getCanvas());
+                UnitLabel unitLabel = new UnitLabel(getFreeColClient(), unit, getGUI());
                 if (isEditable()) {
                     unitLabel.setTransferHandler(defaultTransferHandler);
                     unitLabel.addMouseListener(pressListener);
@@ -1160,7 +1178,7 @@ public final class ColonyPanel extends FreeColPanel
     public final class InPortPanel extends JPanel {
 
         public InPortPanel() {
-            super(new MigLayout("wrap 3, fill"));
+            super(new MigLayout("wrap 3, fill, insets 0"));
             setBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(),
                                                        Messages.message("inPort")));
         }
@@ -1174,7 +1192,7 @@ public final class ColonyPanel extends FreeColPanel
             for (Unit unit : getColony().getTile().getUnitList()) {
                 if (!unit.isCarrier()) continue;
 
-                UnitLabel unitLabel = new UnitLabel(unit, getCanvas());
+                UnitLabel unitLabel = new UnitLabel(getFreeColClient(), unit, getGUI());
                 TradeRoute tradeRoute = unit.getTradeRoute();
                 if (tradeRoute != null) {
                     unitLabel.setDescriptionLabel(Messages.message(Messages.getLabel(unit))
@@ -1220,7 +1238,7 @@ public final class ColonyPanel extends FreeColPanel
          */
         public WarehousePanel(ColonyPanel colonyPanel) {
             this.colonyPanel = colonyPanel;
-            setLayout(new MigLayout("fill, gap push"));
+            setLayout(new MigLayout("fill, gap push, insets 0"));
         }
 
         /**
@@ -1244,18 +1262,21 @@ public final class ColonyPanel extends FreeColPanel
          * Update this WarehousePanel.
          */
         private void update() {
-            final int threshold = getClient().getClientOptions()
-                .getInteger(ClientOptions.MIN_NUMBER_FOR_DISPLAYING_GOODS);
+            final int threshold = (FreeCol.isInDebugMode()) ? 1
+                : getClientOptions().getInteger(ClientOptions.MIN_NUMBER_FOR_DISPLAYING_GOODS);
             removeAll();
-            for (GoodsType goodsType : getSpecification().getGoodsTypeList()) {
-                Goods goods = getColony().getGoodsContainer().getGoods(goodsType);
-                if (goodsType.isStorable() && goods.getAmount() >= threshold) {
-                    GoodsLabel goodsLabel = new GoodsLabel(goods, getCanvas());
-                    if (colonyPanel.isEditable()) {
-                        goodsLabel.setTransferHandler(defaultTransferHandler);
-                        goodsLabel.addMouseListener(pressListener);
+            GoodsContainer container = getColony().getGoodsContainer();
+            for (GoodsType goodsType : getColony().getSpecification().getGoodsTypeList()) {
+                if (goodsType.isStorable()) {
+                    Goods goods = container.getGoods(goodsType);
+                    if (goods.getAmount() >= threshold) {
+                        GoodsLabel goodsLabel = new GoodsLabel(goods, getGUI());
+                        if (colonyPanel.isEditable()) {
+                            goodsLabel.setTransferHandler(defaultTransferHandler);
+                            goodsLabel.addMouseListener(pressListener);
+                        }
+                        add(goodsLabel, false);
                     }
-                    add(goodsLabel, false);
                 }
             }
             revalidate();
@@ -1283,7 +1304,6 @@ public final class ColonyPanel extends FreeColPanel
                     return null;
                 }
                 comp.getParent().remove(comp);
-                ((GoodsLabel) comp).setSmall(false);
                 return comp;
             }
 
@@ -1330,11 +1350,12 @@ public final class ColonyPanel extends FreeColPanel
 
         /**
          * Creates this TilePanel.
+         * @param freeColClient 
          *
          * @param colonyPanel The panel that holds this TilePanel.
          */
-        public TilePanel(ColonyPanel colonyPanel) {
-            super(colonyPanel.getCanvas());
+        public TilePanel(FreeColClient freeColClient, GUI gui, ColonyPanel colonyPanel) {
+            super(freeColClient, gui);
             this.colonyPanel = colonyPanel;
             setBackground(Color.BLACK);
             setBorder(null);
@@ -1343,8 +1364,6 @@ public final class ColonyPanel extends FreeColPanel
 
         @Override
         public void paintComponent(Graphics g) {
-            GUI colonyTileGUI = getCanvas().getColonyTileGUI();
-
             g.setColor(Color.black);
             g.fillRect(0, 0, getWidth(), getHeight());
 
@@ -1358,7 +1377,7 @@ public final class ColonyPanel extends FreeColPanel
                             int xx = ((2 - x) + y) * tileWidth;
                             int yy = (x + y) * tileHeight;
                             g.translate(xx, yy);
-                            colonyTileGUI.displayColonyTile((Graphics2D) g, tiles[x][y], getColony());
+                            getGUI().getColonyTileGUI().displayColonyTile((Graphics2D) g, tiles[x][y], getColony());
                             g.translate(-xx, -yy);
                         }
                     }
@@ -1392,6 +1411,7 @@ public final class ColonyPanel extends FreeColPanel
                     }
                 }
             }
+            repaint();
         }
 
         public void cleanup() {
@@ -1438,10 +1458,11 @@ public final class ColonyPanel extends FreeColPanel
              */
             private void initializeAsCenterTile() {
 
-                setLayout(new MigLayout("wrap 1, center"));
-
-                for (AbstractGoods goods : colony.getProductionInfo(colonyTile).getProduction()) {
-                    add(new ProductionLabel(goods, getCanvas()), "center");
+                setLayout(new GridLayout(2, 1));
+                ProductionInfo info = colony.getProductionInfo(colonyTile);
+                if (info == null) return;
+                for (AbstractGoods goods : info.getProduction()) {
+                    add(new ProductionLabel(getFreeColClient(), getGUI(), goods));
                 }
             }
 
@@ -1482,15 +1503,13 @@ public final class ColonyPanel extends FreeColPanel
 
                 removeAll();
                 UnitLabel unitLabel = null;
-                if (colonyTile.getUnit() != null) {
-                    Unit unit = colonyTile.getUnit();
-                    unitLabel = new UnitLabel(unit, getCanvas());
+                for (Unit unit : colonyTile.getUnitList()) {
+                    unitLabel = new UnitLabel(getFreeColClient(), unit, getGUI());
                     if (colonyPanel.isEditable()) {
                         unitLabel.setTransferHandler(defaultTransferHandler);
                         unitLabel.addMouseListener(pressListener);
                     }
                     super.add(unitLabel);
-
                 }
                 updateDescriptionLabel(unitLabel, true);
 
@@ -1502,9 +1521,6 @@ public final class ColonyPanel extends FreeColPanel
                     setTransferHandler(defaultTransferHandler);
                     addMouseListener(releaseListener);
                 }
-
-                revalidate();
-                repaint();
             }
 
             /**
@@ -1524,7 +1540,7 @@ public final class ColonyPanel extends FreeColPanel
                 Container oldParent = comp.getParent();
                 if (editState) {
                     if (comp instanceof UnitLabel) {
-                        if (tryWorkTile(((UnitLabel) comp).getUnit())) {
+                        if (tryWork(((UnitLabel) comp).getUnit())) {
                             oldParent.remove(comp);
                             ((UnitLabel) comp).setSmall(false);
                         } else {
@@ -1554,50 +1570,45 @@ public final class ColonyPanel extends FreeColPanel
              * @param unit The <code>Unit</code> to work the tile.
              * @return True if the unit succeeds.
              */
-            private boolean tryWorkTile(Unit unit) {
+            private boolean tryWork(Unit unit) {
                 Tile tile = colonyTile.getWorkTile();
                 Colony colony = getColony();
                 Player player = unit.getOwner();
 
-                // May need to acquire the tile before working it.
-                if (tile.getOwningSettlement() != colony
-                    && player.canAcquireForSettlement(tile)) {
-                    if (getController().claimLand(tile, colony, 0)
-                        && tile.getOwningSettlement() == colony) {
-                        logger.info("Colony " + colony.getName()
-                                    + " claims tile " + tile.toString()
-                                    + " with unit " + unit.getId());
-                    } else {
-                        logger.warning("Colony " + colony.getName()
-                                       + " did not claim " + tile.toString()
-                                       + " with unit " + unit.getId());
+                if (tile.getOwningSettlement() != colony) {
+                    // Need to acquire the tile before working it.
+                    NoClaimReason claim
+                        = player.canClaimForSettlementReason(tile);
+                    switch (claim) {
+                    case NONE: case NATIVES:
+                        if (getController().claimLand(tile, colony, 0)
+                            && tile.getOwningSettlement() == colony) {
+                            logger.info("Colony " + colony.getName()
+                                + " claims tile " + tile.toString()
+                                + " with unit " + unit.getId());
+                        } else {
+                            logger.warning("Colony " + colony.getName()
+                                + " did not claim " + tile.toString()
+                                + " with unit " + unit.getId());
+                            return false;
+                        }
+                        break;
+                    default: // Otherwise, can not use land
+                        getGUI().errorMessage("noClaimReason."
+                            + claim.toString().toLowerCase(Locale.US));
                         return false;
                     }
+                    // Check reason again, claim should be satisfied.
+                    if (tile.getOwningSettlement() != colony) {
+                        throw new IllegalStateException("Claim failed");
+                    }
                 }
 
-                Canvas canvas = getCanvas();
-                if (!colonyTile.canAdd(unit)) {
-                    Settlement s = tile.getOwningSettlement();
-                    if (s != null && s != getColony()) {
-                        if (s.getOwner() == player) {
-                            // Its one of ours
-                            canvas.errorMessage("tileTakenSelf");
-                        } else if (s.getOwner().isEuropean()) {
-                            // occupied by a foreign european colony
-                            canvas.errorMessage("tileTakenEuro");
-                        } else if (s instanceof IndianSettlement) {
-                            // occupied by an indian settlement
-                            canvas.errorMessage("tileTakenInd");
-                        }
-                    } else {
-                        if (colonyTile.getUnitCount() > 0) {
-                            ; // Tile is already occupied
-                        } else if (!tile.isLand()) {
-                            canvas.errorMessage("tileNeedsDocks");
-                        } else if (tile.hasLostCityRumour()) {
-                            canvas.errorMessage("tileHasRumour");
-                        }
-                    }
+                // Claim sorted, but complain about other failure.
+                NoAddReason reason = colonyTile.getNoAddReason(unit);
+                if (reason != NoAddReason.NONE) {
+                    getGUI().errorMessage("noAddReason."
+                        + reason.toString().toLowerCase(Locale.US));
                     return false;
                 }
 
@@ -1624,7 +1635,7 @@ public final class ColonyPanel extends FreeColPanel
                     getController().changeWorkType(unit, workType);
                 }
 
-                if (getClient().getClientOptions()
+                if (getClientOptions()
                     .getBoolean(ClientOptions.SHOW_NOT_BEST_TILE)) {
                     ColonyTile best = colony.getVacantColonyTileFor(unit, false,
                                                                     workType);
@@ -1636,7 +1647,7 @@ public final class ColonyPanel extends FreeColPanel
                             .addStringTemplate("%unit%", Messages.getLabel(unit))
                             .add("%goods%", workType.getNameKey())
                             .addStringTemplate("%tile%", best.getLabel());
-                        canvas.showInformationMessage(template);
+                        getGUI().showInformationMessage(template);
                     }
                 }
                 return true;
