@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -19,14 +19,12 @@
 
 package net.sf.freecol.client.gui.panel;
 
-import net.sf.freecol.client.gui.Canvas;
-import net.sf.freecol.client.gui.i18n.Messages;
-import net.sf.freecol.common.model.Building;
-import net.sf.freecol.common.model.Colony;
-import net.sf.freecol.common.model.GoodsType;
-import net.sf.freecol.common.model.StringTemplate;
-import net.sf.freecol.common.model.Unit;
-import net.sf.freecol.common.model.UnitType;
+import java.awt.Color;
+import java.awt.Insets;
+import java.awt.event.ActionEvent;
+import java.awt.event.ActionListener;
+import java.util.LinkedHashSet;
+import java.util.Set;
 
 import javax.swing.ImageIcon;
 import javax.swing.JButton;
@@ -35,16 +33,18 @@ import javax.swing.JPanel;
 import javax.swing.SwingConstants;
 import javax.swing.border.Border;
 import javax.swing.border.EmptyBorder;
-import java.awt.Color;
-import java.awt.Insets;
-import java.awt.event.ActionEvent;
-import java.awt.event.ActionListener;
-import java.util.Collections;
-import java.util.LinkedHashSet;
-import java.util.List;
-import java.util.Set;
 
 import net.miginfocom.swing.MigLayout;
+import net.sf.freecol.client.FreeColClient;
+import net.sf.freecol.client.gui.GUI;
+import net.sf.freecol.client.gui.i18n.Messages;
+import net.sf.freecol.common.model.Ability;
+import net.sf.freecol.common.model.Building;
+import net.sf.freecol.common.model.Colony;
+import net.sf.freecol.common.model.GoodsType;
+import net.sf.freecol.common.model.StringTemplate;
+import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.UnitType;
 
 /**
  * This panel displays the Labour Report.
@@ -83,23 +83,25 @@ public final class CompactLabourReport extends ReportPanel {
 
     /**
      * The constructor that will add the items to this panel.
+     * @param freeColClient 
      *
      * @param parent The parent of this panel.
      */
-    public CompactLabourReport(Canvas parent) {
-        this(parent, null);
+    public CompactLabourReport(FreeColClient freeColClient, GUI gui) {
+        this(freeColClient, gui, null);
 
-        labourData = new LabourData(parent.getClient());
+        labourData = new LabourData(getFreeColClient());
         initialize();
     }
 
     /**
      * The constructor that will add the items to this panel.
+     * @param freeColClient 
      *
      * @param parent The parent of this panel.
      */
-    private CompactLabourReport(Canvas parent, LabourData.UnitData data) {
-        super(parent, data == null ? Messages.message("reportLabourAction.name")
+    public CompactLabourReport(FreeColClient freeColClient, GUI gui, LabourData.UnitData data) {
+        super(freeColClient, gui, data == null ? Messages.message("reportLabourAction.name")
               : Messages.message("report.labour.details"));
         this.unitData = data;
 
@@ -108,37 +110,30 @@ public final class CompactLabourReport extends ReportPanel {
         scrollPane.setColumnHeaderView(headerRow);
     }
 
-    @Override
-    protected Border createBorder() {
-        return new EmptyBorder(0, 20, 20, 20);
-    }
+    public JButton createColonyButton(final Colony colony) {
+        String text = colony.getName();
+        if (!unitData.isSummary()) {
+//            int unitIndex = unitData.getUnitType().getIndex();
+//
+//            int skillLevel = Unit.getSkillLevel(unitIndex);
+//            if (skillLevel <= 0 && skillLevel > -2) {
+//                //settlers and servants can be trained anywwhere a farmer can
+//                unitIndex = Unit.EXPERT_FARMER;
+//            }
 
-
-    /**
-     * @return if this is the location summary, grouped by unit type
-     */
-    private boolean isOverview() {
-        return unitData == null;
-    }
-
-    /**
-     * @return if we are any summary
-     */
-    private boolean isSummary() {
-        return isOverview() || unitData.isSummary();
-    }
-
-    private ImageIcon getUnitIcon(UnitType unit) {
-        Unit.Role role = Unit.Role.DEFAULT;
-        if (unit.hasAbility("model.ability.expertPioneer")) {
-            role = Unit.Role.PIONEER;
-        } else if (unit.hasAbility("model.ability.expertMissionary")) {
-            role = Unit.Role.MISSIONARY;
+            if (colony.canTrain(unitData.getUnitType())) {
+                text = text + "*";
+            }
         }
 
-        return getLibrary().getUnitImageIcon(unit, role);
+        return createButton(text, new ActionListener() {
+            public void actionPerformed(ActionEvent e) {
+                getGUI().showColonyPanel(colony);
+            }
+        });
     }
 
+
     @Override
     public void initialize() {
         if (isOverview()) {
@@ -156,17 +151,16 @@ public final class CompactLabourReport extends ReportPanel {
         }
 
 
-        String layoutConstraints = "fill, gap 0 0";
+        String layoutConstraints = "fill, insets 0, gap 0 0";
         StringBuilder widths = new StringBuilder();
-        widths.append("[160, fill]");
-        widths.append(isOverview() || unitData.isSummary() ? "[150, fill]" : "[]");
-        widths.append("[170, fill]");
-        widths.append(showBuildings ? "[130, fill]" : "[]");
+        widths.append("[175, fill]");
+        widths.append(isOverview() || !unitData.isSummary() ? "[150, fill]" : "[0!]");
+        widths.append("[150, fill]");
+        widths.append(showBuildings ? "[130, fill]" : "[0!]");
         widths.append("[26, fill][33, fill]");
-        widths.append(showProduction ? "[30, fill]" : "[]");
-        widths.append(showProduction ? "[40, fill]" : "[]");
-        widths.append(showNetProduction ? "[40, fill]" : "[]");
-        widths.append(showProductionSymbols ? "[50, fill]" : "[]");
+        widths.append(showProductionSymbols ? "[50, fill]" : "[0!]");
+        widths.append(showProduction ? "[30, fill][40, fill]" : "[0!][0!]");
+        widths.append(showNetProduction ? "[40, fill]" : "[0!]");
         String columnConstraints = widths.toString();
         String rowConstraints = "[fill]";
 
@@ -182,107 +176,9 @@ public final class CompactLabourReport extends ReportPanel {
         }
     }
 
-    private void addUnitTypes() {
-        int row = 1;
-
-        JButton allColonistsButton = createUnitNameButton(Messages.message("report.labour.allColonists"), labourData.getSummary());
-        reportPanel.add(allColonistsButton, "cell " + COLONY_COLUMN + " " + row + " 1 "
-                        + labourData.getSummary().getUnitSummaryRowCount());
-
-        row = addLocationData(labourData.getSummary().getTotal(), null, row);
-
-        for (UnitType unitType : LabourData.getLabourTypes(getMyPlayer())) {
-            LabourData.UnitData unitData = labourData.getUnitData(unitType);
-
-            JButton unitButton = createUnitNameButton(unitData.getUnitName(), unitData);
-            int rows = unitData.getUnitSummaryRowCount();
-            reportPanel.add(unitButton, "cell " + COLONY_COLUMN + " " + row + " " + 1 + " " + rows);
-
-            if (unitData.hasDetails()) {
-                row = addLocationData(unitData.getTotal(), null, row);
-            } else {
-                unitButton.setEnabled(false);
-                unitButton.setDisabledIcon(unitButton.getIcon());
-                unitButton.setForeground(Color.GRAY);
-
-                reportPanel.add(createEmptyLabel(), "cell " + UNIT_TYPE_COLUMN + " " + row + " "
-                                + (COLUMNS - 1) + " " + 1);
-                row++;
-            }
-        }
-    }
-
-    /*
-     * distributes {@code value}, amount the number of {@code pocketCount}
-     *
-     * @param value
-     * @param pocketCount
-     * @return distribution of {@code value}
-     */
-    /*
-    private int[] distribute(int value, int pocketCount) {
-        int[] pockets = new int[pocketCount];
-
-        int pocketIndex = 0;
-        for (int i = value; i > 0; i--) {
-            pockets[pocketIndex]++;
-
-            pocketIndex++;
-            pocketIndex = pocketIndex % pocketCount;
-        }
-        return pockets;
-    }
-     */
-
-    private JLabel createEmptyLabel() {
-        JLabel empty = new JLabel("");
-        empty.setBorder(CELLBORDER);
-        return empty;
-    }
-
-    private void addLocations() {
-        LabourData.LocationData unitTotal = unitData.getTotal();
-
-        int row = 1;
-        JLabel summaryLabel = new JLabel(Messages.message("report.labour.summary"));
-        summaryLabel.setBorder(LEFTCELLBORDER);
-        reportPanel.add(summaryLabel, "cell " + COLONY_COLUMN + " " + row + " " + 1 + " "
-                        + unitTotal.getRowCount());
-
-        row = addLocationData(unitTotal, null, row);
-
-        List<Colony> colonies = getMyPlayer().getColonies();
-
-        Collections.sort(colonies, getClient().getClientOptions().getColonyComparator());
-
-        for (Colony colony : colonies) {
-            LabourData.LocationData colonyData = unitData.getDetails().get(colony);
-            if (colonyData != null) {
-                reportPanel.add(createColonyButton(colony),
-                                "cell " + COLONY_COLUMN + " " + row + " " + 1 + " " + colonyData.getRowCount());
-                row = addLocationData(colonyData, colony, row);
-            }
-        }
-        LabourData.LocationData europe = unitData.getUnitsInEurope();
-        if (europe.getRowCount() > 0) {
-            JButton button = createButton(Messages.message(getMyPlayer().getEurope().getNameKey()),
-                                          new ActionListener() {
-                                              public void actionPerformed(ActionEvent e) {
-                                                  getCanvas().showEuropePanel();
-                                              }
-                                          });
-            reportPanel.add(button, "cell " + COLONY_COLUMN + " " + row + " " + 1 + " " + europe.getRowCount());
-            row = addLocationData(europe, null, row);
-        }
-        row = addNonLinkedLocation(unitData.getUnitsOnLand(), "report.onLand", row);
-        row = addNonLinkedLocation(unitData.getUnitsAtSea(), "report.atSea", row);
-
-        reportPanel.add(new JLabel(Messages.message("report.labour.canTrain")),
-                        "cell " + 1 + " " + row + " " + COLUMNS + " " + 1);
-    }
-
-    private GoodsType getGoodsType() {
-        return isSummary() ? null : unitData.getUnitType().getExpertProduction();
+    @Override
+    protected Border createBorder() {
+        return new EmptyBorder(0, 20, 20, 20);
     }
 
     /**
@@ -318,7 +214,7 @@ public final class CompactLabourReport extends ReportPanel {
         if (isOverview()) {
             JLabel production = new JLabel(Messages.message("report.production"));
             production.setBorder(TOPCELLBORDER);
-            headerRow.add(production, "cell " + PRODUCTION_SYMBOL_COLUMN + " 1 4 1");
+            headerRow.add(production, "cell " + PRODUCTION_SYMBOL_COLUMN + " 1 " + (COLUMNS - PRODUCTION_SYMBOL_COLUMN) + " 1");
         } else if (showProduction) {
             LabourData.UnitData unit = unitData;
             GoodsType goods = unit.getExpertProduction();
@@ -326,8 +222,7 @@ public final class CompactLabourReport extends ReportPanel {
             JLabel production = new JLabel(getLibrary().getGoodsImageIcon(goods));
             production.setBorder(TOPCELLBORDER);
 
-            headerRow.add(production, "cell " + PRODUCTION_SYMBOL_COLUMN + " " + row + " "
-                          + (!goods.isStoredAs() ? 4 : 3) + " 1");
+            headerRow.add(production, "cell " + PRODUCTION_SYMBOL_COLUMN + " 1 " + (COLUMNS - PRODUCTION_SYMBOL_COLUMN + (showNetProduction && goods.isStoredAs() ? 1 : 0)) + " 1");
 
             if (showNetProduction && goods.isStoredAs()) {
                 JLabel netProduction = new JLabel(getLibrary().getGoodsImageIcon(goods.getStoredAs()));
@@ -343,18 +238,6 @@ public final class CompactLabourReport extends ReportPanel {
         }
     }
 
-    private int addNonLinkedLocation(LabourData.LocationData data, String messageKey, int row) {
-        int rows = data.getRowCount();
-        if (rows > 0) {
-            JLabel label = new JLabel(Messages.message(messageKey));
-            label.setBorder(LEFTCELLBORDER);
-            label.setForeground(Color.GRAY);
-            reportPanel.add(label, "cell " + COLONY_COLUMN + " " + row + " " + 1 + " " + rows);
-            return addLocationData(data, null, row);
-        }
-        return row;
-    }
-
     /**
      * add unit data for a given location
      *
@@ -407,12 +290,15 @@ public final class CompactLabourReport extends ReportPanel {
 
         int notProducingStartRow = row;
 
-        if (showBuildings && productionBuilding != null && row > buildingStartRow) {
-            JLabel buildingLabel = localizedLabel(productionBuilding.getNameKey());
-            buildingLabel.setBorder(CELLBORDER);
-            reportPanel.add(buildingLabel, "cell " + buildingStartRow + " " + BUILDING_COLUMN
-                            + " 1 " + (row - buildingStartRow));
-            buildingStartRow = row;
+        if (showBuildings) {
+            if (productionBuilding != null && row > buildingStartRow) {
+                JLabel buildingLabel = localizedLabel(productionBuilding.getNameKey());
+                buildingLabel.setBorder(CELLBORDER);
+                reportPanel.add(buildingLabel, "cell " + BUILDING_COLUMN + " " + buildingStartRow + " 1 " + (row - buildingStartRow));
+                buildingStartRow = row;
+            } else {
+                reportPanel.add(createEmptyLabel(), "cell " + BUILDING_COLUMN + " " + buildingStartRow + " 1 " + (row - buildingStartRow));
+            }
         }
 
         row = addRow(data,
@@ -432,11 +318,10 @@ public final class CompactLabourReport extends ReportPanel {
         }
 
         Building schoolhouse = colony != null && data.isTraining() ?
-            colony.getBuildingWithAbility("model.ability.teach") : null;
+            colony.getBuildingWithAbility(Ability.CAN_TEACH) : null;
 
         if (showBuildings && schoolhouse != null && row > buildingStartRow) {
-            reportPanel.add(createEmptyLabel(), "cell " + buildingStartRow + " " + BUILDING_COLUMN
-                            + " 1 " + (row - buildingStartRow));
+            reportPanel.add(createEmptyLabel(), "cell " + BUILDING_COLUMN + " " + buildingStartRow + " 1 " + (row - buildingStartRow));
             buildingStartRow = row;
         }
 
@@ -486,38 +371,90 @@ public final class CompactLabourReport extends ReportPanel {
             JLabel buildingLabel = new JLabel(schoolhouse != null ?
                                               Messages.message(schoolhouse.getNameKey()) : "");
             buildingLabel.setBorder(CELLBORDER);
-            reportPanel.add(buildingLabel, "cell " + buildingStartRow + " " + BUILDING_COLUMN
-                            + " 1 " + (row - buildingStartRow));
+            reportPanel.add(buildingLabel, "cell " + BUILDING_COLUMN + " " + buildingStartRow + " 1 " + (row - buildingStartRow));
         }
 
         if (data.getUnitData().showProduction() && row > notProducingStartRow) {
-            reportPanel.add(createEmptyLabel(), "cell " + notProducingStartRow + " " + PRODUCTION_COLUMN
-                            + " 1 " + (row - notProducingStartRow));
+            reportPanel.add(createEmptyLabel(), "cell " + PRODUCTION_COLUMN + " " + notProducingStartRow + " 1 " + (row - notProducingStartRow));
         }
 
         return row;
     }
 
+    private void addLocations() {
+        LabourData.LocationData unitTotal = unitData.getTotal();
+
+        int row = 1;
+        JLabel summaryLabel = new JLabel(Messages.message("report.labour.summary"));
+        summaryLabel.setBorder(LEFTCELLBORDER);
+        reportPanel.add(summaryLabel, "cell " + COLONY_COLUMN + " " + row + " 1 " + unitTotal.getRowCount());
+
+        row = addLocationData(unitTotal, null, row);
+
+        for (Colony colony : getSortedColonies()) {
+            LabourData.LocationData colonyData = unitData.getDetails().get(colony);
+            if (colonyData != null) {
+                reportPanel.add(createColonyButton(colony), "cell " + COLONY_COLUMN + " "
+                                + row + " 1 " + colonyData.getRowCount());
+                row = addLocationData(colonyData, colony, row);
+            }
+        }
+        LabourData.LocationData europe = unitData.getUnitsInEurope();
+        if (europe.getRowCount() > 0) {
+            JButton button = createButton(Messages.message(getMyPlayer().getEurope().getNameKey()),
+                                          new ActionListener() {
+                                              public void actionPerformed(ActionEvent e) {
+                                                  getGUI().showEuropePanel();
+                                              }
+                                          });
+            reportPanel.add(button, "cell " + COLONY_COLUMN + " " + row + " 1 " + europe.getRowCount());
+            row = addLocationData(europe, null, row);
+        }
+        row = addNonLinkedLocation(unitData.getUnitsOnLand(), "report.onLand", row);
+        row = addNonLinkedLocation(unitData.getUnitsAtSea(), "report.atSea", row);
+
+        reportPanel.add(new JLabel(Messages.message("report.labour.canTrain")), "cell 1 " + row + " " + COLUMNS + " 1");
+    }
+
+    /*
+     * distributes {@code value}, amount the number of {@code pocketCount}
+     *
+     * @param value
+     * @param pocketCount
+     * @return distribution of {@code value}
+     */
+    /*
+    private int[] distribute(int value, int pocketCount) {
+        int[] pockets = new int[pocketCount];
+
+        int pocketIndex = 0;
+        for (int i = value; i > 0; i--) {
+            pockets[pocketIndex]++;
+
+            pocketIndex++;
+            pocketIndex = pocketIndex % pocketCount;
+        }
+        return pockets;
+    }
+     */
+
     private void addLocationSummary(LabourData.LocationData data, int row) {
         int rows = data.getRowCount();
 
         JLabel colonistsLabel = createNumberLabel(data.getTotalColonists(), null);
         if (data.getUnitData().isSummary()) {
             if (isOverview()) {
-                reportPanel.add(createEmptyLabel(), "cell " + UNIT_TYPE_COLUMN + " " + row
-                                + " " + 1 + " " + rows);
+                reportPanel.add(createEmptyLabel(), "cell " + UNIT_TYPE_COLUMN + " " + row + " 1 " + rows);
             }
         } else {
             colonistsLabel.setToolTipText(Messages.message(StringTemplate.template("report.labour.unitTotal.tooltip")
                                                            .addName("%unit%", data.getUnitData().getUnitName())));
         }
 
-        reportPanel.add(colonistsLabel, "cell " + COLONIST_SUMMARY_COLUMN + " " + row
-                        + " " + 1 + " " + data.getRowCount());
+        reportPanel.add(colonistsLabel, "cell " + COLONIST_SUMMARY_COLUMN + " " + row + " 1 " + data.getRowCount());
 
         if (showProduction && !data.getUnitData().showProduction()) {
-            reportPanel.add(createEmptyLabel(), "cell " + PRODUCTION_SYMBOL_COLUMN + " " + row
-                            + " " + 4 + " " + rows);
+            reportPanel.add(createEmptyLabel(), "cell " + PRODUCTION_SYMBOL_COLUMN + " " + row + " 4 " + rows);
             return;
         }
 
@@ -527,8 +464,7 @@ public final class CompactLabourReport extends ReportPanel {
                 productionLabel.setText("");
             }
 
-            reportPanel.add(productionLabel, "cell " + PRODUCTION_SUMMARY_COLUMN + " " + row
-                            + " " + 1 + " " + rows);
+            reportPanel.add(productionLabel, "cell " + PRODUCTION_SUMMARY_COLUMN + " " + row + " 1 " + rows);
         }
 
         if (showNetProduction) {
@@ -543,8 +479,7 @@ public final class CompactLabourReport extends ReportPanel {
             } else {
                 netProductionLabel.setForeground(Color.RED);
             }
-            reportPanel.add(netProductionLabel, "cell " + NETPRODUCTION_SUMMARY_COLUMN + " " + row
-                            + " " + 1 + " " + rows);
+            reportPanel.add(netProductionLabel, "cell " + NETPRODUCTION_SUMMARY_COLUMN + " " + row + " 1 " + rows);
         }
 
         if (showProductionSymbols) {
@@ -554,25 +489,20 @@ public final class CompactLabourReport extends ReportPanel {
             if (goods != null) {
                 icon.setIcon(getLibrary().getGoodsImageIcon(goods));
             }
-            reportPanel.add(icon, "cell " + PRODUCTION_SYMBOL_COLUMN + " " + row
-                            + " " + 1 + " " + rows);
+            reportPanel.add(icon, "cell " + PRODUCTION_SYMBOL_COLUMN + " " + row + " 1 " + rows);
         }
     }
 
-    private JLabel createNonCountedLabel(int otherAmateurs) {
-        JLabel label = createNumberLabel(otherAmateurs, "report.labour.notCounted.tooltip");
-        label.setForeground(Color.GRAY);
-        return label;
-    }
-
-    private JLabel createNumberLabel(int number, String toolTipKey) {
-        JLabel label = new JLabel(String.valueOf(number));
-        label.setHorizontalAlignment(SwingConstants.TRAILING);
-        label.setBorder(CELLBORDER);
-        if (toolTipKey != null) {
-            label.setToolTipText(Messages.message(toolTipKey));
+    private int addNonLinkedLocation(LabourData.LocationData data, String messageKey, int row) {
+        int rows = data.getRowCount();
+        if (rows > 0) {
+            JLabel label = new JLabel(Messages.message(messageKey));
+            label.setBorder(LEFTCELLBORDER);
+            label.setForeground(Color.GRAY);
+            reportPanel.add(label, "cell " + COLONY_COLUMN + " " + row + " 1 " + rows);
+            return addLocationData(data, null, row);
         }
-        return label;
+        return row;
     }
 
     private int addRow(LabourData.LocationData data, String typeName, String activity, int colonists, int production, int row) {
@@ -597,49 +527,36 @@ public final class CompactLabourReport extends ReportPanel {
         reportPanel.add(colonistLabel, "cell " + COLONIST_COLUMN + " " + row);
 
         if (data.getUnitData().showProduction() && production > 0) {
-            reportPanel.add(createNumberLabel(production, "report.labour.potentialProduction.tooltip"),
-                            "cell " + PRODUCTION_COLUMN + " " + row);
+            reportPanel.add(createNumberLabel(production, "report.labour.potentialProduction.tooltip"), "cell " + PRODUCTION_COLUMN + " " + row);
         }
     }
 
-    public JButton createColonyButton(final Colony colony) {
-        String text = colony.getName();
-        if (!unitData.isSummary()) {
-//            int unitIndex = unitData.getUnitType().getIndex();
-//
-//            int skillLevel = Unit.getSkillLevel(unitIndex);
-//            if (skillLevel <= 0 && skillLevel > -2) {
-//                //settlers and servants can be trained anywwhere a farmer can
-//                unitIndex = Unit.EXPERT_FARMER;
-//            }
+    private void addUnitTypes() {
+        int row = 1;
 
-            if (colony.canTrain(unitData.getUnitType())) {
-                text = text + "*";
+        JButton allColonistsButton = createUnitNameButton(Messages.message("report.labour.allColonists"), labourData.getSummary());
+        reportPanel.add(allColonistsButton, "cell " + COLONY_COLUMN + " " + row + " 1 " + labourData.getSummary().getUnitSummaryRowCount());
+
+        row = addLocationData(labourData.getSummary().getTotal(), null, row);
+
+        for (UnitType unitType : LabourData.getLabourTypes(getMyPlayer())) {
+            LabourData.UnitData unitData = labourData.getUnitData(unitType);
+
+            JButton unitButton = createUnitNameButton(unitData.getUnitName(), unitData);
+            int rows = unitData.getUnitSummaryRowCount();
+            reportPanel.add(unitButton, "cell " + COLONY_COLUMN + " " + row + " 1 " + rows);
+
+            if (unitData.hasDetails()) {
+                row = addLocationData(unitData.getTotal(), null, row);
+            } else {
+                unitButton.setEnabled(false);
+                unitButton.setDisabledIcon(unitButton.getIcon());
+                unitButton.setForeground(Color.GRAY);
+
+                reportPanel.add(createEmptyLabel(), "cell " + UNIT_TYPE_COLUMN + " " + row + " " + (COLUMNS - 1) + " 1");
+                row++;
             }
         }
-
-        return createButton(text, new ActionListener() {
-            public void actionPerformed(ActionEvent e) {
-                getCanvas().showColonyPanel(colony);
-            }
-        });
-    }
-
-    private JButton createUnitNameButton(String name, final LabourData.UnitData unitData) {
-        JButton button = createButton(name, new ActionListener() {
-            public void actionPerformed(ActionEvent e) {
-                CompactLabourReport details = new CompactLabourReport(getCanvas(), unitData);
-                details.initialize();
-                getCanvas().addAsFrame(details);
-                details.requestFocus();
-            }
-        });
-
-        if (!unitData.isSummary()) {
-            button.setIcon(getUnitIcon(unitData.getUnitType()));
-        }
-
-        return button;
     }
 
     private JButton createButton(String name, ActionListener listener) {
@@ -653,5 +570,70 @@ public final class CompactLabourReport extends ReportPanel {
         return button;
     }
 
+    private JLabel createEmptyLabel() {
+        JLabel empty = new JLabel("");
+        empty.setBorder(CELLBORDER);
+        return empty;
+    }
+
+    private JLabel createNonCountedLabel(int otherAmateurs) {
+        JLabel label = createNumberLabel(otherAmateurs, "report.labour.notCounted.tooltip");
+        label.setForeground(Color.GRAY);
+        return label;
+    }
+
+    private JLabel createNumberLabel(int number, String toolTipKey) {
+        JLabel label = new JLabel(String.valueOf(number));
+        label.setHorizontalAlignment(SwingConstants.TRAILING);
+        label.setBorder(CELLBORDER);
+        if (toolTipKey != null) {
+            label.setToolTipText(Messages.message(toolTipKey));
+        }
+        return label;
+    }
+
+    private JButton createUnitNameButton(String name, final LabourData.UnitData unitData) {
+        JButton button = createButton(name, new ActionListener() {
+            public void actionPerformed(ActionEvent e) {
+                getGUI().showCompactLabourReport(unitData);
+            }
+        });
+
+        if (!unitData.isSummary()) {
+            button.setIcon(getUnitIcon(unitData.getUnitType()));
+        }
+
+        return button;
+    }
+
+    private GoodsType getGoodsType() {
+        return isSummary() ? null : unitData.getUnitType().getExpertProduction();
+    }
+
+    private ImageIcon getUnitIcon(UnitType unit) {
+        Unit.Role role = Unit.Role.DEFAULT;
+        if (unit.hasAbility(Ability.EXPERT_PIONEER)) {
+            role = Unit.Role.PIONEER;
+        } else if (unit.hasAbility(Ability.EXPERT_MISSIONARY)) {
+            role = Unit.Role.MISSIONARY;
+        }
+
+        return getLibrary().getUnitImageIcon(unit, role);
+    }
+
+    /**
+     * @return if this is the location summary, grouped by unit type
+     */
+    private boolean isOverview() {
+        return unitData == null;
+    }
+
+    /**
+     * @return if we are any summary
+     */
+    private boolean isSummary() {
+        return isOverview() || unitData.isSummary();
+    }
+
 
 }
