@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -42,8 +42,8 @@ import java.awt.event.InputEvent;
 import java.awt.event.MouseEvent;
 import java.io.PrintWriter;
 import java.io.StringWriter;
-import java.util.logging.Logger;
 import java.util.Set;
+import java.util.logging.Logger;
 
 import javax.swing.ImageIcon;
 import javax.swing.JComponent;
@@ -51,16 +51,21 @@ import javax.swing.JLabel;
 import javax.swing.JPanel;
 import javax.swing.TransferHandler;
 
-import net.sf.freecol.client.gui.Canvas;
+import net.sf.freecol.client.FreeColClient;
+import net.sf.freecol.client.gui.GUI;
 import net.sf.freecol.client.gui.i18n.Messages;
+import net.sf.freecol.common.model.Ability;
 import net.sf.freecol.common.model.AbstractGoods;
 import net.sf.freecol.common.model.EquipmentType;
 import net.sf.freecol.common.model.Goods;
+import net.sf.freecol.common.model.GoodsContainer;
+import net.sf.freecol.common.model.GoodsLocation;
 import net.sf.freecol.common.model.GoodsType;
+import net.sf.freecol.common.model.HighSeas;
 import net.sf.freecol.common.model.Modifier;
 import net.sf.freecol.common.model.StringTemplate;
 import net.sf.freecol.common.model.Unit;
-import net.sf.freecol.common.model.Unit.UnitState;
+
 
 /**
 * The transferhandler that is capable of creating ImageSelection objects.
@@ -73,16 +78,22 @@ public final class DefaultTransferHandler extends TransferHandler {
 
     public static final DataFlavor flavor = new DataFlavor(ImageSelection.class, "ImageSelection");
 
-    private final Canvas canvas;
+
     private final FreeColPanel parentPanel;
 
+    private FreeColClient freeColClient;
+
+    private GUI gui;
+
     /**
     * The constructor to use.
+     * @param freeColClient 
     * @param canvas The <code>Canvas</code>.
     * @param parentPanel The layered pane that holds all kinds of information.
     */
-    public DefaultTransferHandler(Canvas canvas, FreeColPanel parentPanel) {
-        this.canvas = canvas;
+    public DefaultTransferHandler(FreeColClient freeColClient, GUI gui, FreeColPanel parentPanel) {
+        this.freeColClient = freeColClient;
+        this.gui = gui;
         this.parentPanel = parentPanel;
     }
 
@@ -241,22 +252,15 @@ public final class DefaultTransferHandler extends TransferHandler {
                     return false;
                 }
 
-                if ((unit.getState() == UnitState.TO_AMERICA) && (!(comp instanceof EuropePanel.ToEuropePanel))) {
-                    return false;
+                if (unit.getLocation() instanceof HighSeas
+                    && !(comp instanceof EuropePanel.DestinationPanel)) {
+                        return false;
                 }
 
-                if ((unit.getState() == UnitState.TO_EUROPE) && (!(comp instanceof EuropePanel.ToAmericaPanel))) {
-                    return false;
-                }
-
-                if ((unit.getState() != UnitState.TO_AMERICA) && ((comp instanceof EuropePanel.ToEuropePanel))) {
-                    return false;
-                }
-
-                if (!unit.isNaval() && (comp instanceof EuropePanel.InPortPanel
+                if (!unit.isNaval()
+                    && (comp instanceof EuropePanel.InPortPanel
                         || comp instanceof ColonyPanel.InPortPanel
-                        || comp instanceof EuropePanel.ToEuropePanel
-                        || comp instanceof EuropePanel.ToAmericaPanel)) {
+                        || comp instanceof EuropePanel.DestinationPanel)) {
                     return false;
                 }
 
@@ -279,10 +283,8 @@ public final class DefaultTransferHandler extends TransferHandler {
                     // Do this in the 'add'-methods instead:
                     //data.getParent().remove(data);
 
-                    if (comp instanceof EuropePanel.ToEuropePanel) {
-                        ((EuropePanel.ToEuropePanel)comp).add(data, true);
-                    } else if (comp instanceof EuropePanel.ToAmericaPanel) {
-                        ((EuropePanel.ToAmericaPanel)comp).add(data, true);
+                    if (comp instanceof EuropePanel.DestinationPanel) {
+                        ((EuropePanel.DestinationPanel)comp).add(data, true);
                     } else if (comp instanceof EuropePanel.DocksPanel) {
                         ((EuropePanel.DocksPanel)comp).add(data, true);
                     } else if (comp instanceof ColonyPanel.BuildingsPanel.ASingleBuildingPanel) {
@@ -300,7 +302,7 @@ public final class DefaultTransferHandler extends TransferHandler {
                                                             .addName("%object%", modifier.getSource()))
                                     + "\n";
                             }
-                            canvas.showInformationMessage(message);
+                            gui.showInformationMessage(message);
                         }
                     } else if (comp instanceof CargoPanel) {
                         ((CargoPanel)comp).add(data, true);
@@ -329,21 +331,29 @@ public final class DefaultTransferHandler extends TransferHandler {
                     return true;
                 }
             } else if (data instanceof GoodsLabel) {
-
-                // Check if the unit can be dragged to comp.
-
-                GoodsLabel label = ((GoodsLabel)data);
+                // Check if the goods can be dragged to comp.
+                GoodsLabel label = (GoodsLabel)data;
 
                 // Import the data.
-
                 if (label.isPartialChosen()) {
-                    int amount = getAmount(label.getGoods().getType(), label.getGoods().getAmount(), false);
-                    if (amount <= 0) {
-                        return false;
+                    Goods goods = label.getGoods();
+                    int defaultAmount = -1;
+                    if (goods.getLocation() instanceof GoodsLocation) {
+                        GoodsLocation loc = (GoodsLocation)goods.getLocation();
+                        if (goods.getAmount() > loc.getGoodsCapacity()) {
+                            // If over capacity, favour the amount that would
+                            // correct the problem.
+                            defaultAmount = Math.min(goods.getAmount()
+                                - loc.getGoodsCapacity(),
+                                GoodsContainer.CARGO_SIZE);
+                        }
                     }
+                    int amount = getAmount(label.getGoods().getType(),
+                        label.getGoods().getAmount(), defaultAmount, false);
+                    if (amount <= 0) return false;
                     label.getGoods().setAmount(amount);
-                } else if (label.getGoods().getAmount() > 100) {
-                    label.getGoods().setAmount(100);
+                } else if (label.getGoods().getAmount() > GoodsContainer.CARGO_SIZE) {
+                    label.getGoods().setAmount(GoodsContainer.CARGO_SIZE);
                 }
 
                 /*
@@ -359,9 +369,9 @@ public final class DefaultTransferHandler extends TransferHandler {
                 if (comp instanceof UnitLabel) {
                     UnitLabel unitLabel = ((UnitLabel) comp);
                     Unit unit = unitLabel.getUnit();
-                    if (unit.hasAbility("model.ability.canBeEquipped")) {
+                    if (unit.hasAbility(Ability.CAN_BE_EQUIPPED)) {
                         Goods goods = label.getGoods();
-                        for (EquipmentType equipment : canvas.getSpecification()
+                        for (EquipmentType equipment : freeColClient.getGame().getSpecification()
                                  .getEquipmentTypeList()) {
                             if (unit.canBeEquippedWith(equipment) && equipment.getGoodsRequired().size() == 1) {
                                 AbstractGoods requiredGoods = equipment.getGoodsRequired().get(0);
@@ -369,7 +379,7 @@ public final class DefaultTransferHandler extends TransferHandler {
                                     && requiredGoods.getAmount() <= goods.getAmount()) {
                                     int amount = Math.min(goods.getAmount() / requiredGoods.getAmount(),
                                                           equipment.getMaximumCount());
-                                    unitLabel.getCanvas().getClient().getInGameController()
+                                    freeColClient.getInGameController()
                                         .equipUnit(unit, equipment, amount);
                                     unitLabel.updateIcon();
                                     return true;
@@ -416,7 +426,7 @@ public final class DefaultTransferHandler extends TransferHandler {
                 // Import the data.
 
                 if (label.isPartialChosen()) {
-                    int amount = getAmount(label.getType(), label.getAmount(), true);
+                    int amount = getAmount(label.getType(), label.getAmount(), -1, true);
                     if (amount <= 0) {
                         return false;
                     }
@@ -426,8 +436,8 @@ public final class DefaultTransferHandler extends TransferHandler {
                 if (comp instanceof UnitLabel) {
                     UnitLabel unitLabel = (UnitLabel) comp;
                     Unit unit = unitLabel.getUnit();
-                    if (unit.hasAbility("model.ability.canBeEquipped")) {
-                        for (EquipmentType equipment : canvas.getSpecification()
+                    if (unit.hasAbility(Ability.CAN_BE_EQUIPPED)) {
+                        for (EquipmentType equipment : freeColClient.getGame().getSpecification()
                                  .getEquipmentTypeList()) {
                             if (unit.canBeEquippedWith(equipment) && equipment.getGoodsRequired().size() == 1) {
                                 AbstractGoods requiredGoods = equipment.getGoodsRequired().get(0);
@@ -435,7 +445,7 @@ public final class DefaultTransferHandler extends TransferHandler {
                                     && requiredGoods.getAmount() <= label.getAmount()) {
                                     int amount = Math.min(label.getAmount() / requiredGoods.getAmount(),
                                                           equipment.getMaximumCount());
-                                    unitLabel.getCanvas().getClient().getInGameController()
+                                    freeColClient.getInGameController()
                                         .equipUnit(unit, equipment, amount);
                                     unitLabel.updateIcon();
                                     return true;
@@ -479,8 +489,8 @@ public final class DefaultTransferHandler extends TransferHandler {
     /**
     * Displays an input dialog box where the user should specify a goods transfer amount.
     */
-    private int getAmount(GoodsType goodsType, int available, boolean needToPay) {
-        return canvas.showFreeColDialog(new SelectAmountDialog(canvas, goodsType, available, needToPay));
+    private int getAmount(GoodsType goodsType, int available, int defaultAmount, boolean needToPay) {
+        return gui.showSelectAmountDialog(goodsType, available, defaultAmount, needToPay);
     }
 
 
