@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -24,7 +24,6 @@ import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.awt.event.MouseAdapter;
 import java.awt.event.MouseEvent;
-
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
@@ -36,11 +35,14 @@ import javax.swing.JMenuItem;
 import javax.swing.JPopupMenu;
 import javax.swing.TransferHandler;
 
+import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.control.InGameController;
 import net.sf.freecol.client.gui.Canvas;
+import net.sf.freecol.client.gui.GUI;
 import net.sf.freecol.client.gui.ImageLibrary;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.client.gui.panel.UnitLabel.UnitAction;
+import net.sf.freecol.common.model.Ability;
 import net.sf.freecol.common.model.AbstractGoods;
 import net.sf.freecol.common.model.Building;
 import net.sf.freecol.common.model.Colony;
@@ -49,14 +51,17 @@ import net.sf.freecol.common.model.EquipmentType;
 import net.sf.freecol.common.model.Europe;
 import net.sf.freecol.common.model.GameOptions;
 import net.sf.freecol.common.model.Goods;
+import net.sf.freecol.common.model.GoodsContainer;
 import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.StringTemplate;
 import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.Unit.Role;
 import net.sf.freecol.common.model.Unit.UnitState;
 import net.sf.freecol.common.model.UnitType;
 import net.sf.freecol.common.model.UnitTypeChange.ChangeType;
 import net.sf.freecol.common.model.WorkLocation;
 
+
 /**
  * A DragListener should be attached to Swing components that have a
  * TransferHandler attached. The DragListener will make sure that the Swing
@@ -69,13 +74,20 @@ public final class DragListener extends MouseAdapter {
     private final FreeColPanel parentPanel;
     private final Canvas canvas;
 
+    private FreeColClient freeColClient;
+
+    private GUI gui;
+
     /**
      * The constructor to use.
+     * @param freeColClient 
      *
      * @param parentPanel The layered pane that contains the components to which
      *            a DragListener might be attached.
      */
-    public DragListener(FreeColPanel parentPanel) {
+    public DragListener(FreeColClient freeColClient, GUI gui, FreeColPanel parentPanel) {
+        this.freeColClient = freeColClient;
+        this.gui = gui;
         this.parentPanel = parentPanel;
         this.canvas = parentPanel.getCanvas();
     }
@@ -102,7 +114,8 @@ public final class DragListener extends MouseAdapter {
                 } else if (comp instanceof MarketLabel
                            && parentPanel instanceof EuropePanel) {
                     GoodsType goodsType = ((MarketLabel) comp).getType();
-                    if (canvas.getClient().getInGameController().payArrears(goodsType)) {
+                    if (freeColClient.getInGameController()
+                        .payArrears(goodsType)) {
                         ((EuropePanel) parentPanel).revalidate();
                         ((EuropePanel) parentPanel).refresh();
                     }
@@ -114,7 +127,8 @@ public final class DragListener extends MouseAdapter {
                         if (menu.getComponent(lastIndex) instanceof JPopupMenu.Separator) {
                             menu.remove(lastIndex);
                         }
-                        if (System.getProperty("os.name").startsWith("Windows")) {
+                        if (gui.isWindowed()
+                            && System.getProperty("os.name").startsWith("Windows")) {
                             // work-around: JRE on Windows is unable
                             // to display popup menus that extend
                             // beyond the canvas
@@ -145,14 +159,14 @@ public final class DragListener extends MouseAdapter {
                     ((GoodsLabel) comp).setPartialChosen(false);
                 } else if (comp instanceof MarketLabel) {
                     ((MarketLabel) comp).setPartialChosen(false);
-                    ((MarketLabel) comp).setAmount(100);
+                    ((MarketLabel) comp).setAmount(GoodsContainer.CARGO_SIZE);
                 }
             }
 
             if ((comp instanceof UnitLabel) && (((UnitLabel) comp).getUnit().isCarrier())) {
                 Unit u = ((UnitLabel) comp).getUnit();
                 if (parentPanel instanceof EuropePanel) {
-                    if (!u.isBetweenEuropeAndNewWorld()) {
+                    if (!u.isAtSea()) {
                         ((EuropePanel) parentPanel).setSelectedUnitLabel((UnitLabel) comp);
                     }
                 } else if (parentPanel instanceof ColonyPanel) {
@@ -217,7 +231,7 @@ public final class DragListener extends MouseAdapter {
             }
         }
 
-        if (tempUnit.hasAbility("model.ability.canBeEquipped")) {
+        if (tempUnit.hasAbility(Ability.CAN_BE_EQUIPPED)) {
             if (addEquipmentItems(unitLabel, menu)) {
                 menu.addSeparator();
             }
@@ -255,11 +269,23 @@ public final class DragListener extends MouseAdapter {
         ImageLibrary imageLibrary = parentPanel.getLibrary();
         Colony colony = tempUnit.getLocation().getColony();
         boolean separatorNeeded = false;
+        ColonyTile also = (tempUnit.getWorkLocation2() instanceof ColonyTile)
+            ? (ColonyTile) tempUnit.getWorkLocation2()
+            : null;
 
-        List<GoodsType> farmedGoods = canvas.getSpecification().getFarmedGoodsTypeList();
+        List<GoodsType> farmedGoods = freeColClient.getGame().getSpecification().getFarmedGoodsTypeList();
         // Work in Field - automatically find the best location
         for (GoodsType goodsType : farmedGoods) {
             ColonyTile bestTile = colony.getVacantColonyTileFor(tempUnit, false, goodsType);
+            if (also != null) {
+                int alsoProd = also.getProductionOf(tempUnit, goodsType);
+                if (bestTile == null) {
+                    if (alsoProd > 0) bestTile = also;
+                } else {
+                    if (alsoProd > bestTile.getProductionOf(tempUnit, goodsType))
+                        bestTile = also;
+                }
+            }
             if (bestTile != null) {
                 int maxpotential = bestTile.getProductionOf(tempUnit, goodsType);
                 String text = Messages.message(StringTemplate.template(goodsType.getId() + ".workAs")
@@ -275,32 +301,27 @@ public final class DragListener extends MouseAdapter {
 
         // Work at Building - show both max potential and realistic projection
         for (Building building : colony.getBuildings()) {
-            if (tempUnit.getWorkLocation() != building) { // Skip if currently working at this location
-                if (building.canAdd(tempUnit)) {
-                    GoodsType goodsType = building.getGoodsOutputType();
-                    String locName = Messages.message(building.getNameKey());
-                    JMenuItem menuItem = new JMenuItem(locName);
-                    if (goodsType != null) {
-                        menuItem.setIcon(imageLibrary.getScaledGoodsImageIcon(goodsType, 0.66f));
-                        int addOutput = building.getAdditionalProductionNextTurn(tempUnit);
-                        locName += " (" + addOutput;
-                        int potential = building.getAdditionalProduction(tempUnit);
-                        if (addOutput < potential) {
-                            // Not reaching full potential, show full potential
-                            locName += "/" + potential;
-                        }
-                        locName +=  " " + Messages.message(goodsType.getNameKey()) +")";
-                        menuItem.setText(locName);
-                        if (addOutput == 0) {
-                            menuItem.setForeground(FreeColPanel.WARNING_COLOR);
-                        }
-                    }
-                    menuItem.setActionCommand(UnitAction.WORK_BUILDING.toString() + ":" +
-                                              building.getType().getId());
-                    menuItem.addActionListener(unitLabel);
-                    menu.add(menuItem);
-                    separatorNeeded = true;
+            switch (building.getNoAddReason(tempUnit)) {
+            case NONE: case ALREADY_PRESENT:
+                GoodsType goodsType = building.getGoodsOutputType();
+                String locName = Messages.message(building.getNameKey());
+                JMenuItem menuItem = new JMenuItem(locName);
+                if (goodsType != null) {
+                    menuItem.setIcon(imageLibrary.getScaledGoodsImageIcon(goodsType, 0.66f));
+                    StringTemplate t = StringTemplate.template("model.goods.goodsAmount")
+                        .addAmount("%amount%", building.getAdditionalProductionNextTurn(tempUnit))
+                        .addName("%goods%", goodsType);
+                    locName += " (" + Messages.message(t) +")";
+                    menuItem.setText(locName);
                 }
+                menuItem.setActionCommand(UnitAction.WORK_BUILDING.toString() + ":" +
+                    building.getType().getId());
+                menuItem.addActionListener(unitLabel);
+                menu.add(menuItem);
+                separatorNeeded = true;
+                break;
+            default:
+                break;
             }
         }
 
@@ -308,7 +329,7 @@ public final class DragListener extends MouseAdapter {
             JMenuItem menuItem = new JMenuItem(Messages.message("showProduction"));
             menuItem.addActionListener(new ActionListener() {
                     public void actionPerformed(ActionEvent event) {
-                        canvas.showSubPanel(new WorkProductionPanel(canvas, tempUnit));
+                        gui.showWorkProductionPanel(tempUnit);
                     }
                 });
             menu.add(menuItem);
@@ -317,7 +338,7 @@ public final class DragListener extends MouseAdapter {
             JMenuItem menuItem = new JMenuItem(Messages.message("showProductivity"));
             menuItem.addActionListener(new ActionListener() {
                     public void actionPerformed(ActionEvent event) {
-                        canvas.showSubPanel(new WorkProductionPanel(canvas, tempUnit));
+                        gui.showWorkProductionPanel(tempUnit);
                     }
                 });
             menu.add(menuItem);
@@ -365,11 +386,8 @@ public final class DragListener extends MouseAdapter {
 
         int experience = unit.getExperience();
         GoodsType goods = unit.getExperienceType();
-        if (goods == null) {
-            goods = unit.getWorkType();
-        }
         if (experience > 0 && goods != null) {
-            UnitType expertType = canvas.getSpecification().getExpertForProducing(goods);
+            UnitType expertType = freeColClient.getGame().getSpecification().getExpertForProducing(goods);
             if (unit.getType().canBeUpgraded(expertType, ChangeType.EXPERIENCE)) {
                 int maxExperience = unit.getType().getMaximumExperience();
                 double probability = unit.getType().getUnitTypeChange(expertType)
@@ -393,13 +411,13 @@ public final class DragListener extends MouseAdapter {
 
     private boolean addCommandItems(final UnitLabel unitLabel, final JPopupMenu menu) {
         final Unit tempUnit = unitLabel.getUnit();
-        final boolean isUnitBetweenEuropeAndNewWorld = tempUnit.isBetweenEuropeAndNewWorld();
+        final boolean isUnitAtSea = tempUnit.isAtSea();
 
         JMenuItem menuItem = new JMenuItem(Messages.message("activateUnit"));
         menuItem.setActionCommand(UnitAction.ACTIVATE_UNIT.toString());
         menuItem.addActionListener(unitLabel);
-        menuItem.setEnabled(tempUnit.getState() != UnitState.ACTIVE
-                                && !isUnitBetweenEuropeAndNewWorld);
+        menuItem.setEnabled(tempUnit.getState() != Unit.UnitState.ACTIVE
+                                && !isUnitAtSea);
         menu.add(menuItem);
 
         if (!(tempUnit.getLocation() instanceof Europe)) {
@@ -407,8 +425,8 @@ public final class DragListener extends MouseAdapter {
             menuItem.setActionCommand(UnitAction.FORTIFY.toString());
             menuItem.addActionListener(unitLabel);
             menuItem.setEnabled((tempUnit.getMovesLeft() > 0)
-                                && !(tempUnit.getState() == UnitState.FORTIFIED ||
-                                     tempUnit.getState() == UnitState.FORTIFYING));
+                && !(tempUnit.getState() == Unit.UnitState.FORTIFIED
+                    || tempUnit.getState() == Unit.UnitState.FORTIFYING));
             menu.add(menuItem);
         }
 
@@ -416,23 +434,30 @@ public final class DragListener extends MouseAdapter {
         menuItem = new JMenuItem(Messages.message("sentryUnit"));
         menuItem.setActionCommand(UnitAction.SENTRY.toString());
         menuItem.addActionListener(unitLabel);
-        menuItem.setEnabled(unitState != UnitState.SENTRY
-                                && !isUnitBetweenEuropeAndNewWorld);
+        menuItem.setEnabled(unitState != Unit.UnitState.SENTRY
+            && !isUnitAtSea);
         menu.add(menuItem);
 
         boolean hasTradeRoute = tempUnit.getTradeRoute() != null;
         menuItem = new JMenuItem(Messages.message("clearUnitOrders"));
         menuItem.setActionCommand(UnitAction.CLEAR_ORDERS.toString());
         menuItem.addActionListener(unitLabel);
-        menuItem.setEnabled((unitState != UnitState.ACTIVE || hasTradeRoute)
-                                && !isUnitBetweenEuropeAndNewWorld);
+        menuItem.setEnabled((unitState != Unit.UnitState.ACTIVE
+                || hasTradeRoute)
+            && !isUnitAtSea);
         menu.add(menuItem);
 
-        if (tempUnit.canCarryTreasure() && !tempUnit.getColony().isLandLocked()) {
+        menuItem = new JMenuItem(Messages.message("assignTradeRoute"));
+        menuItem.setActionCommand(UnitAction.ASSIGN_TRADE_ROUTE.toString());
+        menuItem.addActionListener(unitLabel);
+        menuItem.setEnabled(tempUnit.isCarrier() && !hasTradeRoute);
+        menu.add(menuItem);
+
+        if (tempUnit.canCarryTreasure() && tempUnit.canCashInTreasureTrain()) {
             menuItem = new JMenuItem(Messages.message("cashInTreasureTrain.order"));
             menuItem.addActionListener(new ActionListener() {
                     public void actionPerformed(ActionEvent e) {
-                        canvas.getClient().getInGameController()
+                        freeColClient.getInGameController()
                             .checkCashInTreasureTrain(tempUnit);
                     }
                 });
@@ -444,7 +469,8 @@ public final class DragListener extends MouseAdapter {
 
     private boolean addEquipmentItems(final UnitLabel unitLabel, final JPopupMenu menu) {
         final Unit tempUnit = unitLabel.getUnit();
-        final InGameController igc = canvas.getClient().getInGameController();
+        final InGameController igc = freeColClient
+            .getInGameController();
         ImageLibrary imageLibrary = parentPanel.getLibrary();
         boolean separatorNeeded = false;
         if (tempUnit.getEquipment().size() > 1) {
@@ -464,7 +490,7 @@ public final class DragListener extends MouseAdapter {
 
         EquipmentType horses = null;
         EquipmentType muskets = null;
-        for (EquipmentType equipmentType : canvas.getSpecification().getEquipmentTypeList()) {
+        for (EquipmentType equipmentType : freeColClient.getGame().getSpecification().getEquipmentTypeList()) {
             int count = tempUnit.getEquipment().getCount(equipmentType);
             if (count > 0) {
                 // "remove current equipment" action
@@ -538,11 +564,13 @@ public final class DragListener extends MouseAdapter {
                 }
             }
         }
+        
         // convenience menu for equipping dragoons
         if (horses != null && muskets != null && horses.isCompatibleWith(muskets)) {
             final EquipmentType horseType = horses;
             final EquipmentType musketType = muskets;
-            JMenuItem newItem = new JMenuItem(Messages.message("model.equipment.dragoon"));
+            JMenuItem newItem = new JMenuItem(Messages.message("model.equipment.dragoon"),
+                imageLibrary.getUnitImageIcon(tempUnit.getType(), Role.DRAGOON, 1.0/3));
             newItem.addActionListener(new ActionListener() {
                     public void actionPerformed(ActionEvent e) {
                         igc.equipUnit(tempUnit, horseType, 1);
@@ -562,7 +590,8 @@ public final class DragListener extends MouseAdapter {
 
         UnitType newUnitType = tempUnit.getType().getTargetType(ChangeType.CLEAR_SKILL, tempUnit.getOwner());
         if (newUnitType != null) {
-            JMenuItem menuItem = new JMenuItem(Messages.message("clearSpeciality"));
+            JMenuItem menuItem = new JMenuItem(Messages.message("clearSpeciality"),
+                imageLibrary.getUnitImageIcon(newUnitType, 1.0/3));
             menuItem.setActionCommand(UnitAction.CLEAR_SPECIALITY.toString());
             menuItem.addActionListener(unitLabel);
             menu.add(menuItem);
@@ -579,7 +608,8 @@ public final class DragListener extends MouseAdapter {
     public JPopupMenu getGoodsMenu(final GoodsLabel goodsLabel) {
 
         final Goods goods = goodsLabel.getGoods();
-        final InGameController inGameController = canvas.getClient().getInGameController();
+        final InGameController inGameController = freeColClient
+            .getInGameController();
         ImageLibrary imageLibrary = parentPanel.getLibrary();
         JPopupMenu menu = new JPopupMenu("Cargo");
         JMenuItem name = new JMenuItem(Messages.message(goods.getNameKey()) + " (" +
@@ -587,14 +617,13 @@ public final class DragListener extends MouseAdapter {
                                        imageLibrary.getScaledGoodsImageIcon(goods.getType(), 0.66f));
         name.addActionListener(new ActionListener() {
                 public void actionPerformed(ActionEvent e) {
-                    canvas.showPanel(new ColopediaPanel(canvas, ColopediaPanel.PanelType.GOODS,
-                                                        goods.getType()));
+                    gui.showColopediaPanel(goods.getType().getId());
                 }
             });
         menu.add(name);
 
         if (!(goods.getLocation() instanceof Colony)) {
-            if (canvas.getClient().getMyPlayer().canTrade(goods)) {
+            if (freeColClient.getMyPlayer().canTrade(goods)) {
                 JMenuItem unload = new JMenuItem(Messages.message("unload"));
                 unload.addActionListener(new ActionListener() {
                         public void actionPerformed(ActionEvent e) {
@@ -618,7 +647,7 @@ public final class DragListener extends MouseAdapter {
                     JMenuItem pay = new JMenuItem(Messages.message("boycottedGoods.payArrears"));
                     pay.addActionListener(new ActionListener() {
                             public void actionPerformed(ActionEvent e) {
-                                inGameController.payArrears(goods);
+                                inGameController.payArrears(goods.getType());
                                 if (parentPanel instanceof CargoPanel) {
                                     CargoPanel cargoPanel = (CargoPanel) parentPanel;
                                     cargoPanel.initialize();
