@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -51,17 +51,22 @@ import javax.swing.text.StyledDocument;
 
 import net.miginfocom.swing.MigLayout;
 import net.sf.freecol.client.ClientOptions;
+import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.gui.Canvas;
+import net.sf.freecol.client.gui.GUI;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.common.model.Europe;
+import net.sf.freecol.common.model.FreeColGameObject;
 import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.Goods;
 import net.sf.freecol.common.model.GoodsType;
+import net.sf.freecol.common.model.HighSeas;
+import net.sf.freecol.common.model.Location;
+import net.sf.freecol.common.model.Map;
 import net.sf.freecol.common.model.Market;
 import net.sf.freecol.common.model.StringTemplate;
 import net.sf.freecol.common.model.TransactionListener;
 import net.sf.freecol.common.model.Unit;
-import net.sf.freecol.common.model.Unit.UnitState;
 
 
 /**
@@ -81,9 +86,9 @@ public final class EuropePanel extends FreeColPanel {
         SAIL
     }
 
-    private final ToAmericaPanel toAmericaPanel;
+    private final DestinationPanel toAmericaPanel;
 
-    private final ToEuropePanel toEuropePanel;
+    private final DestinationPanel toEuropePanel;
 
     private final InPortPanel inPortPanel;
 
@@ -110,11 +115,12 @@ public final class EuropePanel extends FreeColPanel {
 
     /**
      * The constructor for a EuropePanel.
+     * @param freeColClient 
      *
      * @param parent The parent of this panel
      */
-    public EuropePanel(Canvas parent) {
-        super(parent);
+    public EuropePanel(FreeColClient freeColClient, GUI gui, Canvas canvas) {
+        super(freeColClient, gui);
 
         setFocusCycleRoot(true);
 
@@ -132,10 +138,10 @@ public final class EuropePanel extends FreeColPanel {
         EuropeButton sailButton = new EuropeButton(Messages.message("sail"),
                                                    KeyEvent.VK_S, EuropeAction.SAIL.toString(), this);
 
-        toAmericaPanel = new ToAmericaPanel(this);
-        toEuropePanel = new ToEuropePanel(this);
+        toAmericaPanel = new DestinationPanel();
+        toEuropePanel = new DestinationPanel();
         inPortPanel = new InPortPanel();
-        cargoPanel = new EuropeCargoPanel(parent);
+        cargoPanel = new EuropeCargoPanel(freeColClient, getGUI());
         cargoPanel.setParentPanel(this);
         docksPanel = new DocksPanel();
         marketPanel = new MarketPanel(this);
@@ -147,7 +153,7 @@ public final class EuropePanel extends FreeColPanel {
         StyleConstants.setBold(attributes, true);
         log.setParagraphAttributes(attributes, true);
 
-        defaultTransferHandler = new DefaultTransferHandler(parent, this);
+        defaultTransferHandler = new DefaultTransferHandler(freeColClient, getGUI(), this);
         toAmericaPanel.setTransferHandler(defaultTransferHandler);
         toEuropePanel.setTransferHandler(defaultTransferHandler);
         inPortPanel.setTransferHandler(defaultTransferHandler);
@@ -155,7 +161,7 @@ public final class EuropePanel extends FreeColPanel {
         docksPanel.setTransferHandler(defaultTransferHandler);
         marketPanel.setTransferHandler(defaultTransferHandler);
 
-        pressListener = new DragListener(this);
+        pressListener = new DragListener(freeColClient, gui, this);
         MouseListener releaseListener = new DropListener();
         toAmericaPanel.addMouseListener(releaseListener);
         toEuropePanel.addMouseListener(releaseListener);
@@ -215,7 +221,7 @@ public final class EuropePanel extends FreeColPanel {
         setLayout(new MigLayout("wrap 3, insets 20, fill",
                                 "[380:][380:][150:200:]"));
 
-        if (parent.getHeight() > 750) {
+        if (canvas.getHeight() > 750) {
             add(header, "span, center");
         }
         add(toAmericaScroll, "sg, height 124:, grow");
@@ -239,7 +245,7 @@ public final class EuropePanel extends FreeColPanel {
         // of this fake mouse listener.
         addMouseListener(new MouseAdapter() {});
 
-        restoreSavedSize(1000, parent.getHeight() > 750 ? 700 : 600);
+        restoreSavedSize(1000, canvas.getHeight() > 750 ? 700 : 600);
     }
 
     /**
@@ -340,8 +346,8 @@ public final class EuropePanel extends FreeColPanel {
         header.setText(Messages.message(europe.getNameKey()));
 
         // Initialize the subpanels.
-        toAmericaPanel.initialize();
-        toEuropePanel.initialize();
+        toAmericaPanel.initialize(getGame().getMap());
+        toEuropePanel.initialize(getMyPlayer().getEurope());
         // Initialize cargoPanel before inPortPanel calls setSelectedUnit().
         cargoPanel.initialize();
         inPortPanel.initialize();
@@ -369,7 +375,7 @@ public final class EuropePanel extends FreeColPanel {
      */
     private void exitAction() {
         cleanup();
-        getCanvas().remove(this);
+        getGUI().removeFromCanvas(this);
         getController().nextModelMessage();
     }
 
@@ -385,7 +391,7 @@ public final class EuropePanel extends FreeColPanel {
                 if (getMyPlayer().canTrade(goods)) {
                     getController().sellGoods(goods);
                 } else {
-                    getController().payArrears(goods);
+                    getController().payArrears(goods.getType());
                 }
             }
             Iterator<Unit> unitIterator = unit.getUnitIterator();
@@ -418,29 +424,25 @@ public final class EuropePanel extends FreeColPanel {
      * @param event The incoming action event
      */
     public void actionPerformed(ActionEvent event) {
-        try {
-            String command = event.getActionCommand();
-            // Close any open Europe Dialog, and show new one if required
-            EuropeAction act = Enum.valueOf(EuropeAction.class, command);
-            getCanvas().showEuropeDialog(act);
-            switch (act) {
-            case EXIT:
-                exitAction();
-                break;
-            case RECRUIT: case PURCHASE: case TRAIN:
-                requestFocus(); // handled by docks panel
-                break;
-            case UNLOAD:
-                unloadAction();
-                break;
-            case SAIL:
-                sailAction();
-                break;
-            default:
-                logger.warning("Invalid action command");
-            }
-        } catch (NumberFormatException e) {
-            logger.warning("Invalid action number");
+        String command = event.getActionCommand();
+        // Close any open Europe Dialog, and show new one if required
+        EuropeAction act = EuropeAction.valueOf(command);
+        getGUI().showEuropeDialog(act);
+        switch (act) {
+        case EXIT:
+            exitAction();
+            break;
+        case RECRUIT: case PURCHASE: case TRAIN:
+            requestFocus(); // handled by docks panel
+            break;
+        case UNLOAD:
+            unloadAction();
+            break;
+        case SAIL:
+            sailAction();
+            break;
+        default:
+            logger.warning("Invalid action command: " + command);
         }
     }
 
@@ -451,66 +453,75 @@ public final class EuropePanel extends FreeColPanel {
      */
     public final class EuropeCargoPanel extends CargoPanel {
 
-        public EuropeCargoPanel(Canvas canvas) {
-            super(canvas, true);
+        public EuropeCargoPanel(FreeColClient freeColClient, GUI gui) {
+            super(freeColClient, gui, true);
         }
 
     }
 
     /**
      * A panel that holds UnitsLabels that represent Units that are going to
-     * America.
+     * America or Europe.
      */
-    public final class ToAmericaPanel extends JPanel {
+    public final class DestinationPanel extends JPanel {
 
-        private final EuropePanel europePanel;
+        private Location destination;
 
         /**
-         * Creates this ToAmericaPanel.
-         *
-         * @param europePanel The panel that holds this ToAmericaPanel.
+         * Initialize this DestinationPanel.
          */
-        public ToAmericaPanel(EuropePanel europePanel) {
-            this.europePanel = europePanel;
-        }
-
-        /**
-         * Initialize this ToAmericaPanel.
-         */
-        public void initialize() {
+        public void initialize(Location destination) {
+            this.destination = destination;
             update();
         }
 
         /**
-         * Cleans up this ToAmericaPanel.
+         * Cleans up this DestinationPanel.
          */
         public void cleanup() {}
 
         /**
-         * Update this ToAmericaPanel.
+         * Update this DestinationPanel.
          */
         public void update() {
             removeAll();
 
-            for (Unit unit : europe.getUnitList()) {
-                if (unit.isNaval()
-                    && unit.getState() == UnitState.TO_AMERICA) {
-                    UnitLabel unitLabel = new UnitLabel(unit, getCanvas());
-                    unitLabel.setTransferHandler(defaultTransferHandler);
-                    unitLabel.addMouseListener(pressListener);
-                    add(unitLabel);
+            HighSeas highSeas = getMyPlayer().getHighSeas();
+            if (highSeas != null) {
+                for (Unit unit : highSeas.getUnitList()) {
+                    boolean belongs;
+                    if (destination instanceof Europe) {
+                        belongs = unit.getDestination() == destination;
+                    } else if (destination instanceof Map) {
+                        belongs = unit.getDestination() == destination
+                            || (unit.getDestination() != null
+                                && unit.getDestination().getTile() != null
+                                && unit.getDestination().getTile().getMap()
+                                == destination);
+                    } else {
+                        logger.warning("Bogus DestinationPanel location: "
+                            + ((FreeColGameObject) destination)
+                            + " for unit: " + unit);
+                        belongs = false;
+                    }
+                    if (belongs) {
+                        UnitLabel unitLabel = new UnitLabel(getFreeColClient(), unit, getGUI());
+                        unitLabel.setTransferHandler(defaultTransferHandler);
+                        unitLabel.addMouseListener(pressListener);
+                        add(unitLabel);
+                    }
                 }
             }
 
-            String newLandName = Messages.getNewLandName(getMyPlayer());
             StringTemplate t = StringTemplate.template("sailingTo")
-                .addName("%location%", newLandName);
+                .addStringTemplate("%location%",
+                    destination.getLocationNameFor(getMyPlayer()));
             ((TitledBorder) getBorder()).setTitle(Messages.message(t));
             revalidate();
         }
 
         /**
-         * Adds a component to this ToAmericaPanel and makes sure that
+         * Adds a component to this DestinationPanel and makes sure that
          * the unit that the component represents gets modified so
          * that it will sail to America.
          *
@@ -524,116 +535,45 @@ public final class EuropePanel extends FreeColPanel {
         public Component add(Component comp, boolean editState) {
             if (editState) {
                 if (!(comp instanceof UnitLabel)) {
-                    logger.warning("Invalid component dropped on this ToAmericaPanel.");
+                    logger.warning("Invalid component dropped on this DestinationPanel.");
                     return null;
                 }
                 final Unit unit = ((UnitLabel) comp).getUnit();
-                final ClientOptions co = getClient().getClientOptions();
-                boolean autoload = co.getBoolean(ClientOptions.AUTOLOAD_EMIGRANTS);
-                if (!autoload
+
+                Location dest = destination;
+                if (unit.isInEurope()) {
+                    dest = getGUI().showSelectDestinationDialog(unit);
+                    if (dest == null) return null; // user aborted
+                }
+
+                final ClientOptions co = getClientOptions();
+                if (!co.getBoolean(ClientOptions.AUTOLOAD_EMIGRANTS)
+                    && unit.isInEurope()
+                    && !(destination instanceof Europe)
                     && docksPanel.getComponentCount() > 0
                     && unit.getSpaceLeft() > 0) {
-                    boolean leave = getCanvas()
-                        .showConfirmDialog(null,
-                                           StringTemplate.template("europe.leaveColonists")
-                                           .addName("%newWorld%", Messages.getNewLandName(unit.getOwner())),
-                                           "yes", "no");
-                    if (!leave) { // Colonists remain in Europe.
-                        return null;
-                    }
+                    boolean leave = getGUI().showConfirmDialog(null,
+                        StringTemplate.template("europe.leaveColonists")
+                            .addStringTemplate("%newWorld%",
+                                destination.getLocationNameFor(unit.getOwner())),
+                        "yes", "no");
+                    if (!leave) return null; // Colonists remain in Europe.
                 }
+
                 comp.getParent().remove(comp);
-
-                getController().moveToAmerica(unit);
-
+                getController().moveTo(unit, dest);
                 inPortPanel.update();
                 docksPanel.update();
             }
 
             Component c = add(comp);
-            toAmericaPanel.revalidate();
-            europePanel.refresh();
-            return c;
-        }
-    }
-
-    /**
-     * A panel that holds UnitsLabels that represent Units that are
-     * going to Europe.
-     */
-    public final class ToEuropePanel extends JPanel {
-
-        private final EuropePanel europePanel;
-
-        /**
-         * Creates this ToEuropePanel.
-         *
-         * @param europePanel The panel that holds this ToEuropePanel.
-         */
-        public ToEuropePanel(EuropePanel europePanel) {
-            this.europePanel = europePanel;
-        }
-
-        /**
-         * Initialize this ToEuropePanel.
-         */
-        public void initialize() {
-            update();
-        }
-
-        /**
-         * Cleans up this ToEuropePanel.
-         */
-        public void cleanup() {}
-
-        /**
-         * Update this ToEuropePanel.
-         */
-        public void update() {
-            removeAll();
-
-            for (Unit unit : europe.getUnitList()) {
-                if (unit.isNaval()
-                    && unit.getState() == UnitState.TO_EUROPE) {
-                    UnitLabel unitLabel = new UnitLabel(unit, getCanvas());
-                    unitLabel.setTransferHandler(defaultTransferHandler);
-                    unitLabel.addMouseListener(pressListener);
-                    add(unitLabel);
-                }
-            }
             revalidate();
-        }
-
-        /**
-         * Adds a component to this ToEuropePanel and makes sure that
-         * the unit that the component represents gets modified so
-         * that it will sail to Europe.
-         *
-         * @param comp The component to add.
-         * @param editState Must be set to 'true' if the state of the component
-         *            that is added (which should be a dropped component
-         *            representing a Unit) should be changed so that the
-         *            underlying unit is now sailing to Europe.
-         * @return The component argument.
-         */
-        public Component add(Component comp, boolean editState) {
-            if (editState) {
-                if (!(comp instanceof UnitLabel)) {
-                    logger.warning("Invalid component dropped on this ToEuropePanel.");
-                    return null;
-                }
-                comp.getParent().remove(comp);
-                Unit unit = ((UnitLabel) comp).getUnit();
-
-                getController().moveToEurope(unit);
-            }
-
-            Component c = add(comp);
-            europePanel.refresh();
+            EuropePanel.this.refresh();
             return c;
         }
     }
 
+
     /**
      * A panel that holds UnitLabels that represent naval units that are
      * waiting in Europe.
@@ -666,9 +606,9 @@ public final class EuropePanel extends FreeColPanel {
             UnitLabel prevCarrier = null;
             for (Unit unit : europe.getUnitList()) {
                 if (unit.isNaval()
-                    && (unit.getState() == UnitState.ACTIVE
-                        || unit.getState() == UnitState.SENTRY)) {
-                    UnitLabel unitLabel = new UnitLabel(unit, getCanvas());
+                    && (unit.getState() == Unit.UnitState.ACTIVE
+                        || unit.getState() == Unit.UnitState.SENTRY)) {
+                    UnitLabel unitLabel = new UnitLabel(getFreeColClient(), unit, getGUI());
                     unitLabel.setTransferHandler(defaultTransferHandler);
                     unitLabel.addMouseListener(pressListener);
                     add(unitLabel);
@@ -734,7 +674,7 @@ public final class EuropePanel extends FreeColPanel {
             List<Unit> units = europe.getUnitList();
             for (Unit unit : units) {
                 if (!unit.isNaval()) {
-                    UnitLabel unitLabel = new UnitLabel(unit, getCanvas());
+                    UnitLabel unitLabel = new UnitLabel(getFreeColClient(), unit, getGUI());
                     unitLabel.setTransferHandler(defaultTransferHandler);
                     unitLabel.addMouseListener(pressListener);
                     add(unitLabel);
@@ -800,7 +740,7 @@ public final class EuropePanel extends FreeColPanel {
             for (GoodsType goodsType : goodsTypes) {
                 if (goodsType.isStorable()) {
                     MarketLabel marketLabel
-                        = new MarketLabel(goodsType, market, getCanvas());
+                        = new MarketLabel(goodsType, market, getGUI());
                     marketLabel.setTransferHandler(defaultTransferHandler);
                     marketLabel.addMouseListener(pressListener);
                     add(marketLabel);
@@ -833,9 +773,9 @@ public final class EuropePanel extends FreeColPanel {
                 if (getMyPlayer().canTrade(goods)) {
                     getController().sellGoods(goods);
                 } else {
-                    switch (getCanvas().showBoycottedGoodsDialog(goods, europe)) {
+                    switch (getGUI().showBoycottedGoodsDialog(goods, europe)) {
                     case PAY_ARREARS:
-                        getController().payArrears(goods);
+                        getController().payArrears(goods.getType());
                         break;
                     case DUMP_CARGO:
                         getController().unloadCargo(goods, true);
