@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -48,139 +48,86 @@ import javax.swing.border.EmptyBorder;
 import javax.swing.filechooser.FileFilter;
 
 import net.miginfocom.swing.MigLayout;
-import net.sf.freecol.FreeCol;
-import net.sf.freecol.client.gui.Canvas;
+import net.sf.freecol.client.FreeColClient;
+import net.sf.freecol.client.gui.GUI;
 import net.sf.freecol.client.gui.i18n.Messages;
 
 
 /**
-* Superclass for all dialogs in FreeCol. This class also contains
-* methods to create simple dialogs.
-*/
+ * Superclass for all dialogs in FreeCol. This class also contains
+ * methods to create simple dialogs.
+ */
 public class FreeColDialog<T> extends FreeColPanel {
 
+    static final class FreeColFileFilter extends FileFilter {
+
+        private final String  extension1;
+        private final String  extension2;
+        private final String  description;
+
+        FreeColFileFilter( String  extension, String  descriptionMessage ) {
+
+            this.extension1 = extension;
+            this.extension2 = "....";
+            description = Messages.message(descriptionMessage);
+        }
+
+        FreeColFileFilter( String  extension1,
+                           String  extension2,
+                           String  descriptionMessage ) {
+
+            this.extension1 = extension1;
+            this.extension2 = extension2;
+            description = Messages.message(descriptionMessage);
+        }
+
+        public boolean accept(File f) {
+
+            return f.isDirectory() || f.getName().endsWith(extension1)
+                || f.getName().endsWith(extension2);
+        }
+
+        public String getDescription() {
+
+            return description;
+        }
+    }
+
     private static final Logger logger = Logger.getLogger(FreeColDialog.class.getName());
 
     protected static final String CANCEL = "CANCEL";
 
-    // Stores the response from the user:
-    private T response = null;
-
-    // Whether or not the user have made the choice.
-    private boolean responseGiven = false;
-
-    protected JButton cancelButton = new JButton(Messages.message("cancel"));
-
     /**
-     * Constructor.
-     * @param parent <code>Canvas</code>
+     * Creates a new <code>FreeColDialog</code> with a text and a
+     * cancel-button, in addition to buttons for each of the objects
+     * in the given array.
+     *
+     * @param text String, text that explains the choice for the user
+     * @param cancelText String, text displayed on the "cancel"-button
+     * @param choices List of <code>ChoiceItem<T> <code> the choice items
+     * @return <code>FreeColDialog</code>
+     * @see ChoiceItem
      */
-    public FreeColDialog(Canvas parent) {
-        super(parent);
-
-        cancelButton.setActionCommand(CANCEL);
-        cancelButton.addActionListener(this);
-        enterPressesWhenFocused(cancelButton);
-        setCancelComponent(cancelButton);
-    }
-
-    /**
-    * Sets the <code>response</code> and wakes up any thread waiting for this information.
-    *
-    * @param response The object that should be returned by {@link #getResponse}.
-    */
-    public synchronized void setResponse(T response) {
-        this.response = response;
-        responseGiven = true;
-        logger.info("Response has been set to " + response);
-        notifyAll();
-    }
-
-
-    /**
-    * Returns the <code>response</code> when set by <code>setResponse(Object response)</code>.
-    * Waits the thread until then.
-    *
-    * @return The object as set by {@link #setResponse}.
-    */
-    public synchronized T getResponse() {
-        // Wait the thread until 'response' is available. Notice that we have to process
-        // the events manually if the current thread is the Event Dispatch Thread (EDT).
-
-        try {
-            if (SwingUtilities.isEventDispatchThread()) {
-                EventQueue theQueue = getToolkit().getSystemEventQueue();
-
-                while (!responseGiven) {
-                    // This is essentially the body of EventDispatchThread
-                    AWTEvent event = theQueue.getNextEvent();
-                    Object src = event.getSource();
-
-                    // We cannot call theQueue.dispatchEvent, so I pasted its body here:
-                    if (event instanceof ActiveEvent) {
-                        ((ActiveEvent) event).dispatch();
-                    } else if (src instanceof Component) {
-                        ((Component) src).dispatchEvent(event);
-                    } else if (src instanceof MenuComponent) {
-                        ((MenuComponent) src).dispatchEvent(event);
-                    } else {
-                        logger.warning("unable to dispatch event: " + event);
-                    }
-                }
-            } else {
-                while (!responseGiven) {
-                    wait();
-                }
-            }
-        } catch(InterruptedException e){}
-
-        T tempResponse = response;
-        response = null;
-        responseGiven = false;
-
-        return tempResponse;
-    }
-
-    /**
-     * Sets that no response has been given.
-     */
-    public void resetResponse() {
-        response = null;
-        responseGiven = false;
-    }
-
-    /**
-    * Creates a new <code>FreeColDialog</code> with a text and a cancel-button,
-    * in addition to buttons for each of the objects in the given array.
-    *
-    * @param text String, text that explains the choice for the user
-    * @param cancelText String, text displayed on the "cancel"-button
-    * @param choices List of <code>ChoiceItem<T> <code> the choice items
-    * @return <code>FreeColDialog</code>
-    * @see ChoiceItem
-    */
-    public static <T> FreeColDialog<ChoiceItem<T>> createChoiceDialog(String text, String cancelText,
-                                                                      List<ChoiceItem<T>> choices) {
+    public static <T> FreeColDialog<ChoiceItem<T>> createChoiceDialog(FreeColClient freeColClient, GUI gui, String text,
+        String cancelText, List<ChoiceItem<T>> choices) {
 
         if (choices.isEmpty()) {
             throw new IllegalArgumentException("Can not create choice dialog with 0 choices!");
         }
 
-
         final List<JButton> choiceBtnLst = new ArrayList<JButton>();
-
-        final FreeColDialog<ChoiceItem<T>> choiceDialog =
-            new FreeColDialog<ChoiceItem<T>>(FreeCol.getFreeColClient().getCanvas()) {
-           @Override
-           public void requestFocus() {
-            	for(JButton b : choiceBtnLst){
-            		if(b.isEnabled()){
-            			b.requestFocus();
-            			return;
-            		}
-            	}
-            }
-        };
+        final FreeColDialog<ChoiceItem<T>> choiceDialog
+            = new FreeColDialog<ChoiceItem<T>>(freeColClient, gui) {
+                @Override
+                public void requestFocus() {
+                    for (JButton b : choiceBtnLst) {
+                        if (b.isEnabled()) {
+                            b.requestFocus();
+                            return;
+                        }
+                    }
+                }
+            };
 
         choiceDialog.setLayout(new MigLayout("fillx, wrap 1", "[align center]", ""));
         JTextArea textArea = getDefaultTextArea(text);
@@ -197,31 +144,31 @@ public class FreeColDialog<T> extends FreeColPanel {
 
         JPanel choicesPanel = new JPanel(new GridLayout(0, columns, 10, 10));
         choicesPanel.setBorder(new CompoundBorder(choicesPanel.getBorder(),
-                                                  new EmptyBorder(10, 20, 10, 20)));
+                new EmptyBorder(10, 20, 10, 20)));
 
         /*
         final ChoiceItem<T> firstObject = choices.get(0);
         if(firstObject.isEnabled()){
-        	firstButton.addActionListener(new ActionListener() {
-        		public void actionPerformed(ActionEvent event) {
-        			choiceDialog.setResponse(firstObject);
-        		}
-        	});
+            firstButton.addActionListener(new ActionListener() {
+                public void actionPerformed(ActionEvent event) {
+                    choiceDialog.setResponse(firstObject);
+                }
+            });
         }
         firstButton.setEnabled(firstObject.isEnabled());
         choicesPanel.add(firstButton);
         choices.remove(0);
-		*/
+        */
 
         for (final ChoiceItem<T> object : choices) {
             final JButton objectButton = new JButton(object.toString());
-            if(object.isEnabled()){
-            	objectButton.addActionListener(new ActionListener() {
-            		public void actionPerformed(ActionEvent event) {
-            			choiceDialog.setResponse(object);
-            		}
-            	});
-            	enterPressesWhenFocused(objectButton);
+            if (object.isEnabled()) {
+                objectButton.addActionListener(new ActionListener() {
+                    public void actionPerformed(ActionEvent event) {
+                        choiceDialog.setResponse(object);
+                    }
+                });
+                enterPressesWhenFocused(objectButton);
             }
             objectButton.setEnabled(object.isEnabled());
             choiceBtnLst.add(objectButton);
@@ -229,8 +176,8 @@ public class FreeColDialog<T> extends FreeColPanel {
         }
         if (choices.size() > 20) {
             JScrollPane scrollPane = new JScrollPane(choicesPanel,
-                                                     JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
-                                                     JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
+                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
+                JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
             choiceDialog.add(scrollPane, "newline 20");
         } else {
             choicesPanel.setOpaque(false);
@@ -240,6 +187,11 @@ public class FreeColDialog<T> extends FreeColPanel {
         if (cancelText != null) {
             choiceDialog.cancelButton.setText(cancelText);
             choiceDialog.add(choiceDialog.cancelButton, "newline 20, tag cancel");
+            choiceDialog.cancelButton.addActionListener(new ActionListener() {
+                    public void actionPerformed(ActionEvent event) {
+                        choiceDialog.setResponse(null);
+                    }
+                });
         }
 
         choiceDialog.setSize(choiceDialog.getPreferredSize());
@@ -247,26 +199,27 @@ public class FreeColDialog<T> extends FreeColPanel {
         return choiceDialog;
     }
 
-
     /**
-    * Creates a new <code>FreeColDialog</code> with a text and a ok/cancel option.
-    * The "ok"-option calls {@link #setResponse setResponse(new Boolean(true))}
-    * and the "cancel"-option calls {@link #setResponse setResponse(new Boolean(false))}.
-    *
-    * @param text The text that explains the choice for the user.
-    * @param okText The text displayed on the "ok"-button.
-    * @param cancelText The text displayed on the "cancel"-button.
-    * @return The <code>FreeColDialog</code>.
-    */
-    public static FreeColDialog<Boolean> createConfirmDialog(String text, String okText, String cancelText) {
-        return createConfirmDialog(new String[] {text}, null, okText, cancelText);
+     * Creates a new <code>FreeColDialog</code> with a text and a
+     * ok/cancel option.  The "ok"-option calls {@link #setResponse
+     * setResponse(new Boolean(true))} and the "cancel"-option calls
+     * {@link #setResponse setResponse(new Boolean(false))}.
+     *
+     * @param text The text that explains the choice for the user.
+     * @param okText The text displayed on the "ok"-button.
+     * @param cancelText The text displayed on the "cancel"-button.
+     * @return The <code>FreeColDialog</code>.
+     */
+    public static FreeColDialog<Boolean> createConfirmDialog(final FreeColClient freeColClient, GUI gui, String text, String okText, String cancelText) {
+        return createConfirmDialog(freeColClient, gui, new String[] {text}, null, okText, cancelText);
     }
 
-    public static FreeColDialog<Boolean> createConfirmDialog(String[] texts, ImageIcon[] icons,
-                                                             String okText, String cancelText) {
+
+    public static FreeColDialog<Boolean> createConfirmDialog(FreeColClient freeColClient, GUI gui, String[] texts,
+        ImageIcon[] icons, String okText, String cancelText) {
         // create the dialog
-        final FreeColDialog<Boolean> confirmDialog =
-            new FreeColDialog<Boolean>(FreeCol.getFreeColClient().getCanvas());
+        final FreeColDialog<Boolean> confirmDialog
+            = new FreeColDialog<Boolean>(freeColClient, gui);
 
         confirmDialog.setLayout(new MigLayout("wrap 2", "[][fill]", ""));
 
@@ -301,28 +254,28 @@ public class FreeColDialog<T> extends FreeColPanel {
     }
 
     /**
-    * Creates a new <code>FreeColDialog</code> with a text field and a ok/cancel option.
-    * The "ok"-option calls {@link #setResponse setResponse(textField.getText())}
-    * and the "cancel"-option calls {@link #setResponse setResponse(null)}.
-    *
-    * @param text The text that explains the action to the user.
-    * @param defaultValue The default value appearing in the text field.
-    * @param okText The text displayed on the "ok"-button.
-    * @param cancelText The text displayed on the "cancel"-button.
-    * @return The <code>FreeColDialog</code>.
-    */
-    public static FreeColDialog<String> createInputDialog(String text, String defaultValue,
-                                                          String okText, String cancelText) {
+     * Creates a new <code>FreeColDialog</code> with a text field and a
+     * ok/cancel option.  The "ok"-option calls {@link #setResponse
+     * setResponse(textField.getText())} and the "cancel"-option calls
+     * {@link #setResponse setResponse(null)}.
+     *
+     * @param text The text that explains the action to the user.
+     * @param defaultValue The default value appearing in the text field.
+     * @param okText The text displayed on the "ok"-button.
+     * @param cancelText The text displayed on the "cancel"-button.
+     * @return The <code>FreeColDialog</code>.
+     */
+    public static FreeColDialog<String> createInputDialog(FreeColClient freeColClient, GUI gui, String text,
+        String defaultValue, String okText, String cancelText) {
 
         final JTextField input = new JTextField(defaultValue);
-
-        final FreeColDialog<String> inputDialog =
-            new FreeColDialog<String>(FreeCol.getFreeColClient().getCanvas())  {
-            @Override
-            public void requestFocus() {
-                input.requestFocus();
-            }
-        };
+        final FreeColDialog<String> inputDialog
+            = new FreeColDialog<String>(freeColClient, gui)  {
+                @Override
+                public void requestFocus() {
+                    input.requestFocus();
+                }
+            };
 
         inputDialog.setLayout(new MigLayout("wrap 1, gapy 20", "", ""));
 
@@ -347,10 +300,20 @@ public class FreeColDialog<T> extends FreeColPanel {
         if (cancelText == null) {
             inputDialog.add(inputDialog.okButton, "tag ok");
             inputDialog.setCancelComponent(inputDialog.okButton);
+            inputDialog.okButton.addActionListener(new ActionListener() {
+                    public void actionPerformed(ActionEvent event) {
+                        inputDialog.setResponse(null);
+                    }
+                });
         } else {
             inputDialog.cancelButton.setText(cancelText);
             inputDialog.add(inputDialog.okButton, "split 2, tag ok");
             inputDialog.add(inputDialog.cancelButton, "tag cancel");
+            inputDialog.cancelButton.addActionListener(new ActionListener() {
+                    public void actionPerformed(ActionEvent event) {
+                        inputDialog.setResponse(null);
+                    }
+                });
         }
 
         inputDialog.setSize(inputDialog.getPreferredSize());
@@ -358,71 +321,19 @@ public class FreeColDialog<T> extends FreeColPanel {
         return inputDialog;
     }
 
-
-    public static FreeColDialog<Dimension> createMapSizeDialog(final Canvas canvas) {
-
-        final int defaultHeight = 100;
-        final int defaultWidth = 40;
-        final int COLUMNS = 5;
-
-        final String widthText = Messages.message("width");
-        final String heightText = Messages.message("height");
-
-        final JTextField inputWidth = new JTextField(Integer.toString(defaultWidth), COLUMNS);
-        final JTextField inputHeight = new JTextField(Integer.toString(defaultHeight), COLUMNS);
-
-        final FreeColDialog<Dimension> mapSizeDialog = new FreeColDialog<Dimension>(canvas);
-
-        mapSizeDialog.setLayout(new MigLayout("wrap 2"));
-
-        mapSizeDialog.okButton.setText(Messages.message("ok"));
-        mapSizeDialog.okButton.addActionListener(new ActionListener() {
-                public void actionPerformed(ActionEvent event) {
-                    try {
-                        int width = Integer.parseInt(inputWidth.getText());
-                        int height = Integer.parseInt(inputHeight.getText());
-                        if (width <= 0 || height <= 0) {
-                            throw new NumberFormatException();
-                        }
-                        mapSizeDialog.setResponse(new Dimension(width, height));
-                    } catch (NumberFormatException nfe) {
-                        canvas.errorMessage("integerAboveZero");
-                    }
-                }
-            });
-
-
-        JLabel widthLabel = new JLabel(widthText);
-        widthLabel.setLabelFor(inputWidth);
-        JLabel heightLabel = new JLabel(heightText);
-        heightLabel.setLabelFor(inputHeight);
-
-        mapSizeDialog.add(new JLabel(Messages.message("editor.mapSize")), "span, align center");
-        mapSizeDialog.add(widthLabel, "newline 20");
-        mapSizeDialog.add(inputWidth);
-        mapSizeDialog.add(heightLabel);
-        mapSizeDialog.add(inputHeight);
-
-        mapSizeDialog.add(mapSizeDialog.okButton, "newline 20, span, split2, tag ok");
-        mapSizeDialog.add(mapSizeDialog.cancelButton, "tag cancel");
-
-        mapSizeDialog.setSize(mapSizeDialog.getPreferredSize());
-
-        return mapSizeDialog;
-    }
-
     /**
-    * Creates a new <code>FreeColDialog</code> in which the user
-    * may choose a savegame to load.
-    *
-    * @param directory The directory to display when choosing the file.
-    * @param fileFilters The available file filters in the
-    *       dialog.
-    * @return The <code>FreeColDialog</code>.
-    */
-    public static FreeColDialog<File> createLoadDialog(File directory, FileFilter[] fileFilters) {
-        final FreeColDialog<File> loadDialog =
-            new FreeColDialog<File>(FreeCol.getFreeColClient().getCanvas());
+     * Creates a new <code>FreeColDialog</code> in which the user
+     * may choose a savegame to load.
+     *
+     * @param directory The directory to display when choosing the file.
+     * @param fileFilters The available file filters in the
+     *       dialog.
+     * @return The <code>FreeColDialog</code>.
+     */
+    public static FreeColDialog<File> createLoadDialog(FreeColClient freeColClient, GUI gui, File directory,
+                                                       FileFilter[] fileFilters) {
+        final FreeColDialog<File> loadDialog
+            = new FreeColDialog<File>(freeColClient, gui);
         final JFileChooser fileChooser = new JFileChooser(directory);
 
         loadDialog.okButton.addActionListener(new ActionListener() {
@@ -449,28 +360,89 @@ public class FreeColDialog<T> extends FreeColPanel {
         loadDialog.add(fileChooser, "grow");
         loadDialog.add(loadDialog.okButton, "newline 20, split 2, tag ok");
         loadDialog.add(loadDialog.cancelButton, "tag cancel");
+        loadDialog.cancelButton.addActionListener(new ActionListener() {
+                public void actionPerformed(ActionEvent event) {
+                    loadDialog.setResponse(null);
+                }
+            });
         loadDialog.setSize(480, 320);
 
         return loadDialog;
     }
 
+    public static FreeColDialog<Dimension> createMapSizeDialog(FreeColClient freeColClient, final GUI gui) {
+
+        final int defaultHeight = 100;
+        final int defaultWidth = 40;
+        final int COLUMNS = 5;
+
+        final String widthText = Messages.message("width");
+        final String heightText = Messages.message("height");
+
+        final JTextField inputWidth = new JTextField(Integer.toString(defaultWidth), COLUMNS);
+        final JTextField inputHeight = new JTextField(Integer.toString(defaultHeight), COLUMNS);
+
+        final FreeColDialog<Dimension> mapSizeDialog = new FreeColDialog<Dimension>(freeColClient, gui);
+
+        mapSizeDialog.setLayout(new MigLayout("wrap 2"));
+
+        mapSizeDialog.okButton.setText(Messages.message("ok"));
+        mapSizeDialog.okButton.addActionListener(new ActionListener() {
+                public void actionPerformed(ActionEvent event) {
+                    try {
+                        int width = Integer.parseInt(inputWidth.getText());
+                        int height = Integer.parseInt(inputHeight.getText());
+                        if (width <= 0 || height <= 0) {
+                            throw new NumberFormatException();
+                        }
+                        mapSizeDialog.setResponse(new Dimension(width, height));
+                    } catch (NumberFormatException nfe) {
+                        gui.errorMessage("integerAboveZero");
+                    }
+                }
+            });
+
+
+        JLabel widthLabel = new JLabel(widthText);
+        widthLabel.setLabelFor(inputWidth);
+        JLabel heightLabel = new JLabel(heightText);
+        heightLabel.setLabelFor(inputHeight);
+
+        mapSizeDialog.add(new JLabel(Messages.message("editor.mapSize")), "span, align center");
+        mapSizeDialog.add(widthLabel, "newline 20");
+        mapSizeDialog.add(inputWidth);
+        mapSizeDialog.add(heightLabel);
+        mapSizeDialog.add(inputHeight);
+
+        mapSizeDialog.add(mapSizeDialog.okButton, "newline 20, span, split2, tag ok");
+        mapSizeDialog.add(mapSizeDialog.cancelButton, "tag cancel");
+        mapSizeDialog.cancelButton.addActionListener(new ActionListener() {
+                public void actionPerformed(ActionEvent event) {
+                    mapSizeDialog.setResponse(null);
+                }
+            });
+
+        mapSizeDialog.setSize(mapSizeDialog.getPreferredSize());
+
+        return mapSizeDialog;
+    }
 
     /**
-    * Creates a new <code>FreeColDialog</code> in which the user
-    * may choose the destination of the savegame.
-    *
-    * @param directory The directory to display when choosing the name.
-    * @param standardName This extension will be added to the
-    *       specified filename (if not added by the user).
-    * @param fileFilters The available file filters in the
-    *       dialog.
-    * @param defaultName Default filename for the savegame.
-    * @return The <code>FreeColDialog</code>.
-    */
-    public static FreeColDialog<File> createSaveDialog(File directory, final String standardName,
-                                                       FileFilter[] fileFilters, String defaultName) {
-        final FreeColDialog<File> saveDialog =
-            new FreeColDialog<File>(FreeCol.getFreeColClient().getCanvas());
+     * Creates a new <code>FreeColDialog</code> in which the user
+     * may choose the destination of the savegame.
+     *
+     * @param directory The directory to display when choosing the name.
+     * @param standardName This extension will be added to the
+     *       specified filename (if not added by the user).
+     * @param fileFilters The available file filters in the
+     *       dialog.
+     * @param defaultName Default filename for the savegame.
+     * @return The <code>FreeColDialog</code>.
+     */
+    public static FreeColDialog<File> createSaveDialog(FreeColClient freeColClient, GUI gui, File directory,
+        final String standardName, FileFilter[] fileFilters, String defaultName) {
+        final FreeColDialog<File> saveDialog
+            = new FreeColDialog<File>(freeColClient, gui);
         final JFileChooser fileChooser = new JFileChooser(directory);
         final File defaultFile = new File(defaultName);
 
@@ -509,80 +481,59 @@ public class FreeColDialog<T> extends FreeColPanel {
 
 
     /**
-    * Returns a filter accepting "*.fsg".
-    * @return The filter.
-    */
-    public static FileFilter getFSGFileFilter() {
-
-        return new FreeColFileFilter( ".fsg", "filter.savedGames" );
-    }
-
-
-    /**
-    * Returns a filter accepting "*.fgo".
-    * @return The filter.
-    */
+     * Returns a filter accepting "*.fgo".
+     * @return The filter.
+     */
     public static FileFilter getFGOFileFilter() {
-
         return new FreeColFileFilter( ".fgo", "filter.gameOptions" );
     }
 
+    /**
+     * Returns a filter accepting "*.fsg".
+     * @return The filter.
+     */
+    public static FileFilter getFSGFileFilter() {
+        return new FreeColFileFilter( ".fsg", "filter.savedGames" );
+    }
 
     /**
-    * Returns a filter accepting all files containing a
-    * {@link net.sf.freecol.common.model.GameOptions}.
-    * That is; both "*.fgo" and "*.fsg".
-    *
-    * @return The filter.
-    */
+     * Returns a filter accepting all files containing a
+     * {@link net.sf.freecol.common.model.GameOptions}.
+     * That is; both "*.fgo" and "*.fsg".
+     *
+     * @return The filter.
+     */
     public static FileFilter getGameOptionsFileFilter() {
-
         return new FreeColFileFilter( ".fgo", ".fsg", "filter.gameOptionsAndSavedGames" );
     }
 
 
-    static final class FreeColFileFilter extends FileFilter {
+    // Stores the response from the user:
+    private T response = null;
 
-        private final String  extension1;
-        private final String  extension2;
-        private final String  description;
+    // Whether or not the user have made the choice.
+    private boolean responseGiven = false;
 
-        FreeColFileFilter( String  extension1,
-                           String  extension2,
-                           String  descriptionMessage ) {
 
-            this.extension1 = extension1;
-            this.extension2 = extension2;
-            description = Messages.message(descriptionMessage);
-        }
+    protected JButton cancelButton = new JButton(Messages.message("cancel"));
 
-        FreeColFileFilter( String  extension, String  descriptionMessage ) {
+    /**
+     * Constructor.
+     *
+     * @param parent The parent <code>Canvas</code>.
+     */
+    public FreeColDialog(FreeColClient freeColClient, GUI gui) {
+        super(freeColClient, gui);
 
-            this.extension1 = extension;
-            this.extension2 = "....";
-            description = Messages.message(descriptionMessage);
-        }
-
-        public boolean accept(File f) {
-
-            return f.isDirectory() || f.getName().endsWith(extension1)
-                || f.getName().endsWith(extension2);
-        }
-
-        public String getDescription() {
-
-            return description;
-        }
+        cancelButton.setActionCommand(CANCEL);
+        cancelButton.addActionListener(this);
+        enterPressesWhenFocused(cancelButton);
+        setCancelComponent(cancelButton);
     }
 
     /**
-     * Used for Polymorphism in Recruit, Purchase, Train Dialogs
-     */
-    public void initialize() {}
-
-    /**
-     * This function analyses an event and calls the right methods to take care
-     * of the user's requests.
+     * This function analyses an event and calls the right methods to
+     * take care of the user's requests.
      *
      * @param event The incoming ActionEvent.
      */
@@ -596,4 +547,83 @@ public class FreeColDialog<T> extends FreeColPanel {
         }
     }
 
+    /**
+     * Returns the <code>response</code> when set by
+     * <code>setResponse(Object response)</code>.
+     * Waits the thread until then.
+     *
+     * @return The object as set by {@link #setResponse}.
+     */
+    public synchronized T getResponse() {
+        // Wait the thread until 'response' is available. Notice that
+        // we have to process the events manually if the current
+        // thread is the Event Dispatch Thread (EDT).
+
+        try {
+            if (SwingUtilities.isEventDispatchThread()) {
+                EventQueue theQueue = getToolkit().getSystemEventQueue();
+
+                while (!responseGiven) {
+                    // This is essentially the body of EventDispatchThread
+                    AWTEvent event = theQueue.getNextEvent();
+                    Object src = event.getSource();
+
+                    try {
+                        // We cannot call theQueue.dispatchEvent, so I
+                        // pasted its body here:
+                        if (event instanceof ActiveEvent) {
+                            ((ActiveEvent) event).dispatch();
+                        } else if (src instanceof Component) {
+                            ((Component) src).dispatchEvent(event);
+                        } else if (src instanceof MenuComponent) {
+                            ((MenuComponent) src).dispatchEvent(event);
+                        } else {
+                            logger.warning("unable to dispatch event: "
+                                + event);
+                        }
+                    } finally {
+                        continue;
+                    }
+                }
+            } else {
+                while (!responseGiven) {
+                    wait();
+                }
+            }
+        } catch (InterruptedException e) {}
+
+        T tempResponse = response;
+        response = null;
+        responseGiven = false;
+
+        return tempResponse;
+    }
+
+
+    /**
+     * Used for Polymorphism in Recruit, Purchase, Train Dialogs
+     */
+    public void initialize() {}
+
+    /**
+     * Sets that no response has been given.
+     */
+    public void resetResponse() {
+        response = null;
+        responseGiven = false;
+    }
+
+    /**
+     * Sets the <code>response</code> and wakes up any thread waiting
+     * for this information.
+     *
+     * @param response The object that should be returned by
+     *      {@link #getResponse}.
+     */
+    public synchronized void setResponse(T response) {
+        this.response = response;
+        responseGiven = true;
+        logger.info("Response has been set to " + response);
+        notifyAll();
+    }
 }
