@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -21,6 +21,7 @@ package net.sf.freecol.client.gui.panel;
 
 import java.awt.Color;
 import java.awt.Component;
+import java.awt.Cursor;
 import java.awt.Dimension;
 import java.awt.FlowLayout;
 import java.awt.Font;
@@ -31,16 +32,15 @@ import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.awt.event.KeyEvent;
 import java.awt.event.MouseAdapter;
-import java.awt.event.MouseEvent;
-import java.awt.event.MouseListener;
-import java.awt.event.MouseMotionListener;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.text.DecimalFormat;
 import java.util.ArrayList;
 import java.util.EnumMap;
+import java.util.HashMap;
 import java.util.LinkedHashSet;
 import java.util.List;
+import java.util.Map;
 import java.util.Set;
 import java.util.logging.Logger;
 
@@ -65,19 +65,23 @@ import javax.swing.text.Style;
 import javax.swing.text.StyleConstants;
 import javax.swing.text.StyleContext;
 
+import net.sf.freecol.client.ClientOptions;
 import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.control.InGameController;
 import net.sf.freecol.client.gui.Canvas;
+import net.sf.freecol.client.gui.GUI;
 import net.sf.freecol.client.gui.ImageLibrary;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.Game;
+import net.sf.freecol.common.model.HistoryEvent;
 import net.sf.freecol.common.model.Modifier;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Specification;
 import net.sf.freecol.common.model.StringTemplate;
-import net.sf.freecol.common.option.Option;
+import net.sf.freecol.common.model.Turn;
 import net.sf.freecol.common.option.IntegerOption;
+import net.sf.freecol.common.option.Option;
 import net.sf.freecol.common.resources.ResourceManager;
 
 /**
@@ -89,27 +93,23 @@ public abstract class FreeColPanel extends JPanel implements ActionListener {
     private static final Logger logger = Logger.getLogger(FreeColPanel.class.getName());
 
     protected static final String OK = "OK";
+    protected static final String HELP = "HELP";
 
     public static final Insets emptyMargin = new Insets(0,0,0,0);
 
     private static final int cancelKeyCode = KeyEvent.VK_ESCAPE;
 
     // The decimal format to use for Modifiers
-    private static final DecimalFormat modifierFormat = new DecimalFormat("0.00");
-
-    /**
-     * The canvas all panels belong to.
-     */
-    private Canvas canvas = null;
+    protected static final DecimalFormat modifierFormat = new DecimalFormat("0.00");
 
     // Font to use for text areas
     protected static final Font defaultFont = ResourceManager.getFont("NormalFont", 13f);
 
     // Fonts to use for report headers, etc.
     protected static final Font  smallHeaderFont = ResourceManager.getFont("HeaderFont", 24f);
+
     protected static final Font mediumHeaderFont = ResourceManager.getFont("HeaderFont", 36f);
     protected static final Font    bigHeaderFont = ResourceManager.getFont("HeaderFont", 48f);
-
     // How many columns (em-widths) to use in the text area
     protected static final int COLUMNS = 20;
 
@@ -123,7 +123,8 @@ public abstract class FreeColPanel extends JPanel implements ActionListener {
     // The color to use for links
     protected static final Color LINK_COLOR
         = ResourceManager.getColor("lookAndFeel.link.color");
-
+    
+    
     // The color to use for borders
     protected static final Color BORDER_COLOR
         = ResourceManager.getColor("lookAndFeel.border.color");
@@ -132,9 +133,11 @@ public abstract class FreeColPanel extends JPanel implements ActionListener {
     public static final Border TOPCELLBORDER =
         BorderFactory.createCompoundBorder(BorderFactory.createMatteBorder(1, 0, 1, 1, BORDER_COLOR),
                                            BorderFactory.createEmptyBorder(2, 2, 2, 2));
+
     public static final Border CELLBORDER =
         BorderFactory.createCompoundBorder(BorderFactory.createMatteBorder(0, 0, 1, 1, BORDER_COLOR),
                                            BorderFactory.createEmptyBorder(2, 2, 2, 2));
+
     public static final Border LEFTCELLBORDER =
         BorderFactory.createCompoundBorder(BorderFactory.createMatteBorder(0, 1, 1, 1, BORDER_COLOR),
                                            BorderFactory.createEmptyBorder(2, 2, 2, 2));
@@ -142,243 +145,18 @@ public abstract class FreeColPanel extends JPanel implements ActionListener {
         BorderFactory.createCompoundBorder(BorderFactory.createMatteBorder(1, 1, 1, 1, BORDER_COLOR),
                                            BorderFactory.createEmptyBorder(2, 2, 2, 2));
 
-
-    protected boolean editable = true;
-
-    protected JButton okButton = new JButton(Messages.message("ok"));
-
-    protected static final String okCommand = "OK";
-
-    protected static StyleContext styleContext = new StyleContext();
-
-    static {
-        Style defaultStyle = StyleContext.getDefaultStyleContext()
-            .getStyle(StyleContext.DEFAULT_STYLE);
-
-        Style regular = styleContext.addStyle("regular", defaultStyle);
-        StyleConstants.setFontFamily(regular, "NormalFont");
-        StyleConstants.setFontSize(regular, 13);
-
-        Style buttonStyle = styleContext.addStyle("button", regular);
-        StyleConstants.setForeground(buttonStyle, LINK_COLOR);
-
-        Style right = styleContext.addStyle("right", regular);
-        StyleConstants.setAlignment(right, StyleConstants.ALIGN_RIGHT);
-    }
-
     /**
-     * Constructor.
+     * Registers enter key for a JButton.
      *
-     * @param parent <code>Canvas</code>
+     * @param button
      */
-    public FreeColPanel(Canvas parent) {
-        this(parent, new FlowLayout());
-    }
+    public static void enterPressesWhenFocused(JButton button) {
+        button.registerKeyboardAction(
+                button.getActionForKeyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_SPACE, 0, false)), KeyStroke
+                        .getKeyStroke(KeyEvent.VK_ENTER, 0, false), JComponent.WHEN_FOCUSED);
 
-    /**
-     * Default constructor.
-     *
-     * @param parent The <code>Canvas</code> all panels belong to.
-     * @param layout The <code>LayoutManager</code> to be used.
-     */
-    public FreeColPanel(Canvas parent, LayoutManager layout) {
-        super(layout);
-
-        this.canvas = parent;
-        setFocusCycleRoot(true);
-
-        setBorder(FreeColImageBorder.imageBorder);
-
-        // See the message of Ulf Onnen for more information about the presence
-        // of this fake mouse listener.
-        addMouseListener(new MouseAdapter() {
-        });
-
-        okButton.setActionCommand(okCommand);
-        okButton.addActionListener(this);
-        enterPressesWhenFocused(okButton);
-        setCancelComponent(okButton);
-
-    }
-
-
-    /**
-     * Return an <code>int</code> associated with the name of the
-     * panel's class plus the given key from the saved ClientOptions.
-     *
-     * @param key a <code>String</code> value
-     * @return an <code>int</code> value
-     */
-    private int getInteger(String key) {
-        return getClient().getClientOptions().getInteger(getClass().getName() + key);
-    }
-
-    /**
-     * Returns the saved size of this panel, null by default.
-     *
-     * @return a <code>Dimension</code> value
-     */
-    public final Dimension getSavedSize() {
-        try {
-            return new Dimension(getInteger(".w"), getInteger(".h"));
-        } catch(Exception e) {
-            return null;
-        }
-    }
-
-
-    /**
-     * Save an <code>int</code> value to the saved ClientOptions,
-     * using the name of the panel's class plus the given key as and
-     * ID.
-     *
-     * @param key a <code>String</code> value
-     * @param value an <code>int</code> value
-     */
-    private void saveInteger(String key, int value) {
-        Option o = getClient().getClientOptions().getOption(getClass().getName() + key);
-        if (o == null) {
-            IntegerOption io = new IntegerOption(getClass().getName() + key);
-            io.setValue(value);
-            getClient().getClientOptions().add(io);
-        } else if (o instanceof IntegerOption) {
-            ((IntegerOption) o).setValue(value);
-        }
-    }
-
-    /**
-     * Set preferred size to saved size, or to the given
-     * <code>Dimension</code> if no saved size was found. Call this
-     * method in the constructor of a FreeColPanel in order to
-     * remember its size and position.
-     *
-     * @param d a <code>Dimension</code> value
-     */
-    protected void restoreSavedSize(Dimension d) {
-        Dimension size = getSavedSize();
-        if (size == null) {
-            size = d;
-            saveSize(size);
-        }
-        if (!getPreferredSize().equals(size)) {
-            setPreferredSize(size);
-        }
-    }
-
-    /**
-     * Set preferred size to saved size, or to [w, h] if no saved size
-     * was found. Call this method in the constructor of a
-     * FreeColPanel in order to remember its size and position.
-     *
-     * @param w an <code>int</code> value
-     * @param h an <code>int</code> value
-     */
-    protected void restoreSavedSize(int w, int h) {
-        restoreSavedSize(new Dimension(w, h));
-    }
-
-
-    /**
-     * Returns the saved position of this panel, null by default.
-     *
-     * @return a <code>Point</code> value
-     */
-    public Point getSavedPosition() {
-        try {
-            return new Point(getInteger(".x"), getInteger(".y"));
-        } catch(Exception e) {
-            return null;
-        }
-    }
-
-
-    /**
-     * Get the <code>Canvas</code> value.
-     *
-     * @return a <code>Canvas</code> value
-     */
-    public final Canvas getCanvas() {
-        return canvas;
-    }
-
-    /**
-     * Returns the ImageLibrary.
-     *
-     * @return the ImageLibrary.
-     */
-    public ImageLibrary getLibrary() {
-        return canvas.getImageLibrary();
-    }
-
-    /**
-     * Describe <code>getClient</code> method here.
-     *
-     * @return a <code>FreeColClient</code> value
-     */
-    public FreeColClient getClient() {
-        return canvas.getClient();
-    }
-
-    /**
-     * Describe <code>getGame</code> method here.
-     *
-     * @return a <code>Game</code> value
-     */
-    public Game getGame() {
-        return canvas.getClient().getGame();
-    }
-
-    /**
-     * Describe <code>getSpecification</code> method here.
-     *
-     * @return a <code>Specification</code> value
-     */
-    public Specification getSpecification() {
-        return canvas.getSpecification();
-    }
-
-    /**
-     * Describe <code>getController</code> method here.
-     *
-     * @return an <code>InGameController</code> value
-     */
-    public InGameController getController() {
-        return canvas.getClient().getInGameController();
-    }
-
-    /**
-     * Describe <code>getMyPlayer</code> method here.
-     *
-     * @return a <code>Player</code> value
-     */
-    public Player getMyPlayer() {
-        return canvas.getClient().getMyPlayer();
-    }
-
-    /**
-     * Checks if this panel is editable
-     * @return boolean
-     */
-    public boolean isEditable() {
-        return editable;
-    }
-
-    /**
-     * Set the <code>Editable</code> value.
-     *
-     * @param newEditable boolean, the new Editable value
-     */
-    public void setEditable(boolean newEditable) {
-        this.editable = newEditable;
-    }
-
-    /**
-     * The OK button requests focus.
-     *
-     */
-    @Override
-    public void requestFocus() {
-        okButton.requestFocus();
+        button.registerKeyboardAction(button.getActionForKeyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_SPACE, 0, true)),
+                KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0, true), JComponent.WHEN_FOCUSED);
     }
 
     /**
@@ -423,42 +201,6 @@ public abstract class FreeColPanel extends JPanel implements ActionListener {
         return textPane;
     }
 
-
-
-    /**
-     * Returns a text area with standard settings suitable for use in FreeCol
-     * dialogs.
-     *
-     * @param text The text to display in the text area.
-     * @return a text area with standard settings suitable for use in FreeCol
-     *         dialogs.
-     */
-    public static JTextArea getDefaultTextArea(String text) {
-        return getDefaultTextArea(text, COLUMNS);
-    }
-
-    /**
-     * Returns a text area with standard settings suitable for use in FreeCol
-     * dialogs.
-     *
-     * @param text The text to display in the text area.
-     * @param columns an <code>int</code> value
-     * @return a text area with standard settings suitable for use in FreeCol
-     *         dialogs.
-     */
-    public static JTextArea getDefaultTextArea(String text, int columns) {
-        JTextArea textArea = new JTextArea(text);
-        textArea.setColumns(columns);
-        textArea.setOpaque(false);
-        textArea.setLineWrap(true);
-        textArea.setWrapStyleWord(true);
-        textArea.setFocusable(false);
-        textArea.setFont(defaultFont);
-        // necessary because of resizing
-        textArea.setSize(textArea.getPreferredSize());
-        return textArea;
-    }
-
     /**
      * Return a button suitable for linking to another panel
      * (e.g. ColopediaPanel).
@@ -476,49 +218,202 @@ public abstract class FreeColPanel extends JPanel implements ActionListener {
         button.setAlignmentY(0.8f);
         button.setBorder(BorderFactory.createEmptyBorder());
         button.setActionCommand(action);
+        button.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
         return button;
     }
 
-    public JButton createColonyButton(Colony colony) {
-        JButton button = getLinkButton(colony.getName(), null, colony.getId());
-        button.addActionListener(this);
-        return button;
+    /**
+     * Returns the default modifier value format.
+     *
+     * @return a <code>DecimalFormat</code> value
+     */
+    public static final DecimalFormat getModifierFormat() {
+        return modifierFormat;
     }
 
-
     /**
      * Returns the default header for panels.
      *
      * @param text a <code>String</code> value
      * @return a <code>JLabel</code> value
      */
-    public static JLabel getDefaultHeader(String text) {
+    protected static JLabel getDefaultHeader(String text) {
         JLabel header = new JLabel(text, JLabel.CENTER);
         header.setFont(bigHeaderFont);
         header.setBorder(BorderFactory.createEmptyBorder(20, 0, 20, 0));
         return header;
     }
 
+
     /**
-     * Return a JLabel with Messages.message(key) as text.
+     * Returns a text area with standard settings suitable for use in FreeCol
+     * dialogs.
      *
-     * @param key a <code>String</code> value
-     * @return a <code>JLabel</code> value
+     * @param text The text to display in the text area.
+     * @return a text area with standard settings suitable for use in FreeCol
+     *         dialogs.
      */
-    public JLabel localizedLabel(String key) {
-        return new JLabel(Messages.message(key));
+    protected static JTextArea getDefaultTextArea(String text) {
+        return getDefaultTextArea(text, COLUMNS);
     }
 
     /**
-     * Return a JLabel with Messages.localize(template) as text.
+     * Returns a text area with standard settings suitable for use in FreeCol
+     * dialogs.
      *
-     * @param template a <code>StringTemplate</code> value
-     * @return a <code>JLabel</code> value
+     * @param text The text to display in the text area.
+     * @param columns an <code>int</code> value
+     * @return a text area with standard settings suitable for use in FreeCol
+     *         dialogs.
      */
-    public JLabel localizedLabel(StringTemplate template) {
-        return new JLabel(Messages.message(template));
+    protected static JTextArea getDefaultTextArea(String text, int columns) {
+        JTextArea textArea = new JTextArea(text);
+        textArea.setColumns(columns);
+        textArea.setOpaque(false);
+        textArea.setLineWrap(true);
+        textArea.setWrapStyleWord(true);
+        textArea.setFocusable(false);
+        textArea.setFont(defaultFont);
+        // necessary because of resizing
+        textArea.setSize(textArea.getPreferredSize());
+        return textArea;
     }
 
+    protected boolean editable = true;
+
+    protected JButton okButton = new JButton(Messages.message("ok"));
+
+
+    private FreeColClient freeColClient;
+
+    private GUI gui;
+
+    protected static StyleContext styleContext = new StyleContext();
+
+    static {
+        Style defaultStyle = StyleContext.getDefaultStyleContext()
+            .getStyle(StyleContext.DEFAULT_STYLE);
+
+        Style regular = styleContext.addStyle("regular", defaultStyle);
+        StyleConstants.setFontFamily(regular, "NormalFont");
+        StyleConstants.setFontSize(regular, 13);
+
+        Style buttonStyle = styleContext.addStyle("button", regular);
+        StyleConstants.setForeground(buttonStyle, LINK_COLOR);
+
+        Style right = styleContext.addStyle("right", regular);
+        StyleConstants.setAlignment(right, StyleConstants.ALIGN_RIGHT);
+    }
+
+
+    /**
+     * Constructor.
+     *
+     * @param parent <code>Canvas</code>
+     */
+    public FreeColPanel(FreeColClient freeColClient, GUI gui) {
+        this(freeColClient, gui, new FlowLayout());
+    }
+
+
+    /**
+     * Default constructor.
+     *
+     * @param parent The <code>Canvas</code> all panels belong to.
+     * @param layout The <code>LayoutManager</code> to be used.
+     */
+    public FreeColPanel(FreeColClient freeColClient, GUI gui, LayoutManager layout) {
+        super(layout);
+
+        this.freeColClient = freeColClient;
+        this.gui = gui;
+        setFocusCycleRoot(true);
+
+        setBorder(FreeColImageBorder.imageBorder);
+
+        // See the message of Ulf Onnen for more information about the presence
+        // of this fake mouse listener.
+        addMouseListener(new MouseAdapter() {
+        });
+
+        okButton.setActionCommand(OK);
+        okButton.addActionListener(this);
+        enterPressesWhenFocused(okButton);
+        setCancelComponent(okButton);
+
+    }
+
+    /**
+     * This function analyses an event and calls the right methods to take care
+     * of the user's requests.
+     *
+     * @param event The incoming ActionEvent.
+     */
+    public void actionPerformed(ActionEvent event) {
+        String command = event.getActionCommand();
+        if (command.equals(OK)) {
+            getGUI().removeFromCanvas(this);
+        }
+    }
+
+    /**
+     * Add a routine to be called when this panel closes.
+     * Triggered by notifyClose() above.
+     *
+     * @param runnable Some code to run on close.
+     */
+    public void addClosingCallback(final Runnable runnable) {
+        final FreeColPanel fcp = this;
+        addPropertyChangeListener(new PropertyChangeListener() {
+                public void propertyChange(PropertyChangeEvent e) {
+                    if ("closing".equals(e.getPropertyName())) {
+                        runnable.run();
+                        fcp.removePropertyChangeListener(this);
+                    }
+                }
+            });
+    }
+
+    /**
+     * Returns the saved position of this panel, null by default.
+     *
+     * @return a <code>Point</code> value
+     */
+    public Point getSavedPosition() {
+        try {
+            return new Point(getInteger(".x"), getInteger(".y"));
+        } catch(Exception e) {
+            return null;
+        }
+    }
+
+    /**
+     * Notify this panel that it is being removed from the
+     * canvas. Saves the current size and position of the panel to the
+     * ClientOptions, which are included in the savegame file.
+     *
+     * @see Canvas#remove(Component)
+     */
+    public void notifyClose() {
+        firePropertyChange("closing", false, true);
+        Component frame = SwingUtilities.getAncestorOfClass(JInternalFrame.class, this);
+        if (frame != null
+            && getClientOptions() != null
+            && getClientOptions().getBoolean("model.option.rememberPanelPositions")) {
+            saveInteger(".x", frame.getLocation().x);
+            saveInteger(".y", frame.getLocation().y);
+        }
+        saveSize();
+    }
+
+    /**
+     * The OK button requests focus.
+     *
+     */
+    @Override
+    public void requestFocus() {
+        okButton.requestFocus();
+    }
 
     /**
      * Make the given button the CANCEL button.
@@ -537,27 +432,202 @@ public abstract class FreeColPanel extends JPanel implements ActionListener {
         getActionMap().put("release", cancelAction);
     }
 
-    /**
-     * Registers enter key for a JButton.
-     *
-     * @param button
-     */
-    public static void enterPressesWhenFocused(JButton button) {
-        button.registerKeyboardAction(
-                button.getActionForKeyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_SPACE, 0, false)), KeyStroke
-                        .getKeyStroke(KeyEvent.VK_ENTER, 0, false), JComponent.WHEN_FOCUSED);
+    protected JButton createColonyButton(Colony colony) {
+        JButton button = getLinkButton(colony.getName(), null, colony.getId());
+        button.addActionListener(this);
+        return button;
+    }
 
-        button.registerKeyboardAction(button.getActionForKeyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_SPACE, 0, true)),
-                KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0, true), JComponent.WHEN_FOCUSED);
+
+
+    /**
+     * Get the <code>Canvas</code> value.
+     *
+     * @return a <code>Canvas</code> value
+     */
+    protected final Canvas getCanvas() {
+        return gui.getCanvas();
     }
 
     /**
-     * Returns the default modifier value format.
+     * Return the client's <code>ClientOptions</code>.
      *
-     * @return a <code>DecimalFormat</code> value
+     * @return a <code>ClientOptions</code> value
      */
-    public static final DecimalFormat getModifierFormat() {
-        return modifierFormat;
+    protected ClientOptions getClientOptions() {
+        return freeColClient == null ? null : freeColClient.getClientOptions();
+    }
+
+    /**
+     * Describe <code>getController</code> method here.
+     *
+     * @return an <code>InGameController</code> value
+     */
+    protected InGameController getController() {
+        return freeColClient.getInGameController();
+    }
+
+    /**
+     * Returns the <code>Turn</code>s during which a Player's
+     * FoundingFathers were elected to the Continental Congress
+     *
+     * @return a <code>Turn</code> value
+     */
+    protected Map<String, Turn> getElectionTurns() {
+        Map<String, Turn> result = new HashMap<String, Turn>();
+        if (!getMyPlayer().getFathers().isEmpty()) {
+            for (HistoryEvent event : getMyPlayer().getHistory()) {
+                if (event.getEventType() == HistoryEvent.EventType.FOUNDING_FATHER) {
+                    result.put(event.getReplacement("%father%").getId(), event.getTurn());
+                }
+            }
+        }
+        return result;
+    }
+
+    /**
+     * Gets the FreeColClient from the canvas.
+     *
+     * @return A current <code>FreeColClient</code>.
+     */
+    protected FreeColClient getFreeColClient() {
+        return freeColClient;
+    }
+
+
+
+    /**
+     * Describe <code>getGame</code> method here.
+     *
+     * @return a <code>Game</code> value
+     */
+    protected Game getGame() {
+        return freeColClient.getGame();
+    }
+
+    protected GUI getGUI() {
+        return gui;
+    }
+
+    /**
+     * Returns the ImageLibrary.
+     *
+     * @return the ImageLibrary.
+     */
+    protected ImageLibrary getLibrary() {
+        return gui.getImageLibrary();
+    }
+
+    /**
+     * Describe <code>getMyPlayer</code> method here.
+     *
+     * @return a <code>Player</code> value
+     */
+    protected Player getMyPlayer() {
+        return freeColClient.getMyPlayer();
+    }
+
+
+    /**
+     * Returns the saved size of this panel, null by default.
+     *
+     * @return a <code>Dimension</code> value
+     */
+    protected final Dimension getSavedSize() {
+        try {
+            return new Dimension(getInteger(".w"), getInteger(".h"));
+        } catch(Exception e) {
+            return null;
+        }
+    }
+
+    /**
+     * Return the player's Colonies, sorted according to player's
+     * <code>ClientOptions</code>.
+     *
+     * @return a sorted List of Colonies
+     */
+    protected List<Colony> getSortedColonies() {
+        return freeColClient.getClientOptions()
+            .getSortedColonies(getMyPlayer());
+    }
+
+    /**
+     * Describe <code>getSpecification</code> method here.
+     *
+     * @return a <code>Specification</code> value
+     */
+    protected Specification getSpecification() {
+        return freeColClient.getGame().getSpecification();
+    }
+
+
+    /**
+     * Checks if this panel is editable
+     * @return boolean
+     */
+    protected boolean isEditable() {
+        return editable;
+    }
+
+    /**
+     * Return a JLabel with Messages.message(key) as text.
+     *
+     * @param key a <code>String</code> value
+     * @return a <code>JLabel</code> value
+     */
+    protected JLabel localizedLabel(String key) {
+        return new JLabel(Messages.message(key));
+    }
+
+    /**
+     * Return a JLabel with Messages.localize(template) as text.
+     *
+     * @param template a <code>StringTemplate</code> value
+     * @return a <code>JLabel</code> value
+     */
+    protected JLabel localizedLabel(StringTemplate template) {
+        return new JLabel(Messages.message(template));
+    }
+
+    /**
+     * Set preferred size to saved size, or to the given
+     * <code>Dimension</code> if no saved size was found. Call this
+     * method in the constructor of a FreeColPanel in order to
+     * remember its size and position.
+     *
+     * @param d a <code>Dimension</code> value
+     */
+    protected void restoreSavedSize(Dimension d) {
+        Dimension size = getSavedSize();
+        if (size == null) {
+            size = d;
+            saveSize(size);
+        }
+        if (!getPreferredSize().equals(size)) {
+            setPreferredSize(size);
+        }
+    }
+
+    /**
+     * Set preferred size to saved size, or to [w, h] if no saved size
+     * was found. Call this method in the constructor of a
+     * FreeColPanel in order to remember its size and position.
+     *
+     * @param w an <code>int</code> value
+     * @param h an <code>int</code> value
+     */
+    protected void restoreSavedSize(int w, int h) {
+        restoreSavedSize(new Dimension(w, h));
+    }
+
+    /**
+     * Set the <code>Editable</code> value.
+     *
+     * @param newEditable boolean, the new Editable value
+     */
+    protected void setEditable(boolean newEditable) {
+        this.editable = newEditable;
     }
 
     /**
@@ -566,7 +636,7 @@ public abstract class FreeColPanel extends JPanel implements ActionListener {
      * @param result Set of <code>Modifier</code>
      * @return a sorted Set of <code>Modifier</code>
      */
-    public Set<Modifier> sortModifiers(Set<Modifier> result) {
+    protected Set<Modifier> sortModifiers(Set<Modifier> result) {
         EnumMap<Modifier.Type, List<Modifier>> modifierMap =
             new EnumMap<Modifier.Type, List<Modifier>>(Modifier.Type.class);
         for (Modifier.Type type : Modifier.Type.values()) {
@@ -583,33 +653,41 @@ public abstract class FreeColPanel extends JPanel implements ActionListener {
     }
 
     /**
-     * This function analyses an event and calls the right methods to take care
-     * of the user's requests.
+     * Return an <code>int</code> associated with the name of the
+     * panel's class plus the given key from the saved ClientOptions.
      *
-     * @param event The incoming ActionEvent.
+     * @param key a <code>String</code> value
+     * @return an <code>int</code> value
      */
-    public void actionPerformed(ActionEvent event) {
-        String command = event.getActionCommand();
-        if (command.equals(okCommand)) {
-            getCanvas().remove(this);
-        }
+    private int getInteger(String key) {
+        return freeColClient.getClientOptions()
+            .getInteger(getClass().getName() + key);
     }
 
+
+
+
     /**
-     * Notify this panel that it is being removed from the
-     * canvas. Saves the current size and position of the panel to the
-     * ClientOptions, which are included in the savegame file.
+     * Save an <code>int</code> value to the saved ClientOptions,
+     * using the name of the panel's class plus the given key as and
+     * ID.
      *
-     * @see Canvas#remove(Component)
+     * @param key a <code>String</code> value
+     * @param value an <code>int</code> value
      */
-    public void notifyClose() {
-        firePropertyChange("closing", false, true);
-        Component frame = SwingUtilities.getAncestorOfClass(JInternalFrame.class, this);
-        if (frame != null) {
-            saveInteger(".x", frame.getLocation().x);
-            saveInteger(".y", frame.getLocation().y);
+    private void saveInteger(String key, int value) {
+        if (freeColClient != null
+            && freeColClient.getClientOptions() != null) {
+            Option o = freeColClient.getClientOptions()
+                .getOption(getClass().getName() + key);
+            if (o == null) {
+                IntegerOption io = new IntegerOption(getClass().getName() + key);
+                io.setValue(value);
+                freeColClient.getClientOptions().add(io);
+            } else if (o instanceof IntegerOption) {
+                ((IntegerOption) o).setValue(value);
+            }
         }
-        saveSize();
     }
 
     /**
@@ -629,78 +707,4 @@ public abstract class FreeColPanel extends JPanel implements ActionListener {
         saveInteger(".w", size.width);
         saveInteger(".h", size.height);
     }
-
-
-
-
-    /**
-     * Add a routine to be called when this panel closes.
-     * Triggered by notifyClose() above.
-     *
-     * @param runnable Some code to run on close.
-     */
-    public void addClosingCallback(final Runnable runnable) {
-        addPropertyChangeListener(new PropertyChangeListener() {
-                public void propertyChange(PropertyChangeEvent e) {
-                    if ("closing".equals(e.getPropertyName())) {
-                        runnable.run();
-                    }
-                }
-            });
-    }
-
-    /**
-     * Creates a <code>MouseListener</code> which forwards events
-     * to the given <code>Component</code>.
-     *
-     * @param c The <code>Component</code> the events should be forwarded to.
-     * @return <code>MouseListener</code>
-     */
-    public static MouseListener createEventForwardingMouseListener(final Component c) {
-        final MouseListener ml = new MouseListener() {
-            private void forward(MouseEvent e) {
-                c.dispatchEvent(javax.swing.SwingUtilities.convertMouseEvent(e.getComponent(), e, c));
-            }
-
-            public void mouseClicked(MouseEvent e) {
-                forward(e);
-            }
-            public void mouseEntered(MouseEvent e) {
-                forward(e);
-            }
-            public void mouseExited(MouseEvent e) {
-                forward(e);
-            }
-            public void mousePressed(MouseEvent e) {
-                forward(e);
-            }
-            public void mouseReleased(MouseEvent e) {
-                forward(e);
-            }
-        };
-        return ml;
-    }
-
-    /**
-     * Creates a <code>MouseMotionListener</code> which forwards events
-     * to the given <code>Component</code>.
-     *
-     * @param c The <code>Component</code> the events should be forwarded to
-     * @return <code>MouseMotionListener</code>
-     */
-    public static MouseMotionListener createEventForwardingMouseMotionListener(final Component c) {
-        final MouseMotionListener ml = new MouseMotionListener() {
-            private void forward(MouseEvent e) {
-                c.dispatchEvent(javax.swing.SwingUtilities.convertMouseEvent(e.getComponent(), e, c));
-            }
-
-            public void mouseDragged(MouseEvent e) {
-                forward(e);
-            }
-            public void mouseMoved(MouseEvent e) {
-                forward(e);
-            }
-        };
-        return ml;
-    }
 }
