@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -24,7 +24,6 @@ import java.awt.Color;
 import java.awt.Graphics;
 import java.awt.Graphics2D;
 import java.awt.Image;
-import java.awt.Insets;
 import java.awt.Rectangle;
 import java.awt.RenderingHints;
 import java.awt.event.MouseEvent;
@@ -39,6 +38,7 @@ import javax.swing.event.MouseInputListener;
 
 import net.sf.freecol.client.ClientOptions;
 import net.sf.freecol.client.FreeColClient;
+import net.sf.freecol.client.gui.GUI;
 import net.sf.freecol.client.gui.ImageLibrary;
 import net.sf.freecol.client.gui.action.MiniMapZoomInAction;
 import net.sf.freecol.client.gui.action.MiniMapZoomOutAction;
@@ -67,6 +67,7 @@ public final class MiniMap extends JPanel implements MouseInputListener {
 
     private static final int MAP_WIDTH = 220;
     private static final int MAP_HEIGHT = 128;
+    private static final int GAP = 4;
 
     /**
      * The part of the panel that is used for the map. If a skin is
@@ -78,6 +79,10 @@ public final class MiniMap extends JPanel implements MouseInputListener {
     private final JButton miniMapZoomOutButton;
     private final JButton miniMapZoomInButton;
     private Color backgroundColor;
+    private Image back = ResourceManager.getImage("MiniMap.back");
+    private Image skin = ResourceManager.getImage("MiniMap.skin");
+    private Color newBackground = ResourceManager.getColor("miniMapBackground.color");
+    private boolean useSkin = true;
 
     private int tileSize; //tileSize is the size (in pixels) that each tile will take up on the mini map
 
@@ -93,60 +98,66 @@ public final class MiniMap extends JPanel implements MouseInputListener {
      */
     private int adjustX = 0, adjustY = 0;
 
+    private GUI gui;
+
     /**
      * The constructor that will initialize this component.
      *
      * @param freeColClient The main controller object for the client
+     * @param gui a <code>GUI</code> value
      */
-    public MiniMap(FreeColClient freeColClient) {
+    public MiniMap(FreeColClient freeColClient, GUI gui) {
+        this(freeColClient, gui, true);
+    }
+
+    /**
+     * The constructor that will initialize this component.
+     *
+     * @param freeColClient The main controller object for the client
+     * @param gui a <code>GUI</code> value
+     */
+    public MiniMap(FreeColClient freeColClient, GUI gui, boolean useSkin) {
         this.freeColClient = freeColClient;
+        this.gui = gui;
+        this.useSkin = useSkin;
         backgroundColor = Color.BLACK;
+        setLayout(null);
 
         tileSize = 4 * (freeColClient.getClientOptions().getInteger(ClientOptions.DEFAULT_MINIMAP_ZOOM) + 1);
 
         addMouseListener(this);
         addMouseMotionListener(this);
-        setLayout(null);
 
-        Image skin = ResourceManager.getImage("MiniMap.skin");
-        if (skin == null) {
-            try {
-                BevelBorder border = new BevelBorder(BevelBorder.RAISED);
-                setBorder(border);
-            } catch (Exception e) {}
-            setSize(MAP_WIDTH, MAP_HEIGHT);
-            setOpaque(true);
-            mapWindow = new Rectangle(MAP_WIDTH, MAP_HEIGHT);
-        } else {
+        // Add buttons:
+        miniMapZoomOutButton = new UnitButton(freeColClient.getActionManager(), MiniMapZoomOutAction.id);
+        miniMapZoomInButton = new UnitButton(freeColClient.getActionManager(), MiniMapZoomInAction.id);
+
+        miniMapZoomOutButton.setFocusable(false);
+        miniMapZoomInButton.setFocusable(false);
+
+        add(miniMapZoomInButton);
+        add(miniMapZoomOutButton);
+
+        if (useSkin && skin != null) {
             setBorder(null);
             setSize(skin.getWidth(null), skin.getHeight(null));
             setOpaque(false);
             // TODO-LATER: The values below should be specified by a skin-configuration-file:
             mapWindow = new Rectangle(38, 75, MAP_WIDTH, MAP_HEIGHT);
+            miniMapZoomInButton.setLocation(4, 174);
+            miniMapZoomOutButton.setLocation(264, 174);
+        } else {
+            int width = miniMapZoomOutButton.getWidth()
+                + miniMapZoomInButton.getWidth() + 4 * GAP;
+            setBorder(new BevelBorder(BevelBorder.RAISED));
+            setSize(MAP_WIDTH + width, MAP_HEIGHT + 2 * GAP);
+            setOpaque(true);
+            mapWindow = new Rectangle(width/2, GAP, MAP_WIDTH, MAP_HEIGHT);
+            miniMapZoomInButton.setLocation(GAP, MAP_HEIGHT + GAP - miniMapZoomInButton.getHeight());
+            miniMapZoomOutButton.setLocation(getWidth() - GAP - miniMapZoomOutButton.getWidth(),
+                                             MAP_HEIGHT + GAP - miniMapZoomOutButton.getHeight());
         }
 
-        // Add buttons:
-        miniMapZoomOutButton = new UnitButton(freeColClient.getActionManager().getFreeColAction(MiniMapZoomOutAction.id));
-        miniMapZoomInButton = new UnitButton(freeColClient.getActionManager().getFreeColAction(MiniMapZoomInAction.id));
-
-        miniMapZoomOutButton.setFocusable(false);
-        miniMapZoomInButton.setFocusable(false);
-
-        int bh = mapWindow.y + MAP_HEIGHT - Math.max(miniMapZoomOutButton.getHeight(),
-                                                     miniMapZoomInButton.getHeight());
-        int bw = mapWindow.x;
-        if (getBorder() != null) {
-            Insets insets = getBorder().getBorderInsets(this);
-            bh -= insets.bottom;
-            bw += insets.left;
-        }
-
-        // TODO-LATER: The values below should be specified by a skin-configuration-file:
-        miniMapZoomInButton.setLocation(4, 174);
-        miniMapZoomOutButton.setLocation(264, 174);
-
-        add(miniMapZoomInButton);
-        add(miniMapZoomOutButton);
     }
 
     /**
@@ -224,21 +235,15 @@ public final class MiniMap extends JPanel implements MouseInputListener {
             || freeColClient.getGame().getMap() == null) {
             return;
         }
-        Image back = ResourceManager.getImage("MiniMap.back");
-        Image skin = ResourceManager.getImage("MiniMap.skin");
-
-    	Color newBackground = ResourceManager.getColor("miniMapBackground.color");
     	this.setBackgroundColor(newBackground);
 
-        if (skin == null) {
-            paintMap(graphics, getWidth(), getHeight());
-        } else {
+        if (useSkin && skin != null) {
             graphics.drawImage(back, 0, 0, null);
-            graphics.translate(mapWindow.x, mapWindow.y);
-            paintMap(graphics, MAP_WIDTH, MAP_HEIGHT);
-            graphics.translate(-mapWindow.x, -mapWindow.y);
             graphics.drawImage(skin, 0, 0, null);
         }
+        graphics.translate(mapWindow.x, mapWindow.y);
+        paintMap(graphics, MAP_WIDTH, MAP_HEIGHT);
+        graphics.translate(-mapWindow.x, -mapWindow.y);
     }
 
     private Color getMinimapColor(TileType type) {
@@ -257,7 +262,7 @@ public final class MiniMap extends JPanel implements MouseInputListener {
         final Graphics2D g = (Graphics2D) graphics;
         final AffineTransform originTransform = g.getTransform();
         final Map map = freeColClient.getGame().getMap();
-        final ImageLibrary library = freeColClient.getImageLibrary();
+        final ImageLibrary library = gui.getImageLibrary();
 
         g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                            RenderingHints.VALUE_ANTIALIAS_ON);
@@ -268,7 +273,7 @@ public final class MiniMap extends JPanel implements MouseInputListener {
         g.setColor(ResourceManager.getColor("miniMapBackground.color"));
         g.fillRect(0, 0, width, height);
 
-        if (freeColClient.getGUI() == null || freeColClient.getGUI().getFocus() == null) {
+        if (gui.getFocus() == null) {
             return;
         }
 
@@ -278,8 +283,8 @@ public final class MiniMap extends JPanel implements MouseInputListener {
         int ySize = (height / tileSize) * 4;
 
         /* Center the mini map correctly based on the map's focus */
-        firstColumn = freeColClient.getGUI().getFocus().getX() - (xSize / 2);
-        firstRow = freeColClient.getGUI().getFocus().getY() - (ySize / 2);
+        firstColumn = gui.getFocus().getX() - (xSize / 2);
+        firstRow = gui.getFocus().getY() - (ySize / 2);
 
         /* Make sure the mini map won't try to display tiles off the
          * bounds of the world map */
@@ -380,8 +385,8 @@ public final class MiniMap extends JPanel implements MouseInputListener {
          * x/yTiles are the number of tiles that fit on the large map */
         if (getParent() != null) {
             TileType tileType = freeColClient.getGame().getSpecification().getTileTypeList().get(0);
-            int miniRectX = (freeColClient.getGUI().getFocus().getX() - firstColumn) * tileSize;
-            int miniRectY = (freeColClient.getGUI().getFocus().getY() - firstRow) * tileSize / 4;
+            int miniRectX = (gui.getFocus().getX() - firstColumn) * tileSize;
+            int miniRectY = (gui.getFocus().getY() - firstRow) * tileSize / 4;
             int miniRectWidth = (getParent().getWidth() / library.getTerrainImageWidth(tileType) + 1) * tileSize;
             int miniRectHeight = (getParent().getHeight() / library.getTerrainImageHeight(tileType) + 1) * tileSize / 2;
             if (miniRectX + miniRectWidth / 2 > width) {
@@ -424,7 +429,7 @@ public final class MiniMap extends JPanel implements MouseInputListener {
         int tileX = ((x - adjustX) / tileSize) + firstColumn;
         int tileY = ((y - adjustY) / tileSize * 4) + firstRow;
 
-        freeColClient.getGUI().setFocus(freeColClient.getGame().getMap().getTile(tileX,tileY));
+        gui.setFocus(freeColClient.getGame().getMap().getTile(tileX,tileY));
     }
 
     private void focus(MouseEvent e) {
