@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -23,16 +23,17 @@ import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.util.logging.Logger;
 
+import javax.swing.JButton;
 import javax.swing.JLabel;
 
-import net.sf.freecol.client.gui.Canvas;
+import net.miginfocom.swing.MigLayout;
+import net.sf.freecol.client.FreeColClient;
+import net.sf.freecol.client.gui.GUI;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.common.model.Monarch.MonarchAction;
 import net.sf.freecol.common.model.Nation;
 import net.sf.freecol.common.model.StringTemplate;
 
-import net.miginfocom.swing.MigLayout;
-
 /**
  * This panel is used to show information about a tile.
  */
@@ -42,26 +43,26 @@ public final class MonarchPanel extends FreeColDialog<Boolean> implements Action
 
     /**
      * The constructor that will add the items to this panel.
+     * @param freeColClient 
      *
      * @param parent The parent panel.
      * @param action The MonarchAction
+     * @param template The StringTemplate to use
      */
-    public MonarchPanel(Canvas parent, MonarchAction action) {
-        this(parent, action, null);
-    }
+    public MonarchPanel(FreeColClient freeColClient, GUI gui, MonarchAction action,
+                        StringTemplate template) {
+        super(freeColClient, gui);
 
-    /**
-     * The constructor that will add the items to this panel.
-     *
-     * @param parent The parent panel.
-     * @param action The MonarchAction
-     * @param replace The StringTemplate to use
-     */
-    public MonarchPanel(Canvas parent, MonarchAction action, StringTemplate replace) {
+        String messageId = "model.monarch.action." + action.toString();
+        String yesId = messageId + ".yes";
+        String noId = messageId + ".no";
 
-        super(parent);
+        if (!Messages.containsKey(messageId)) {
+            logger.warning("Unrecognized monarch action: " + action);
+            return;
+        }
 
-        setLayout(new MigLayout("wrap 2"));
+        setLayout(new MigLayout("wrap 2, insets 10", "[]20[]"));
 
         JLabel header = new JLabel(Messages.message("aMessageFromTheCrown"));
         header.setFont(mediumHeaderFont);
@@ -69,60 +70,32 @@ public final class MonarchPanel extends FreeColDialog<Boolean> implements Action
 
         Nation nation = getMyPlayer().getNation();
         add(new JLabel(getLibrary().getMonarchImageIcon(nation)));
+        add(getDefaultTextArea((template == null)
+                ? Messages.message(messageId)
+                : Messages.message(new StringTemplate(messageId, template))));
 
-        String messageID;
-        String okText = "ok";
-        String cancelText = null;
-        switch (action) {
-        case RAISE_TAX:
-            messageID = "model.monarch.raiseTax";
-            okText = "model.monarch.acceptTax";
-            cancelText = "model.monarch.rejectTax";
-            break;
-        case LOWER_TAX:
-            messageID = "model.monarch.lowerTax";
-            break;
-        case WAIVE_TAX:
-            messageID = "model.monarch.waiveTax";
-            break;
-        case ADD_TO_REF:
-            messageID = "model.monarch.addToREF";
-            break;
-        case DECLARE_WAR:
-            messageID = "model.monarch.declareWar";
-            break;
-        case SUPPORT_SEA:
-            messageID = "model.monarch.supportSea";
-            cancelText = "display";
-            break;
-        case SUPPORT_LAND:
-            messageID = "model.monarch.supportLand";
-            cancelText = "display";
-            break;
-        case OFFER_MERCENARIES:
-            messageID = "model.monarch.offerMercenaries";
-            okText = "model.monarch.acceptMercenaries";
-            cancelText = "model.monarch.rejectMercenaries";
-            break;
-        default:
-            messageID = "Unknown monarch action: " + action;
+        boolean haveOK = false;
+        if (Messages.containsKey(yesId)) {
+            okButton.setText(Messages.message(yesId));
+            haveOK = true;
         }
-
-        if (replace == null) {
-            add(getDefaultTextArea(Messages.message(messageID)));
-        } else {
-            add(getDefaultTextArea(Messages.message(new StringTemplate(messageID, replace))));
-        }
-
-        okButton.setText(Messages.message(okText));
-        if (cancelText == null) {
-            add(okButton, "newline 20, span, tag ok");
-        } else {
-            add(okButton, "newline 20, span, tag ok, split 2");
-            cancelButton.setText(Messages.message(cancelText));
+        if (!Messages.containsKey(noId)) noId = "close";
+        cancelButton.setText(Messages.message(noId));
+        if (haveOK) {
+            if (action == MonarchAction.RAISE_TAX_ACT
+                || action == MonarchAction.RAISE_TAX_WAR) {
+                add(okButton, "newline 20, span, tag ok, split 3");
+                JButton helpButton = new JButton(Messages.message("help"));
+                helpButton.setActionCommand(HELP);
+                helpButton.addActionListener(this);
+                add(helpButton, "tag help");
+            } else {
+                add(okButton, "newline 20, span, tag ok, split 2");
+            }
             add(cancelButton, "tag cancel");
+        } else {
+            add(cancelButton, "newline 20, span, tag cancel");
         }
-
         setSize(getPreferredSize());
     }
 
@@ -139,9 +112,10 @@ public final class MonarchPanel extends FreeColDialog<Boolean> implements Action
             setResponse(Boolean.TRUE);
         } else if (CANCEL.equals(command)) {
             setResponse(Boolean.FALSE);
+        } else if (HELP.equals(command)) {
+            getGUI().showColopediaPanel("colopedia.concepts.taxes");
         } else {
             logger.warning("Invalid action command: " + command);
         }
     }
-
 }
