@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -23,7 +23,6 @@ import java.awt.Color;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.util.ArrayList;
-import java.util.Collections;
 import java.util.Iterator;
 import java.util.List;
 import java.util.logging.Logger;
@@ -40,7 +39,8 @@ import javax.swing.text.StyleConstants;
 import javax.swing.text.StyledDocument;
 
 import net.miginfocom.swing.MigLayout;
-import net.sf.freecol.client.gui.Canvas;
+import net.sf.freecol.client.FreeColClient;
+import net.sf.freecol.client.gui.GUI;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.ColonyTradeItem;
@@ -65,9 +65,11 @@ import net.sf.freecol.common.model.UnitTradeItem;
  */
 public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> implements ActionListener {
 
+    private static Logger logger = Logger.getLogger(NegotiationDialog.class.getName());
+
     private static final String SEND = "send", ACCEPT = "accept", CANCEL = "cancel";
 
-    private static Logger logger = Logger.getLogger(NegotiationDialog.class.getName());
+    private static final int HUGE_DEMAND = 100000;
 
     private DiplomaticTrade agreement;
 
@@ -90,24 +92,23 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
     /**
      * Creates a new <code>NegotiationDialog</code> instance.
      *
-     * @param parent a <code>Canvas</code> value
      * @param unit an <code>Unit</code> value
      * @param settlement a <code>Settlement</code> value
      */
-    public NegotiationDialog(Canvas parent, Unit unit, Settlement settlement) {
-        this(parent, unit, settlement, null);
+    public NegotiationDialog(FreeColClient freeColClient, GUI gui, Unit unit, Settlement settlement) {
+        this(freeColClient, gui, unit, settlement, null);
     }
 
     /**
      * Creates a new <code>NegotiationDialog</code> instance.
+     * @param freeColClient 
      *
-     * @param parent a <code>Canvas</code> value
      * @param unit an <code>Unit</code> value
      * @param settlement a <code>Settlement</code> value
      * @param agreement a <code>DiplomaticTrade</code> with the offer
      */
-    public NegotiationDialog(Canvas parent, Unit unit, Settlement settlement, DiplomaticTrade agreement) {
-        super(parent);
+    public NegotiationDialog(FreeColClient freeColClient, GUI gui, Unit unit, Settlement settlement, DiplomaticTrade agreement) {
+        super(freeColClient, gui);
         setFocusCycleRoot(true);
 
         this.unit = unit;
@@ -145,7 +146,7 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
      */
     @Override
     public void initialize() {
-        
+
         sendButton = new JButton(Messages.message("negotiationDialog.send"));
         sendButton.addActionListener(this);
         sendButton.setActionCommand(SEND);
@@ -164,8 +165,10 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
         FreeColPanel.enterPressesWhenFocused(cancelButton);
 
         stance = new StanceTradeItemPanel(this, player, otherPlayer);
-        goldDemand = new GoldTradeItemPanel(this, otherPlayer, 0);
-        goldOffer = new GoldTradeItemPanel(this, player, 0);
+        goldDemand = new GoldTradeItemPanel(this, otherPlayer, HUGE_DEMAND);
+        goldOffer = new GoldTradeItemPanel(this, player,
+            ((player.getGold() == Player.GOLD_NOT_ACCOUNTED) ? HUGE_DEMAND
+                : player.getGold()));
         colonyDemand = new ColonyTradeItemPanel(this, otherPlayer);
         colonyOffer = new ColonyTradeItemPanel(this, player);
         /** TODO: UnitTrade
@@ -198,7 +201,7 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
         add(sendButton, "newline 20, span, split 3");
         add(acceptButton, "tag ok");
         add(cancelButton, "tag cancel");
-            
+
         updateDialog();
     }
 
@@ -249,19 +252,19 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
             logger.warning("Failed to update summary: " + e.toString());
         }
     }
-    
+
     private void updateOfferItems(){
         // Update stance options
         stance.updateStanceBox();
-        
+
         // Update the gold amount that can be demanded
         goldOffer.setAvailableGold(player.getGold());
-        
+
         if(unit.isCarrier()){
             Iterator<Goods> goodsInAgreement = agreement.getGoodsGivenBy(player).iterator();
             List<Goods> goodsAvail = new ArrayList<Goods>();
             goodsAvail.addAll(unit.getGoodsContainer().getGoods());
-                
+
             //remove the ones already on the table
             while(goodsInAgreement.hasNext()){
                 Goods goods = goodsInAgreement.next();
@@ -275,7 +278,7 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
                     }
                 }
             }
-                
+
             // Update the list of goods available to add to agreement
             goodsOffer.updateGoodsBox(goodsAvail);
         } else{
@@ -283,18 +286,18 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
             colonyOffer.updateColonyBox();
         }
     }
-    
+
     private void updateDemandItems() {
         // Update the gold amount that can be demanded
         NationSummary ns = getController().getNationSummary(otherPlayer);
         int foreignGold = (ns == null) ? 0 : ns.getGold();
         goldDemand.setAvailableGold(foreignGold);
-        
+
         if(unit.isCarrier()){
             Iterator<Goods> goodsInAgreement = agreement.getGoodsGivenBy(otherPlayer).iterator();
             List<Goods> goodsAvail = new ArrayList<Goods>();
             goodsAvail.addAll(settlement.getGoodsContainer().getGoods());
-                
+
             //remove the ones already on the table
             while(goodsInAgreement.hasNext()){
                 Goods goods = goodsInAgreement.next();
@@ -308,7 +311,7 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
                     }
                 }
             }
-                
+
             // Update the list of goods available to add to agreement
             goodsDemand.updateGoodsBox(goodsAvail);
         }
@@ -317,7 +320,7 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
             colonyDemand.updateColonyBox();
         }
     }
-    
+
     public void updateDialog(){
         updateOfferItems();
         updateDemandItems();
@@ -469,7 +472,7 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
     /**
      * Analyzes an event and calls the right external methods to take care of
      * the user's request.
-     * 
+     *
      * @param event The incoming action event
      */
     @Override
@@ -498,7 +501,7 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
         private Player player;
         private NegotiationDialog negotiationDialog;
         private JLabel textLabel;
-        
+
         /**
          * Creates a new <code>ColonyTradeItemPanel</code> instance.
          *
@@ -521,9 +524,9 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
             add(this.textLabel);
             add(colonyBox);
             add(addButton);
-            
+
         }
-        
+
         private void updateColonyBox() {
 
             if (!player.isEuropean()) {
@@ -537,11 +540,10 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
                 colonyBox.removeActionListener(al);
             }
             colonyBox.removeAllItems();
-                
+
             Iterator<Colony> coloniesInAgreement = agreement.getColoniesGivenBy(player).iterator();
-            List<Colony> coloniesAvail = new ArrayList<Colony>();
-            coloniesAvail.addAll(player.getColonies());
-                
+            List<Colony> coloniesAvail = getClientOptions().getSortedColonies(player);
+
             //remove the ones already on the table
             while(coloniesInAgreement.hasNext()){
                 Colony colony = coloniesInAgreement.next();
@@ -554,15 +556,13 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
                     }
                 }
             }
-            
+
             if (coloniesAvail.isEmpty()){
                 addButton.setEnabled(false);
                 colonyBox.setEnabled(false);
             } else {
-                Collections.sort(coloniesAvail, getClient().getClientOptions().getColonyComparator());
-                Iterator<Colony> colonyIterator = coloniesAvail.iterator();
-                while (colonyIterator.hasNext()) {
-                    colonyBox.addItem(colonyIterator.next());
+                for (Colony c : coloniesAvail) {
+                    colonyBox.addItem(c);
                 }
                 for(ActionListener al : listeners) {
                     colonyBox.addActionListener(al);
@@ -575,7 +575,7 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
         /**
          * Analyzes an event and calls the right external methods to take care of
          * the user's request.
-         * 
+         *
          * @param event The incoming action event
          */
         public void actionPerformed(ActionEvent event) {
@@ -595,7 +595,7 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
         private Player player;
         private NegotiationDialog negotiationDialog;
         private JLabel label;
-        
+
         /**
          * Creates a new <code>GoodsTradeItemPanel</code> instance.
          *
@@ -632,7 +632,7 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
                 goodsBox.removeActionListener(al);
             }
             goodsBox.removeAllItems();
-            
+
             if(allGoods != null && !allGoods.isEmpty()){
                 Iterator<Goods> goodsIterator = allGoods.iterator();
                 while (goodsIterator.hasNext()) {
@@ -644,7 +644,7 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
                 for(ActionListener al : listeners) {
                     goodsBox.addActionListener(al);
                 }
-                
+
                 this.label.setEnabled(true);
                 addButton.setEnabled(true);
                 goodsBox.setEnabled(true);
@@ -652,13 +652,13 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
                 this.label.setEnabled(false);
                 addButton.setEnabled(false);
                 goodsBox.setEnabled(false);
-            }            
+            }
         }
 
         /**
          * Analyzes an event and calls the right external methods to take care of
          * the user's request.
-         * 
+         *
          * @param event The incoming action event
          */
         public void actionPerformed(ActionEvent event) {
@@ -681,16 +681,16 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
                   throw new NullPointerException();
                this.value = value;
             }
-            
+
             @Override
             public String toString() {
                 return Messages.getStanceAsString(value);
             }
-            
+
             Stance getValue() {
                 return value;
             }
-            
+
             @Override
             public boolean equals(Object other) {
                 if (other == null || !(other instanceof StanceItem)) {
@@ -705,13 +705,13 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
                return value.hashCode();
             }
         }
-        
+
         private JComboBox stanceBox;
         private JButton addButton;
         private NegotiationDialog negotiationDialog;
         private Player source;
         private Player target;
-        
+
         /**
          * Creates a new <code>StanceTradeItemPanel</code> instance.
          *
@@ -723,12 +723,12 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
             this.negotiationDialog = parent;
             this.source = source;
             this.target = target;
-            
+
             addButton = new JButton(Messages.message("negotiationDialog.add"));
             addButton.addActionListener(this);
             addButton.setActionCommand("add");
             stanceBox = new JComboBox();
-            
+
             updateStanceBox();
 
             setBorder(BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(Color.BLACK),
@@ -742,7 +742,7 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
         /**
          * Analyzes an event and calls the right external methods to take care of
          * the user's request.
-         * 
+         *
          * @param event The incoming action event
          */
         public void actionPerformed(ActionEvent event) {
@@ -754,7 +754,7 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
             }
 
         }
-        
+
         public void updateStanceBox(){
             stanceBox.removeAllItems();
             Stance stance = source.getStance(target);
@@ -791,7 +791,7 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
             spinner = new JSpinner(new SpinnerNumberModel(0, 0, gold, 1));
             // adjust entry size
             ((JSpinner.DefaultEditor)spinner.getEditor()).getTextField().setColumns(5);
-            
+
             setBorder(BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(Color.BLACK),
                                                          BorderFactory.createEmptyBorder(5, 5, 5, 5)));
             setLayout(new MigLayout("wrap 1", "", ""));
@@ -801,9 +801,9 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
         }
 
         /**
-         * Analyzes an event and calls the right external methods to take care of
-         * the user's request.
-         * 
+         * Analyzes an event and calls the right external methods to
+         * take care of the user's request.
+         *
          * @param event The incoming action event
          */
         public void actionPerformed(ActionEvent event) {
@@ -815,7 +815,7 @@ public final class NegotiationDialog extends FreeColDialog<DiplomaticTrade> impl
             }
 
         }
-        
+
         public void setAvailableGold(int gold) {
             SpinnerNumberModel model = (SpinnerNumberModel) spinner.getModel();
             model.setMaximum(new Integer(gold));
