@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -47,17 +47,18 @@ import javax.swing.table.TableCellEditor;
 import javax.swing.table.TableCellRenderer;
 import javax.swing.table.TableColumn;
 
+import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.control.PreGameController;
-import net.sf.freecol.client.gui.Canvas;
+import net.sf.freecol.client.gui.GUI;
 import net.sf.freecol.client.gui.ImageLibrary;
+import net.sf.freecol.client.gui.action.ColopediaAction.PanelType;
 import net.sf.freecol.client.gui.i18n.Messages;
-import net.sf.freecol.client.gui.panel.ColopediaPanel.PanelType;
 import net.sf.freecol.common.model.EuropeanNationType;
 import net.sf.freecol.common.model.Nation;
 import net.sf.freecol.common.model.NationOptions;
+import net.sf.freecol.common.model.NationOptions.NationState;
 import net.sf.freecol.common.model.NationType;
 import net.sf.freecol.common.model.Player;
-import net.sf.freecol.common.model.NationOptions.NationState;
 import net.sf.freecol.common.resources.ResourceManager;
 
 
@@ -93,17 +94,18 @@ public final class PlayersTable extends JTable {
 
     /**
      * The constructor that will add the items to this panel.
-     * 
+     * @param gui 
+     *
      * @param canvas a <code>Canvas</code> value
      * @param nationOptions a <code>NationOptions</code> value
      * @param myPlayer a <code>Player</code> value
      */
-    public PlayersTable(final Canvas canvas, NationOptions nationOptions, Player myPlayer) {
+    public PlayersTable(final FreeColClient freeColClient, final GUI gui, NationOptions nationOptions, Player myPlayer) {
         super();
 
-        library = canvas.getImageLibrary();
+        library = gui.getImageLibrary();
 
-        setModel(new PlayersTableModel(canvas.getClient().getPreGameController(), nationOptions, myPlayer));
+        setModel(new PlayersTableModel(freeColClient.getPreGameController(), nationOptions, myPlayer));
         setRowHeight(47);
 
         JButton nationButton = new JButton(Messages.message("nation"));
@@ -114,13 +116,13 @@ public final class PlayersTable extends JTable {
 
         nationButton.addActionListener(new ActionListener() {
                 public void actionPerformed(ActionEvent event) {
-                    canvas.showPanel(new ColopediaPanel(canvas, PanelType.NATIONS, null));
+                    gui.showColopediaPanel(PanelType.NATIONS.toString());
                 }
             });
 
         advantageButton.addActionListener(new ActionListener() {
                 public void actionPerformed(ActionEvent event) {
-                    canvas.showPanel(new ColopediaPanel(canvas, PanelType.NATION_TYPES, null));
+                    gui.showColopediaPanel(PanelType.NATION_TYPES.toString());
                 }
             });
 
@@ -139,10 +141,10 @@ public final class PlayersTable extends JTable {
         TableColumn availableColumn = getColumnModel().getColumn(AVAILABILITY_COLUMN);
         availableColumn.setCellRenderer(new AvailableCellRenderer());
         availableColumn.setCellEditor(new AvailableCellEditor());
-        
+
         TableColumn advantagesColumn = getColumnModel().getColumn(ADVANTAGE_COLUMN);
         if (nationOptions.getNationalAdvantages() == NationOptions.Advantages.SELECTABLE) {
-            advantagesColumn.setCellEditor(new AdvantageCellEditor(canvas.getSpecification().getEuropeanNationTypes()));
+            advantagesColumn.setCellEditor(new AdvantageCellEditor(freeColClient.getGame().getSpecification().getEuropeanNationTypes()));
         }
         advantagesColumn.setCellRenderer(new AdvantageCellRenderer(nationOptions.getNationalAdvantages()));
         advantagesColumn.setHeaderRenderer(renderer);
@@ -153,7 +155,7 @@ public final class PlayersTable extends JTable {
         TableColumn playerColumn = getColumnModel().getColumn(PLAYER_COLUMN);
         playerColumn.setCellEditor(new PlayerCellEditor());
         playerColumn.setCellRenderer(new PlayerCellRenderer());
-        
+
     }
 
     public void update() {
@@ -233,7 +235,13 @@ public final class PlayersTable extends JTable {
         }
     }
 
-    class AvailableCellRenderer extends JLabel implements TableCellRenderer {
+    class AvailableCellRenderer implements TableCellRenderer {
+
+        private JComboBox box = new JComboBox(allStates);
+
+        public AvailableCellRenderer() {
+            box.setRenderer(new NationStateRenderer());
+        }
 
         /**
          * Returns the component used to render the cell's value.
@@ -246,8 +254,8 @@ public final class PlayersTable extends JTable {
          */
         public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected,
                                                        boolean hasFocus, int row, int column) {
-            setText(Messages.message("nationState." + ((NationState) value).toString()));
-            return this;
+            box.setSelectedItem(value);
+            return box;
         }
     }
 
@@ -379,7 +387,7 @@ public final class PlayersTable extends JTable {
 
         /**
          * A standard constructor.
-         * 
+         *
          * @param pgc The PreGameController to use when updates need to be notified
          *            across the network.
          * @param nationOptions a <code>NationOptions</code> value
@@ -413,7 +421,7 @@ public final class PlayersTable extends JTable {
 
         /**
          * Returns the Class of the objects in the given column.
-         * 
+         *
          * @param column The column to return the Class of.
          * @return The Class of the objects in the given column.
          */
@@ -435,7 +443,7 @@ public final class PlayersTable extends JTable {
 
         /**
          * Returns the amount of columns in this statesTable.
-         * 
+         *
          * @return The amount of columns in this statesTable.
          */
         public int getColumnCount() {
@@ -444,7 +452,7 @@ public final class PlayersTable extends JTable {
 
         /**
          * Returns the name of the specified column.
-         * 
+         *
          * @return The name of the specified column.
          */
         public String getColumnName(int column) {
@@ -453,7 +461,7 @@ public final class PlayersTable extends JTable {
 
         /**
          * Returns the amount of rows in this statesTable.
-         * 
+         *
          * @return The amount of rows in this statesTable.
          */
         public int getRowCount() {
@@ -462,7 +470,7 @@ public final class PlayersTable extends JTable {
 
         /**
          * Returns the value at the requested location.
-         * 
+         *
          * @param row The requested row.
          * @param column The requested column.
          * @return The value at the requested location.
@@ -492,7 +500,7 @@ public final class PlayersTable extends JTable {
 
         /**
          * Returns 'true' if the specified cell is editable, 'false' otherwise.
-         * 
+         *
          * @param row The specified row.
          * @param column The specified column.
          * @return 'true' if the specified cell is editable, 'false' otherwise.
@@ -516,7 +524,7 @@ public final class PlayersTable extends JTable {
 
         /**
          * Sets the value at the specified location.
-         * 
+         *
          * @param value The new value.
          * @param row The specified row.
          * @param column The specified column.
