@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -23,7 +23,8 @@ import java.awt.event.ActionListener;
 
 import javax.swing.JPanel;
 
-import net.sf.freecol.client.gui.Canvas;
+import net.sf.freecol.client.FreeColClient;
+import net.sf.freecol.client.gui.GUI;
 import net.sf.freecol.client.gui.i18n.Messages;
 
 
@@ -34,10 +35,11 @@ public final class ReportLabourDetailPanel extends ReportPanel implements Action
     
     /**
      * The constructor that will add the items to this panel.
+     * @param freeColClient 
      * @param parent The parent of this panel.
      */
-    public ReportLabourDetailPanel(Canvas parent) {
-        super(parent, Messages.message("report.labour.details"));
+    public ReportLabourDetailPanel(FreeColClient freeColClient, GUI gui) {
+        super(freeColClient, gui, Messages.message("report.labour.details"));
     }
 
     /**
