@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -22,12 +22,12 @@ package net.sf.freecol.client.gui.panel;
 import java.awt.Image;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
-import java.util.Hashtable;
 import java.util.Arrays;
 import java.util.Comparator;
-
+import java.util.Hashtable;
 import java.util.List;
 import java.util.Vector;
+
 import javax.swing.BorderFactory;
 import javax.swing.ImageIcon;
 import javax.swing.JButton;
@@ -37,8 +37,10 @@ import javax.swing.JTextPane;
 import javax.swing.text.StyleConstants;
 import javax.swing.text.StyledDocument;
 
+import net.miginfocom.swing.MigLayout;
 import net.sf.freecol.client.ClientOptions;
-import net.sf.freecol.client.gui.Canvas;
+import net.sf.freecol.client.FreeColClient;
+import net.sf.freecol.client.gui.GUI;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.Europe;
@@ -54,8 +56,6 @@ import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.option.BooleanOption;
 
-import net.miginfocom.swing.MigLayout;
-
 /**
  * This panel displays the Turn Report.
  */
@@ -67,26 +67,28 @@ public final class ReportTurnPanel extends ReportPanel {
 
     /**
      * The constructor that will add the items to this panel.
+     * @param freeColClient 
      *
      * @param parent The parent of this panel.
      * @param messages A variable number of ModelMessages
      */
-    public ReportTurnPanel(Canvas parent, ModelMessage... messages) {
-        super(parent, Messages.message("reportTurnAction.name"));
+    public ReportTurnPanel(FreeColClient freeColClient, GUI gui, ModelMessage... messages) {
+        super(freeColClient, gui, Messages.message("reportTurnAction.name"));
 
+        ClientOptions options = getClientOptions();
         this._messages = messages;
 
-        Comparator<ModelMessage> comparator = getClient().getClientOptions().getModelMessageComparator();
+        Comparator<ModelMessage> comparator
+            = options.getModelMessageComparator();
         if (comparator != null) {
             Arrays.sort(messages, comparator);
         }
 
-        ClientOptions options = getClient().getClientOptions();
         int groupBy = options.getInteger(ClientOptions.MESSAGES_GROUP_BY);
 
         Object source = this;
         ModelMessage.MessageType type = null;
-        Game game = getClient().getGame();
+        Game game = getFreeColClient().getGame();
 
         // count number of headlines
         for (final ModelMessage message : messages) {
@@ -128,7 +130,7 @@ public final class ReportTurnPanel extends ReportPanel {
             FreeColObject messageDisplay = game.getMessageDisplay(message);
             if (messageDisplay != null) {
                 // TODO: Scale icons relative to font size.
-                ImageIcon icon = getCanvas().getImageIcon(messageDisplay, false);
+                ImageIcon icon = getGUI().getImageIcon(messageDisplay, false);
                 if (icon != null && icon.getIconHeight() > 40) {
                     Image image = icon.getImage();
                     int newWidth = (int)((double)image.getWidth(null)/image.getHeight(null)*40.0);
@@ -197,7 +199,7 @@ public final class ReportTurnPanel extends ReportPanel {
             if (filterOption != null) {
                 JButton filterButton = new JButton("X");
                 filterButton.setToolTipText(Messages.message(StringTemplate.template("model.message.filter")
-                                                             .addName("%type%", message.getMessageTypeName())));
+                                                             .add("%type%", message.getMessageTypeName())));
                 filterButton.addActionListener(new ActionListener() {
 
                     public void actionPerformed(ActionEvent event) {
@@ -306,7 +308,7 @@ public final class ReportTurnPanel extends ReportPanel {
             while ((end = input.indexOf('%', start + 1)) >= 0) {
                 String var = input.substring(start, end + 1);
                 String[] item = findReplacementData(message, var);
-                FreeColGameObject messageSource = getClient().getGame().getMessageSource(message);
+                FreeColGameObject messageSource = getFreeColClient().getGame().getMessageSource(message);
                 if (item != null && var.equals(item[0])) {
                     // found variable to replace
                     if (var.equals("%colony%")) {
