@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -48,15 +48,19 @@ import javax.swing.event.ChangeEvent;
 import javax.swing.event.ChangeListener;
 
 import net.miginfocom.swing.MigLayout;
-import net.sf.freecol.client.gui.Canvas;
+import net.sf.freecol.client.FreeColClient;
+import net.sf.freecol.client.gui.GUI;
+import net.sf.freecol.client.gui.ImageLibrary;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.client.gui.plaf.FreeColComboBoxRenderer;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.Europe;
+import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.Goods;
 import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.IndianSettlement;
 import net.sf.freecol.common.model.Location;
+import net.sf.freecol.common.model.Map;
 import net.sf.freecol.common.model.Market;
 import net.sf.freecol.common.model.PathNode;
 import net.sf.freecol.common.model.Player;
@@ -94,63 +98,26 @@ public final class SelectDestinationDialog extends FreeColDialog<Location>
 
     /**
      * The constructor to use.
+     * @param freeColClient 
      */
-    public SelectDestinationDialog(Canvas parent, Unit unit) {
-        super(parent);
-
-        final Settlement inSettlement = unit.getSettlement();
+    public SelectDestinationDialog(FreeColClient freeColClient, GUI gui, Unit unit) {
+        super(freeColClient, gui);
 
         // Collect the goods the unit is carrying.
-        List<GoodsType> goodsTypeList = new ArrayList<GoodsType>();
+        final List<GoodsType> goodsTypes = new ArrayList<GoodsType>();
         for (Goods goods : unit.getGoodsList()) {
-            goodsTypeList.add(goods.getType());
-        }
-        final List<GoodsType> goodsTypes = goodsTypeList;
-
-        // Search for destinations we can reach:
-        getGame().getMap().search(unit, unit.getTile(), new GoalDecider() {
-                public PathNode getGoal() {
-                    return null;
-                }
-
-                public boolean check(Unit u, PathNode p) {
-                    Settlement settlement = p.getTile().getSettlement();
-                    if (settlement != null && settlement != inSettlement) {
-                        String extras = (settlement.getOwner() != u.getOwner())
-                            ? getExtras(u, settlement, goodsTypes) : "";
-                        destinations.add(new Destination(settlement, p.getTurns(), extras));
-                    }
-                    return false;
-                }
-
-                public boolean hasSubGoals() {
-                    return false;
-                }
-            }, CostDeciders.avoidIllegal(), Integer.MAX_VALUE);
-
-
-        if (destinationComparator == null) {
-            destinationComparator = new DestinationComparator(getMyPlayer());
-        }
-        Collections.sort(destinations, destinationComparator);
-
-        if (unit.isNaval() && unit.getOwner().canMoveToEurope()) {
-            PathNode path = unit.findPathToEurope();
-            int turns = (path != null)
-                ? unit.getSailTurns() + path.getTotalTurns()
-                : (unit.getTile() != null
-                   && (unit.getTile().canMoveToEurope()
-                       || unit.getTile().isAdjacentToMapEdge()))
-                ? unit.getSailTurns()
-                : -1;
-            if (turns >= 0) {
-                Europe europe = getMyPlayer().getEurope();
-                destinations.add(0,
-                    new Destination(europe, turns,
-                        getExtras(unit, europe, goodsTypes)));
+            if (!goodsTypes.contains(goods.getType())) {
+                goodsTypes.add(goods.getType());
             }
         }
 
+        destinations.clear();
+        if (unit.isInEurope()) {
+            collectDestinationsFromEurope(unit, goodsTypes);
+        } else {
+            collectDestinationsFromAmerica(unit, goodsTypes);
+        }
+
         MigLayout layout = new MigLayout("wrap 1, fill", "[align center]", "");
         setLayout(layout);
 
@@ -171,13 +138,14 @@ public final class SelectDestinationDialog extends FreeColDialog<Location>
                     if (d != null) {
                         setResponse((Location) d.location);
                     }
-                    getCanvas().remove(SelectDestinationDialog.this);
+                    getGUI().removeFromCanvas(SelectDestinationDialog.this);
                 }
             };
 
         Action quitAction = new AbstractAction(Messages.message("selectDestination.cancel")) {
                 public void actionPerformed(ActionEvent e) {
-                    getCanvas().remove(SelectDestinationDialog.this);
+                    getGUI().removeFromCanvas(SelectDestinationDialog.this);
+                    setResponse(null);
                 }
             };
 
@@ -193,7 +161,7 @@ public final class SelectDestinationDialog extends FreeColDialog<Location>
                         if (d != null) {
                             setResponse((Location) d.location);
                         }
-                        getCanvas().remove(SelectDestinationDialog.this);
+                        getGUI().removeFromCanvas(SelectDestinationDialog.this);
                     }
                 }
             };
@@ -233,6 +201,77 @@ public final class SelectDestinationDialog extends FreeColDialog<Location>
         setSize(getPreferredSize());
     }
 
+    private void collectDestinationsFromEurope(Unit unit,
+                                               List<GoodsType> goodsTypes) {
+        Game game = getGame();
+        Map map = game.getMap();
+        int sailTurns = unit.getSailTurns();
+        for (Player p : game.getPlayers()) {
+            for (Settlement s : p.getSettlements()) {
+                if (!s.isConnected()) continue;
+                PathNode path = unit.findPathToEurope(s.getTile());
+                if (path != null) {
+                    String extras = (s.getOwner() != unit.getOwner())
+                        ? getExtras(unit, s, goodsTypes) : "";
+                    destinations.add(new Destination(s,
+                            sailTurns + path.getTurns(), extras));
+                }
+            }
+        }
+
+        if (destinationComparator == null) {
+            destinationComparator = new DestinationComparator(getMyPlayer());
+        }
+        Collections.sort(destinations, destinationComparator);
+
+        destinations.add(0, new Destination(map, sailTurns, ""));
+    }
+
+    private void collectDestinationsFromAmerica(Unit unit,
+        final List<GoodsType> goodsTypes) {
+        final Settlement inSettlement = unit.getSettlement();
+
+        unit.search(unit.getTile(), new GoalDecider() {
+                public PathNode getGoal() {
+                    return null;
+                }
+
+                public boolean check(Unit u, PathNode p) {
+                    Settlement settlement = p.getTile().getSettlement();
+                    if (settlement != null && settlement != inSettlement) {
+                        String extras = (settlement.getOwner() != u.getOwner())
+                            ? getExtras(u, settlement, goodsTypes) : "";
+                        destinations.add(new Destination(settlement, p.getTurns(), extras));
+                    }
+                    return false;
+                }
+
+                public boolean hasSubGoals() {
+                    return false;
+                }
+            }, CostDeciders.avoidIllegal(), Integer.MAX_VALUE, null);
+
+        if (destinationComparator == null) {
+            destinationComparator = new DestinationComparator(getMyPlayer());
+        }
+        Collections.sort(destinations, destinationComparator);
+
+        if (unit.isNaval() && unit.getOwner().canMoveToEurope()) {
+            PathNode path = unit.findPathToEurope();
+            int turns = (path != null) ? unit.getSailTurns()
+                + path.getTotalTurns()
+                : (unit.getTile() != null
+                    && unit.getTile().canMoveToEurope()) ? unit.getSailTurns()
+                : -1;
+            if (turns >= 0) {
+                Europe europe = getMyPlayer().getEurope();
+                destinations.add(0,
+                    new Destination(europe, turns,
+                        getExtras(unit, europe, goodsTypes)));
+            }
+        }
+    }
+
     @Override
     public void requestFocus() {
         destinationList.requestFocus();
@@ -281,6 +320,9 @@ public final class SelectDestinationDialog extends FreeColDialog<Location>
             if (!sales.isEmpty()) {
                 return "[" + Utils.join(", ", sales) + "]";
             }
+        } else if (loc instanceof Settlement
+            && ((Settlement)loc).getOwner().atWarWith(unit.getOwner())) {
+            return "[" + Messages.message("model.stance.war") + "]";
         } else if (loc instanceof Settlement && !goodsTypes.isEmpty()) {
             List<String> sales = new ArrayList<String>();
             for (GoodsType goodsType : goodsTypes) {
@@ -311,6 +353,7 @@ public final class SelectDestinationDialog extends FreeColDialog<Location>
         for (Destination d : destinations) {
             if (showOnlyMyColonies) {
                 if (d.location instanceof Europe
+                    || d.location instanceof Map
                     || (d.location instanceof Colony
                         && ((Colony) d.location).getOwner() == getMyPlayer())) {
                     model.addElement(d);
@@ -326,16 +369,17 @@ public final class SelectDestinationDialog extends FreeColDialog<Location>
     }
 
     public int compareNames(Location dest1, Location dest2) {
+        Player player = getMyPlayer();
         String name1 = "";
         if (dest1 instanceof Settlement) {
-            name1 = ((Settlement) dest1).getName();
-        } else if (dest1 instanceof Europe) {
+            name1 = ((Settlement) dest1).getNameFor(player);
+        } else if (dest1 instanceof Europe || dest1 instanceof Map) {
             return -1;
         }
         String name2 = "";
         if (dest2 instanceof Settlement) {
-            name2 = ((Settlement) dest2).getName();
-        } else if (dest2 instanceof Europe) {
+            name2 = ((Settlement) dest2).getNameFor(player);
+        } else if (dest2 instanceof Europe || dest2 instanceof Map) {
             return 1;
         }
         return name1.compareTo(name2);
@@ -360,17 +404,22 @@ public final class SelectDestinationDialog extends FreeColDialog<Location>
 
             Destination d = (Destination) value;
             Location location = d.location;
+            Player player = getMyPlayer();
             String name = "";
+            ImageLibrary lib = getLibrary();
             if (location instanceof Europe) {
                 Europe europe = (Europe) location;
                 name = Messages.message(europe.getNameKey());
-                label.setIcon(new ImageIcon(getLibrary().getCoatOfArmsImage(europe.getOwner().getNation())
-                                            .getScaledInstance(-1, 48, Image.SCALE_SMOOTH)));
+                label.setIcon(new ImageIcon(lib.getCoatOfArmsImage(europe.getOwner().getNation())
+                        .getScaledInstance(-1, 48, Image.SCALE_SMOOTH)));
+            } else if (location instanceof Map) {
+                name = Messages.message(location.getLocationNameFor(player));
+                label.setIcon(lib.getMiscImageIcon(ImageLibrary.LOST_CITY_RUMOUR));
             } else if (location instanceof Settlement) {
                 Settlement settlement = (Settlement) location;
-                name = settlement.getName();
-                label.setIcon(new ImageIcon(getLibrary().getSettlementImage(settlement)
-                                            .getScaledInstance(64, -1, Image.SCALE_SMOOTH)));
+                name = Messages.message(settlement.getNameFor(player));
+                label.setIcon(new ImageIcon(lib.getSettlementImage(settlement)
+                        .getScaledInstance(64, -1, Image.SCALE_SMOOTH)));
             }
             label.setText(Messages.message(StringTemplate.template("selectDestination.destinationTurns")
                                            .addName("%location%", name)
@@ -392,7 +441,7 @@ public final class SelectDestinationDialog extends FreeColDialog<Location>
             Location dest2 = choice2.location;
 
             int score1 = 100;
-            if (dest1 instanceof Europe) {
+            if (dest1 instanceof Europe || dest1 instanceof Map) {
                 score1 = 10;
             } else if (dest1 instanceof Colony) {
                 if (((Colony) dest1).getOwner() == owner) {
@@ -404,7 +453,7 @@ public final class SelectDestinationDialog extends FreeColDialog<Location>
                 score1 = 40;
             }
             int score2 = 100;
-            if (dest2 instanceof Europe) {
+            if (dest2 instanceof Europe || dest2 instanceof Map) {
                 score2 = 10;
             } else if (dest2 instanceof Colony) {
                 if (((Colony) dest2).getOwner() == owner) {
