@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -24,18 +24,16 @@ package net.sf.freecol.common.model;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashSet;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Set;
 import java.util.logging.Logger;
 
-import javax.xml.stream.XMLStreamConstants;
 import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
+import net.sf.freecol.common.model.Player.NoClaimReason;
 
-import org.w3c.dom.Element;
 
 /**
  * Represents a work location on a tile. Each ColonyTile except the
@@ -44,23 +42,21 @@ import org.w3c.dom.Element;
  * produces two different of goods, one food type and one new world
  * raw material.
  */
-public class ColonyTile extends FreeColGameObject
-    implements WorkLocation, Ownable {
+public class ColonyTile extends WorkLocation implements Ownable {
 
     @SuppressWarnings("unused")
     private static final Logger logger = Logger.getLogger(ColonyTile.class.getName());
 
     public static final String UNIT_CHANGE = "UNIT_CHANGE";
 
-    /** The colony this colony tile belongs to. */
-    protected Colony colony;
+    /**
+     * The maximum number of units a ColonyTile can hold.
+     */
+    public static final int UNIT_CAPACITY = 1;
 
     /** The tile to work. */
     protected Tile workTile;
 
-    /** The unit working the tile. */
-    protected Unit unit;
-
     /** Is this colony tile at the center of the colony. */
     protected boolean colonyCenterTile;
 
@@ -76,13 +72,20 @@ public class ColonyTile extends FreeColGameObject
      * Constructor for ServerColonyTile.
      *
      * @param game The <code>Game</code> this object belongs to.
+     * @param colony The <code>Colony</code> this object belongs to.
+     * @param workTile The tile in which this <code>ColonyTile</code>
+     *                 represents a <code>WorkLocation</code> for.
      */
-    public ColonyTile(Game game) {
+    protected ColonyTile(Game game, Colony colony, Tile workTile) {
         super(game);
+
+        setColony(colony);
+        this.workTile = workTile;
+        colonyCenterTile = (getTile() == workTile);
     }
 
     /**
-     * Initiates a new <code>Building</code> from an
+     * Initiates a new <code>ColonyTile</code> from an
      * XML representation.
      *
      * @param game The <code>Game</code> this object belongs to.
@@ -95,26 +98,11 @@ public class ColonyTile extends FreeColGameObject
         readFromXML(in);
     }
 
-    /**
-     * Initiates a new <code>Building</code> from an
-     * XML representation.
-     *
-     * @param game The <code>Game</code> this object belongs to.
-     * @param e An XML-element that will be used to initialize
-     *      this object.
-     */
-    public ColonyTile(Game game, Element e) {
-        super(game, e);
-
-        readFromXMLElement(e);
-    }
-
     /**
      * Initiates a new <code>ColonyTile</code>
      * with the given ID. The object should later be
      * initialized by calling either
-     * {@link #readFromXML(XMLStreamReader)} or
-     * {@link #readFromXMLElement(Element)}.
+     * {@link #readFromXML(XMLStreamReader)}.
      *
      * @param game The <code>Game</code> in which this object belong.
      * @param id The unique identifier for this object.
@@ -133,20 +121,12 @@ public class ColonyTile extends FreeColGameObject
             return StringTemplate.name(name);
         } else {
             return StringTemplate.template("nearLocation")
+                .add("%direction%", "direction."
+                    + getTile().getDirection(workTile).toString())
                 .addName("%location%", name);
         }
     }
 
-    /**
-     * Returns the name of this ColonyTile for a particular player.
-     *
-     * @param player The <code>Player</code> to prepare the name for.
-     * @return The name of this ColonyTile.
-     */
-    public StringTemplate getLocationNameFor(Player player) {
-        return getLocationName();
-    }
-
     /**
      * Returns a description of the tile, with the name of the tile
      * and any improvements made to it (road/plow).
@@ -159,31 +139,10 @@ public class ColonyTile extends FreeColGameObject
 
 
     /**
-    * Gets the owner of this <code>Ownable</code>.
-    *
-    * @return The <code>Player</code> controlling this
-    *         {@link Ownable}.
-    */
-    public Player getOwner() {
-        return colony.getOwner();
-    }
-
-    /**
-     * Sets the owner of this <code>Ownable</code>.
+     * Checks if this is the tile where the <code>Colony</code> is located.
      *
-     * @param p The <code>Player</code> that should take ownership
-     *      of this {@link Ownable}.
-     * @exception UnsupportedOperationException is always thrown by
-     *      this method.
+     * @return True if this is the colony center tile.
      */
-    public void setOwner(Player p) {
-        throw new UnsupportedOperationException();
-    }
-
-    /**
-    * Checks if this is the tile where the <code>Colony</code> is located.
-    * @return The result.
-    */
     public boolean isColonyCenterTile() {
         return colonyCenterTile;
     }
@@ -199,62 +158,127 @@ public class ColonyTile extends FreeColGameObject
     }
 
     /**
-     * Gets the tile where the colony is located.
-     * @return The <code>Tile</code>.
-     */
-    public Tile getTile() {
-        return colony.getTile();
-    }
-
-    public GoodsContainer getGoodsContainer() {
-        return null;
-    }
-
-
-    /**
-     * Gets a pointer to the settlement containing this tile.
-     *
-     * @return This settlement.
-     */
-    public Settlement getSettlement() {
-        return colony;
-    }
-
-    /**
-     * Gets a pointer to the colony containing this tile.
-     *
-     * @return The <code>Colony</code>.
-     */
-    public Colony getColony() {
-        return colony;
-    }
-
-    /**
-     * Gets the <code>Unit</code> currently working on this <code>ColonyTile</code>.
+     * Gets the <code>Unit</code> currently working on this
+     * <code>ColonyTile</code>.
      *
      * @return The <code>Unit</code> or <i>null</i> if no unit is present.
-     * @see #setUnit
+     * TODO: deprecate this in favour of using the unit list, as we are
+     * unnecessarily encoding the assumption that there can only be one unit.
      */
     public Unit getUnit() {
-        return unit;
+        return (isEmpty()) ? null : getUnitList().get(0);
     }
 
     /**
-     * Sets a <code>Unit</code> to this <code>ColonyTile</code>.
+     * {@inheritDoc}
+     */
+    @Override
+    public int getUnitCapacity() {
+        return (isColonyCenterTile()) ? 0 : UNIT_CAPACITY;
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public NoAddReason getNoWorkReason() {
+        Tile tile = getWorkTile();
+        NoClaimReason claim;
+
+        return (isColonyCenterTile())
+            ? NoAddReason.COLONY_CENTER
+            : (tile.getOccupyingUnit() != null)
+            ? NoAddReason.OCCUPIED_BY_ENEMY
+            : (!getColony().hasAbility(Ability.PRODUCE_IN_WATER)
+                && !tile.isLand())
+            ? NoAddReason.MISSING_ABILITY
+            : (tile.getOwningSettlement() == getColony())
+            ? NoAddReason.NONE
+            : ((claim = getOwner().canClaimForSettlementReason(tile))
+                == NoClaimReason.NONE)
+            ? NoAddReason.CLAIM_REQUIRED
+            : (claim == NoClaimReason.TERRAIN
+                || claim == NoClaimReason.RUMOUR
+                || claim == NoClaimReason.WATER)
+            ? NoAddReason.MISSING_ABILITY
+            : (claim == NoClaimReason.SETTLEMENT)
+            ? ((tile.getSettlement().getOwner() == getOwner())
+                ? NoAddReason.ANOTHER_COLONY
+                : NoAddReason.OWNED_BY_ENEMY)
+            : (claim == NoClaimReason.WORKED)
+            ? NoAddReason.ANOTHER_COLONY
+            : (claim == NoClaimReason.EUROPEANS)
+            ? NoAddReason.OWNED_BY_ENEMY
+            : (claim == NoClaimReason.NATIVES)
+            ? NoAddReason.CLAIM_REQUIRED
+            : NoAddReason.WRONG_TYPE;
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public NoAddReason getNoAddReason(Locatable locatable) {
+        NoAddReason reason = getNoWorkReason();
+        return (reason != NoAddReason.NONE) ? reason
+            : super.getNoAddReason(locatable);
+    }
+
+    /**
+     * Add the specified locatable to this colony tile.
      *
-     * @param unit The <code>Unit</code>.
-     * @see #getUnit
+     * @param locatable The <code>Locatable</code> to add.
      */
-    public void setUnit(Unit unit) {
-        this.unit = unit;
+    public boolean add(final Locatable locatable) {
+        NoAddReason reason = getNoAddReason(locatable);
+        if (reason != NoAddReason.NONE) {
+            throw new IllegalStateException("Can not add " + locatable
+                + " to " + toString() + " because " + reason);
+        }
+        Unit unit = (Unit) locatable;
+        if (contains(unit)) return true;
+
+        if (super.add(unit)) {
+            unit.setState(Unit.UnitState.IN_COLONY);
+
+            // Choose a sensible work type only if none already specified.
+            if (unit.getWorkType() == null) {
+                AbstractGoods goods = workTile.getType().getPrimaryGoods();
+                if (goods == null) {
+                    goods = workTile.getType().getSecondaryGoods();
+                    if (goods == null
+                        && !workTile.getType().getProduction().isEmpty()) {
+                        goods = workTile.getType().getProduction().get(0);
+                    }
+                }
+                if (goods != null) unit.setWorkType(goods.getType());
+            }
+
+            getColony().invalidateCache();
+            return true;
+        }
+        return false;
     }
 
     /**
-     * Gets the amount of Units at this <code>ColonyTile</code>.
-     * @return The amount of Units at this <code>ColonyTile</code>.
+     * Remove the specified locatable from this colony tile.
+     *
+     * @param locatable The <code>Locatable</code> to be removed.
      */
-    public int getUnitCount() {
-        return (getUnit() != null) ? 1 : 0;
+    public boolean remove(final Locatable locatable) {
+        if (!(locatable instanceof Unit)) {
+            throw new IllegalStateException("Not a unit: " + locatable);
+        }
+        Unit unit = (Unit) locatable;
+        if (!contains(unit)) return true;
+
+        if (super.remove(unit)) {
+            unit.setState(Unit.UnitState.ACTIVE);
+            unit.setMovesLeft(0);
+
+            getColony().invalidateCache();
+            return true;
+        }
+        return false;
     }
 
     /**
@@ -263,179 +287,16 @@ public class ColonyTile extends FreeColGameObject
      * within the {@link Colony}.
      */
     public void relocateWorkers() {
-        if (getUnit() != null) {
-            for (WorkLocation wl : getColony().getWorkLocations()) {
-                if (wl != this && wl.canAdd(getUnit())) {
-                    getUnit().setLocation(wl);
+        for (Unit unit : getUnitList()) {
+            for (WorkLocation wl : getColony().getCurrentWorkLocations()) {
+                if (wl != this && wl.canAdd(unit)) {
+                    unit.setLocation(wl);
                     break;
                 }
             }
         }
     }
 
-    /**
-     * Check if this <code>WorkLocation</code> is available to the colony.
-     * Used by canAdd() and the gui to decide whether to draw a border
-     * on this tile in the colony panel.
-     *
-     * @return True if the location can be worked.
-     */
-    public boolean canBeWorked() {
-        Player player = getOwner();
-
-        // Not workable if there is a settlement, hostile occupation,
-        // or can not be claimed.
-        Tile tile = getWorkTile();
-        if (tile.getSettlement() != null
-            || tile.getOccupyingUnit() != null
-            || (tile.getOwningSettlement() != colony
-                && !player.canClaimForSettlement(tile))
-            || (!tile.isLand()
-                && !colony.hasAbility("model.ability.produceInWater"))) {
-            return false;
-        }
-        return true;
-    }
-
-
-    /**
-     * Checks if the specified <code>Locatable</code> may be added to
-     * this <code>WorkLocation</code>.
-     *
-     * @param locatable the <code>Locatable</code>.
-     * @return <code>true</code> if the <code>Unit</code> may be added
-     *         and <code>false</code> otherwise.
-     */
-    public boolean canAdd(Locatable locatable) {
-        if (!canBeWorked()) {
-            return false;
-        }
-        if (!(locatable instanceof Unit)) {
-            return false;
-        }
-        Unit unit = (Unit) locatable;
-        if (!unit.getType().hasSkill()) {
-            return false;
-        }
-        return getUnit() == null || unit == getUnit();
-    }
-
-
-    /**
-     * Add the specified locatable to this colony tile.
-     *
-     * @param locatable The <code>Locatable</code> to add.
-     */
-    public void add(final Locatable locatable) {
-        if (isColonyCenterTile()) {
-            throw new IllegalStateException("Can not add to colony center: "
-                                            + toString());
-        } else if (unit != null) {
-            throw new IllegalStateException("Can not add to occupied tile: "
-                                            + toString());
-        } else if (getWorkTile().getOwningSettlement() != getColony()) {
-            throw new IllegalStateException("Can not add to tile owned by another colony: "
-                                            + toString());
-        } else if (!canAdd(locatable)) {
-            throw new IllegalStateException("Can not add " + locatable
-                                            + " to " + toString());
-        }
-
-        final Unit unit = (Unit) locatable;
-        setUnit(unit);
-        unit.setState(Unit.UnitState.IN_COLONY);
-        getColony().invalidateCache();
-
-        // Choose a sensible work type only if none already specified.
-        if (unit.getWorkType() == null) {
-            AbstractGoods goods = workTile.getType().getPrimaryGoods();
-            if (goods == null) {
-                goods = workTile.getType().getSecondaryGoods();
-                if (goods == null
-                    && !workTile.getType().getProduction().isEmpty()) {
-                    goods = workTile.getType().getProduction().get(0);
-                }
-            }
-            if (goods != null) unit.setWorkType(goods.getType());
-        }
-
-        Unit teacher = unit.getTeacher();
-        if (teacher == null
-            && (teacher = getColony().findTeacher(unit)) != null) {
-            unit.setTeacher(teacher);
-            teacher.setStudent(unit);
-        }
-    }
-
-    /**
-     * Remove the specified locatable from this colony tile.
-     *
-     * @param locatable The <code>Locatable</code> to be removed.
-     */
-    public void remove(final Locatable locatable) {
-        if (getUnit() == null || !getUnit().equals(locatable)) {
-            return;
-        }
-
-        final Unit unit = getUnit();
-        setUnit(null);
-        unit.setMovesLeft(0);
-        unit.setState(Unit.UnitState.ACTIVE);
-        getColony().invalidateCache();
-
-        Unit teacher = unit.getTeacher();
-        if (teacher != null) {
-            teacher.setStudent(null);
-            unit.setTeacher(null);
-        }
-    }
-
-
-    public List<Unit> getUnitList() {
-        if(getUnit() == null) {
-            return new ArrayList<Unit>();
-        } else {
-            return Collections.singletonList(getUnit());
-        }
-    }
-
-    public Iterator <Unit> getUnitIterator() {
-        return getUnitList().iterator();
-    }
-
-
-    /**
-     * Checks if this <code>ColonyTile</code> contains the given <code>Locatable</code>.
-     *
-     * @param locatable The <code>Locatable</code>.
-     * @return The result.
-     */
-    public boolean contains(Locatable locatable) {
-        return (locatable == unit) ? true:false;
-    }
-
-
-    /**
-     * Gets the <code>Unit</code> currently working on this <code>ColonyTile</code>.
-     *
-     * @return The <code>Unit</code> or <i>null</i> if no unit is present.
-     * @see #setUnit
-     */
-    public Unit getFirstUnit() {
-        return getUnit();
-    }
-
-
-    /**
-     * Gets the <code>Unit</code> currently working on this <code>ColonyTile</code>.
-     *
-     * @return The <code>Unit</code> or <i>null</i> if no unit is present.
-     * @see #setUnit
-     */
-    public Unit getLastUnit() {
-        return getUnit();
-    }
-
     /**
      * Returns the unit who is occupying the tile
      * @return the unit who is occupying the tile
@@ -534,13 +395,41 @@ public class ColonyTile extends FreeColGameObject
             if (secondaryProduction != null) {
                 result.add(secondaryProduction);
             }
-        } else if (getUnit() != null) {
-            GoodsType goodsType = getUnit().getWorkType();
-            result.add(new AbstractGoods(goodsType, getProductionOf(getUnit(), goodsType)));
+        } else {
+            for (Unit unit : getUnitList()) {
+                GoodsType goodsType = unit.getWorkType();
+                result.add(new AbstractGoods(goodsType, getProductionOf(unit,
+                            goodsType)));
+            }
         }
         return result;
     }
 
+    /**
+     * Gets the production modifiers for the given type of goods and unit.
+     *
+     * @param goodsType The <code>GoodsType</code> to produce.
+     * @param unitType The <code>unitType</code> to produce them.
+     * @return A set of the applicable modifiers.
+     */
+    public Set<Modifier> getProductionModifiers(GoodsType goodsType, UnitType unitType) {
+        if (goodsType == null) {
+            throw new IllegalArgumentException("GoodsType must not be 'null'.");
+        }
+        Set<Modifier> result = new HashSet<Modifier>();
+        for (Unit unit : getUnitList()) {
+            if (isColonyCenterTile()
+                && (workTile.getType().isPrimaryGoodsType(goodsType)
+                    || workTile.getType().isSecondaryGoodsType(goodsType))) {
+                result.addAll(workTile.getProductionBonus(goodsType, null));
+                result.addAll(getColony().getFeatureContainer().getModifierSet(goodsType.getId()));
+            } else if (goodsType.equals(unit.getWorkType())) {
+                result.addAll(workTile.getProductionBonus(goodsType, unitType));
+                result.addAll(unit.getModifierSet(goodsType.getId()));
+            }
+        }
+        return result;
+    }
 
     /**
      * Returns the production of the given type of goods.
@@ -549,52 +438,26 @@ public class ColonyTile extends FreeColGameObject
      * @return an <code>int</code> value
      */
     public int getProductionOf(GoodsType goodsType) {
-        if (isColonyCenterTile()) {
-            if (workTile.getType().getPrimaryGoods() != null
-                && workTile.getType().getPrimaryGoods().getType() == goodsType) {
-                return getPrimaryProduction().getAmount();
-            } else if (workTile.getType().getSecondaryGoods() != null
-                       && workTile.getType().getSecondaryGoods().getType() == goodsType) {
-                return getSecondaryProduction().getAmount();
-            } else {
-                return 0;
-            }
-        } else if (goodsType == null) {
-            throw new IllegalArgumentException("GoodsType must not be 'null'.");
-        } else if (getUnit() == null) {
-            return 0;
-        } else if (goodsType.equals(getUnit().getWorkType())) {
-            return getProductionOf(getUnit(), goodsType);
-        } else {
-            return 0;
-        }
-    }
-
-    /**
-     * Returns the production of the given type of goods.
-     *
-     * @param goodsType a <code>GoodsType</code> value
-     * @param unitType a <code>unitType</code> value
-     * @return an <code>int</code> value
-     */
-    public Set<Modifier> getProductionModifiers(GoodsType goodsType, UnitType unitType) {
         if (goodsType == null) {
             throw new IllegalArgumentException("GoodsType must not be 'null'.");
-        } else {
-            Set<Modifier> result = new HashSet<Modifier>();
-            if (getUnit() == null) {
-                if (isColonyCenterTile() &&
-                    (workTile.getType().isPrimaryGoodsType(goodsType)
-                     || workTile.getType().isSecondaryGoodsType(goodsType))) {
-                    result.addAll(workTile.getProductionBonus(goodsType, null));
-                    result.addAll(getColony().getFeatureContainer().getModifierSet(goodsType.getId()));
-                }
-            } else if (goodsType.equals(getUnit().getWorkType())) {
-                result.addAll(workTile.getProductionBonus(goodsType, unitType));
-                result.addAll(getUnit().getModifierSet(goodsType.getId()));
-            }
-            return result;
         }
+        if (isColonyCenterTile()) {
+            TileType type = workTile.getType();
+            return (type.getPrimaryGoods() != null
+                && type.getPrimaryGoods().getType() == goodsType)
+                ? getPrimaryProduction().getAmount()
+                : (type.getSecondaryGoods() != null
+                    && type.getSecondaryGoods().getType() == goodsType)
+                ? getSecondaryProduction().getAmount()
+                : 0;
+        }
+        int production = 0;
+        for (Unit unit : getUnitList()) {
+            if (goodsType.equals(unit.getWorkType())) {
+                production += getProductionOf(unit, goodsType);
+            }
+        }
+        return production;
     }
 
     /**
@@ -608,11 +471,11 @@ public class ColonyTile extends FreeColGameObject
     public int getProductionOf(Unit unit, GoodsType goodsType) {
         if (unit == null) {
             throw new IllegalArgumentException("Unit must not be 'null'.");
-        } else if (workTile.isLand() || colony.hasAbility("model.ability.produceInWater")) {
+        } else if (workTile.isLand() || getColony().hasAbility(Ability.PRODUCE_IN_WATER)) {
             Set<Modifier> modifiers = workTile.getProductionBonus(goodsType, unit.getType());
             if (FeatureContainer.applyModifierSet(0f, getGame().getTurn(), modifiers) > 0) {
                 modifiers.addAll(unit.getModifierSet(goodsType.getId()));
-                modifiers.add(colony.getProductionModifier(goodsType));
+                modifiers.add(getColony().getProductionModifier(goodsType));
                 modifiers.addAll(getColony().getModifierSet(goodsType.getId()));
                 List<Modifier> modifierList = new ArrayList<Modifier>(modifiers);
                 Collections.sort(modifierList);
@@ -626,24 +489,52 @@ public class ColonyTile extends FreeColGameObject
     }
 
     /**
-     * Removes all references to this object.
+     * Gets the potential production of a given goods type from using
+     * a unit of a given type in this building.
      *
-     * @return A list of disposed objects.
+     * @param unitType The optional <code>UnitType</code> to check.
+     * @param goodsType The <code>GoodsType</code> to check.
+     * @return The amount of goods potentially produced.
      */
-    public List<FreeColGameObject> disposeList() {
-        List<FreeColGameObject> objects = new ArrayList<FreeColGameObject>();
-        if (unit != null) {
-            objects.addAll(unit.disposeList());
+    public int getPotentialProduction(UnitType unitType, GoodsType goodsType) {
+        int production = 0;
+        if (isColonyCenterTile()) {
+            TileType type = workTile.getType();
+            return (type.getPrimaryGoods() != null
+                && type.getPrimaryGoods().getType() == goodsType)
+                ? getPrimaryProduction().getAmount()
+                : (type.getSecondaryGoods() != null
+                    && type.getSecondaryGoods().getType() == goodsType)
+                ? getSecondaryProduction().getAmount()
+                : 0;
         }
-        objects.addAll(super.disposeList());
-        return objects;
+        if (workTile.isLand()
+            || getColony().hasAbility(Ability.PRODUCE_IN_WATER)) {
+            Set<Modifier> modifiers = workTile.getProductionBonus(goodsType,
+                                                                  unitType);
+            if (FeatureContainer.applyModifierSet(0f, getGame().getTurn(),
+                    modifiers) > 0) {
+                if (unitType != null) {
+                    modifiers.addAll(unitType.getModifierSet(goodsType.getId()));
+                }
+                modifiers.add(getColony().getProductionModifier(goodsType));
+                modifiers.addAll(getColony().getModifierSet(goodsType.getId()));
+                List<Modifier> modifierList = new ArrayList<Modifier>(modifiers);
+                Collections.sort(modifierList);
+                production = Math.max((int) FeatureContainer.applyModifiers(0,
+                        getGame().getTurn(), modifierList), 1);
+            }
+        }
+        return production;
     }
 
     /**
-     * Dispose of this ColonyTile.
+     * Colony tiles can not auto-produce except for the center tile.
+     *
+     * @return True if this work location can produce goods without workers.
      */
-    public void dispose() {
-        disposeList();
+    public boolean canAutoProduce() {
+        return isColonyCenterTile();
     }
 
     /**
@@ -668,18 +559,20 @@ public class ColonyTile extends FreeColGameObject
      * @throws XMLStreamException if there are any problems writing
      *      to the stream.
      */
-    protected void toXMLImpl(XMLStreamWriter out, Player player, boolean showAll, boolean toSavedGame) throws XMLStreamException {
-        // Start element:
+    protected void toXMLImpl(XMLStreamWriter out, Player player,
+                             boolean showAll, boolean toSavedGame)
+        throws XMLStreamException {
+        // Start
         out.writeStartElement(getXMLElementTagName());
 
-        // Add attributes:
-        out.writeAttribute("ID", getId());
-        out.writeAttribute("colony", colony.getId());
+        // Attributes
+        super.writeAttributes(out);
         out.writeAttribute("workTile", workTile.getId());
 
-        writeFreeColGameObject(unit, out, player, showAll, toSavedGame);
+        // Children
+        super.writeChildren(out, player, showAll, toSavedGame);
 
-        // End element:
+        // End
         out.writeEndElement();
     }
 
@@ -690,19 +583,12 @@ public class ColonyTile extends FreeColGameObject
      *      during parsing.
      */
     protected void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException {
-        setId(in.getAttributeValue(null, "ID"));
+        super.readAttributes(in);
 
-        colony = getFreeColGameObject(in, "colony", Colony.class);
         workTile = getFreeColGameObject(in, "workTile", Tile.class);
-        colonyCenterTile = (colony.getTile() == workTile);
-
-        unit = null;
-        while (in.nextTag() != XMLStreamConstants.END_ELEMENT) {
-            if (in.getLocalName().equals(Unit.getXMLElementTagName())) {
-                unit = updateFreeColGameObject(in, Unit.class);
-            }
-        }
+        colonyCenterTile = (getTile() == workTile);
 
+        super.readChildren(in);
     }
 
     /**
