@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -19,11 +19,7 @@
 
 package net.sf.freecol.common.model;
 
-import java.util.ArrayList;
-import java.util.Collections;
 import java.util.HashMap;
-import java.util.Iterator;
-import java.util.List;
 import java.util.Map.Entry;
 import java.util.logging.Logger;
 
@@ -32,7 +28,7 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
-import org.w3c.dom.Element;
+import net.sf.freecol.common.model.Unit.UnitState;
 
 /**
  * Represents Europe in the game. Each <code>Player</code> has it's
@@ -43,7 +39,7 @@ import org.w3c.dom.Element;
  * <p>In Europe, you can recruit, train and purchase new units. You
  * can also equip units, as well as sell and buy goods.
  */
-public class Europe extends FreeColGameObject implements Location, Ownable, Named {
+public class Europe extends UnitLocation implements Ownable, Named {
 
     private static final Logger logger = Logger.getLogger(Europe.class.getName());
 
@@ -51,8 +47,6 @@ public class Europe extends FreeColGameObject implements Location, Ownable, Name
 
     private static final int LOWER_CAP_INITIAL = 80;
 
-    public static final String UNITS_TAG_NAME = "units";
-
     public static final String UNIT_CHANGE = "unitChange";
 
     /**
@@ -78,13 +72,10 @@ public class Europe extends FreeColGameObject implements Location, Ownable, Name
 
     private int recruitLowerCap;
 
-    /**
-     * Contains the units on this location.
-     */
-    private List<Unit> units = Collections.emptyList();
-
     private Player owner;
 
+    private FeatureContainer featureContainer;
+
 
     /**
      * Constructor for ServerEurope.
@@ -121,18 +112,6 @@ public class Europe extends FreeColGameObject implements Location, Ownable, Name
         readFromXML(in);
     }
 
-    /**
-     * Initializes this object from an XML-representation of this object.
-     *
-     * @param game The <code>Game</code> in which this object belong.
-     * @param e An XML-element that will be used to initialize this object.
-     */
-    public Europe(Game game, Element e) {
-        super(game, e);
-
-        readFromXMLElement(e);
-    }
-
     /**
      * Initiates a new <code>Europe</code> with the given ID. The object
      * should later be initialized by calling either
@@ -146,6 +125,52 @@ public class Europe extends FreeColGameObject implements Location, Ownable, Name
         super(game, id);
     }
 
+    /**
+     * Describe <code>hasAbility</code> method here.
+     *
+     * @param id a <code>String</code> value
+     * @return a <code>boolean</code> value
+     */
+    @Override
+    public boolean hasAbility(String id) {
+        return featureContainer.hasAbility(id);
+    }
+
+    /**
+     * Returns the FeatureContainer.
+     *
+     * @return a <code>FeatureContainer</code> value
+     */
+    public FeatureContainer getFeatureContainer() {
+        return featureContainer;
+    }
+
+    /**
+     * Sets the FeatureContainer.
+     *
+     * @param container a <code>FeatureContainer</code> value
+     */
+    protected void setFeatureContainer(FeatureContainer container) {
+        featureContainer = container;
+    }
+
+    /**
+     * Checks if there is a useable carrier unit with a specified
+     * minimum amount of space available docked in this European port.
+     *
+     * @param space The amount of space to require.
+     * @return True if there is a suitable unit present.
+     * @see Unit#isCarrier
+     */
+    public boolean hasCarrierWithSpace(int space) {
+        for (Unit u : getUnitList()) {
+            if (u.isCarrier()
+                && !u.isUnderRepair()
+                && u.getSpaceLeft() >= space) return true;
+        }
+        return false;
+    }
+
     /**
      * Return true if this Europe could build at least one item of the
      * given EquipmentType.
@@ -211,85 +236,19 @@ public class Europe extends FreeColGameObject implements Location, Ownable, Name
         }
     }
 
-    /**
-     * Returns <i>null</i>.
-     *
-     * @return <i>null</i>.
-     */
-    public Tile getTile() {
-        return null;
-    }
-
-
-    /**
-     * Europe does not belong to a settlement.
-     *
-     * @return Null.
-     */
-    public Settlement getSettlement() {
-        return null;
-    }
-
-    /**
-     * Europe does not belong to a colony.
-     *
-     * @return Null.
-     */
-    public Colony getColony() {
-        return null;
-    }
-
     /**
      * Adds a <code>Locatable</code> to this Location.
      *
      * @param locatable The <code>Locatable</code> to add to this Location.
      */
-    public void add(Locatable locatable) {
-        if (!(locatable instanceof Unit)) {
-            throw new IllegalArgumentException("Only units can be added to Europe.");
-        } else if (!units.contains(locatable)) {
-            if (units.equals(Collections.emptyList())) {
-                units = new ArrayList<Unit>();
-            }
-            Unit newUnit = (Unit) locatable;
-            units.add(newUnit);
-            if (!newUnit.isBetweenEuropeAndNewWorld()) {
-                newUnit.setState(Unit.UnitState.SENTRY);
-            }
+    public boolean add(Locatable locatable) {
+        boolean result = super.add(locatable);
+        if (result && locatable instanceof Unit) {
+            Unit unit = (Unit) locatable;
+            unit.setState((unit.canCarryUnits()) ? UnitState.ACTIVE
+                : UnitState.SENTRY);
         }
-    }
-
-    /**
-     * Removes a <code>Locatable</code> from this Location.
-     *
-     * @param locatable The <code>Locatable</code> to remove from this
-     *            Location.
-     */
-    public void remove(Locatable locatable) {
-        if (locatable instanceof Unit) {
-            units.remove(locatable);
-        } else {
-            logger.warning("Tried to remove an unrecognized 'Locatable' from a europe.");
-        }
-    }
-
-    /**
-     * Checks if the specified <code>Locatable</code> is at this
-     * <code>Location</code>.
-     *
-     * @param locatable The <code>Locatable</code> to test the presence of.
-     * @return The result.
-     */
-    public boolean contains(Locatable locatable) {
-        if (locatable instanceof Unit) {
-            return units.contains(locatable);
-        }
-
-        return false;
-    }
-
-    public GoodsContainer getGoodsContainer() {
-        return null;
+        return result;
     }
 
     /**
@@ -300,86 +259,10 @@ public class Europe extends FreeColGameObject implements Location, Ownable, Name
      * @return <i>true</i>.
      */
     public boolean canAdd(Locatable locatable) {
-        return true;
-    }
-
-    /**
-     * Gets the amount of Units at this Location.
-     *
-     * @return The amount of Units at this Location.
-     */
-    public int getUnitCount() {
-        return units.size();
-    }
-
-    /**
-     * Gets a <code>List</code> of every <code>Unit</code> directly located
-     * in this <code>Europe</code>. This does not include <code>Unit</code>s
-     * on ships.
-     *
-     * @return The <code>List</code>.
-     */
-    public List<Unit> getUnitList() {
-        return units;
-    }
-
-    /**
-     * Dispose of this <code>Europe</code>.
-     *
-     * @return A list of disposed objects.
-     */
-    public List<FreeColGameObject> disposeList() {
-        List<FreeColGameObject> objects = new ArrayList<FreeColGameObject>();
-        while (units.size() > 0) {
-            objects.addAll(units.remove(0).disposeList());
-        }
-        units = null;
-
-        objects.addAll(super.disposeList());
-        return objects;
-    }
-
-    /**
-     * Removes all references to this object.
-     */
-    public void dispose() {
-        disposeList();
-    }
-
-    /**
-     * Gets an <code>Iterator</code> of every <code>Unit</code> directly
-     * located in this <code>Europe</code>. This does not include
-     * <code>Unit</code>s on ships.
-     *
-     * @return The <code>Iterator</code>.
-     */
-    public Iterator<Unit> getUnitIterator() {
-        return units.iterator();
-    }
-
-    /**
-     * Gets the first <code>Unit</code> in this <code>Europe</code>.
-     *
-     * @return The first <code>Unit</code> in this <code>Europe</code>.
-     */
-    public Unit getFirstUnit() {
-        if (units.isEmpty()) {
-            return null;
+        if (locatable instanceof Goods) {
+            return true;
         } else {
-            return units.get(0);
-        }
-    }
-
-    /**
-     * Gets the last <code>Unit</code> in this <code>Europe</code>.
-     *
-     * @return The last <code>Unit</code> in this <code>Europe</code>.
-     */
-    public Unit getLastUnit() {
-        if (units.isEmpty()) {
-            return null;
-        } else {
-            return units.get(units.size() - 1);
+            return super.canAdd(locatable);
         }
     }
 
@@ -449,16 +332,6 @@ public class Europe extends FreeColGameObject implements Location, Ownable, Name
         return StringTemplate.key(getNameKey());
     }
 
-    /**
-     * Returns the name of this location for a particular player.
-     *
-     * @param player The <code>Player</code> to prepare the name for.
-     * @return The name of this location.
-     */
-    public StringTemplate getLocationNameFor(Player player) {
-        return getLocationName();
-    }
-
     /**
      * Returns the name of the owner's home port.
      *
@@ -468,23 +341,6 @@ public class Europe extends FreeColGameObject implements Location, Ownable, Name
         return getOwner().getEuropeNameKey();
     }
 
-    /**
-     * Returns a suitable name.
-     */
-    public String toString() {
-        return "Europe";
-    }
-
-    private void unitsToXML(XMLStreamWriter out, Player player, boolean showAll, boolean toSavedGame)
-            throws XMLStreamException {
-        if (!units.isEmpty()) {
-            out.writeStartElement(UNITS_TAG_NAME);
-            for (Unit unit : units) {
-                unit.toXML(out, player, showAll, toSavedGame);
-            }
-            out.writeEndElement();
-        }
-    }
 
     /**
      * This method writes an XML-representation of this object to the given
@@ -509,12 +365,13 @@ public class Europe extends FreeColGameObject implements Location, Ownable, Name
      * @throws XMLStreamException if there are any problems writing to the
      *             stream.
      */
-    protected void toXMLImpl(XMLStreamWriter out, Player player, boolean showAll, boolean toSavedGame)
+    protected void toXMLImpl(XMLStreamWriter out, Player player,
+                             boolean showAll, boolean toSavedGame)
             throws XMLStreamException {
         // Start element:
         out.writeStartElement(getXMLElementTagName());
 
-        out.writeAttribute(ID_ATTRIBUTE, getId());
+        super.writeAttributes(out);
         for (int index = 0; index < recruitables.length; index++) {
             if (recruitables[index] != null) {
                 out.writeAttribute("recruit" + index, recruitables[index].getId());
@@ -531,7 +388,7 @@ public class Europe extends FreeColGameObject implements Location, Ownable, Name
             out.writeEndElement();
         }
 
-        unitsToXML(out, player, showAll, toSavedGame);
+        super.writeChildren(out, player, showAll, toSavedGame);
 
         out.writeEndElement();
     }
@@ -542,10 +399,19 @@ public class Europe extends FreeColGameObject implements Location, Ownable, Name
      * @param in The input stream with the XML.
      * @throws XMLStreamException if a problem was encountered during parsing.
      */
-    protected void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException {
+    protected void readFromXMLImpl(XMLStreamReader in)
+        throws XMLStreamException {
         setId(in.getAttributeValue(null, ID_ATTRIBUTE));
 
         Specification spec = getSpecification();
+
+        // @compat 0.10.0
+        if (featureContainer == null) {
+            featureContainer = new FeatureContainer();
+            featureContainer.addAbility(new Ability("model.ability.dressMissionary"));
+        }
+        // end compatibility code
+
         for (int index = 0; index < recruitables.length; index++) {
             String unitTypeId = in.getAttributeValue(null, "recruit" + index);
             if (unitTypeId != null) {
@@ -558,22 +424,33 @@ public class Europe extends FreeColGameObject implements Location, Ownable, Name
         recruitPrice = getAttribute(in, "recruitPrice", RECRUIT_PRICE_INITIAL);
         recruitLowerCap = getAttribute(in, "recruitLowerCap", LOWER_CAP_INITIAL);
 
-        units.clear();
         unitPrices.clear();
-        while (in.nextTag() != XMLStreamConstants.END_ELEMENT) {
-            if (in.getLocalName().equals(UNITS_TAG_NAME)) {
-                units = new ArrayList<Unit>();
-                while (in.nextTag() != XMLStreamConstants.END_ELEMENT) {
-                    if (in.getLocalName().equals(Unit.getXMLElementTagName())) {
-                        units.add(updateFreeColGameObject(in, Unit.class));
+        readChildren(in);
+    }
+
+    protected void readChild(XMLStreamReader in) throws XMLStreamException {
+        if (in.getLocalName().equals(UNITS_TAG_NAME)) {
+            Unit unit;
+            while (in.nextTag() != XMLStreamConstants.END_ELEMENT) {
+                if (in.getLocalName().equals(Unit.getXMLElementTagName())) {
+                    unit = updateFreeColGameObject(in, Unit.class);
+                    // @compat 0.10.1
+                    if (unit.getLocation() == null) {
+                        // sometimes units in a Europe element have a missing
+                        //  location. it should always be this Europe instance.
+                        unit.setLocationNoUpdate(this);
                     }
+                    // end compatibility code
+                    add(unit);
                 }
-            } else if (in.getLocalName().equals("unitPrice")) {
-                String unitTypeId = in.getAttributeValue(null, "unitType");
-                Integer price = new Integer(in.getAttributeValue(null, "price"));
-                unitPrices.put(spec.getUnitType(unitTypeId), price);
-                in.nextTag(); // close "unitPrice" tag
             }
+        } else if (in.getLocalName().equals("unitPrice")) {
+            String unitTypeId = in.getAttributeValue(null, "unitType");
+            Integer price = new Integer(in.getAttributeValue(null, "price"));
+            unitPrices.put(getSpecification().getUnitType(unitTypeId), price);
+            in.nextTag(); // close "unitPrice" tag
+        } else {
+            super.readChild(in);
         }
     }
 
@@ -602,6 +479,13 @@ public class Europe extends FreeColGameObject implements Location, Ownable, Name
         readFromXMLPartialByClass(in, getClass());
     }
 
+    /**
+     * Returns a suitable name.
+     */
+    public String toString() {
+        return "Europe";
+    }
+
     /**
      * Gets the tag name of the root element representing this object.
      *
@@ -610,5 +494,4 @@ public class Europe extends FreeColGameObject implements Location, Ownable, Name
     public static String getXMLElementTagName() {
         return "europe";
     }
-
 }
