@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -36,6 +36,7 @@ import net.sf.freecol.server.model.ServerGame;
 
 import org.w3c.dom.Element;
 
+
 /**
  * The superclass of all game objects in FreeCol.
  */
@@ -43,6 +44,8 @@ abstract public class FreeColGameObject extends FreeColObject {
 
     private static final Logger logger = Logger.getLogger(FreeColGameObject.class.getName());
 
+    public static final String UNITS_TAG_NAME = "units";
+
     private Game game;
     private boolean disposed = false;
     private boolean uninitialized;
@@ -62,13 +65,13 @@ abstract public class FreeColGameObject extends FreeColObject {
     public FreeColGameObject(Game game) {
         this.game = game;
 
-        if (game != null && game instanceof ServerGame) {
+        if (game != null && game instanceof Game) {
             setDefaultId(game);
         } else if (this instanceof Game) {
             setId("0");
         } else {
             logger.warning("Created 'FreeColGameObject' with 'game == null':"
-                           + this);
+                + this.getId());
         }
 
         uninitialized = false;
@@ -224,18 +227,194 @@ abstract public class FreeColGameObject extends FreeColObject {
         return uninitialized;
     }
 
+    /**
+     * Gets the ID's integer part of this object. The age of two
+     * FreeColGameObjects can be compared by comparing their integer
+     * IDs.
+     *
+     * @return The unique ID of this object.
+     */
+    // TODO: remove compatibility code: use established instead
+    public Integer getIntegerID() {
+        String stringPart = getRealXMLElementTagName() + ":";
+        return new Integer(getId().substring(stringPart.length()));
+    }
+
+    /**
+     * Sets the unique ID of this object. When setting a new ID to this object,
+     * it it automatically registered at the corresponding <code>Game</code>
+     * with the new ID.
+     *
+     * @param newID the unique ID of this object,
+     */
+    @Override
+    public final void setId(String newID) {
+        if (game != null && !(this instanceof Game)) {
+            if (!newID.equals(getId())) {
+                if (getId() != null) {
+                    game.removeFreeColGameObject(getId());
+                }
+
+                super.setId(newID);
+                game.setFreeColGameObject(newID, this);
+            }
+        } else {
+            super.setId(newID);
+        }
+    }
+
+    /**
+     * Checks if the given <code>FreeColGameObject</code> equals this object.
+     *
+     * @param o The <code>FreeColGameObject</code> to compare against this object.
+     * @return <i>true</i> if the two <code>FreeColGameObject</code> are equal and <i>false</i> otherwise.
+     */
+    public boolean equals(FreeColGameObject o) {
+        if (o != null) {
+            return Utils.equals(this.getGame(), o.getGame()) && getId().equals(o.getId());
+        } else {
+            return false;
+        }
+    }
+
+    /**
+     * Checks if the given <code>FreeColGameObject</code> equals this object.
+     *
+     * @param o The <code>FreeColGameObject</code> to compare against this object.
+     * @return <i>true</i> if the two <code>FreeColGameObject</code> are equal and <i>false</i> otherwise.
+     */
+    @Override
+    public boolean equals(Object o) {
+        return (o instanceof FreeColGameObject) ? equals((FreeColGameObject) o) : false;
+    }
+
+    @Override
+    public int hashCode() {
+        return getId().hashCode();
+    }
+
+
+    public <T extends FreeColGameObject> T getFreeColGameObject(XMLStreamReader in, String attributeName,
+                                                                Class<T> returnClass) {
+        final String attributeString = in.getAttributeValue(null, attributeName);
+        if (attributeString == null) {
+            return null;
+        } else {
+            T returnValue = returnClass.cast(getGame().getFreeColGameObject(attributeString));
+            try {
+                if (returnValue == null) {
+                    Constructor<T> c = returnClass.getConstructor(Game.class, String.class);
+                    returnValue = returnClass.cast(c.newInstance(getGame(), attributeString));
+                }
+                return returnValue;
+            } catch(Exception e) {
+                logger.warning("Failed to create FreeColGameObject with ID " + attributeString);
+                return null;
+            }
+        }
+    }
+
+    public <T extends FreeColGameObject> T getFreeColGameObject(XMLStreamReader in, String attributeName,
+                                                                Class<T> returnClass, T defaultValue) {
+        final String attributeString = in.getAttributeValue(null, attributeName);
+        if (attributeString != null) {
+            return returnClass.cast(getGame().getFreeColGameObject(attributeString));
+        } else {
+            return defaultValue;
+        }
+    }
+
+    public <T extends FreeColGameObject> T updateFreeColGameObject(XMLStreamReader in, Class<T> returnClass) {
+        String idString = in.getAttributeValue(null, ID_ATTRIBUTE);
+        if (idString == null) {
+            idString = in.getAttributeValue(null, ID_ATTRIBUTE_TAG);
+        }
+        if (idString == null) {
+            return null;
+        }
+        FreeColGameObject fcgo = getGame().getFreeColGameObject(idString);
+        T returnValue = (fcgo == null) ? null : returnClass.cast(fcgo);
+        try {
+            if (returnValue == null) {
+                Constructor<T> c = returnClass.getConstructor(Game.class, XMLStreamReader.class);
+                returnValue = returnClass.cast(c.newInstance(getGame(), in));
+            } else {
+                returnValue.readFromXML(in);
+            }
+            return returnValue;
+        } catch (Exception e) {
+            logger.warning("Failed to update FreeColGameObject with ID "
+                           + idString);
+            e.printStackTrace();
+            return null;
+        }
+    }
+
+    protected Location newLocation(String locationString) {
+        Location destination = null;
+        if (locationString != null) {
+            destination = (Location) game.getFreeColGameObject(locationString);
+            if (destination == null) {
+                String XMLElementTag = locationString.substring(0, locationString.indexOf(':'));
+                if (XMLElementTag.equals(Tile.getXMLElementTagName())) {
+                    return new Tile(game, locationString);
+                } else if (XMLElementTag.equals(ColonyTile.getXMLElementTagName())) {
+                    return new ColonyTile(game, locationString);
+                } else if (XMLElementTag.equals(Colony.getXMLElementTagName())) {
+                    return new Colony(game, locationString);
+                } else if (XMLElementTag.equals(IndianSettlement.getXMLElementTagName())) {
+                    return new IndianSettlement(game, locationString);
+                } else if (XMLElementTag.equals(Europe.getXMLElementTagName())) {
+                    return new Europe(game, locationString);
+                } else if (XMLElementTag.equals(Building.getXMLElementTagName())) {
+                    return new Building(game, locationString);
+                } else if (XMLElementTag.equals(Unit.getXMLElementTagName())) {
+                    return new Unit(game, locationString);
+                } else if (XMLElementTag.equals(HighSeas.getXMLElementTagName())) {
+                    return new HighSeas(game, locationString);
+                } else if (XMLElementTag.equals(Map.getXMLElementTagName())) {
+                    return new Map(game, locationString);
+                } else if (XMLElementTag.equals("newWorld")) {
+                    // do nothing
+                } else {
+                    logger.warning("Unknown type of Location: " + locationString);
+                    return new Tile(game, locationString);
+                }
+            }
+        }
+        return destination;
+    }
+
     /**
      * This method writes an XML-representation of this object to
-     * the given stream for the purpose of storing this object
-     * as a part of a saved game.
+     * the given stream.
+     *
+     * <br><br>
+     *
+     * Only attributes visible to the given <code>Player</code> will
+     * be added to that representation if <code>showAll</code> is
+     * set to <code>false</code>.
      *
      * @param out The target stream.
+     * @param player The <code>Player</code> this XML-representation
+     *      should be made for, or <code>null</code> if
+     *      <code>showAll == true</code>.
+     * @param showAll Only attributes visible to <code>player</code>
+     *      will be added to the representation if <code>showAll</code>
+     *      is set to <i>false</i>.
+     * @param toSavedGame If <code>true</code> then information that
+     *      is only needed when saving a game is added.
      * @throws XMLStreamException if there are any problems writing
      *      to the stream.
-     * @see #toXML(XMLStreamWriter, Player, boolean, boolean)
      */
-    public void toSavedXML(XMLStreamWriter out) throws XMLStreamException {
-        toXML(out, null, true, true);
+    @Override
+    public final void toXML(XMLStreamWriter out, Player player,
+                            boolean showAll, boolean toSavedGame)
+        throws XMLStreamException {
+        if (toSavedGame && !showAll) {
+            throw new IllegalArgumentException("'showAll' should be true when saving a game.");
+        }
+        toXMLImpl(out, player, showAll, toSavedGame);
     }
 
     /**
@@ -271,40 +450,9 @@ abstract public class FreeColGameObject extends FreeColObject {
      * @throws XMLStreamException if there are any problems writing
      *      to the stream.
      */
-    abstract protected void toXMLImpl(XMLStreamWriter out, Player player, boolean showAll,
-                                      boolean toSavedGame) throws XMLStreamException;
-
-
-    /**
-     * This method writes an XML-representation of this object to
-     * the given stream.
-     *
-     * <br><br>
-     *
-     * Only attributes visible to the given <code>Player</code> will
-     * be added to that representation if <code>showAll</code> is
-     * set to <code>false</code>.
-     *
-     * @param out The target stream.
-     * @param player The <code>Player</code> this XML-representation
-     *      should be made for, or <code>null</code> if
-     *      <code>showAll == true</code>.
-     * @param showAll Only attributes visible to <code>player</code>
-     *      will be added to the representation if <code>showAll</code>
-     *      is set to <i>false</i>.
-     * @param toSavedGame If <code>true</code> then information that
-     *      is only needed when saving a game is added.
-     * @throws XMLStreamException if there are any problems writing
-     *      to the stream.
-     */
-    @Override
-    public final void toXML(XMLStreamWriter out, Player player, boolean showAll,
-                            boolean toSavedGame) throws XMLStreamException {
-        if (toSavedGame && !showAll) {
-            throw new IllegalArgumentException("'showAll' should be true when saving a game.");
-        }
-        toXMLImpl(out, player, showAll, toSavedGame);
-    }
+    abstract protected void toXMLImpl(XMLStreamWriter out, Player player,
+                                      boolean showAll, boolean toSavedGame)
+        throws XMLStreamException;
 
     /**
      * Initialize this object from an XML-representation of this object.
@@ -318,19 +466,6 @@ abstract public class FreeColGameObject extends FreeColObject {
         super.readFromXML(in);
     }
 
-    /**
-     * Gets the ID's integer part of this object. The age of two
-     * FreeColGameObjects can be compared by comparing their integer
-     * IDs.
-     *
-     * @return The unique ID of this object.
-     */
-    // TODO: remove compatibility code: use established instead
-    public Integer getIntegerID() {
-        String stringPart = getRealXMLElementTagName() + ":";
-        return new Integer(getId().substring(stringPart.length()));
-    }
-
     private String getRealXMLElementTagName() {
         String tagName = "";
         try {
@@ -341,133 +476,6 @@ abstract public class FreeColGameObject extends FreeColObject {
     }
     // end TODO
 
-    /**
-     * Sets the unique ID of this object. When setting a new ID to this object,
-     * it it automatically registered at the corresponding <code>Game</code>
-     * with the new ID.
-     *
-     * @param newID the unique ID of this object,
-     */
-    @Override
-    public final void setId(String newID) {
-        if (game != null && !(this instanceof Game)) {
-            if (!newID.equals(getId())) {
-                if (getId() != null) {
-                    game.removeFreeColGameObject(getId());
-                }
-
-                super.setId(newID);
-                game.setFreeColGameObject(newID, this);
-            }
-        } else {
-            super.setId(newID);
-        }
-    }
-
-
-    /**
-     * Checks if this object has the specified ID.
-     *
-     * @param id The ID to check against.
-     * @return <i>true</i> if the specified ID match the ID of this object and
-     *         <i>false</i> otherwise.
-     */
-    public boolean hasID(String id) {
-        return getId().equals(id);
-    }
-
-    /**
-     * Checks if the given <code>FreeColGameObject</code> equals this object.
-     *
-     * @param o The <code>FreeColGameObject</code> to compare against this object.
-     * @return <i>true</i> if the two <code>FreeColGameObject</code> are equal and <i>false</i> otherwise.
-     */
-    public boolean equals(FreeColGameObject o) {
-        if (o != null) {
-            return Utils.equals(this.getGame(), o.getGame()) && getId().equals(o.getId());
-        } else {
-            return false;
-        }
-    }
-
-    /**
-     * Checks if the given <code>FreeColGameObject</code> equals this object.
-     *
-     * @param o The <code>FreeColGameObject</code> to compare against this object.
-     * @return <i>true</i> if the two <code>FreeColGameObject</code> are equal and <i>false</i> otherwise.
-     */
-    @Override
-    public boolean equals(Object o) {
-        return (o instanceof FreeColGameObject) ? equals((FreeColGameObject) o) : false;
-    }
-
-    @Override
-    public int hashCode() {
-        return getId().hashCode();
-    }
-
-
-    /**
-     * Returns a string representation of the object.
-     * @return The <code>String</code>
-     */
-    @Override
-    public String toString() {
-        return getClass().getName() + ": " + getId() + " (super's hash code: " +
-            Integer.toHexString(super.hashCode()) + ")";
-    }
-
-    public <T extends FreeColGameObject> T getFreeColGameObject(XMLStreamReader in, String attributeName,
-                                                                Class<T> returnClass) {
-        final String attributeString = in.getAttributeValue(null, attributeName);
-        if (attributeString == null) {
-            return null;
-        } else {
-            T returnValue = returnClass.cast(getGame().getFreeColGameObject(attributeString));
-            try {
-                if (returnValue == null) {
-                    Constructor<T> c = returnClass.getConstructor(Game.class, String.class);
-                    returnValue = returnClass.cast(c.newInstance(getGame(), attributeString));
-                }
-                return returnValue;
-            } catch(Exception e) {
-                logger.warning("Failed to create FreeColGameObject with ID " + attributeString);
-                return null;
-            }
-        }
-    }
-
-    public <T extends FreeColGameObject> T getFreeColGameObject(XMLStreamReader in, String attributeName,
-                                                                Class<T> returnClass, T defaultValue) {
-        final String attributeString = in.getAttributeValue(null, attributeName);
-        if (attributeString != null) {
-            return returnClass.cast(getGame().getFreeColGameObject(attributeString));
-        } else {
-            return defaultValue;
-        }
-    }
-
-    public <T extends FreeColGameObject> T updateFreeColGameObject(XMLStreamReader in, Class<T> returnClass) {
-        final String idString = in.getAttributeValue(null, ID_ATTRIBUTE);
-        if (idString == null) return null;
-        FreeColGameObject fcgo = getGame().getFreeColGameObject(idString);
-        T returnValue = (fcgo == null) ? null : returnClass.cast(fcgo);
-        try {
-            if (returnValue == null) {
-                Constructor<T> c = returnClass.getConstructor(Game.class, XMLStreamReader.class);
-                returnValue = returnClass.cast(c.newInstance(getGame(), in));
-            } else {
-                returnValue.readFromXML(in);
-            }
-            return returnValue;
-        } catch (Exception e) {
-            logger.warning("Failed to update FreeColGameObject with ID "
-                           + idString);
-            e.printStackTrace();
-            return null;
-        }
-    }
-
     /**
      * Common routine for FreeColGameObject descendants to write an
      * XML-representation of this object to the given stream,
@@ -551,6 +559,18 @@ abstract public class FreeColGameObject extends FreeColObject {
         while (in.nextTag() != XMLStreamConstants.END_ELEMENT);
     }
 
+    /**
+     * Gets a string representation of the object.
+     *
+     * @return A string representation of the object.
+     */
+    @Override
+    public String toString() {
+        return getClass().getName() + ": "
+            + getId() + " (super's hash code: "
+            + Integer.toHexString(super.hashCode()) + ")";
+    }
+
     /**
      * Gets the tag name of the root element representing this object.
      * This method should be overwritten by any sub-class, preferably
@@ -561,5 +581,4 @@ abstract public class FreeColGameObject extends FreeColObject {
     public static String getXMLElementTagName() {
         return "unknown";
     }
-
 }
