@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -34,15 +34,14 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
+import net.sf.freecol.common.model.NationOptions.Advantages;
 import net.sf.freecol.common.model.NationOptions.NationState;
 import net.sf.freecol.common.option.BooleanOption;
 import net.sf.freecol.common.option.IntegerOption;
+import net.sf.freecol.common.option.MapGeneratorOptions;
 import net.sf.freecol.common.option.Option;
 import net.sf.freecol.common.option.OptionGroup;
-import net.sf.freecol.server.generator.MapGeneratorOptions;
-import net.sf.freecol.server.model.ServerModelObject;
 
-import net.sf.freecol.common.model.NationOptions.Advantages;
 
 /**
  * The main component of the game model.
@@ -126,8 +125,6 @@ public class Game extends FreeColGameObject {
      */
     private Specification specification;
 
-
-
     /**
      * This constructor is used by the Server to create a new Game
      * with the given Specification.
@@ -237,17 +234,6 @@ public class Game extends FreeColGameObject {
         return result;
     }
 
-    /**
-     * Check if the clients are trusted or if the server should keep secrets in
-     * order to prevent cheating.
-     *
-     * @return true if clients are to be trusted.
-     */
-    public boolean isClientTrusted() {
-        // Trust the clients in order to prevent certain bugs, fix this later
-        return false;
-    }
-
     /**
      * Returns the owner of this view of the game, or <code>null</code> if
      * this game has all the information. <br>
@@ -341,7 +327,8 @@ public class Game extends FreeColGameObject {
                 currentPlayer = player;
             }
         } else {
-            logger.warning("Tried to add a new player, but the game was already full.");
+            logger.warning("Game already full, but tried to add: "
+                + player.getName());
         }
     }
 
@@ -480,6 +467,11 @@ public class Game extends FreeColGameObject {
      */
     public void setMap(Map map) {
         this.map = map;
+        for (Player player : getPlayers()) {
+            if (player.getHighSeas() != null) {
+                player.getHighSeas().addDestination(map);
+            }
+        }
     }
 
     /**
@@ -622,21 +614,6 @@ public class Game extends FreeColGameObject {
         }
     }
 
-    /**
-     * Collects a list of all the ServerModelObjects in this game.
-     *
-     * @return A list of all the ServerModelObjects in this game.
-     */
-    public List<ServerModelObject> getServerModelObjects() {
-        List<ServerModelObject> objs = new ArrayList<ServerModelObject>();
-        for (WeakReference<FreeColGameObject> wr :freeColGameObjects.values()) {
-            if (wr.get() instanceof ServerModelObject) {
-                objs.add((ServerModelObject) wr.get());
-            }
-        }
-        return objs;
-    }
-
     /**
      * Gets an <code>Iterator</code> of every registered
      * <code>FreeColGameObject</code>.
@@ -937,6 +914,50 @@ public class Game extends FreeColGameObject {
         return this == o;
     }
 
+
+    /**
+     * Gets the statistics of this game.
+     *
+     * @return A <code>Map</code> of the statistics.
+     */
+    public java.util.Map<String, String> getStatistics() {
+        java.util.Map<String, String> stats = new HashMap<String, String>();
+
+        // Memory
+        System.gc();
+        long free = Runtime.getRuntime().freeMemory()/(1024*1024);
+        long total = Runtime.getRuntime().totalMemory()/(1024*1024);
+        long max = Runtime.getRuntime().maxMemory()/(1024*1024);
+        stats.put("freeMemory", Long.toString(free));
+        stats.put("totalMemory", Long.toString(total));
+        stats.put("maxMemory", Long.toString(max));
+
+        // Game objects
+        java.util.Map<String, Long> objStats = new HashMap<String, Long>();
+        long disposed = 0;
+        Iterator<FreeColGameObject> iter = getFreeColGameObjectIterator();
+        while (iter.hasNext()) {
+            FreeColGameObject obj = iter.next();
+            String className = obj.getClass().getSimpleName();
+            if (objStats.containsKey(className)) {
+                Long count = objStats.get(className);
+                count++;
+                objStats.put(className, count);
+            } else {
+                Long count = new Long(1);
+                objStats.put(className, count);
+            }
+            if (obj.isDisposed()) disposed++;
+        }
+        stats.put("disposed", Long.toString(disposed));
+        for (String k : objStats.keySet()) {
+            stats.put(k, Long.toString(objStats.get(k)));
+        }
+
+        return stats;
+    }
+
+
     // comment of Janet: latter is a sick implementation as long as hashCode
     // cannot be rooted back to class Object's functionality
 
@@ -963,7 +984,8 @@ public class Game extends FreeColGameObject {
      * @throws XMLStreamException if there are any problems writing to the
      *             stream.
      */
-    protected void toXMLImpl(XMLStreamWriter out, Player player, boolean showAll, boolean toSavedGame)
+    protected void toXMLImpl(XMLStreamWriter out, Player player,
+                             boolean showAll, boolean toSavedGame)
         throws XMLStreamException {
         // Start element:
         out.writeStartElement(getXMLElementTagName());
@@ -972,7 +994,7 @@ public class Game extends FreeColGameObject {
             throw new IllegalArgumentException("showAll must be set to true when toSavedGame is true.");
         }
 
-        out.writeAttribute("ID", getId());
+        out.writeAttribute(ID_ATTRIBUTE, getId());
         out.writeAttribute("UUID", getUUID().toString());
         out.writeAttribute("turn", Integer.toString(getTurn().getNumber()));
         out.writeAttribute("spanishSuccession", Boolean.toString(spanishSuccession));
@@ -999,17 +1021,18 @@ public class Game extends FreeColGameObject {
             Player p = playerIterator.next();
             p.toXML(out, player, showAll, toSavedGame);
         }
-        writeFreeColGameObject(getUnknownEnemy(), out, player, showAll, toSavedGame);
+        Player enemy = getUnknownEnemy();
+        if (enemy != null) enemy.toXML(out, player, showAll, toSavedGame);
 
         // serialize map
-        writeFreeColGameObject(map, out, player, showAll, toSavedGame);
+        if (map != null) map.toXML(out, player, showAll, toSavedGame);
 
         /* Moved to within player.  Last used in 0.9.x.
         // serialize messages
         playerIterator = getPlayerIterator();
         while (playerIterator.hasNext()) {
             Player p = playerIterator.next();
-            if (this.isClientTrusted() || showAll || p.equals(player)) {
+            if (showAll || p.equals(player)) {
                 for (ModelMessage message : p.getModelMessages()) {
                     message.toXML(out);
                 }
@@ -1026,8 +1049,9 @@ public class Game extends FreeColGameObject {
      * @param in The input stream with the XML.
      */
     @Override
-    protected void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException {
-        setId(in.getAttributeValue(null, "ID"));
+    protected void readFromXMLImpl(XMLStreamReader in)
+        throws XMLStreamException {
+        setId(in.getAttributeValue(null, ID_ATTRIBUTE));
 
         String hs = in.getAttributeValue(null, "UUID");
         if (hs != null) {
@@ -1060,15 +1084,16 @@ public class Game extends FreeColGameObject {
             String tagName = in.getLocalName();
             logger.finest("Found tag " + tagName);
             if (tagName.equals("gameOptions") || tagName.equals("game-options")) {
-                // TODO: remove 0.9.x compatibility code
-                gameOptions = new OptionGroup(in);
+                // @compat 0.9.x
+                gameOptions = new OptionGroup(specification);
+                gameOptions.readFromXML(in);
             } else if (tagName.equals(NationOptions.getXMLElementTagName())) {
                 if (nationOptions == null) {
                     nationOptions = new NationOptions(specification, Advantages.SELECTABLE);
                 }
                 nationOptions.readFromXML(in);
             } else if (tagName.equals(Player.getXMLElementTagName())) {
-                Player player = (Player) getFreeColGameObject(in.getAttributeValue(null, "ID"));
+                Player player = (Player) getFreeColGameObject(in.getAttributeValue(null, ID_ATTRIBUTE));
                 if (player == null) {
                     player = new Player(this, in);
                     if (player.isUnknownEnemy()) {
@@ -1080,16 +1105,14 @@ public class Game extends FreeColGameObject {
                     player.readFromXML(in);
                 }
             } else if (tagName.equals(Map.getXMLElementTagName())) {
-                String mapId = in.getAttributeValue(null, "ID");
+                String mapId = in.getAttributeValue(null, ID_ATTRIBUTE);
                 map = (Map) getFreeColGameObject(mapId);
-                if (map != null) {
-                    map.readFromXML(in);
-                } else {
+                if (map == null) {
                     map = new Map(this, mapId);
-                    map.readFromXML(in);
                 }
+                map.readFromXML(in);
             } else if (tagName.equals(ModelMessage.getXMLElementTagName())) {
-                // 0.9.x save format compatibility.  Remove one day.
+                // @compat 0.9.x
                 ModelMessage m = new ModelMessage();
                 m.readFromXML(in);
                 // When this goes, remove getOwnerId().
@@ -1099,18 +1122,20 @@ public class Game extends FreeColGameObject {
                     player.addModelMessage(m);
                 }
             } else if (tagName.equals("citiesOfCibola")) {
-                // TODO: remove support for old format
+                // @compat 0.9.x
                 citiesOfCibola = readFromListElement("citiesOfCibola", in, String.class);
-            } else if (CIBOLA_TAG.equals(tagName)) {
+            } else if (tagName.equals(CIBOLA_TAG)) {
                 citiesOfCibola.add(in.getAttributeValue(null, ID_ATTRIBUTE_TAG));
                 in.nextTag();
             } else if (OptionGroup.getXMLElementTagName().equals(tagName)
                        || "difficultyLevel".equals(tagName)) {
-                // remove compatibility code after 0.10.0
-                OptionGroup difficultyLevel = new OptionGroup(in);
+                // @compat 0.9.x
+                OptionGroup difficultyLevel = new OptionGroup(specification);
+                difficultyLevel.readFromXML(in);
             } else if (MapGeneratorOptions.getXMLElementTagName().equals(tagName)) {
-                // TODO: remove 0.9.x compatibility code
-                mapGeneratorOptions = new OptionGroup(in);
+                // @compat 0.9.x
+                mapGeneratorOptions = new OptionGroup(specification);
+                mapGeneratorOptions.readFromXML(in);
             } else if (Specification.getXMLElementTagName().equals(tagName)) {
                 Specification spec = new Specification();
                 spec.readFromXMLImpl(in);
@@ -1129,7 +1154,7 @@ public class Game extends FreeColGameObject {
                            "found instead: " + in.getLocalName());
         }
 
-        // TODO: remove compatibility code post 0.10.0
+        // @compat 0.9.x
         if (gameOptions != null) {
             addOldOptions(gameOptions);
         }
@@ -1137,10 +1162,9 @@ public class Game extends FreeColGameObject {
             addOldOptions(mapGeneratorOptions);
         }
         // end compatibility code
-
     }
 
-    // TODO: remove compatibility code post 0.10.0
+    // @compat 0.9.x
     private void addOldOptions(OptionGroup group) {
         Iterator<Option> iterator = group.iterator();
         while (iterator.hasNext()) {
@@ -1166,7 +1190,6 @@ public class Game extends FreeColGameObject {
     }
     // end compatibility code
 
-
     /**
      * Partial writer, so that simple updates can be brief.
      *
@@ -1195,10 +1218,9 @@ public class Game extends FreeColGameObject {
     /**
      * Returns the tag name of the root element representing this object.
      *
-     * @return the tag name.
+     * @return "game".
      */
     public static String getXMLElementTagName() {
         return "game";
     }
-
 }
