@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -34,9 +34,9 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
-
 import org.w3c.dom.Element;
 
+
 /**
  * Contains goods and can be used by a {@link Location} to make certain
  * tasks easier.
@@ -165,8 +165,8 @@ public class GoodsContainer extends FreeColGameObject implements Ownable {
      * Adds a <code>Goods</code> to this containter.
      * @param g The Goods to add to this container.
      */
-    public void addGoods(AbstractGoods g) {
-        addGoods(g.getType(), g.getAmount());
+    public boolean addGoods(AbstractGoods g) {
+        return addGoods(g.getType(), g.getAmount());
     }
 
     /**
@@ -174,7 +174,7 @@ public class GoodsContainer extends FreeColGameObject implements Ownable {
      * @param type The type of goods to add.
      * @param amount The type of amount to add.
      */
-    public void addGoods(GoodsType type, int amount) {
+    public boolean addGoods(GoodsType type, int amount) {
         int oldAmount = getGoodsCount(type);
         int newAmount = oldAmount + amount;
 
@@ -186,6 +186,7 @@ public class GoodsContainer extends FreeColGameObject implements Ownable {
         } else {
             storedGoods.put(type, newAmount);
         }
+        return true;
     }
 
     /**
@@ -471,12 +472,13 @@ public class GoodsContainer extends FreeColGameObject implements Ownable {
      * @throws XMLStreamException if there are any problems writing
      *      to the stream.
      */
-    protected void toXMLImpl(XMLStreamWriter out, Player player, boolean showAll, boolean toSavedGame)
+    protected void toXMLImpl(XMLStreamWriter out, Player player,
+                             boolean showAll, boolean toSavedGame)
         throws XMLStreamException {
         // Start element:
         out.writeStartElement(getXMLElementTagName());
 
-        out.writeAttribute("ID", getId());
+        out.writeAttribute(ID_ATTRIBUTE, getId());
         if (showAll || toSavedGame || player == getOwner()) {
             writeStorage(out, STORED_GOODS_TAG, storedGoods);
             writeStorage(out, OLD_STORED_GOODS_TAG, oldStoredGoods);
@@ -503,7 +505,7 @@ public class GoodsContainer extends FreeColGameObject implements Ownable {
      * @param in The input stream with the XML.
      */
     protected void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException {
-        setId(in.getAttributeValue(null, "ID"));
+        setId(in.getAttributeValue(null, ID_ATTRIBUTE));
         storedGoods.clear();
         oldStoredGoods.clear();
         Map<GoodsType, Integer> storage;
@@ -564,6 +566,7 @@ public class GoodsContainer extends FreeColGameObject implements Ownable {
         for (Map.Entry<GoodsType, Integer> entry : storedGoods.entrySet()) {
             sb.append(entry.getKey() + "=" + entry.getValue() + ", ");
         }
+        sb.setLength(sb.length() - 2);
         return sb.toString();
     }
 
