@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -32,11 +32,10 @@ import javax.xml.stream.XMLStreamWriter;
 
 import net.sf.freecol.common.model.Player.PlayerType;
 import net.sf.freecol.common.model.Unit.Role;
+import net.sf.freecol.common.option.UnitListOption;
 import net.sf.freecol.common.util.RandomChoice;
 import net.sf.freecol.common.util.Utils;
 
-import org.w3c.dom.Element;
-
 
 /**
  * This class implements the player's monarch, whose functions prior
@@ -71,6 +70,9 @@ public final class Monarch extends FreeColGameObject implements Named {
     /** Whether a frigate has been provided. */
     private boolean supportSea = false;
 
+    /** Whether displeasure has been incurred. */
+    private boolean displeasure = false;
+
     // Internal variables that do not need serialization.
     /** The space required to transport all land units. */
     private int spaceRequired;
@@ -82,14 +84,17 @@ public final class Monarch extends FreeColGameObject implements Named {
     /** Constants describing monarch actions. */
     public static enum MonarchAction {
         NO_ACTION,
-        RAISE_TAX,
-        LOWER_TAX,
+        RAISE_TAX_ACT,
+        RAISE_TAX_WAR,
+        FORCE_TAX,
+        LOWER_TAX_WAR,
+        LOWER_TAX_OTHER,
         WAIVE_TAX,
         ADD_TO_REF,
         DECLARE_WAR,
         SUPPORT_LAND,
         SUPPORT_SEA,
-        OFFER_MERCENARIES
+        OFFER_MERCENARIES, DISPLEASURE,
     }
 
 
@@ -113,22 +118,22 @@ public final class Monarch extends FreeColGameObject implements Named {
         this.name = name;
 
         Specification spec = getSpecification();
-        int number = spec.getIntegerOption("model.option.refStrength")
-            .getValue() + 3;
-
-        for (UnitType unitType : spec.getUnitTypeList()) {
+        List<AbstractUnit> ref =
+            ((UnitListOption) spec.getOption("model.option.refSize")).getOptionValues();
+        for (AbstractUnit unit : ref) {
+            UnitType unitType = unit.getUnitType(spec);
             if (unitType.hasAbility("model.ability.refUnit")) {
-                if (unitType.hasAbility("model.ability.navalUnit")) {
-                    navalUnits.add(new AbstractUnit(unitType, Role.DEFAULT, number));
-                } else if (unitType.hasAbility("model.ability.canBeEquipped")) {
-                    landUnits.add(new AbstractUnit(unitType, Role.SOLDIER, number));
-                    landUnits.add(new AbstractUnit(unitType, Role.DRAGOON, number));
+                if (unitType.hasAbility(Ability.NAVAL_UNIT)) {
+                    navalUnits.add(unit);
                 } else {
-                    landUnits.add(new AbstractUnit(unitType, Role.DEFAULT, number));
+                    landUnits.add(unit);
                 }
+            } else {
+                logger.warning("Found REF unit lacking ability \"model.ability.refUnit\": "
+                               + unit.toString());
             }
         }
-        updateSpaceAndCapacity();
+        //updateSpaceAndCapacity();
     }
 
     /**
@@ -145,25 +150,11 @@ public final class Monarch extends FreeColGameObject implements Named {
         readFromXML(in);
     }
 
-    /**
-     * Initiates a new <code>Monarch</code> from an <code>Element</code>
-     * and registers this <code>Monarch</code> at the specified game.
-     *
-     * @param game The <code>Game</code> this object belongs to.
-     * @param e An XML-element that will be used to initialize
-     *      this object.
-     */
-    public Monarch(Game game, Element e) {
-        super(game, e);
-        readFromXMLElement(e);
-    }
-
     /**
      * Initiates a new <code>Monarch</code>
      * with the given ID. The object should later be
      * initialized by calling either
-     * {@link #readFromXML(XMLStreamReader)} or
-     * {@link #readFromXMLElement(Element)}.
+     * {@link #readFromXML(XMLStreamReader)}.
      *
      * @param game The <code>Game</code> in which this object belong.
      * @param id The unique identifier for this object.
@@ -207,6 +198,24 @@ public final class Monarch extends FreeColGameObject implements Named {
         this.supportSea = supportSea;
     }
 
+    /**
+     * Gets the displeasure status.
+     *
+     * @return Gets the displeasure status.
+     */
+    public boolean getDispleasure() {
+        return displeasure;
+    }
+
+    /**
+     * Sets the displeasure status.
+     *
+     * @param displeasure The new displeasure status.
+     */
+    public void setDispleasure(boolean displeasure) {
+        this.displeasure = displeasure;
+    }
+
 
     /**
      * Return the name key of this Monarch.
@@ -218,9 +227,9 @@ public final class Monarch extends FreeColGameObject implements Named {
     }
 
     /**
-     * Returns a List of all REF units.
+     * Gets the REF units.
      *
-     * @return a List of all REF units.
+     * @return A list of all REF units.
      */
     public List<AbstractUnit> getREF() {
         List<AbstractUnit> result = new ArrayList<AbstractUnit>(landUnits);
@@ -229,20 +238,20 @@ public final class Monarch extends FreeColGameObject implements Named {
     }
 
     /**
-     * Returns only the naval units.
+     * Gets the REF naval units.
      *
-     * @return the naval units
+     * @return A list of the REF naval units.
      */
-    public List<AbstractUnit> getNavalUnits() {
+    public List<AbstractUnit> getREFNavalUnits() {
         return navalUnits;
     }
 
     /**
-     * Returns only the land units.
+     * Gets the REF land units.
      *
-     * @return the land units
+     * @return A list of the REF land units.
      */
-    public List<AbstractUnit> getLandUnits() {
+    public List<AbstractUnit> getREFLandUnits() {
         return landUnits;
     }
 
@@ -282,9 +291,13 @@ public final class Monarch extends FreeColGameObject implements Named {
         switch (action) {
         case NO_ACTION:
             return true;
-        case RAISE_TAX:
+        case RAISE_TAX_ACT:
+        case RAISE_TAX_WAR:
             return player.getTax() < taxMaximum();
-        case LOWER_TAX:
+        case FORCE_TAX:
+            return false;
+        case LOWER_TAX_WAR:
+        case LOWER_TAX_OTHER:
             return player.getTax() > MINIMUM_TAX_RATE + 10;
         case WAIVE_TAX:
             return true;
@@ -293,9 +306,12 @@ public final class Monarch extends FreeColGameObject implements Named {
         case DECLARE_WAR:
             return !collectPotentialEnemies().isEmpty();
         case SUPPORT_SEA:
-            return player.getAttackedByPrivateers() && !getSupportSea();
+            return player.getAttackedByPrivateers() && !getSupportSea()
+                && !getDispleasure();
         case SUPPORT_LAND: case OFFER_MERCENARIES:
-            return player.isAtWar();
+            return player.isAtWar() && !getDispleasure();
+        case DISPLEASURE:
+            return false;
         default:
             throw new IllegalArgumentException("Bogus monarch action: "
                                                + action);
@@ -323,53 +339,31 @@ public final class Monarch extends FreeColGameObject implements Named {
             return choices;
         }
 
-        if (actionIsValid(MonarchAction.NO_ACTION)) {
-            // The more time has passed, the less likely the monarch
-            // will do nothing.
-            choices.add(new RandomChoice<MonarchAction>(MonarchAction.NO_ACTION,
-                                                        Math.max(200 - turn, 100)));
-        }
-
-        if (actionIsValid(MonarchAction.RAISE_TAX)) {
-            choices.add(new RandomChoice<MonarchAction>(MonarchAction.RAISE_TAX,
-                                                        10 + dx));
-        }
-
-        if (actionIsValid(MonarchAction.LOWER_TAX)) {
-            choices.add(new RandomChoice<MonarchAction>(MonarchAction.LOWER_TAX,
-                                                        10 - dx));
-        }
-
-        if (actionIsValid(MonarchAction.ADD_TO_REF)) {
-            choices.add(new RandomChoice<MonarchAction>(MonarchAction.ADD_TO_REF,
-                                                        10 + dx));
-        }
-
-        if (actionIsValid(MonarchAction.DECLARE_WAR)) {
-            choices.add(new RandomChoice<MonarchAction>(MonarchAction.DECLARE_WAR,
-                                                        5 + dx));
-        }
-
-        if (actionIsValid(MonarchAction.SUPPORT_LAND)) {
-            if (!player.checkGold(MINIMUM_PRICE)) {
-                if (dx < 3) {
-                    choices.add(new RandomChoice<MonarchAction>(MonarchAction.SUPPORT_LAND,
-                                                                3 - dx));
-                }
-            } else {
-                choices.add(new RandomChoice<MonarchAction>(MonarchAction.OFFER_MERCENARIES,
-                                                            6 - dx));
-            }
-        }
-
-        if (actionIsValid(MonarchAction.SUPPORT_SEA)) {
-            choices.add(new RandomChoice<MonarchAction>(MonarchAction.SUPPORT_SEA,
-                                                        6 - dx));
+        // The more time has passed, the less likely the monarch will
+        // do nothing.
+        addIfValid(choices, MonarchAction.NO_ACTION, Math.max(200 - turn, 100));
+        addIfValid(choices, MonarchAction.RAISE_TAX_ACT, 5 + dx);
+        addIfValid(choices, MonarchAction.RAISE_TAX_WAR, 5 + dx);
+        addIfValid(choices, MonarchAction.LOWER_TAX_WAR, 5 - dx);
+        addIfValid(choices, MonarchAction.LOWER_TAX_OTHER, 5 - dx);
+        addIfValid(choices, MonarchAction.ADD_TO_REF, 10 + dx);
+        addIfValid(choices, MonarchAction.DECLARE_WAR, 5 + dx);
+        if (player.checkGold(MINIMUM_PRICE)) {
+            addIfValid(choices, MonarchAction.OFFER_MERCENARIES, 6 - dx);
+        } else if (dx < 3) {
+            addIfValid(choices, MonarchAction.SUPPORT_LAND, 3 - dx);
         }
+        addIfValid(choices, MonarchAction.SUPPORT_SEA, 6 - dx);
 
         return choices;
     }
 
+    private void addIfValid(List<RandomChoice<MonarchAction>> choices,
+                            MonarchAction action, int weight) {
+        if (actionIsValid(action)) {
+            choices.add(new RandomChoice<MonarchAction>(action, weight));
+        }
+    }
 
     /**
      * Calculates a tax raise.
@@ -383,8 +377,8 @@ public final class Monarch extends FreeColGameObject implements Named {
         int turn = getGame().getTurn().getNumber();
         int oldTax = player.getTax();
         int adjust = Math.max(1, (6 - taxAdjustment) * 10); // 20-60
-        adjust = Utils.randomInt(logger, "Tax rise", random, 5 + turn/adjust)
-            + 1;
+        adjust = 1 + Utils.randomInt(logger, "Tax rise", random,
+                                     5 + turn/adjust);
         return Math.min(oldTax + adjust, taxMaximum());
     }
 
@@ -399,7 +393,7 @@ public final class Monarch extends FreeColGameObject implements Named {
             .getIntegerOption("model.option.taxAdjustment").getValue();
         int oldTax = player.getTax();
         int adjust = Math.max(1, 10 - taxAdjustment); // 5-10
-        adjust = Utils.randomInt(logger, "Tax reduction", random, adjust) + 1;
+        adjust = 1 + Utils.randomInt(logger, "Tax reduction", random, adjust);
         return Math.max(oldTax - adjust, Monarch.MINIMUM_TAX_RATE);
     }
 
@@ -409,8 +403,8 @@ public final class Monarch extends FreeColGameObject implements Named {
      * @param random The <code>Random</code> number source to use.
      * @return An addition to the Royal Expeditionary Force.
      */
-    public List<AbstractUnit> chooseForREF(Random random) {
-        ArrayList<AbstractUnit> result = new ArrayList<AbstractUnit>();
+    public AbstractUnit chooseForREF(Random random) {
+        AbstractUnit result = null;
         // Preserve some extra naval capacity so that not all the REF
         // navy is completely loaded
         logger.info("Add to REF: capacity=" + capacity
@@ -419,15 +413,13 @@ public final class Monarch extends FreeColGameObject implements Named {
                     + " unit");
         // TODO: magic number 2.5 * Manowar-capacity = 15
         if (capacity < spaceRequired + 15) {
-            AbstractUnit unit = Utils.getRandomMember(logger, "Choose naval",
-                                                      navalUnits, random);
-            result.add(new AbstractUnit(unit.getId(), unit.getRole(), 1));
+            result = Utils.getRandomMember(logger, "Choose naval",
+                                           navalUnits, random);
+            result.setNumber(1);
         } else {
-            AbstractUnit unit = Utils.getRandomMember(logger, "Choose land",
-                                                      landUnits, random);
-            result.add(new AbstractUnit(unit.getId(), unit.getRole(),
-                                        Utils.randomInt(logger, "Choose land#",
-                                                        random, 3) + 1));
+            result = Utils.getRandomMember(logger, "Choose land",
+                                           landUnits, random);
+            result.setNumber(Utils.randomInt(logger, "Choose land#", random, 3) + 1);
         }
         return result;
     }
@@ -437,29 +429,27 @@ public final class Monarch extends FreeColGameObject implements Named {
      *
      * @param units The addition to the Royal Expeditionary Force.
      */
-    public void addToREF(List<AbstractUnit> units) {
+    public void addToREF(AbstractUnit units) {
         Specification spec = getSpecification();
-        for (AbstractUnit unitToAdd : units) {
-            UnitType unitType = spec.getUnitType(unitToAdd.getId());
-            int n = unitToAdd.getNumber();
-            if (unitType.hasAbility("model.ability.navalUnit")) {
-                for (AbstractUnit refUnit : navalUnits) {
-                    if (refUnit.getId().equals(unitToAdd.getId())) {
-                        refUnit.setNumber(refUnit.getNumber() + n);
-                        if (unitType.canCarryUnits()) {
-                            capacity += unitType.getSpace() * n;
-                        }
-                        break;
+        UnitType unitType = spec.getUnitType(units.getId());
+        int n = units.getNumber();
+        if (unitType.hasAbility(Ability.NAVAL_UNIT)) {
+            for (AbstractUnit refUnit : navalUnits) {
+                if (refUnit.getId().equals(units.getId())) {
+                    refUnit.setNumber(refUnit.getNumber() + n);
+                    if (unitType.canCarryUnits()) {
+                        capacity += unitType.getSpace() * n;
                     }
+                    break;
                 }
-            } else {
-                for (AbstractUnit refUnit : landUnits) {
-                    if (refUnit.getId().equals(unitToAdd.getId())
-                        && refUnit.getRole().equals(unitToAdd.getRole())) {
-                        refUnit.setNumber(refUnit.getNumber() + n);
-                        spaceRequired += unitType.getSpaceTaken() * n;
-                        break;
-                    }
+            }
+        } else {
+            for (AbstractUnit refUnit : landUnits) {
+                if (refUnit.getId().equals(units.getId())
+                    && refUnit.getRole().equals(units.getRole())) {
+                    refUnit.setNumber(refUnit.getNumber() + n);
+                    spaceRequired += unitType.getSpaceTaken() * n;
+                    break;
                 }
             }
         }
@@ -481,11 +471,11 @@ public final class Monarch extends FreeColGameObject implements Named {
         List<UnitType> mountedTypes = new ArrayList<UnitType>();
         for (UnitType unitType : spec.getUnitTypeList()) {
             if (unitType.hasAbility("model.ability.supportUnit")) {
-                if (unitType.hasAbility("model.ability.navalUnit")) {
+                if (unitType.hasAbility(Ability.NAVAL_UNIT)) {
                     navalTypes.add(unitType);
-                } else if (unitType.hasAbility("model.ability.bombard")) {
+                } else if (unitType.hasAbility(Ability.BOMBARD)) {
                     bombardTypes.add(unitType);
-                } else if (unitType.hasAbility("model.ability.canBeEquipped")) {
+                } else if (unitType.hasAbility(Ability.CAN_BE_EQUIPPED)) {
                     mountedTypes.add(unitType);
                 }
             }
@@ -572,7 +562,7 @@ public final class Monarch extends FreeColGameObject implements Named {
         for (int count = 0; count < limit; count++) {
             unitType = Utils.getRandomMember(logger, "Choose unit", unitTypes,
                                              random);
-            if (unitType.hasAbility("model.ability.canBeEquipped")) {
+            if (unitType.hasAbility(Ability.CAN_BE_EQUIPPED)) {
                 for (int number = 3; number > 0; number--) {
                     au = new AbstractUnit(unitType, Role.DRAGOON, number);
                     int newPrice = player.getPrice(au) * mercPrice / 100;
@@ -607,7 +597,7 @@ public final class Monarch extends FreeColGameObject implements Named {
 
         /* Try to always return something, even if it is not affordable */
         if (mercs.isEmpty() && unitType != null) {
-            Role role = (unitType.hasAbility("model.ability.canBeEquipped"))
+            Role role = (unitType.hasAbility(Ability.CAN_BE_EQUIPPED))
                 ? Role.SOLDIER
                 : Role.DEFAULT;
             mercs.add(new AbstractUnit(unitType, role, 1));
@@ -637,15 +627,17 @@ public final class Monarch extends FreeColGameObject implements Named {
      * @throws XMLStreamException if there are any problems writing
      *      to the stream.
      */
-    protected void toXMLImpl(XMLStreamWriter out, Player player, boolean showAll, boolean toSavedGame)
+    protected void toXMLImpl(XMLStreamWriter out, Player player,
+                             boolean showAll, boolean toSavedGame)
         throws XMLStreamException {
         // Start element:
         out.writeStartElement(getXMLElementTagName());
 
-        out.writeAttribute("ID", getId());
+        out.writeAttribute(ID_ATTRIBUTE, getId());
         out.writeAttribute("player", this.player.getId());
         out.writeAttribute("name", name);
         out.writeAttribute("supportSea", String.valueOf(supportSea));
+        out.writeAttribute("displeasure", String.valueOf(displeasure));
 
         out.writeStartElement("navalUnits");
         for (AbstractUnit unit : navalUnits) {
@@ -668,14 +660,15 @@ public final class Monarch extends FreeColGameObject implements Named {
      * @param in The input stream with the XML.
      */
     protected void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException {
-        setId(in.getAttributeValue(null, "ID"));
+        setId(in.getAttributeValue(null, ID_ATTRIBUTE));
 
         player = (Player) getGame().getFreeColGameObject(in.getAttributeValue(null, "player"));
         if (player == null) {
             player = new Player(getGame(), in.getAttributeValue(null, "player"));
         }
         name = getAttribute(in, "name", player.getNation().getRulerNameKey());
-        supportSea = Boolean.valueOf(in.getAttributeValue(null, "supportSea")).booleanValue();
+        supportSea = getAttribute(in, "supportSea", false);
+        displeasure = getAttribute(in, "displeasure", false);
 
         navalUnits.clear();
         landUnits.clear();
