@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -31,6 +31,15 @@ public class Nation extends FreeColGameObjectType {
 
     public static String UNKNOWN_NATION_ID = "model.nation.unknownEnemy";
 
+    public static final String[] EUROPEAN_NATIONS = new String[] {
+        // the original game's nations
+        "english", "french", "spanish", "dutch",
+        // FreeCol's additions
+        "portuguese", "danish", "swedish", "russian",
+        // other Europeans, used to generate Monarch messages
+        "german", "austrian", "prussian", "turkish"
+    };
+
     /**
      * Describe type here.
      */
@@ -41,41 +50,27 @@ public class Nation extends FreeColGameObjectType {
      */
     private boolean selectable;
 
-    /**
-     * TODO: create audio resource and move all audio resources into
-     * ResourceManager.
-     */
-    private String anthem;
-
     /**
      * Describe refNation here.
      */
     private Nation refNation;
 
+    /**
+     * Describe preferredLatitude here.
+     */
+    private int preferredLatitude = 0;
+
+    /**
+     * Describe startsOnEastCoast here.
+     */
+    private boolean startsOnEastCoast = true;
+
 
     public Nation(String id, Specification specification) {
         super(id, specification);
     }
 
 
-    /**
-     * Get the <code>Anthem</code> value.
-     *
-     * @return a <code>String</code> value
-     */
-    public final String getAnthem() {
-        return anthem;
-    }
-
-    /**
-     * Set the <code>Anthem</code> value.
-     *
-     * @param newAnthem The new Anthem value.
-     */
-    public final void setAnthem(final String newAnthem) {
-        this.anthem = newAnthem;
-    }
-
     /**
      * Get the <code>Type</code> value.
      *
@@ -139,16 +134,41 @@ public class Nation extends FreeColGameObjectType {
         this.selectable = newSelectable;
     }
 
-    public void readAttributes(XMLStreamReader in) throws XMLStreamException {
-        super.readAttributes(in);
-        type = getSpecification().getNationType(in.getAttributeValue(null, "nation-type"));
-        selectable = getAttribute(in, "selectable", false);
-        String refId = getAttribute(in, "ref", null);
-        if (refId != null) {
-            refNation = getSpecification().getNation(refId);
-        }
-        anthem = in.getAttributeValue(null, "anthem");
-   }
+    /**
+     * Get the <code>PreferredLatitude</code> value.
+     *
+     * @return an <code>int</code> value
+     */
+    public final int getPreferredLatitude() {
+        return preferredLatitude;
+    }
+
+    /**
+     * Set the <code>PreferredLatitude</code> value.
+     *
+     * @param newPreferredLatitude The new PreferredLatitude value.
+     */
+    public final void setPreferredLatitude(final int newPreferredLatitude) {
+        this.preferredLatitude = newPreferredLatitude;
+    }
+
+    /**
+     * Get the <code>StartsOnEastCoast</code> value.
+     *
+     * @return a <code>boolean</code> value
+     */
+    public final boolean startsOnEastCoast() {
+        return startsOnEastCoast;
+    }
+
+    /**
+     * Set the <code>StartsOnEastCoast</code> value.
+     *
+     * @param newStartsOnEastCoast The new StartsOnEastCoast value.
+     */
+    public final void setStartsOnEastCoast(final boolean newStartsOnEastCoast) {
+        this.startsOnEastCoast = newStartsOnEastCoast;
+    }
 
     /**
      * Makes an XML-representation of this object.
@@ -157,25 +177,60 @@ public class Nation extends FreeColGameObjectType {
      * @throws XMLStreamException if there are any problems writing to the
      *             stream.
      */
-    public void toXMLImpl(XMLStreamWriter out) throws XMLStreamException {
+    protected void toXMLImpl(XMLStreamWriter out) throws XMLStreamException {
         super.toXML(out, getXMLElementTagName());
     }
 
-    public void writeAttributes(XMLStreamWriter out) throws XMLStreamException {
+    /**
+     * Write the attributes of this object to a stream.
+     *
+     * @param out The target stream.
+     * @throws XMLStreamException if there are any problems writing to
+     *     the stream.
+     */
+    @Override
+    protected void writeAttributes(XMLStreamWriter out)
+        throws XMLStreamException {
         super.writeAttributes(out);
+
         out.writeAttribute("nation-type", type.getId());
         out.writeAttribute("selectable", Boolean.toString(selectable));
-        if (anthem != null) {
-            out.writeAttribute("anthem", anthem);
-        }
+        out.writeAttribute("preferredLatitude", Integer.toString(preferredLatitude));
+        out.writeAttribute("startsOnEastCoast", Boolean.toString(startsOnEastCoast));
         if (refNation != null) {
             out.writeAttribute("ref", refNation.getId());
         }
     }
 
+    /**
+     * Reads the attributes of this object from an XML stream.
+     *
+     * @param in The XML input stream.
+     * @exception XMLStreamException if a problem was encountered
+     *     during parsing.
+     */
+    @Override
+    protected void readAttributes(XMLStreamReader in)
+        throws XMLStreamException {
+        super.readAttributes(in);
+
+        type = getSpecification().getNationType(in.getAttributeValue(null,
+                "nation-type"));
+        selectable = getAttribute(in, "selectable", false);
+        preferredLatitude = getAttribute(in, "preferredLatitude", 0);
+        startsOnEastCoast = getAttribute(in, "startsOnEastCoast", true);
+        String refId = getAttribute(in, "ref", null);
+        if (refId != null) {
+            refNation = getSpecification().getNation(refId);
+        }
+    }
+
+    /**
+     * Returns the tag name of the root element representing this object.
+     *
+     * @return "nation".
+     */
     public static String getXMLElementTagName() {
         return "nation";
     }
-
-
 }
