@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -20,7 +20,6 @@
 package net.sf.freecol.common.model;
 
 import java.util.ArrayList;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Random;
 import java.util.Set;
@@ -30,16 +29,15 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
-import net.sf.freecol.common.model.Unit.UnitState;
 import net.sf.freecol.common.model.UnitTypeChange.ChangeType;
 
-import org.w3c.dom.Element;
-
 
 /**
- * The super class of all settlements on the map (that is colonies and indian settlements).
+ * The super class of all settlements on the map (that is colonies and
+ * indian settlements).
  */
-abstract public class Settlement extends FreeColGameObject implements Location, Named, Ownable {
+abstract public class Settlement extends GoodsLocation
+    implements Named, Ownable {
 
     private static final Logger logger = Logger.getLogger(Settlement.class.getName());
 
@@ -54,13 +52,12 @@ abstract public class Settlement extends FreeColGameObject implements Location,
     /** The <code>Tile</code> where this <code>Settlement</code> is located. */
     protected Tile tile;
 
-    protected GoodsContainer goodsContainer;
-
-    /**
-     * Contains the abilities and modifiers of this Colony.
-     */
+    /** Contains the abilities and modifiers of this Settlement. */
     private FeatureContainer featureContainer;
 
+    /** The tiles this settlement owns. */
+    private List<Tile> ownedTiles = new ArrayList<Tile>();
+
     /**
      * Describe type here.
      */
@@ -88,10 +85,8 @@ abstract public class Settlement extends FreeColGameObject implements Location,
         this.name = name;
         this.tile = tile;
 
-        featureContainer = new FeatureContainer(game.getSpecification());
+        featureContainer = new FeatureContainer();
         setType(owner.getNationType().getSettlementType(false));
-
-        owner.addSettlement(this);
     }
 
 
@@ -108,17 +103,6 @@ abstract public class Settlement extends FreeColGameObject implements Location,
         super(game, in);
     }
 
-    /**
-     * Initiates a new <code>Settlement</code> from an <code>Element</code>.
-     *
-     * @param game The <code>Game</code> in which this object belong.
-     * @param e An XML-element that will be used to initialize
-     *      this object.
-     */
-    public Settlement(Game game, Element e) {
-        super(game, e);
-    }
-
     /**
      * Initiates a new <code>Settlement</code>
      * with the given ID. The object should later be
@@ -243,7 +227,9 @@ abstract public class Settlement extends FreeColGameObject implements Location,
      * @see Player#canSee(Tile)
      */
     public int getLineOfSight() {
-        return getType().getVisibleRadius();
+        return (int) getFeatureContainer()
+            .applyModifier(getType().getVisibleRadius(),
+                           "model.modifier.lineOfSightBonus");
     }
 
 
@@ -288,11 +274,29 @@ abstract public class Settlement extends FreeColGameObject implements Location,
     }
 
     /**
-     * Put a prepared settlement onto the map.
+     * Returns this Settlement.
+     *
+     * @return this Settlement
      */
-    public void placeSettlement() {
-        List<Tile> tiles = getGame().getMap().getClaimableTiles(owner, tile,
-                                                                getRadius());
+    public Settlement getSettlement() {
+        return this;
+    }
+
+    /**
+     * Put a prepared settlement onto the map.
+     *
+     * @param maximal If true, also claim all the tiles possible.
+     */
+    public void placeSettlement(boolean maximal) {
+        List<Tile> tiles;
+        if (maximal) {
+            tiles = getGame().getMap()
+                .getClaimableTiles(owner, tile, getRadius());
+        } else {
+            tiles = new ArrayList<Tile>();
+            tiles.add(tile);
+        }
+
         tile.setSettlement(this);
         for (Tile t : tiles) {
             t.changeOwnership(owner, this);
@@ -342,8 +346,10 @@ abstract public class Settlement extends FreeColGameObject implements Location,
 
         List<Unit> units = getUnitList();
         units.addAll(getTile().getUnitList());
-        for (Unit u : units) {
-            u.setState(UnitState.ACTIVE);
+        while (!units.isEmpty()) {
+            Unit u = units.remove(0);
+            units.addAll(u.getUnitList());
+            u.setState(Unit.UnitState.ACTIVE);
             UnitType type = u.getTypeChange((newOwner.isUndead())
                                             ? ChangeType.UNDEAD
                                             : ChangeType.CAPTURE, newOwner);
@@ -363,59 +369,31 @@ abstract public class Settlement extends FreeColGameObject implements Location,
         }
     }
 
-    /**
-     * Gets the goods container for this settlement.
-     *
-     * @return The settlements goods container.
-     */
-    public GoodsContainer getGoodsContainer() {
-        return goodsContainer;
-    }
-
-    /**
-     * Sets the goods container for this settlement.
-     *
-     * @param goodsContainer The new <code>GoodsContainer</code> to use.
-     */
-    public void setGoodsContainer(GoodsContainer goodsContainer) {
-        this.goodsContainer = goodsContainer;
-    }
-
-    /**
-     * Gets an <code>Iterator</code> of every <code>Goods</code> in this
-     * <code>GoodsContainer</code>. Each <code>Goods</code> have a maximum
-     * amount of 100.
-     *
-     * @return The <code>Iterator</code>.
-     */
-    public Iterator<Goods> getGoodsIterator() {
-        return goodsContainer.getGoodsIterator();
-    }
-
-    /**
-     * Gets an <code>List</code> with every <code>Goods</code> in this
-     * <code>Colony</code>. There is only one <code>Goods</code> for each
-     * type of goods.
-     *
-     * @return The <code>Iterator</code>.
-     */
-    public List<Goods> getCompactGoods() {
-        return goodsContainer.getCompactGoods();
-    }
-
     /**
      * Get the tiles this settlement owns.
      *
      * @return A list of tiles.
      */
     public List<Tile> getOwnedTiles() {
-        Tile settlementTile = getTile();
-        ArrayList<Tile> tiles = new ArrayList<Tile>();
-        for (Tile t : settlementTile.getSurroundingTiles(getRadius())) {
-            if (t.getOwningSettlement() == this) tiles.add(t);
-        }
-        tiles.add(settlementTile);
-        return tiles;
+        return new ArrayList<Tile>(ownedTiles);
+    }
+
+    /**
+     * Adds a tile to this settlement.
+     *
+     * @param tile The <code>Tile</code> to add.
+     */
+    public void addTile(Tile tile) {
+        ownedTiles.add(tile);
+    }
+
+    /**
+     * Removes a tile from this settlement.
+     *
+     * @param tile The <code>Tile</code> to remove.
+     */
+    public void removeTile(Tile tile) {
+        ownedTiles.remove(tile);
     }
 
     /**
@@ -425,12 +403,8 @@ abstract public class Settlement extends FreeColGameObject implements Location,
      */
     public List<FreeColGameObject> disposeList() {
         List<FreeColGameObject> objects = new ArrayList<FreeColGameObject>();
-        if (owner != null && goodsContainer != null) {
-            objects.addAll(goodsContainer.disposeList());
-            goodsContainer = null;
-        }
-
-        if (owner != null && getTile() != null
+        if (owner != null
+            && getTile() != null
             && getTile().getSettlement() != null) {
             // Defensive tests to handle transition from calling dispose()
             // on both sides to when it is only called on server-side.
@@ -454,13 +428,6 @@ abstract public class Settlement extends FreeColGameObject implements Location,
         return objects;
     }
 
-    /**
-     * Dispose of this <code>Settlement</code>.
-     */
-    public void dispose() {
-        disposeList();
-    }
-
     /**
      * Gets the radius of what the <code>Settlement</code> considers
      * as it's own land.
@@ -471,6 +438,20 @@ abstract public class Settlement extends FreeColGameObject implements Location,
         return getType().getClaimableRadius();
     }
 
+    /**
+     * Returns whether this settlement is connected by water to Europe.
+     *
+     * @return <code>true</code> if this <code>Settlement</code> is connected
+     *         to Europe.
+     */
+    public boolean isConnected() {
+        for (Tile t : getTile().getSurroundingTiles(1)) {
+            if (t.isExplored() && t.getType().isWater()
+                && t.isConnected()) return true;
+        }
+        return false;
+    }
+
     /**
      * Gets the current Sons of Liberty in this settlement.
      */
@@ -488,58 +469,6 @@ abstract public class Settlement extends FreeColGameObject implements Location,
      */
     public abstract boolean propagateAlarm(Player player, int addToAlarm);
 
-    /**
-     * Removes a specified amount of a type of Goods from this Settlement.
-     *
-     * @param type The type of Goods to remove from this settlement.
-     * @param amount The amount of Goods to remove from this settlement.
-     */
-    public void removeGoods(GoodsType type, int amount) {
-        goodsContainer.removeGoods(type, amount);
-    }
-
-    /**
-     * Removes the given Goods from the Settlement.
-     *
-     * @param goods a <code>Goods</code> value
-     */
-    public void removeGoods(AbstractGoods goods) {
-        goodsContainer.removeGoods(goods);
-    }
-
-    /**
-     * Removes all Goods of the given type from the Settlement.
-     *
-     * @param type a <code>GoodsType</code> value
-     */
-    public void removeGoods(GoodsType type) {
-        goodsContainer.removeGoods(type);
-    }
-
-    /**
-     * Describe <code>addGoods</code> method here.
-     *
-     * @param type a <code>GoodsType</code> value
-     * @param amount an <code>int</code> value
-     */
-    public void addGoods(GoodsType type, int amount) {
-        goodsContainer.addGoods(type, amount);
-    }
-
-    public void addGoods(AbstractGoods goods) {
-        addGoods(goods.getType(), goods.getAmount());
-    }
-
-    /**
-     * Gets the amount of one type of Goods at this Settlement.
-     *
-     * @param type The type of goods to look for.
-     * @return The amount of this type of Goods at this Location.
-     */
-    public int getGoodsCount(GoodsType type) {
-        return goodsContainer.getGoodsCount(type);
-    }
-
     /**
      * Returns the production of the given type of goods.
      *
@@ -595,8 +524,8 @@ abstract public class Settlement extends FreeColGameObject implements Location,
      * Return true if this Colony could build at least one item of the
      * given EquipmentType.
      *
-     * @param equipmentType an <code>EquipmentType</code> value
-     * @return a <code>boolean</code> value
+     * @param equipmentType The <code>EquipmentType</code> to build.
+     * @return True if the equipment can be built.
      */
     public boolean canBuildEquipment(EquipmentType equipmentType) {
         for (AbstractGoods requiredGoods : equipmentType.getGoodsRequired()) {
@@ -607,34 +536,90 @@ abstract public class Settlement extends FreeColGameObject implements Location,
         return true;
     }
 
-    protected void readAttributes(XMLStreamReader in) throws XMLStreamException {
-        super.readAttributes(in);
-        setName(in.getAttributeValue(null, "name"));
-        owner = getFreeColGameObject(in, "owner", Player.class);
-        tile = getFreeColGameObject(in, "tile", Tile.class);
-        featureContainer = new FeatureContainer(getGame().getSpecification());
+    /**
+     * Determines if this settlement can build the given type of equipment.
+     * Unlike canBuildEquipment, this takes goods "reserved"
+     * for other purposes into account (e.g. breeding).
+     *
+     * @param equipmentType an <code>EquipmentType</code> value
+     * @return True if the settlement can provide the equipment.
+     * @see Settlement#canBuildEquipment(EquipmentType equipmentType)
+     */
+    public boolean canProvideEquipment(EquipmentType equipmentType) {
+        for (AbstractGoods goods : equipmentType.getGoodsRequired()) {
+            int available = getGoodsCount(goods.getType());
 
-        // TODO: remove 0.9.x compatibility code
-        String typeStr = in.getAttributeValue(null, "settlementType");
-        SettlementType settlementType;
-        if (typeStr == null) {
-            // must be old style
-            String capital = in.getAttributeValue(null, "isCapital");
-            settlementType = owner.getNationType()
-                .getSettlementType("true".equals(capital));
-        } else {
-            settlementType = owner.getNationType().getSettlementType(typeStr);
+            int breedingNumber = goods.getType().getBreedingNumber();
+            if (breedingNumber != GoodsType.INFINITY) {
+                available -= breedingNumber;
+            }
+
+            if (available < goods.getAmount()) return false;
         }
-        // end compatibility code
-        setType(settlementType);
+        return true;
     }
 
+    /**
+     * Return true if this Settlement could provide at least one item of
+     * all the given EquipmentTypes.  This is designed specifically to
+     * mesh with getRoleEquipment().
+     *
+     * @param equipment A list of <code>EquipmentType</code>s to build.
+     * @return True if the settlement can provide all the equipment.
+     */
+    public boolean canProvideEquipment(List<EquipmentType> equipment) {
+        for (EquipmentType e : equipment) {
+            if (!canProvideEquipment(e)) return false;
+        }
+        return true;
+    }
+
+    /**
+     * Write the attributes of this object to a stream.
+     *
+     * @param out The target stream.
+     * @throws XMLStreamException if there are any problems writing
+     *     to the stream.
+     */
+    @Override
     protected void writeAttributes(XMLStreamWriter out)
         throws XMLStreamException {
-        out.writeAttribute(ID_ATTRIBUTE, getId());
+        super.writeAttributes(out);
         out.writeAttribute("name", getName());
+        out.writeAttribute("owner", owner.getId());
         out.writeAttribute("tile", tile.getId());
         out.writeAttribute("settlementType", getType().getId());
         // Not owner, it is subject to PlayerExploredTile handling.
     }
+
+    /**
+     * Reads the attributes of this object from an XML stream.
+     *
+     * @param in The XML input stream.
+     * @exception XMLStreamException if a problem was encountered
+     *     during parsing.
+     */
+    @Override
+    protected void readAttributes(XMLStreamReader in)
+        throws XMLStreamException {
+        super.readAttributes(in);
+
+        setName(in.getAttributeValue(null, "name"));
+        owner = getFreeColGameObject(in, "owner", Player.class);
+        tile = getFreeColGameObject(in, "tile", Tile.class);
+        featureContainer = new FeatureContainer();
+
+        // @compat 0.9.x
+        String typeStr = in.getAttributeValue(null, "settlementType");
+        SettlementType settlementType;
+        if (typeStr == null) {
+            String capital = in.getAttributeValue(null, "isCapital");
+            settlementType = owner.getNationType()
+                .getSettlementType("true".equals(capital));
+        // end compatibility code
+        } else {
+            settlementType = owner.getNationType().getSettlementType(typeStr);
+        }
+        setType(settlementType);
+    }
 }
