@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -26,7 +26,7 @@ import java.util.Random;
 import java.util.Set;
 import java.util.logging.Logger;
 
-import net.sf.freecol.common.model.Unit.UnitState;
+import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.model.UnitTypeChange.ChangeType;
 import net.sf.freecol.common.util.Utils;
 
@@ -57,10 +57,11 @@ public class SimpleCombatModel extends CombatModel {
         = "model.modifier.artilleryAgainstRaid";
     public static final String AMPHIBIOUS_ATTACK
         = "model.modifier.amphibiousAttack";
+    public static final String BOMBARD_BONUS
+        = "model.modifier.bombardBonus";
     public static final Modifier UNKNOWN_DEFENCE_MODIFIER
         = new Modifier("bogus", Modifier.UNKNOWN, Modifier.Type.ADDITIVE);
 
-
     public SimpleCombatModel() {}
 
 
@@ -97,8 +98,8 @@ public class SimpleCombatModel extends CombatModel {
                                  FreeColGameObject defender) {
         float result = 0.0f;
         if (attacker == null) {
-            throw new IllegalArgumentException("Null attacker");
-        } else if (combatIsMeasurement(attacker, defender)
+            throw new IllegalStateException("Null attacker");
+        } else if (combatIsAttackMeasurement(attacker, defender)
             || combatIsAttack(attacker, defender)
             || combatIsSettlementAttack(attacker, defender)) {
             result = FeatureContainer.applyModifierSet(0,
@@ -109,7 +110,7 @@ public class SimpleCombatModel extends CombatModel {
             Settlement attackerSettlement = (Settlement) attacker;
             if (attackerSettlement.hasAbility("model.ability.bombardShips")) {
                 for (Unit unit : attackerSettlement.getTile().getUnitList()) {
-                    if (unit.hasAbility("model.ability.bombard")) {
+                    if (unit.hasAbility(Ability.BOMBARD)) {
                         result += unit.getType().getOffence();
                     }
                 }
@@ -131,9 +132,8 @@ public class SimpleCombatModel extends CombatModel {
     public float getDefencePower(FreeColGameObject attacker,
                                  FreeColGameObject defender) {
         float result;
-        if (attacker == null) {
-            throw new IllegalArgumentException("Null attacker");
-        } else if (combatIsAttack(attacker, defender)
+        if (combatIsDefenceMeasurement(attacker, defender)
+            || combatIsAttack(attacker, defender)
             || combatIsSettlementAttack(attacker, defender)) {
             result = FeatureContainer.applyModifierSet(0,
                     defender.getGame().getTurn(),
@@ -157,7 +157,9 @@ public class SimpleCombatModel extends CombatModel {
     public Set<Modifier> getOffensiveModifiers(FreeColGameObject attacker,
                                                FreeColGameObject defender) {
         Set<Modifier> result = new LinkedHashSet<Modifier>();
-        if (combatIsMeasurement(attacker, defender)
+        if (attacker == null) {
+            throw new IllegalStateException("Null attacker");
+        } else if (combatIsAttackMeasurement(attacker, defender)
             || combatIsAttack(attacker, defender)
             || combatIsSettlementAttack(attacker, defender)) {
             Unit attackerUnit = (Unit) attacker;
@@ -244,19 +246,19 @@ public class SimpleCombatModel extends CombatModel {
             result.addAll(spec.getModifiers(AMPHIBIOUS_ATTACK));
         }
 
-        if (combatIsMeasurement(attacker, defender)) {
+        if (combatIsAttackMeasurement(attacker, defender)) {
             ; // No defender information available
         } else if (combatIsSettlementAttack(attacker, defender)) {
             // Settlement present, REF bombardment bonus
             result.addAll(attackerUnit
-                          .getModifierSet("model.modifier.bombardBonus"));
+                          .getModifierSet(BOMBARD_BONUS));
         } else if (combatIsAttack(attacker, defender)) {
             Unit defenderUnit = (Unit) defender;
             Tile tile = defenderUnit.getTile();
             if (tile != null) {
                 if (tile.getSettlement() != null) {
                     result.addAll(attackerUnit
-                                  .getModifierSet("model.modifier.bombardBonus"));
+                                  .getModifierSet(BOMBARD_BONUS));
                 } else {
                     // Ambush bonus in the open = defender's defence
                     // bonus, if defender is REF, or attacker is indian.
@@ -269,8 +271,10 @@ public class SimpleCombatModel extends CombatModel {
                             result.add(modifier);
                         }
                     }
-                    // Artillery in the open penalty
-                    if (attackerUnit.hasAbility("model.ability.bombard")
+                    // Artillery in the open penalty, must be on a
+                    // tile and not in a settlement.
+                    if (attackerUnit.hasAbility(Ability.BOMBARD)
+                        && attackerUnit.getLocation() instanceof Tile
                         && attackerUnit.getSettlement() == null) {
                         result.addAll(spec.getModifiers(ARTILLERY_IN_THE_OPEN));
                     }
@@ -291,7 +295,8 @@ public class SimpleCombatModel extends CombatModel {
     public Set<Modifier> getDefensiveModifiers(FreeColGameObject attacker,
                                                FreeColGameObject defender) {
         Set<Modifier> result = new LinkedHashSet<Modifier>();
-        if (combatIsAttack(attacker, defender)) {
+        if (combatIsDefenceMeasurement(attacker, defender)
+            || combatIsAttack(attacker, defender)) {
             Unit defenderUnit = (Unit) defender;
             result.add(new Modifier(Modifier.DEFENCE,
                                     Specification.BASE_DEFENCE_SOURCE,
@@ -353,7 +358,6 @@ public class SimpleCombatModel extends CombatModel {
     private void addLandDefensiveModifiers(FreeColGameObject attacker,
                                            FreeColGameObject defender,
                                            Set<Modifier> result) {
-        Unit attackerUnit = (Unit) attacker;
         Unit defenderUnit = (Unit) defender;
         Specification spec = defender.getSpecification();
         // Auto-equip and equipment bonuses.
@@ -370,7 +374,7 @@ public class SimpleCombatModel extends CombatModel {
             }
         }
         // Fortify bonus
-        if (defenderUnit.getState() == UnitState.FORTIFIED) {
+        if (defenderUnit.getState() == Unit.UnitState.FORTIFIED) {
             result.addAll(spec.getModifiers(FORTIFIED));
         }
         Tile tile = defenderUnit.getTile();
@@ -381,16 +385,17 @@ public class SimpleCombatModel extends CombatModel {
                     result.addAll(tile.getType().getDefenceBonus());
                 }
                 // Artillery in the Open penalty
-                if (defenderUnit.hasAbility("model.ability.bombard")
-                    && defenderUnit.getState() != UnitState.FORTIFIED) {
+                if (defenderUnit.hasAbility(Ability.BOMBARD)
+                    && defenderUnit.getState() != Unit.UnitState.FORTIFIED) {
                     result.addAll(spec.getModifiers(ARTILLERY_IN_THE_OPEN));
                 }
             } else { // In settlement
                 result.addAll(tile.getSettlement()
                               .getModifierSet(Modifier.DEFENCE));
                 // Artillery defence bonus against an Indian raid
-                if (defenderUnit.hasAbility("model.ability.bombard")
-                    && attackerUnit.getOwner().isIndian()) {
+                if (defenderUnit.hasAbility(Ability.BOMBARD)
+                    && attacker != null
+                    && ((Unit)attacker).getOwner().isIndian()) {
                     result.addAll(spec.getModifiers(ARTILLERY_AGAINST_RAID));
                 }
             }
@@ -407,8 +412,7 @@ public class SimpleCombatModel extends CombatModel {
      * @return The results of the combat.
      */
     public List<CombatResult> generateAttackResult(Random random,
-                                                   FreeColGameObject attacker,
-                                                   FreeColGameObject defender) {
+        FreeColGameObject attacker, FreeColGameObject defender) {
         ArrayList<CombatResult> crs = new ArrayList<CombatResult>();
         CombatOdds odds = calculateCombatOdds(attacker, defender);
         float r = random.nextFloat();
@@ -439,8 +443,8 @@ public class SimpleCombatModel extends CombatModel {
                 great = r < 0.1f * odds.win; // Great Win
                 crs.add(CombatResult.WIN);
                 resolveAttack(attackerUnit, defenderUnit, great,
-                              r / (0.1f * odds.win), // Rescale to 0 <= r < 1
-                              crs);
+                    // Rescale to 0 <= r < 1
+                    r / (0.1f * odds.win), crs);
             } else if (r < 0.8f * odds.win + 0.2f
                     && defenderUnit.hasAbility("model.ability.evadeAttack")) {
                 crs.add(CombatResult.NO_RESULT);
@@ -449,10 +453,9 @@ public class SimpleCombatModel extends CombatModel {
                 great = r >= 0.1f * odds.win + 0.9f; // Great Loss
                 crs.add(CombatResult.LOSE);
                 resolveAttack(defenderUnit, attackerUnit, great,
-                              // Rescaling to 0 <= r < 1
-                              // (rearrange: 0.8 * odds.win + 0.2 <= r < 1.0)
-                              (1.25f * r - 0.25f - odds.win)/(1.0f - odds.win),
-                              crs);
+                    // Rescaling to 0 <= r < 1
+                    // (rearrange: 0.8 * odds.win + 0.2 <= r < 1.0)
+                    (1.25f * r - 0.25f - odds.win)/(1.0f - odds.win), crs);
             }
 
         } else if (combatIsBombard(attacker, defender)) {
@@ -533,7 +536,8 @@ public class SimpleCombatModel extends CombatModel {
                 && !loser.getGoodsList().isEmpty()) {
                 crs.add(CombatResult.LOOT_SHIP);
             }
-            if (great || loser.getRepairLocation() == null
+            if (great
+                || loser.getRepairLocation() == null
                 || isBeached(loser)) {
                 crs.add(CombatResult.SINK_SHIP_ATTACK);
             } else {
@@ -565,23 +569,27 @@ public class SimpleCombatModel extends CombatModel {
                 // Ships in a falling colony will be damaged or sunk
                 // if they have no repair location.
                 Colony colony = (Colony) settlement;
-                CombatResult shipResult
-                    = (colony.getShipList().isEmpty()) ? null
-                    : (colony.getShipList().get(0).getRepairLocation() == null)
-                    ? CombatResult.SINK_COLONY_SHIPS
-                    : CombatResult.DAMAGE_COLONY_SHIPS;
-                if (winnerPlayer.isEuropean()) {
-                    if (shipResult != null) crs.add(shipResult);
-                    crs.add(CombatResult.CAPTURE_COLONY);
-                } else if (!great && colony.canBePillaged(winner)) {
-                    crs.add(CombatResult.PILLAGE_COLONY);
-                } else if (colony.getUnitCount() > 1
-                           || loser.getLocation() == tile) {
+                CombatResult colonyResult = (winnerPlayer.isEuropean())
+                    ? CombatResult.CAPTURE_COLONY
+                    : (!great && colony.canBePillaged(winner))
+                    ? CombatResult.PILLAGE_COLONY
+                    : (colony.getUnitCount() > 1
+                        || loser.getLocation() == tile)
+                    ? CombatResult.SLAUGHTER_UNIT
+                    : CombatResult.DESTROY_COLONY;
+                if (colonyResult == CombatResult.DESTROY_COLONY) {
                     crs.add(CombatResult.SLAUGHTER_UNIT);
-                } else {
-                    if (shipResult != null) crs.add(shipResult);
-                    crs.add(CombatResult.DESTROY_COLONY);
                 }
+                if (colonyResult == CombatResult.CAPTURE_COLONY
+                    || colonyResult == CombatResult.DESTROY_COLONY) {
+                    CombatResult shipResult = null;
+                    shipResult = (colony.getShipList().isEmpty()) ? null
+                        : (colony.getShipList().get(0).getRepairLocation() == null)
+                        ? CombatResult.SINK_COLONY_SHIPS
+                        : CombatResult.DAMAGE_COLONY_SHIPS;
+                    if (shipResult != null) crs.add(shipResult);
+                }
+                crs.add(colonyResult);
 
             // Failed to defend a native settlement.
             } else if (settlement instanceof IndianSettlement) {
@@ -629,6 +637,9 @@ public class SimpleCombatModel extends CombatModel {
                         : (winner.canCaptureEquipment(equip, loser) != null)
                         ? CombatResult.CAPTURE_EQUIP
                         : CombatResult.LOSE_EQUIP);
+                if (loser.losingEquipmentDemotesUnit(equip)) {
+                    crs.add(CombatResult.DEMOTE_UNIT);
+                }
 
             // Consume autoequipment.
             } else if (settlement instanceof Colony && autoEquip != null) {
@@ -644,8 +655,8 @@ public class SimpleCombatModel extends CombatModel {
                 crs.add(CombatResult.DEMOTE_UNIT);
 
             // Capture suitable units if the winner is capable.
-            } else if (loser.hasAbility("model.ability.canBeCaptured")
-                       && winner.hasAbility("model.ability.captureUnits")
+            } else if (loser.hasAbility(Ability.CAN_BE_CAPTURED)
+                       && winner.hasAbility(Ability.CAPTURE_UNITS)
                        && !combatIsAmphibious(winner, loser)) {
                 crs.add(CombatResult.CAPTURE_UNIT);
 
