@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -17,7 +17,6 @@
  *  along with FreeCol.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
 package net.sf.freecol.common.model;
 
 import java.util.ArrayList;
@@ -28,6 +27,7 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
+
 /**
  * The <code>StringTemplate</code> represents a non-localized string
  * that can be localized by looking up its value in a message bundle
@@ -436,27 +436,49 @@ public class StringTemplate extends FreeColObject {
         return result;
     }
 
-    public void toXMLImpl(XMLStreamWriter out) throws XMLStreamException {
-        out.writeStartElement(getXMLElementTagName());
-        writeAttributes(out);
-        writeChildren(out);
-        out.writeEndElement();
+
+    /**
+     * This method writes an XML-representation of this object to
+     * the given stream.
+     *
+     * @param out The target stream.
+     * @throws XMLStreamException if there are any problems writing
+     *      to the stream.
+     */
+    protected void toXMLImpl(XMLStreamWriter out) throws XMLStreamException {
+        super.toXML(out, getXMLElementTagName());
     }
 
-    public static String getXMLElementTagName() {
-        return "stringTemplate";
-    }
+    /**
+     * Write the attributes of this object to a stream.
+     *
+     * @param out The target stream.
+     * @throws XMLStreamException if there are any problems writing to
+     *     the stream.
+     */
+    @Override
+    protected void writeAttributes(XMLStreamWriter out)
+        throws XMLStreamException {
+        super.writeAttributes(out);
 
-    public void writeAttributes(XMLStreamWriter out) throws XMLStreamException {
-        out.writeAttribute(ID_ATTRIBUTE_TAG, getId());
         out.writeAttribute("templateType", templateType.toString());
         if (defaultId != null) {
             out.writeAttribute("defaultId", defaultId);
         }
     }
 
+    /**
+     * Write the children of this object to a stream.
+     *
+     * @param out The target stream.
+     * @throws XMLStreamException if there are any problems writing to
+     *     the stream.
+     */
+    @Override
+    protected void writeChildren(XMLStreamWriter out)
+        throws XMLStreamException {
+        super.writeChildren(out);
 
-    public void writeChildren(XMLStreamWriter out) throws XMLStreamException {
         if (keys != null) {
             for (String key : keys) {
                 out.writeStartElement("key");
@@ -471,8 +493,17 @@ public class StringTemplate extends FreeColObject {
         }
     }
 
-    public void readAttributes(XMLStreamReader in) throws XMLStreamException {
-        // TODO: remove compatibility code
+    /**
+     * Reads the attributes of this object from an XML stream.
+     *
+     * @param in The XML input stream.
+     * @exception XMLStreamException if a problem was encountered
+     *     during parsing.
+     */
+    @Override
+    protected void readAttributes(XMLStreamReader in)
+        throws XMLStreamException {
+        // @compat 0.10.x
         String id = in.getAttributeValue(null, ID_ATTRIBUTE_TAG);
         if (id == null) {
             id = in.getAttributeValue(null, ID_ATTRIBUTE);
@@ -494,8 +525,15 @@ public class StringTemplate extends FreeColObject {
         }
     }
 
-
-    public void readChildren(XMLStreamReader in) throws XMLStreamException {
+    /**
+     * Reads the children of this object from an XML stream.
+     *
+     * @param in The XML input stream.
+     * @exception XMLStreamException if a problem was encountered
+     *     during parsing.
+     */
+    @Override
+    protected void readChildren(XMLStreamReader in) throws XMLStreamException {
         while (in.nextTag() != XMLStreamConstants.END_ELEMENT) {
             if ("key".equals(in.getLocalName())) {
                 keys.add(in.getAttributeValue(null, VALUE_TAG));
@@ -504,11 +542,10 @@ public class StringTemplate extends FreeColObject {
                 StringTemplate replacement = new StringTemplate();
                 replacement.readFromXMLImpl(in);
                 replacements.add(replacement);
-            } else if ("data".equals(in.getLocalName())) {
-                // TODO: remove compatibility code for ModelMessage
+            } else if ("data".equals(in.getLocalName())) { // @compat 0.9.x
                 readOldFormat(readFromArrayElement("data", in, new String[0]));
                 // end compatibility code
-            } else if ("strings".equals(in.getLocalName())) {
+            } else if ("strings".equals(in.getLocalName())) { // @compat 0.9.x
                 // TODO: remove compatibility code for HistoryEvent
                 readOldFormat(readFromArrayElement("strings", in, new String[0]));
                 // end compatibility code
@@ -516,12 +553,21 @@ public class StringTemplate extends FreeColObject {
         }
     }
 
-    // TODO: remove compatibility code
+    // @compat 0.9.x
     private void readOldFormat(String[] data) {
         for (int index = 0; index < data.length; index += 2) {
             keys.add(data[index]);
             replacements.add(new StringTemplate(data[index + 1], TemplateType.NAME));
         }
     }
+    // end compatibility code
 
+    /**
+     * Returns the tag name of the root element representing this object.
+     *
+     * @return "stringTemplate".
+     */
+    public static String getXMLElementTagName() {
+        return "stringTemplate";
+    }
 }
