@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -20,6 +20,8 @@
 
 package net.sf.freecol.common.model;
 
+import java.util.Locale;
+
 
 public class Tension {
 
@@ -139,6 +141,15 @@ public class Tension {
         setValue(value + newTension);
     }
 
+    /**
+     * Gets a string suitable for use in a message id.
+     *
+     * @return A message id string.
+     */
+    public String getKey() {
+        return toString().toLowerCase(Locale.US);
+    }
+
     /**
      * Returns the current tension level as a string.
      *
