@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -35,7 +35,6 @@ import javax.xml.stream.XMLStreamWriter;
 
 import net.sf.freecol.common.model.PlayerExploredTile;
 
-import org.w3c.dom.Element;
 
 /**
  * Contains <code>TileItem</code>s and can be used by a {@link Tile}
@@ -85,7 +84,7 @@ public class TileItemContainer extends FreeColGameObject {
 
 
     /**
-     * Initiates a new <code>TileItemContainer</code> from an <code>Element</code>.
+     * Initiates a new <code>TileItemContainer</code> from an XML stream.
      *
      * @param game The <code>Game</code> in which this <code>TileItemContainer</code>
      *       belong.
@@ -106,27 +105,6 @@ public class TileItemContainer extends FreeColGameObject {
         readFromXML(in);
     }
 
-    /**
-     * Initiates a new <code>TileItemContainer</code> from an <code>Element</code>.
-     *
-     * @param game The <code>Game</code> in which this <code>TileItemContainer</code>
-     *       belong.
-     * @param tile The <code>Tile</code> using this <code>TileItemContainer</code>
-     *       for storing it's TileItem.
-     * @param e An XML-element that will be used to initialize
-     *      this object.
-     */
-    public TileItemContainer(Game game, Tile tile, Element e) {
-        super(game, e);
-
-        if (tile == null) {
-            throw new IllegalArgumentException("Tile must not be 'null'.");
-        }
-
-        this.tile = tile;
-        readFromXMLElement(e);
-    }
-
     // ------------------------------------------------------------ checking/retrieval functions
 
     /**
@@ -182,6 +160,23 @@ public class TileItemContainer extends FreeColGameObject {
         return null;
     }
 
+    /**
+     * Gets the tile improvement of the given type if any.
+     *
+     * @param type The <code>TileImprovementType</code> to look for.
+     * @return The tile improvement of the given type if present,
+     *     otherwise null.
+     */
+    public TileImprovement getImprovement(TileImprovementType type) {
+        for (TileItem item : tileItems) {
+            if (item instanceof TileImprovement
+                && ((TileImprovement) item).getType() == type) {
+                return (TileImprovement) item;
+            }
+        }
+        return null;
+    }
+
     /**
      * Returns the road improvement or null.
      *
@@ -339,42 +334,26 @@ public class TileItemContainer extends FreeColGameObject {
     }
 
     /**
-     * Determine the movement cost to this <code>Tile</code> from another <code>Tile</code>.
+     * Determine the movement cost to this <code>Tile</code> from
+     * another <code>Tile</code>.
      * Does not consider special unit abilities.
      *
-     * @param basicMoveCost an <code>int</code> value
-     * @param fromTile a <code>Tile</code> value
-     * @return The movement cost
+     * @param basicMoveCost The basic cost.
+     * @param fromTile The <code>Tile</code> to move from.
+     * @return The movement cost.
      */
     public int getMoveCost(int basicMoveCost, Tile fromTile) {
         int moveCost = basicMoveCost;
         for (TileItem item : tileItems) {
-            if (item instanceof TileImprovement) {
-                moveCost = ((TileImprovement) item).getMovementCost(moveCost, fromTile);
+            if (item instanceof TileImprovement
+                && ((TileImprovement) item).isComplete()) {
+                moveCost = ((TileImprovement) item).getMovementCost(moveCost,
+                                                                    fromTile);
             }
         }
         return moveCost;
     }
 
-    /**
-     * Returns a description of the tile, with the name of the tile
-     * and any improvements made to it (road/plow)
-     *
-     * @return The description label for this tile
-     */
-    public StringTemplate getLabel() {
-        StringTemplate label = StringTemplate.label("/");
-        for (TileItem item : tileItems) {
-            if (item instanceof Resource) {
-                label.add(((Resource) item).getType().getNameKey());
-            } else if (item instanceof TileImprovement
-                       && ((TileImprovement) item).isComplete()) {
-                label.add(((TileImprovement) item).getType().getNameKey());
-            }
-        }
-        return label;
-    }
-
     // ------------------------------------------------------------ add/remove from container
 
     /**
@@ -562,7 +541,6 @@ public class TileItemContainer extends FreeColGameObject {
         return null;
     }
 
-    // ------------------------------------------------------------ API methods
 
     /**
      * This method writes an XML-representation of this object to
@@ -586,19 +564,19 @@ public class TileItemContainer extends FreeColGameObject {
      * @throws XMLStreamException if there are any problems writing
      *      to the stream.
      */
-    protected void toXMLImpl(XMLStreamWriter out, Player player, boolean showAll, boolean toSavedGame)
+    protected void toXMLImpl(XMLStreamWriter out, Player player,
+                             boolean showAll, boolean toSavedGame)
         throws XMLStreamException {
-        PlayerExploredTile pet = (showAll) ? null
+        PlayerExploredTile pet = (showAll || toSavedGame) ? null
             : tile.getPlayerExploredTile(player);
 
         // Start element:
         out.writeStartElement(getXMLElementTagName());
 
-        out.writeAttribute("ID", getId());
+        out.writeAttribute(ID_ATTRIBUTE, getId());
         out.writeAttribute("tile", tile.getId());
 
-        if (getGame().isClientTrusted() || showAll || toSavedGame
-            || player.canSee(tile)) {
+        if (showAll || toSavedGame || player.canSee(tile)) {
             for (TileItem item : tileItems) {
                 item.toXML(out, player, showAll, toSavedGame);
             }
@@ -615,10 +593,12 @@ public class TileItemContainer extends FreeColGameObject {
 
     /**
      * Initialize this object from an XML-representation of this object.
+     *
      * @param in The input stream with the XML.
      */
-    protected void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException {
-        setId(in.getAttributeValue(null, "ID"));
+    protected void readFromXMLImpl(XMLStreamReader in)
+        throws XMLStreamException {
+        setId(in.getAttributeValue(null, ID_ATTRIBUTE));
 
         tile = (Tile) getGame().getFreeColGameObject(in.getAttributeValue(null, "tile"));
         if (tile == null) {
@@ -628,7 +608,7 @@ public class TileItemContainer extends FreeColGameObject {
         tileItems.clear();
 
         while (in.nextTag() != XMLStreamConstants.END_ELEMENT) {
-            TileItem item = (TileItem) getGame().getFreeColGameObject(in.getAttributeValue(null, "ID"));
+            TileItem item = (TileItem) getGame().getFreeColGameObject(in.getAttributeValue(null, ID_ATTRIBUTE));
             if (item == null) {
                 if (in.getLocalName().equals(Resource.getXMLElementTagName())) {
                     item = new Resource(getGame(), in);
@@ -647,20 +627,11 @@ public class TileItemContainer extends FreeColGameObject {
         Collections.sort(tileItems, tileItemComparator);
     }
 
-
-    /**
-     * Gets the tag name of the root element representing this object.
-     * @return "tileitemcontainer".
-     */
-    public static String getXMLElementTagName() {
-        return "tileitemcontainer";
-    }
-
-
     /**
      * Creates a <code>String</code> representation of this
      * <code>TileItemContainer</code>.
      */
+    @Override
     public String toString() {
         StringBuffer sb = new StringBuffer(60);
         sb.append("TileItemContainer with: ");
@@ -670,4 +641,12 @@ public class TileItemContainer extends FreeColGameObject {
         return sb.toString();
     }
 
+    /**
+     * Gets the tag name of the root element representing this object.
+     *
+     * @return "tileitemcontainer".
+     */
+    public static String getXMLElementTagName() {
+        return "tileitemcontainer";
+    }
 }
