@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -30,63 +30,75 @@ import javax.xml.stream.XMLStreamWriter;
 
 import net.sf.freecol.common.model.UnitTypeChange.ChangeType;
 
-public final class UnitType extends BuildableType implements Comparable<UnitType>, Consumer {
 
+public final class UnitType extends BuildableType
+    implements Comparable<UnitType>, Consumer {
+
+    /**
+     * The default offence value.
+     */
     public static final int DEFAULT_OFFENCE = 0;
+
+    /**
+     * The default offence value.
+     */
     public static final int DEFAULT_DEFENCE = 1;
 
     /**
-     * Describe offence here.
+     * The offence of this UnitType. Only Units with an offence value
+     * greater than zero can attack.
      */
     private int offence = DEFAULT_OFFENCE;
 
     /**
-     * Describe defence here.
+     * The defence of this UnitType.
      */
     private int defence = DEFAULT_DEFENCE;
 
     /**
-     * Describe space here.
+     * The capacity of this UnitType.
      */
     private int space = 0;
 
     /**
-     * Describe hitPoints here.
+     * The number of hit points this UnitType has. At the moment, this
+     * is only used for ships. All other UnitTypes are downgraded or
+     * destroyed if they lose a battle.
      */
     private int hitPoints = 0;
 
     /**
-     * Describe spaceTaken here.
+     * The space taken by this UnitType.
      */
     private int spaceTaken = 1;
 
     /**
-     * Describe skill here.
+     * The skill level of this UnitType.
      */
     private int skill = UNDEFINED;
 
     /**
-     * Describe price here.
+     * The price of this UnitType.
      */
     private int price = UNDEFINED;
 
     /**
-     * Describe movement here.
+     * The initial moves of this UnitType.
      */
     private int movement = 3;
 
     /**
-     * Describe lineOfSight here.
+     * The maximum distance of tiles this UnitType can observe.
      */
     private int lineOfSight = 1;
 
     /**
-     * Describe recruitProbability here.
+     * The probability of recruiting a Unit of this type in Europe.
      */
     private int recruitProbability = 0;
 
     /**
-     * Describe expertProduction here.
+     * The expert production of this UnitType.
      */
     private GoodsType expertProduction;
 
@@ -101,7 +113,8 @@ public final class UnitType extends BuildableType implements Comparable<UnitType
     private int maximumExperience = 0;
 
     /**
-     * Describe maximumAttrition here.
+     * The maximum attrition this UnitType can accumulate without
+     * being destroyed.
      */
     private int maximumAttrition = INFINITY;
 
@@ -145,7 +158,7 @@ public final class UnitType extends BuildableType implements Comparable<UnitType
      * @return a <code>boolean</code> value
      */
     public boolean canCarryUnits() {
-        return hasAbility("model.ability.carryUnits");
+        return hasAbility(Ability.CARRY_UNITS);
     }
 
     /**
@@ -154,7 +167,7 @@ public final class UnitType extends BuildableType implements Comparable<UnitType
      * @return a <code>boolean</code> value
      */
     public boolean canCarryGoods() {
-        return hasAbility("model.ability.carryGoods");
+        return hasAbility(Ability.CARRY_GOODS);
     }
 
     /**
@@ -441,7 +454,7 @@ public final class UnitType extends BuildableType implements Comparable<UnitType
     }
 
     public EquipmentType[] getDefaultEquipment() {
-        if (hasAbility("model.ability.canBeEquipped") && defaultEquipment != null) {
+        if (hasAbility(Ability.CAN_BE_EQUIPPED) && defaultEquipment != null) {
             int count = defaultEquipment.getMaximumCount();
             EquipmentType[] result = new EquipmentType[count];
             for (int index = 0; index < count; index++) {
@@ -620,156 +633,7 @@ public final class UnitType extends BuildableType implements Comparable<UnitType
      * @return True if this is a naval unit type.
      */
     public boolean isNaval() {
-        return hasAbility("model.ability.navalUnit");
-    }
-
-    public void readAttributes(XMLStreamReader in) throws XMLStreamException {
-        super.readAttributes(in);
-        String extendString = in.getAttributeValue(null, "extends");
-        UnitType parent = (extendString == null) ? this :
-            getSpecification().getUnitType(extendString);
-        offence = getAttribute(in, "offence", parent.offence);
-        defence = getAttribute(in, "defence", parent.defence);
-        movement = getAttribute(in, "movement", parent.movement);
-        lineOfSight = getAttribute(in, "lineOfSight", parent.lineOfSight);
-        scoreValue = getAttribute(in, "scoreValue", parent.scoreValue);
-        space = getAttribute(in, "space", parent.space);
-        hitPoints = getAttribute(in, "hitPoints", parent.hitPoints);
-        spaceTaken = getAttribute(in, "spaceTaken", parent.spaceTaken);
-        maximumExperience = getAttribute(in, "maximumExperience", parent.maximumExperience);
-        maximumAttrition = getAttribute(in, "maximumAttrition", parent.maximumAttrition);
-        String skillString = in.getAttributeValue(null, "skillTaught");
-        skillTaught = (skillString == null) ? this : getSpecification().getUnitType(skillString);
-
-        recruitProbability = getAttribute(in, "recruitProbability", parent.recruitProbability);
-        skill = getAttribute(in, "skill", parent.skill);
-
-        setPopulationRequired(getAttribute(in, "population-required", parent.getPopulationRequired()));
-
-        price = getAttribute(in, "price", parent.price);
-
-        expertProduction = getSpecification().getType(in, "expert-production", GoodsType.class,
-                                                 parent.expertProduction);
-
-        if (parent != this) {
-            typeChanges.addAll(parent.typeChanges);
-            defaultEquipment = parent.defaultEquipment;
-            consumption.putAll(parent.consumption);
-            getFeatureContainer().add(parent.getFeatureContainer());
-            if (parent.isAbstractType()) {
-                getFeatureContainer().replaceSource(parent, this);
-            }
-        }
-    }
-
-    public void readChild(XMLStreamReader in) throws XMLStreamException {
-        String nodeName = in.getLocalName();
-        if ("downgrade".equals(nodeName)
-            || "upgrade".equals(nodeName)) {
-            if (getAttribute(in, "delete", false)) {
-                String unitId = in.getAttributeValue(null, "unit");
-                Iterator<UnitTypeChange> iterator = typeChanges.iterator();
-                while (iterator.hasNext()) {
-                    if (unitId.equals(iterator.next().getNewUnitType().getId())) {
-                        iterator.remove();
-                        break;
-                    }
-                }
-                in.nextTag();
-            } else {
-                UnitTypeChange change = new UnitTypeChange(in, getSpecification());
-                if ("downgrade".equals(nodeName)
-                    && change.getChangeTypes().isEmpty()) {
-                    // add default downgrade type
-                    change.getChangeTypes().put(ChangeType.CLEAR_SKILL, 100);
-                }
-                typeChanges.add(change);
-            }
-        } else if ("default-equipment".equals(nodeName)) {
-            String equipmentString = in.getAttributeValue(null, ID_ATTRIBUTE_TAG);
-            if (equipmentString != null) {
-                defaultEquipment = getSpecification().getEquipmentType(equipmentString);
-            }
-            in.nextTag(); // close this element
-        } else if ("consumes".equals(nodeName)) {
-            String typeString = in.getAttributeValue(null, ID_ATTRIBUTE_TAG);
-            String valueString = in.getAttributeValue(null, VALUE_TAG);
-            if (typeString != null && valueString != null) {
-                try {
-                    GoodsType type = getSpecification().getGoodsType(typeString);
-                    int amount = Integer.parseInt(valueString);
-                    consumption.incrementCount(type, amount);
-                } catch(Exception e) {
-                    logger.warning("Failed to parse integer " + valueString);
-                }
-            }
-            in.nextTag(); // close this element
-        } else {
-            super.readChild(in);
-        }
-    }
-
-
-    /**
-     * Makes an XML-representation of this object.
-     *
-     * @param out The output stream.
-     * @throws XMLStreamException if there are any problems writing to the
-     *             stream.
-     */
-    public void toXMLImpl(XMLStreamWriter out) throws XMLStreamException {
-        super.toXML(out, getXMLElementTagName());
-    }
-
-    public void writeAttributes(XMLStreamWriter out) throws XMLStreamException {
-        super.writeAttributes(out);
-        out.writeAttribute("offence", Integer.toString(offence));
-        out.writeAttribute("defence", Integer.toString(defence));
-        out.writeAttribute("movement", Integer.toString(movement));
-        out.writeAttribute("lineOfSight", Integer.toString(lineOfSight));
-        out.writeAttribute("scoreValue", Integer.toString(scoreValue));
-        out.writeAttribute("space", Integer.toString(space));
-        out.writeAttribute("spaceTaken", Integer.toString(spaceTaken));
-        out.writeAttribute("hitPoints", Integer.toString(hitPoints));
-        out.writeAttribute("maximumExperience", Integer.toString(maximumExperience));
-        if (maximumAttrition < INFINITY) {
-            out.writeAttribute("maximumAttrition", Integer.toString(maximumAttrition));
-        }
-        out.writeAttribute("recruitProbability", Integer.toString(recruitProbability));
-        if (skill != UNDEFINED) {
-            out.writeAttribute("skill", Integer.toString(skill));
-        }
-        if (price != UNDEFINED) {
-            out.writeAttribute("price", Integer.toString(price));
-        }
-        out.writeAttribute("skillTaught", skillTaught.getId());
-        if (expertProduction != null) {
-            out.writeAttribute("expert-production", expertProduction.getId());
-        }
-    }
-
-    protected void writeChildren(XMLStreamWriter out) throws XMLStreamException {
-        super.writeChildren(out);
-        for (UnitTypeChange change : typeChanges) {
-            change.toXMLImpl(out);
-        }
-        if (defaultEquipment != null) {
-            out.writeStartElement("default-equipment");
-            out.writeAttribute(ID_ATTRIBUTE_TAG, defaultEquipment.getId());
-            out.writeEndElement();
-        }
-        if (!consumption.isEmpty()) {
-            for (GoodsType goodsType : consumption.keySet()) {
-                out.writeStartElement("consumes");
-                out.writeAttribute(ID_ATTRIBUTE_TAG, goodsType.getId());
-                out.writeAttribute(VALUE_TAG, Integer.toString(consumption.getCount(goodsType)));
-                out.writeEndElement();
-            }
-        }
-    }
-
-    public static String getXMLElementTagName() {
-        return "unit-type";
+        return hasAbility(Ability.NAVAL_UNIT);
     }
 
 
@@ -803,16 +667,7 @@ public final class UnitType extends BuildableType implements Comparable<UnitType
         return price != UNDEFINED;
     }
 
-    public int getProductionFor(GoodsType goodsType, int base) {
-        if (base == 0) {
-            return 0;
-        }
-
-        base = (int) getFeatureContainer().applyModifier(base, goodsType.getId());
-        return Math.max(base, 1);
-    }
-
-     /**
+    /**
      * Returns the number of units of the given GoodsType this
      * UnitType consumes per turn (when in a settlement).
      *
@@ -859,4 +714,211 @@ public final class UnitType extends BuildableType implements Comparable<UnitType
     }
 
 
+    /**
+     * Makes an XML-representation of this object.
+     *
+     * @param out The output stream.
+     * @throws XMLStreamException if there are any problems writing to the
+     *             stream.
+     */
+    protected void toXMLImpl(XMLStreamWriter out) throws XMLStreamException {
+        super.toXML(out, getXMLElementTagName());
+    }
+
+    /**
+     * Write the attributes of this object to a stream.
+     *
+     * @param out The target stream.
+     * @throws XMLStreamException if there are any problems writing to
+     *     the stream.
+     */
+    @Override
+    protected void writeAttributes(XMLStreamWriter out)
+        throws XMLStreamException {
+        super.writeAttributes(out);
+
+        out.writeAttribute("offence", Integer.toString(offence));
+        out.writeAttribute("defence", Integer.toString(defence));
+        out.writeAttribute("movement", Integer.toString(movement));
+        out.writeAttribute("lineOfSight", Integer.toString(lineOfSight));
+        out.writeAttribute("scoreValue", Integer.toString(scoreValue));
+        out.writeAttribute("space", Integer.toString(space));
+        out.writeAttribute("spaceTaken", Integer.toString(spaceTaken));
+        out.writeAttribute("hitPoints", Integer.toString(hitPoints));
+        out.writeAttribute("maximumExperience",
+            Integer.toString(maximumExperience));
+        if (maximumAttrition < INFINITY) {
+            out.writeAttribute("maximumAttrition",
+                Integer.toString(maximumAttrition));
+        }
+        out.writeAttribute("recruitProbability",
+            Integer.toString(recruitProbability));
+        if (skill != UNDEFINED) {
+            out.writeAttribute("skill", Integer.toString(skill));
+        }
+        if (price != UNDEFINED) {
+            out.writeAttribute("price", Integer.toString(price));
+        }
+        out.writeAttribute("skillTaught", skillTaught.getId());
+        if (expertProduction != null) {
+            out.writeAttribute("expert-production", expertProduction.getId());
+        }
+    }
+
+    /**
+     * Write the children of this object to a stream.
+     *
+     * @param out The target stream.
+     * @throws XMLStreamException if there are any problems writing to
+     *     the stream.
+     */
+    @Override
+    protected void writeChildren(XMLStreamWriter out)
+        throws XMLStreamException {
+        super.writeChildren(out);
+
+        for (UnitTypeChange change : typeChanges) {
+            change.toXMLImpl(out);
+        }
+        if (defaultEquipment != null) {
+            out.writeStartElement("default-equipment");
+            out.writeAttribute(ID_ATTRIBUTE_TAG, defaultEquipment.getId());
+            out.writeEndElement();
+        }
+        if (!consumption.isEmpty()) {
+            for (GoodsType goodsType : consumption.keySet()) {
+                out.writeStartElement("consumes");
+                out.writeAttribute(ID_ATTRIBUTE_TAG, goodsType.getId());
+                out.writeAttribute(VALUE_TAG, Integer.toString(consumption.getCount(goodsType)));
+                out.writeEndElement();
+            }
+        }
+    }
+
+    /**
+     * Reads the attributes of this object from an XML stream.
+     *
+     * @param in The XML input stream.
+     * @throws XMLStreamException if a problem was encountered
+     *     during parsing.
+     */
+    @Override
+    protected void readAttributes(XMLStreamReader in)
+        throws XMLStreamException {
+        super.readAttributes(in);
+
+        String extendString = in.getAttributeValue(null, "extends");
+        UnitType parent = (extendString == null) ? this :
+            getSpecification().getUnitType(extendString);
+        offence = getAttribute(in, "offence", parent.offence);
+        defence = getAttribute(in, "defence", parent.defence);
+        movement = getAttribute(in, "movement", parent.movement);
+        lineOfSight = getAttribute(in, "lineOfSight", parent.lineOfSight);
+        scoreValue = getAttribute(in, "scoreValue", parent.scoreValue);
+        space = getAttribute(in, "space", parent.space);
+        hitPoints = getAttribute(in, "hitPoints", parent.hitPoints);
+        spaceTaken = getAttribute(in, "spaceTaken", parent.spaceTaken);
+        maximumExperience = getAttribute(in, "maximumExperience",
+            parent.maximumExperience);
+        maximumAttrition = getAttribute(in, "maximumAttrition",
+            parent.maximumAttrition);
+        String skillString = in.getAttributeValue(null, "skillTaught");
+        skillTaught = (skillString == null) ? this
+            : getSpecification().getUnitType(skillString);
+
+        recruitProbability = getAttribute(in, "recruitProbability",
+            parent.recruitProbability);
+        skill = getAttribute(in, "skill", parent.skill);
+
+        setPopulationRequired(getAttribute(in, "population-required",
+                parent.getPopulationRequired()));
+
+        price = getAttribute(in, "price", parent.price);
+
+        expertProduction = getSpecification().getType(in, "expert-production",
+            GoodsType.class, parent.expertProduction);
+
+        if (parent != this) {
+            typeChanges.addAll(parent.typeChanges);
+            defaultEquipment = parent.defaultEquipment;
+            consumption.putAll(parent.consumption);
+            getFeatureContainer().add(parent.getFeatureContainer());
+            if (parent.isAbstractType()) {
+                getFeatureContainer().replaceSource(parent, this);
+            }
+        }
+    }
+
+    /**
+     * Reads a child object.
+     *
+     * @param in The XML stream to read.
+     * @exception XMLStreamException if an error occurs
+     */
+    @Override
+    protected void readChild(XMLStreamReader in) throws XMLStreamException {
+        String nodeName = in.getLocalName();
+        Specification spec = getSpecification();
+        if ("downgrade".equals(nodeName)
+            || "upgrade".equals(nodeName)) {
+            if (getAttribute(in, "delete", false)) {
+                String unitId = in.getAttributeValue(null, "unit");
+                Iterator<UnitTypeChange> iterator = typeChanges.iterator();
+                while (iterator.hasNext()) {
+                    if (unitId.equals(iterator.next()
+                            .getNewUnitType().getId())) {
+                        iterator.remove();
+                        break;
+                    }
+                }
+                in.nextTag();
+            } else {
+                UnitTypeChange change = new UnitTypeChange(in, spec);
+                if ("downgrade".equals(nodeName)
+                    && change.getChangeTypes().isEmpty()) {
+                    // add default downgrade type
+                    change.getChangeTypes().put(ChangeType.CLEAR_SKILL, 100);
+                }
+                typeChanges.add(change);
+            }
+        } else if ("default-equipment".equals(nodeName)) {
+            String equipmentString
+                = in.getAttributeValue(null, ID_ATTRIBUTE_TAG);
+            if (equipmentString != null) {
+                defaultEquipment = spec.getEquipmentType(equipmentString);
+            }
+            in.nextTag(); // close this element
+        } else if ("consumes".equals(nodeName)) {
+            String typeString = in.getAttributeValue(null, ID_ATTRIBUTE_TAG);
+            String valueString = in.getAttributeValue(null, VALUE_TAG);
+            if (typeString != null && valueString != null) {
+                try {
+                    GoodsType type = spec.getGoodsType(typeString);
+                    int amount = Integer.parseInt(valueString);
+                    consumption.incrementCount(type, amount);
+                } catch(Exception e) {
+                    logger.warning("Failed to parse integer " + valueString);
+                }
+            }
+            in.nextTag(); // close this element
+        } else {
+            super.readChild(in);
+        }
+    }
+
+    /**
+     * Debug print helper.
+     */
+    public String toString() {
+        return getId();
+    }
+
+    /**
+     * Returns the tag name of the root element representing this object.
+     *
+     * @return "unit-type".
+     */
+    public static String getXMLElementTagName() {
+        return "unit-type";
+    }
 }
