@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -40,8 +40,11 @@ import javax.xml.transform.TransformerFactory;
 import javax.xml.transform.dom.DOMSource;
 import javax.xml.transform.stream.StreamResult;
 
+import net.sf.freecol.FreeCol;
+
 import org.w3c.dom.Element;
 
+
 /**
  * A network connection. Responsible for both sending and receiving network
  * messages.
@@ -74,18 +77,19 @@ public class Connection {
 
     private int currentQuestionID = -1;
 
-    private String threadName;
+    private String name = null;
 
 
     /**
-     * Dead constructor, for DummyConnection purposes.
+     * Trivial constructor for DummyConnection to use.
      */
-    protected Connection() {
+    protected Connection(String name) {
         out = null;
         in = null;
         socket = null;
         thread = null;
         xmlTransformer = null;
+        this.name = name;
     }
 
     /**
@@ -98,8 +102,9 @@ public class Connection {
      *            received.
      * @throws IOException
      */
-    public Connection(String host, int port, MessageHandler messageHandler, String threadName) throws IOException {
-        this(createSocket(host, port), messageHandler, threadName);
+    public Connection(String host, int port, MessageHandler messageHandler,
+                      String name) throws IOException {
+        this(createSocket(host, port), messageHandler, name);
     }
 
     /**
@@ -111,10 +116,11 @@ public class Connection {
      *            received.
      * @throws IOException
      */
-    public Connection(Socket socket, MessageHandler messageHandler, String threadName) throws IOException {
+    public Connection(Socket socket, MessageHandler messageHandler,
+                      String name) throws IOException {
         this.messageHandler = messageHandler;
         this.socket = socket;
-        this.threadName = threadName;
+        this.name = name;
 
         out = socket.getOutputStream();
         in = socket.getInputStream();
@@ -130,7 +136,7 @@ public class Connection {
         }
         xmlTransformer = myTransformer;
 
-        thread = new ReceivingThread(this, in, threadName);
+        thread = new ReceivingThread(this, in, name);
         thread.start();
     }
 
@@ -142,13 +148,32 @@ public class Connection {
         return socket;
     }
 
+    /**
+     * Gets the socket.
+     *
+     * @return The <code>Socket</code> used while communicating with the other
+     *         peer.
+     */
+    public Socket getSocket() {
+        return socket;
+    }
+
+    /**
+     * Gets the connection name.
+     *
+     * @return The connection name.
+     */
+    public String getName() {
+        return name;
+    }
+
     /**
      * Sends a "disconnect"-message and closes this connection.
      *
      * @throws IOException
      */
     public void close() throws IOException {
-        Element disconnectElement = Message.createNewRootElement("disconnect");
+        Element disconnectElement = DOMMessage.createNewRootElement("disconnect");
         send(disconnectElement);
 
         reallyClose();
@@ -240,7 +265,7 @@ public class Connection {
         } else {
             NetworkReplyObject nro = thread.waitForNetworkReply(networkReplyId);
             send(questionElement);
-            Message response = (Message) nro.getResponse();
+            DOMMessage response = (DOMMessage) nro.getResponse();
             if (response == null) return null;
             Element rootElement = response.getDocument().getDocumentElement();
             return (Element) rootElement.getFirstChild();
@@ -492,7 +517,7 @@ public class Connection {
             if (!messagedConsumed) {
                 xmlIn.close();
                 in.reset();
-                final Message msg = new Message(in);
+                final DOMMessage msg = new DOMMessage(in);
 
                 final Connection connection = this;
                 Thread t = new Thread(msg.getType()) {
@@ -505,7 +530,7 @@ public class Connection {
                                 Element reply = messageHandler.handle(connection, (Element) element.getFirstChild());
 
                                 if (reply == null) {
-                                    reply = Message.createNewRootElement("reply");
+                                    reply = DOMMessage.createNewRootElement("reply");
                                     reply.setAttribute("networkReplyId", networkReplyId);
                                     logger.finest("reply == null");
                                 } else {
@@ -529,7 +554,7 @@ public class Connection {
                         }
                     }
                 };
-                t.setName(threadName+"MessageHandler:" + t.getName());
+                t.setName(name + "MessageHandler:" + t.getName());
                 t.start();
             }
         } catch (Exception e) {
@@ -568,13 +593,45 @@ public class Connection {
      */
 
     /**
-     * Gets the socket.
+     * Dumping version of ask().
+     * Dumps to System.err with a faked-XML prefix so the whole line can
+     * be fed to an XML-pretty printer if required.
      *
-     * @return The <code>Socket</code> used while communicating with the other
-     *         peer.
+     * @param request The <code>Element</code> to send.
+     * @return The reply element.
+     * @exception Throws IOException if ask() fails.
      */
-    public Socket getSocket() {
-        return socket;
+    public Element askDumping(Element request) throws IOException {
+        boolean dump = FreeCol.getDebugLevel() >= FreeCol.DEBUG_FULL_COMMS;
+        if (dump) {
+            try {
+                System.err.println("<" + getName() + "-request>"
+                    + DOMMessage.elementToString(request) + "\n");
+            } catch (Exception e) {}
+        }
+
+        Element reply;
+        if (dump) {
+            try {
+                reply = ask(request);
+                try {
+                    System.err.println("<" + getName() + "-reply>"
+                        + ((reply == null) ? ""
+                            : DOMMessage.elementToString(reply))
+                        + "\n");
+                } catch (Exception x) {}
+            } catch (IOException e) {
+                try {
+                    System.err.println("<" + getName() + "-reply><exception "
+                        + e.getMessage() + "\n");
+                } catch (Exception x) {}
+                throw e;
+            }
+        } else {
+            reply = ask(request);
+        }
+
+        return reply;
     }
 
     /**
