@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -33,8 +33,6 @@ import java.util.logging.Logger;
 import javax.swing.ImageIcon;
 
 import net.sf.freecol.FreeCol;
-import net.sf.freecol.client.gui.FAFile;
-import net.sf.freecol.client.gui.video.Video;
 import net.sf.freecol.common.io.sza.SimpleZippedAnimation;
 
 
@@ -418,6 +416,19 @@ public class ResourceManager {
         return (r != null) ? r.getColor() : null;
     }
 
+    /**
+     * Returns the <code>Color</code> for the given production bonus.
+     *
+     * @param bonus The production bonus to look up.
+     * @return An <code>Color</code> created with the image
+     *      identified by <code>resource</code> or
+     *      <code>null</code> if there is no color identified
+     *      by that name.
+     */
+    public static Color getProductionColor(int bonus) {
+        return ResourceManager.getColor("productionBonus." + bonus + ".color");
+    }
+
     /**
      * Returns the <code>Image</code> with the given name.
      *
@@ -517,5 +528,4 @@ public class ResourceManager {
         final AudioResource r = getResource(resource, AudioResource.class);
         return (r == null) ? null : r.getAudio();
     }
-
 }
