@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -17,9 +17,9 @@
  *  along with FreeCol.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
 package net.sf.freecol.server.ai;
 
+import java.util.Comparator;
 import java.util.logging.Logger;
 
 import javax.xml.stream.XMLStreamException;
@@ -29,6 +29,7 @@ import javax.xml.stream.XMLStreamWriter;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.Europe;
 import net.sf.freecol.common.model.Goods;
+import net.sf.freecol.common.model.GoodsContainer;
 import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.Locatable;
 import net.sf.freecol.common.model.Location;
@@ -39,21 +40,28 @@ import org.w3c.dom.Element;
 
 
 /**
-* Objects of this class contains AI-information for a single {@link Goods}.
-*/
+ * Objects of this class contains AI-information for a single {@link Goods}.
+ */
 public class AIGoods extends AIObject implements Transportable {
+
     private static final Logger logger = Logger.getLogger(AIGoods.class.getName());
 
+    private static final Comparator<AIGoods> aiGoodsPriorityComparator
+        = new Comparator<AIGoods>() {
+            public int compare(AIGoods g1, AIGoods g2) {
+                return g2.getTransportPriority() - g1.getTransportPriority();
+            }
+        };
 
     public static final int IMPORTANT_DELIVERY = 110;
     public static final int FULL_DELIVERY = 100;
-    
+
     /**
      * The priority of tools intended for a Colony with none stored
      * at the present (and with no special needs).
      */
     public static final int TOOLS_FOR_COLONY_PRIORITY = 10;
-    
+
     /**
      * The extra priority value added to the base value of
      * {@link #TOOLS_FOR_COLONY_PRIORITY}
@@ -75,7 +83,7 @@ public class AIGoods extends AIObject implements Transportable {
      * is also added to the total amount.
      */
     public static final int TOOLS_FOR_BUILDING = 100;
-    
+
     private Goods goods;
     private Location destination;
     private int transportPriority;
@@ -84,7 +92,7 @@ public class AIGoods extends AIObject implements Transportable {
 
     /**
      * Creates a new <code>AIGoods</code>.
-     * 
+     *
      * @param aiMain The main AI-object.
      * @param location The location of the goods.
      * @param type The type of goods.
@@ -102,42 +110,52 @@ public class AIGoods extends AIObject implements Transportable {
 
     /**
      * Creates a new <code>AIGoods</code>.
-     * 
+     *
      * @param aiMain The main AI-object.
      * @param element An <code>Element</code> containing an
      *      XML-representation of this object.
-     */    
+     */
     public AIGoods(AIMain aiMain, Element element) {
-        super(aiMain, element.getAttribute("ID"));
+        super(aiMain, element.getAttribute(ID_ATTRIBUTE));
         readFromXMLElement(element);
     }
-    
+
     /**
      * Creates a new <code>AIGoods</code>.
-     * 
+     *
      * @param aiMain The main AI-object.
      * @param in The input stream containing the XML.
      * @throws XMLStreamException if a problem was encountered
      *      during parsing.
-     */    
+     */
     public AIGoods(AIMain aiMain, XMLStreamReader in) throws XMLStreamException {
-        super(aiMain, in.getAttributeValue(null, "ID"));
+        super(aiMain, in.getAttributeValue(null, ID_ATTRIBUTE));
         readFromXML(in);
     }
-    
+
     /**
      * Creates a new <code>AIGoods</code>.
-     * 
+     *
      * @param aiMain The main AI-object.
      * @param id The unique ID of this object.
-     */    
+     */
     public AIGoods(AIMain aiMain, String id) {
         super(aiMain, id);
         uninitialized = true;
     }
 
+    /**
+     * Gets a comparator that sorts by priority.
+     *
+     * @return A priority comparator.
+     */
+    public static Comparator<AIGoods> getAIGoodsPriorityComparator() {
+        return aiGoodsPriorityComparator;
+    }
+
     /**
      * Aborts the given <code>Wish</code>.
+     *
      * @param w The <code>Wish</code> to be aborted.
      */
     public void abortWish(Wish w) {
@@ -150,73 +168,71 @@ public class AIGoods extends AIObject implements Transportable {
     }
 
     /**
-    * Returns the source for this <code>Transportable</code>.
-    * This is normally the location of the
-    * {@link #getTransportLocatable locatable}.
-    *
-    * @return The source for this <code>Transportable</code>.
-    */
+     * Returns the source for this <code>Transportable</code>.
+     * This is normally the location of the
+     * {@link #getTransportLocatable locatable}.
+     *
+     * @return The source for this <code>Transportable</code>.
+     */
     public Location getTransportSource() {
         return goods.getLocation();
     }
 
 
     /**
-    * Returns the destination for this <code>Transportable</code>.
-    * This can either be the target {@link Tile} of the transport
-    * or the target for the entire <code>Transportable</code>'s
-    * mission. The target for the tansport is determined by
-    * {@link TransportMission} in the latter case.
-    *
-    * @return The destination for this <code>Transportable</code>.
-    */
+     * Returns the destination for this <code>Transportable</code>.
+     * This can either be the target {@link Tile} of the transport
+     * or the target for the entire <code>Transportable</code>'s
+     * mission. The target for the tansport is determined by
+     * {@link TransportMission} in the latter case.
+     *
+     * @return The destination for this <code>Transportable</code>.
+     */
     public Location getTransportDestination() {
         return destination;
     }
-    
 
     /**
-    * Gets the <code>Locatable</code> which should be transported.
-    * @return The <code>Locatable</code>.
-    */
+     * Gets the <code>Locatable</code> which should be transported.
+     * @return The <code>Locatable</code>.
+     */
     public Locatable getTransportLocatable() {
         return getGoods();
     }
 
-
     /**
-    * Gets the priority of transporting this <code>Transportable</code>
-    * to it's destination.
-    *
-    * @return The priority of the transport.
-    */
+     * Gets the priority of transporting this <code>Transportable</code>
+     * to it's destination.
+     *
+     * @return The priority of the transport.
+     */
     public int getTransportPriority() {
-        if (goods.getAmount() <= 100) {
+        if (goods.getAmount() <= GoodsContainer.CARGO_SIZE) {
             return goods.getAmount();
         } else {
             return transportPriority;
         }
     }
 
-    
     /**
-    * Increases the transport priority of this <code>Transportable</code>.
-    * This method gets called every turn the <code>Transportable</code>
-    * have not been put on a carrier's transport list.
-    */    
+     * Increases the transport priority of this <code>Transportable</code>.
+     * This method gets called every turn the <code>Transportable</code>
+     * have not been put on a carrier's transport list.
+     */
     public void increaseTransportPriority() {
         transportPriority++;
     }
 
-    
     /**
-    * Gets the carrier responsible for transporting this <code>Transportable</code>.
-    *
-    * @return The <code>AIUnit</code> which has this <code>Transportable</code>
-    *         in it's transport list. This <code>Transportable</code> has not been
-    *         scheduled for transport if this value is <code>null</code>.
-    *
-    */
+     * Gets the carrier responsible for transporting this
+     * <code>Transportable</code>.
+     *
+     * @return The <code>AIUnit</code> which has this
+     *         <code>Transportable</code> in it's transport list. This
+     *         <code>Transportable</code> has not been scheduled for
+     *         transport if this value is <code>null</code>.
+     *
+     */
     public AIUnit getTransport() {
         return transport;
     }
@@ -237,19 +253,22 @@ public class AIGoods extends AIObject implements Transportable {
         }
         super.dispose();
     }
-    
+
     /**
-    * Sets the carrier responsible for transporting this <code>Transportable</code>.
-    *
-    * @param transport The <code>AIUnit</code> which has this <code>Transportable</code>
-    *         in it's transport list. This <code>Transportable</code> has not been
-    *         scheduled for transport if this value is <code>null</code>.
-    *
-    */
+     * Sets the carrier responsible for transporting this
+     * <code>Transportable</code>.
+     *
+     * @param transport The <code>AIUnit</code> which has this
+     *         <code>Transportable</code> in it's transport list. This
+     *         <code>Transportable</code> has not been scheduled for
+     *         transport if this value is <code>null</code>.
+     *
+     */
     public void setTransport(AIUnit transport) {
+        if (this.transport == transport) return;
         AIUnit oldTransport = this.transport;
         this.transport = transport;
-        
+
         if (oldTransport != null) {
             // Remove from old carrier:
             if (oldTransport.getMission() != null
@@ -260,7 +279,7 @@ public class AIGoods extends AIObject implements Transportable {
                 }
             }
         }
-            
+
         if (transport != null
                 && transport.getMission() instanceof TransportMission
                 && !((TransportMission) transport.getMission()).isOnTransportList(this)) {
@@ -268,21 +287,24 @@ public class AIGoods extends AIObject implements Transportable {
             ((TransportMission) transport.getMission()).addToTransportList(this);
         }
     }
-    
+
 
     /**
-     * Sets the priority of getting the goods to the {@link #getTransportDestination}.
+     * Sets the priority of getting the goods to the {@link
+     * #getTransportDestination}.
+     *
      * @param transportPriority The priority.
      */
     public void setTransportPriority(int transportPriority) {
         this.transportPriority = transportPriority;
     }
 
-    
+
     /**
-    * Gets the goods this <code>AIGoods</code> is controlling.
-    * @return The <code>Goods</code>.
-    */
+     * Gets the goods this <code>AIGoods</code> is controlling.
+     *
+     * @return The <code>Goods</code>.
+     */
     public Goods getGoods() {
         return goods;
     }
@@ -290,16 +312,17 @@ public class AIGoods extends AIObject implements Transportable {
 
     /**
      * Sets the goods this <code>AIGoods</code> is controlling.
+     *
      * @param goods The <code>Goods</code>.
-     */    
-    public void setGoods(Goods goods) {        
+     */
+    public void setGoods(Goods goods) {
         if (goods == null) {
             throw new NullPointerException();
         }
         this.goods = goods;
     }
-    
-    
+
+
     /**
      * Writes this object to an XML stream.
      *
@@ -308,9 +331,11 @@ public class AIGoods extends AIObject implements Transportable {
      *      to the stream.
      */
     protected void toXMLImpl(XMLStreamWriter out) throws XMLStreamException {
+        if (goods == null) return; // Workaround for BR#3456180
+
         out.writeStartElement(getXMLElementTagName());
 
-        out.writeAttribute("ID", getId());
+        out.writeAttribute(ID_ATTRIBUTE, getId());
         if (destination != null) {
             out.writeAttribute("destination", destination.getId());
         }
@@ -333,12 +358,14 @@ public class AIGoods extends AIObject implements Transportable {
 
     /**
      * Reads information for this object from an XML stream.
+     *
      * @param in The input stream with the XML.
      * @throws XMLStreamException if there are any problems reading
      *      from the stream.
      */
-    protected void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException {
-        setId(in.getAttributeValue(null, "ID"));
+    protected void readFromXMLImpl(XMLStreamReader in)
+        throws XMLStreamException {
+        setId(in.getAttributeValue(null, ID_ATTRIBUTE));
         final String destinationStr = in.getAttributeValue(null, "destination");
         if (destinationStr != null) {
             destination = (Location) getAIMain().getFreeColGameObject(destinationStr);
@@ -359,7 +386,7 @@ public class AIGoods extends AIObject implements Transportable {
         } else {
             transport = null;
         }
-        
+
         in.nextTag();
 
         if (goods != null) {
@@ -370,20 +397,23 @@ public class AIGoods extends AIObject implements Transportable {
         in.nextTag();
     }
 
-    
     /**
      * Returns a <code>String</code>-representation of this object.
-     * @return A <code>String</code> representing this objecy for debugging purposes.
+     *
+     * @return A <code>String</code> representing this object for
+     *     debugging purposes.
      */
+    @Override
     public String toString() {
-        return "AIGoods@" + hashCode() + ": " + goods + " (" + transportPriority + ")";
+        return "AIGoods@" + hashCode() + "[ " + goods
+            + " /" + transportPriority + "]";
     }
-    
 
     /**
-    * Returns the tag name of the root element representing this object.
-    * @return "aiGoods"
-    */
+     * Returns the tag name of the root element representing this object.
+     *
+     * @return "aiGoods"
+     */
     public static String getXMLElementTagName() {
         return "aiGoods";
     }
