@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -17,14 +17,13 @@
  *  along with FreeCol.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
 package net.sf.freecol.server.ai;
 
 import java.io.PrintWriter;
 import java.io.StringWriter;
+import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Iterator;
-import java.util.Random;
 import java.util.logging.Logger;
 
 import javax.xml.stream.XMLStreamConstants;
@@ -32,7 +31,6 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
-import net.sf.freecol.FreeCol;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.FreeColGameObject;
 import net.sf.freecol.common.model.FreeColGameObjectListener;
@@ -40,30 +38,29 @@ import net.sf.freecol.common.model.FreeColObject;
 import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.util.Utils;
 import net.sf.freecol.server.FreeColServer;
 import net.sf.freecol.server.model.ServerPlayer;
 
 import org.w3c.dom.Element;
 
-/**
-* The main AI-class. Keeps references to all other AI-classes.
-*/
-public class AIMain extends FreeColObject implements FreeColGameObjectListener {
-    private static final Logger logger = Logger.getLogger(AIMain.class.getName());
 
+/**
+ * The main AI-class. Keeps references to all other AI-classes.
+ */
+public class AIMain extends FreeColObject
+    implements FreeColGameObjectListener {
+    private static final Logger logger = Logger.getLogger(AIMain.class.getName());
 
     private FreeColServer freeColServer;
     private int nextID = 1;
 
     /**
-    * Contains mappings between <code>FreeColGameObject</code>s
-    * and <code>AIObject</code>s.
-    */
+     * Contains mappings between <code>FreeColGameObject</code>s
+     * and <code>AIObject</code>s.
+     */
     private HashMap<String, AIObject> aiObjects = new HashMap<String, AIObject>();
 
-
-
-
     /**
     * Creates a new <code>AIMain</code> and searches the current
     * game for <code>FreeColGameObject</code>s.
@@ -76,13 +73,13 @@ public class AIMain extends FreeColObject implements FreeColGameObjectListener {
         this.freeColServer = freeColServer;
         findNewObjects();
     }
-    
+
     /**
     * Creates a new <code>AIMain</code> and reads the given element.
-    * 
+    *
     * @param freeColServer The main controller object for the
     *       server.
-    * @param element The <code>Element</code> (in a DOM-parsed XML-tree) 
+    * @param element The <code>Element</code> (in a DOM-parsed XML-tree)
     *       that describes this object.
     * @see #readFromXMLElement
     */
@@ -93,7 +90,7 @@ public class AIMain extends FreeColObject implements FreeColGameObjectListener {
 
     /**
      * Creates a new <code>AIMain</code> and reads the given element.
-     * 
+     *
      * @param freeColServer The main controller object for the
      *       server.
      * @param in The input stream containing the XML.
@@ -105,7 +102,7 @@ public class AIMain extends FreeColObject implements FreeColGameObjectListener {
          this(freeColServer);
          readFromXML(in);
      }
-     
+
 
     /**
      * Gets the main controller object for the server.
@@ -125,21 +122,51 @@ public class AIMain extends FreeColObject implements FreeColGameObjectListener {
         return id;
     }
 
+    /**
+     * Gets a random value from the server to use for individual AI player
+     * PRNG seeds.
+     *
+     * @param logMe A logging string.
+     * @return A random seed.
+     */
+    public int getRandomSeed(String logMe) {
+        return Utils.randomInt(logger, logMe, freeColServer.getServerRandom(),
+            Integer.MAX_VALUE);
+    }
+
     /**
      * Checks the integrity of this <code>AIMain</code>
      * by checking if there are any
      * {@link AIObject#isUninitialized() uninitialized objects}.
-     * 
+     *
      * Detected problems gets written to the log.
-     * 
+     *
      * @return <code>true</code> if the <code>Game</code> has
      *      been loaded properly.
      */
     public boolean checkIntegrity() {
+        // Workaround for BR#3456180.
+        // Remove when GoodsWishes are working properly again.
+        for (AIObject ao : new ArrayList<AIObject>(aiObjects.values())) {
+            if (ao instanceof GoodsWish) {
+                GoodsWish gw = (GoodsWish)ao;
+                Transportable tr = gw.getTransportable();
+                if (tr != null
+                    && tr instanceof AIGoods
+                    && ((AIGoods)tr).isUninitialized()) {
+                    gw.setTransportable(null);
+                    ((AIGoods)tr).dispose();
+                    logger.warning("Dropping bad GoodsWish: " + gw);
+                    gw.dispose();                    
+                }
+            }
+        }
+
         boolean ok = true;
         for (AIObject ao : aiObjects.values()) {
             if (ao.isUninitialized()) {
-                logger.warning("Uninitialized object: " + ao.getId() + " (" + ao.getClass() + ")");
+                logger.warning("Uninitialized object: " + ao.getId()
+                    + " (" + ao.getClass() + ")");
                 ok = false;
             }
         }
@@ -171,16 +198,6 @@ public class AIMain extends FreeColObject implements FreeColGameObjectListener {
     }
 
 
-    /**
-     * Gets the random number generator to be used in the AI.
-     *
-     * @return The AI random number generator.
-     */
-    public Random getAIRandom() {
-        return freeColServer.getServerRandom();
-    }
-
-
     /**
     * Searches for new {@link FreeColGameObject FreeColGameObjects}. An AI-object is
     * created for each object.
@@ -232,7 +249,7 @@ public class AIMain extends FreeColObject implements FreeColGameObjectListener {
      * @see #getAIObject(FreeColGameObject)
      * @return The <code>AIObject</code>.
      */
-    public AIObject getAIObject(String id) {        
+    public AIObject getAIObject(String id) {
         return aiObjects.get(id);
     }
 
@@ -340,16 +357,13 @@ public class AIMain extends FreeColObject implements FreeColGameObjectListener {
         if (freeColGameObject instanceof Unit) {
             new AIUnit(this, (Unit) freeColGameObject);
         } else if (freeColGameObject instanceof ServerPlayer) {
-            if (FreeCol.usesExperimentalAI()) {
-                ServerPlayer p = (ServerPlayer) freeColGameObject;
-                if (!p.isREF() && !p.isIndian() && p.isEuropean()) {
-                    logger.info("Using experimental ColonialAIPlayer for "+p.getName());
-                    new ColonialAIPlayer(this, p);
-                } else {
-                    new StandardAIPlayer(this, p);
-                }
-            } else {
-                new StandardAIPlayer(this, (ServerPlayer) freeColGameObject);
+            ServerPlayer p = (ServerPlayer) freeColGameObject;
+            if (p.isIndian()) {
+                new NativeAIPlayer(this, p);
+            } else if (p.isREF()) {
+                new REFAIPlayer(this, p);
+            } else if (p.isEuropean()) {
+                new EuropeanAIPlayer(this, p);
             }
         } else if (freeColGameObject instanceof Colony) {
             new AIColony(this, (Colony) freeColGameObject);
@@ -370,7 +384,35 @@ public class AIMain extends FreeColObject implements FreeColGameObjectListener {
     }
 
     /**
-     * Writes all of the <code>AIObject</code>s and other AI-related 
+     * Computes how many objects of each class have been created, to
+     * track memory leaks over time
+     */
+    public HashMap<String, String> getAIStatistics() {
+        HashMap<String, String> stats = new HashMap<String, String>();
+        HashMap<String, Long> objStats = new HashMap<String, Long>();
+        Iterator<AIObject> iter = aiObjects.values().iterator();
+        while (iter.hasNext()) {
+            AIObject obj = iter.next();
+            String className = obj.getClass().getSimpleName();
+            if (objStats.containsKey(className)) {
+                Long count = objStats.get(className);
+                count++;
+                objStats.put(className, count);
+            } else {
+                Long count = new Long(1);
+                objStats.put(className, count);
+            }
+        }
+        for (String k : objStats.keySet()) {
+            stats.put(k, Long.toString(objStats.get(k)));
+        }
+
+        return stats;
+    }
+
+
+    /**
+     * Writes all of the <code>AIObject</code>s and other AI-related
      * information to an XML-stream.
      *
      * @param out The target stream.
@@ -378,14 +420,37 @@ public class AIMain extends FreeColObject implements FreeColGameObjectListener {
      *      to the stream.
      */
     protected void toXMLImpl(XMLStreamWriter out) throws XMLStreamException {
-        out.writeStartElement(getXMLElementTagName());
+        super.toXML(out, getXMLElementTagName());
+    }
+
+    /**
+     * Write the attributes of this object to a stream.
+     *
+     * @param out The target stream.
+     * @throws XMLStreamException if there are any problems writing
+     *     to the stream.
+     */
+    @Override
+    protected void writeAttributes(XMLStreamWriter out)
+        throws XMLStreamException {
+        super.writeAttributes(out);
 
         out.writeAttribute("nextID", Integer.toString(nextID));
+    }
 
-        Iterator<AIObject> i = aiObjects.values().iterator();
-        while (i.hasNext()) {
-            AIObject aio = i.next();
-            
+    /**
+     * Write the children of this object to a stream.
+     *
+     * @param out The target stream.
+     * @throws XMLStreamException if there are any problems writing
+     *     to the stream.
+     */
+    @Override
+    protected void writeChildren(XMLStreamWriter out)
+        throws XMLStreamException {
+        super.writeChildren(out);
+
+        for (AIObject aio : new ArrayList<AIObject>(aiObjects.values())) {
             if ((aio instanceof Wish) && !((Wish) aio).shouldBeStored()) {
                 continue;
             }
@@ -394,7 +459,8 @@ public class AIMain extends FreeColObject implements FreeColGameObjectListener {
                 if (aio.getId() != null) {
                     aio.toXML(out);
                 } else {
-                    logger.warning("aio.getId() == null, for: " + aio.getClass().getName());
+                    logger.warning("aio.getId() == null, for: "
+                        + aio.getClass().getName());
                 }
             } catch (Exception e) {
                 StringWriter sw = new StringWriter();
@@ -402,19 +468,19 @@ public class AIMain extends FreeColObject implements FreeColGameObjectListener {
                 logger.warning(sw.toString());
             }
         }
-
-        out.writeEndElement();
     }
 
     /**
      * Reads all the <code>AIObject</code>s and other AI-related information
      * from XML data.
+     *
      * @param in The input stream with the XML.
      * @throws XMLStreamException if an error occured during parsing.
      */
-    protected void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException {
+    protected void readFromXMLImpl(XMLStreamReader in)
+        throws XMLStreamException {
         aiObjects.clear();
-        
+
         if (!in.getLocalName().equals(getXMLElementTagName())) {
             logger.warning("Expected element name, got: " + in.getLocalName());
         }
@@ -422,85 +488,84 @@ public class AIMain extends FreeColObject implements FreeColGameObjectListener {
         if (nextIDStr != null) {
             nextID = Integer.parseInt(nextIDStr);
         }
-        
+
         String lastTag = "";
         while (in.nextTag() != XMLStreamConstants.END_ELEMENT) {
             final String tagName = in.getLocalName();
-            try {         
-                final String oid = in.getAttributeValue(null, "ID");
+            final String oid = in.getAttributeValue(null, ID_ATTRIBUTE);
+            try {
                 if (oid != null && aiObjects.containsKey(oid)) {
                     getAIObject(oid).readFromXML(in);
                 } else if (tagName.equals(AIUnit.getXMLElementTagName())) {
                     new AIUnit(this, in);
                 } else if (tagName.equals(AIPlayer.getXMLElementTagName())) {
-                    new StandardAIPlayer(this, in);
-                } else if (tagName.equals(ColonialAIPlayer.getXMLElementTagName())) {
-                    new ColonialAIPlayer(this, in);
+                    Player p = (Player) getGame().getFreeColGameObject(oid);
+                    if (p != null) {
+                        if (p.isIndian()) {
+                            new NativeAIPlayer(this, in);
+                        } else if (p.isREF()) {
+                            new REFAIPlayer(this, in);
+                        } else if (p.isEuropean()) {
+                            new EuropeanAIPlayer(this, in);
+                        } else {
+                            logger.warning("Bogus AIPlayer: " + p);
+                            in.nextTag();
+                        }
+                    }
+                // @compat 0.10.1
+                } else if (tagName.equals("colonialAIPlayer")) {
+                    new EuropeanAIPlayer(this, in);
+                // end compatibility code
                 } else if (tagName.equals(AIColony.getXMLElementTagName())) {
                     new AIColony(this, in);
                 } else if (tagName.equals(AIGoods.getXMLElementTagName())) {
                     new AIGoods(this, in);
                 } else if (tagName.equals(WorkerWish.getXMLElementTagName())) {
                     new WorkerWish(this, in);
-                } else if (tagName.equals(GoodsWish.getXMLElementTagName())) {
+                } else if (tagName.equals(GoodsWish.getXMLElementTagName())
+                    // @compat 0.10.3
+                    || tagName.equals("GoodsWish")
+                    // end compatibility code
+                           ) {
                     new GoodsWish(this, in);
-                } else if (tagName.equals(TileImprovementPlan.getXMLElementTagName())) {
-                    new TileImprovementPlan(this, in);                
+                } else if (tagName.equals(TileImprovementPlan.getXMLElementTagName())
+                    // @compat 0.10.3
+                    || tagName.equals("tileimprovementplan")
+                    // end compatibility code
+                           ) {
+                    new TileImprovementPlan(this, in);
                 } else {
-                    logger.warning("Unkown AI-object read: " + tagName + "(" + lastTag + ")");
+                    logger.warning("Unknown AI-object read: " + tagName + "(" + lastTag + ")");
                 }
                 lastTag = in.getLocalName();
             } catch (Exception e) {
                 StringWriter sw = new StringWriter();
                 e.printStackTrace(new PrintWriter(sw));
-                logger.warning("Exception while reading an AIObject: " + sw.toString());
+                logger.warning("Exception while reading an AIObject(" + tagName
+                    + ", " + oid + "): " + sw.toString());
                 while (!in.getLocalName().equals(tagName) && !in.getLocalName().equals(getXMLElementTagName())) {
-                    in.nextTag();                    
+                    in.nextTag();
                 }
                 if (!in.getLocalName().equals(getXMLElementTagName())) {
                     in.nextTag();
                 }
             }
         }
-        
+
         if (!in.getLocalName().equals(getXMLElementTagName())) {
             logger.warning("Expected element name (2), got: " + in.getLocalName());
         }
-        
+
         // This should not be necessary - but just in case:
         findNewObjects(false);
     }
 
-
     /**
-    * Returns the tag name of the root element representing this object.
-    * @return "aiMain"
-    */
+     * Returns the tag name of the root element representing this object.
+     *
+     * @return "aiMain"
+     */
     public static String getXMLElementTagName() {
         return "aiMain";
     }
-    
-    /**
-     * Computes how many objects of each class have been created, 
-     * to track memory leaks over time
-     */
-    public HashMap<String, Long> getAIStatistics() {
-        
-        HashMap<String, Long> map = new HashMap<String, Long>();
-        Iterator<AIObject> iter = aiObjects.values().iterator();
-        while (iter.hasNext()) {
-            AIObject obj = iter.next();
-            String className = obj.getClass().getSimpleName();
-            if (map.containsKey(className)) {
-                Long count = map.get(className);
-                count++;
-                map.put(className, count);
-            } else {
-                Long count = new Long(1);
-                map.put(className, count);
-            }
-        }
-        
-        return map;
-    }
 }
