@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -29,6 +29,7 @@ import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.EquipmentType;
 import net.sf.freecol.common.model.Goods;
 import net.sf.freecol.common.model.GoodsType;
+import net.sf.freecol.common.model.Location;
 import net.sf.freecol.common.model.Map.Direction;
 import net.sf.freecol.common.model.Settlement;
 import net.sf.freecol.common.model.Tile;
@@ -48,6 +49,7 @@ import net.sf.freecol.common.networking.ClaimLandMessage;
 import net.sf.freecol.common.networking.ClearSpecialityMessage;
 import net.sf.freecol.common.networking.CloseTransactionMessage;
 import net.sf.freecol.common.networking.Connection;
+import net.sf.freecol.common.networking.DOMMessage;
 import net.sf.freecol.common.networking.DeliverGiftMessage;
 import net.sf.freecol.common.networking.DisembarkMessage;
 import net.sf.freecol.common.networking.EmbarkMessage;
@@ -56,11 +58,10 @@ import net.sf.freecol.common.networking.EquipUnitMessage;
 import net.sf.freecol.common.networking.GetTransactionMessage;
 import net.sf.freecol.common.networking.IndianDemandMessage;
 import net.sf.freecol.common.networking.LoadCargoMessage;
-import net.sf.freecol.common.networking.Message;
+import net.sf.freecol.common.networking.LootCargoMessage;
 import net.sf.freecol.common.networking.MissionaryMessage;
 import net.sf.freecol.common.networking.MoveMessage;
-import net.sf.freecol.common.networking.MoveToAmericaMessage;
-import net.sf.freecol.common.networking.MoveToEuropeMessage;
+import net.sf.freecol.common.networking.MoveToMessage;
 import net.sf.freecol.common.networking.PutOutsideColonyMessage;
 import net.sf.freecol.common.networking.ScoutIndianSettlementMessage;
 import net.sf.freecol.common.networking.SellGoodsMessage;
@@ -76,10 +77,43 @@ import org.w3c.dom.Element;
  * Wrapper class for AI message handling.
  */
 public class AIMessage {
+
     private static final Logger logger = Logger.getLogger(AIMessage.class.getName());
 
     /**
-     * Send a message to the server.
+     * Ask the server a question.
+     *
+     * @param connection The <code>Connection</code> to use
+     *     when communicating with the server.
+     * @param request The <code>Element</code> to send.
+     * @return The reply element.
+     */
+    private static Element askMessage(Connection connection,
+                                      Element request) {
+        Element reply;
+        try {
+            reply = connection.askDumping(request);
+        } catch (IOException e) {
+            logger.log(Level.WARNING, "Could not send \""
+                + request.getTagName() + "\"-message.", e);
+            return null;
+        }
+        
+        if (reply != null && "error".equals(reply.getTagName())) {
+            String msgID = reply.getAttribute("messageID");
+            String msg = reply.getAttribute("message");
+            String logMessage = "AIMessage." + request.getTagName()
+                + " error,"
+                + " messageID: " + ((msgID == null) ? "(null)" : msgID)
+                + " message: " + ((msg == null) ? "(null)" : msg);
+            logger.warning(logMessage);
+            return null;
+        }
+        return reply;
+    }
+
+    /**
+     * Sends a DOMMessage to the server.
      *
      * @param connection The <code>Connection</code> to use
      *     when communicating with the server.
@@ -87,27 +121,9 @@ public class AIMessage {
      * @return True if the message was sent, and a non-null, non-error
      *     reply returned.
      */
-    private static boolean sendMessage(Connection connection, Element request) {
-        try {
-            Element reply = connection.ask(request);
-            if (reply == null) {
-                return false;
-            } else if ("error".equals(reply.getTagName())) {
-                String msgID = reply.getAttribute("messageID");
-                String msg = reply.getAttribute("message");
-                String logMessage = "AIMessage." + request.getTagName()
-                    + " error,"
-                    + " messageID: " + ((msgID == null) ? "(null)" : msgID)
-                    + " message: " + ((msg == null) ? "(null)" : msg);
-                logger.warning(logMessage);
-                return false;
-            }
-            return true;
-        } catch (IOException e) {
-            logger.log(Level.WARNING, "Could not send \""
-                       + request.getTagName() + "\"-message!", e);
-        }
-        return false;
+    private static boolean sendMessage(Connection connection,
+                                       Element request) {
+        return askMessage(connection, request) != null;
     }
 
     /**
@@ -120,7 +136,7 @@ public class AIMessage {
      *     reply returned.
      */
     private static boolean sendMessage(Connection connection,
-                                       Message message) {
+                                       DOMMessage message) {
         return (connection != null && message != null)
             ? sendMessage(connection, message.toXMLElement())
             : false;
@@ -150,7 +166,7 @@ public class AIMessage {
         if ((attributes.length & 1) == 1) {
             throw new IllegalArgumentException("Attributes list must have even length");
         }
-        Element element = Message.createNewRootElement(tag);
+        Element element = DOMMessage.createNewRootElement(tag);
         for (int i = 0; i < attributes.length; i += 2) {
             element.setAttribute(attributes[i], attributes[i+1]);
         }
@@ -328,11 +344,14 @@ public class AIMessage {
      *
      * @param aiUnit The <code>AIUnit</code> carrier.
      * @param unit The <code>Unit</code> that is embarking.
+     * @param direction The <code>Direction</code> to embark in (may be null).
      * @return True if the message was sent, and a non-error reply returned.
      */
-    public static boolean askEmbark(AIUnit aiUnit, Unit unit) {
+    public static boolean askEmbark(AIUnit aiUnit, Unit unit,
+                                    Direction direction) {
         return sendMessage(aiUnit.getConnection(),
-                           new EmbarkMessage(unit, aiUnit.getUnit(), null));
+                           new EmbarkMessage(unit, aiUnit.getUnit(),
+                                             direction));
     }
 
 
@@ -345,7 +364,18 @@ public class AIMessage {
      */
     public static boolean askEmigrate(Connection connection, int slot) {
         return sendMessage(connection,
-                           new EmigrateUnitMessage(slot));
+            new EmigrateUnitMessage(slot));
+    }
+
+
+    /**
+     * Ends the player turn.
+     *
+     * @param connection The <code>Connection</code> to the server.
+     * @return True if the message was sent, and a non-error reply returned.
+     */
+    public static boolean askEndTurn(Connection connection) {
+        return sendTrivial(connection, "endTurn");
     }
 
 
@@ -404,13 +434,17 @@ public class AIMessage {
      * @param colony The <code>Colony</code> to demand of.
      * @param goods The <code>Goods</code> to demand.
      * @param gold The amount of gold to demand.
-     * @return True if the message was sent, and a non-error reply returned.
+     * @return True if the message was sent, a non-error reply returned, and
+     *     the demand was accepted.
      */
     public static boolean askIndianDemand(AIUnit aiUnit, Colony colony,
                                           Goods goods, int gold) {
-        return sendMessage(aiUnit.getConnection(),
-                           new IndianDemandMessage(aiUnit.getUnit(), colony,
-                                                   goods, gold));
+        Element reply = askMessage(aiUnit.getConnection(),
+            new IndianDemandMessage(aiUnit.getUnit(), colony,
+                goods, gold).toXMLElement());
+        IndianDemandMessage message
+            = new IndianDemandMessage(colony.getGame(), reply);
+        return (message == null) ? false : message.getResult();
     }
 
 
@@ -427,6 +461,21 @@ public class AIMessage {
     }
 
 
+    /**
+     * An AI unit loots some cargo.
+     *
+     * @param aiUnit The <code>AIUnit</code> that is looting.
+     * @param defenderId The id of the defending unit.
+     * @param goods A list of <code>Goods</code> to loot.
+     * @return True if the message was sent, and a non-error reply returned.
+     */
+    public static boolean askLoot(AIUnit aiUnit, String defenderId,
+                                  List<Goods> goods) {
+        return sendMessage(aiUnit.getConnection(),
+            new LootCargoMessage(aiUnit.getUnit(), defenderId, goods));
+    }
+
+
     /**
      * Moves an AIUnit in the given direction.
      *
@@ -441,26 +490,15 @@ public class AIMessage {
 
 
     /**
-     * Moves an AIUnit to America.
+     * Moves an AIUnit across the high seas.
      *
      * @param aiUnit The <code>AIUnit</code> to move.
+     * @param destination The <code>Location</code> to move to.
      * @return True if the message was sent, and a non-error reply returned.
      */
-    public static boolean askMoveToAmerica(AIUnit aiUnit) {
+    public static boolean askMoveTo(AIUnit aiUnit, Location destination) {
         return sendMessage(aiUnit.getConnection(),
-                           new MoveToAmericaMessage(aiUnit.getUnit()));
-    }
-
-
-    /**
-     * Moves an AIUnit to Europe.
-     *
-     * @param aiUnit The <code>AIUnit</code> to move.
-     * @return True if the message was sent, and a non-error reply returned.
-     */
-    public static boolean askMoveToEurope(AIUnit aiUnit) {
-        return sendMessage(aiUnit.getConnection(),
-                           new MoveToEuropeMessage(aiUnit.getUnit()));
+                           new MoveToMessage(aiUnit.getUnit(), destination));
     }
 
 
