@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -22,47 +22,58 @@ package net.sf.freecol.server.ai;
 import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Random;
+import java.util.logging.Level;
 import java.util.logging.Logger;
 
 import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
-import net.sf.freecol.common.model.Ability;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.DiplomaticTrade;
-import net.sf.freecol.common.model.Europe;
 import net.sf.freecol.common.model.FoundingFather;
 import net.sf.freecol.common.model.Goods;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Player.PlayerType;
 import net.sf.freecol.common.model.Player.Stance;
 import net.sf.freecol.common.model.Settlement;
+import net.sf.freecol.common.model.Tension;
+import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.Unit;
-import net.sf.freecol.common.model.UnitType;
 import net.sf.freecol.common.networking.Connection;
+import net.sf.freecol.server.ai.mission.Mission;
 import net.sf.freecol.server.model.ServerPlayer;
 import net.sf.freecol.server.networking.DummyConnection;
 
+
 /**
- *
- * Objects of this class contains AI-information for a single {@link Player} and
- * is used for controlling this player.
- *
- * <br />
- * <br />
+ * Objects of this class contains AI-information for a single {@link
+ * Player} and is used for controlling this player.
  *
  * The method {@link #startWorking} gets called by the
  * {@link AIInGameInputHandler} when it is this player's turn.
  */
 public abstract class AIPlayer extends AIObject {
+
     private static final Logger logger = Logger.getLogger(AIPlayer.class.getName());
 
+    public static final int MAX_DISTANCE_TO_BRING_GIFT = 5;
+
+    public static final int MAX_NUMBER_OF_GIFTS_BEING_DELIVERED = 1;
+
+    public static final int MAX_DISTANCE_TO_MAKE_DEMANDS = 5;
+
+    public static final int MAX_NUMBER_OF_DEMANDS = 1;
+
     /**
      * The FreeColGameObject this AIObject contains AI-information for.
      */
     private ServerPlayer player;
 
+    // A PRNG to use for this AI player.
+    private Random aiRandom = null;
+
     /**
      * Temporary variable, used for debugging purposes only.
      * See setDebuggingConnection()
@@ -80,6 +91,7 @@ public abstract class AIPlayer extends AIObject {
 
     public AIPlayer(AIMain aiMain, String id) {
         super(aiMain, id);
+        aiRandom = new Random(aiMain.getRandomSeed("Seed for " + id));
     }
 
     /**
@@ -111,6 +123,28 @@ public abstract class AIPlayer extends AIObject {
         return player.getId();
     }
 
+    /**
+     * Gets the PRNG to use for this player.
+     *
+     * @return A <code>Random<code> to use for this player.
+     */
+    public Random getAIRandom() {
+        return aiRandom;
+    }
+
+    /**
+     * Gets the advantage of this AI player from the nation type.
+     *
+     * @return A short string stating the national advantage.
+     */
+    protected String getAIAdvantage() {
+        final String prefix = "model.nationType.";
+        String id = (player == null || player.getNationType() == null) ? ""
+            : player.getNationType().getId();
+        return (id.startsWith(prefix)) ? id.substring(prefix.length())
+            : "";
+    }
+
     /**
      * Gets the connection to the server.
      *
@@ -160,6 +194,44 @@ public abstract class AIPlayer extends AIObject {
         }
     }
 
+    /**
+     * Gets the AI colony corresponding to a given colony, if any.
+     *
+     * @param colony The <code>Colony</code> to look up.
+     * @return The corresponding AI colony or null if not found.
+     */
+    protected AIColony getAIColony(Colony colony) {
+        return getAIMain().getAIColony(colony);
+    }
+
+    /**
+     * Gets a list of the players AI colonies.
+     *
+     * @return A list of AI colonies.
+     */
+    protected List<AIColony> getAIColonies() {
+        List<AIColony> ac = new ArrayList<AIColony>();
+        for (Colony colony : getPlayer().getColonies()) {
+            AIColony a = getAIColony(colony);
+            if (a != null) {
+                ac.add(a);
+            } else {
+                logger.warning("Could not find the AIColony for: " + colony);
+            }
+        }
+        return ac;
+    }
+        
+    /**
+     * Gets the AI unit corresponding to a given unit, if any.
+     *
+     * @param unit The <code>Unit</code> to look up.
+     * @return The corresponding AI unit or null if not found.
+     */
+    protected AIUnit getAIUnit(Unit unit) {
+        return getAIMain().getAIUnit(unit);
+    }
+
     /**
      * Gets a list of AIUnits for the player.
      *
@@ -181,103 +253,6 @@ public abstract class AIPlayer extends AIObject {
         return aiUnits.iterator();
     }
 
-    /**
-     * Returns an iterator over all the <code>AIColony</code>s owned by this
-     * player.
-     *
-     * @return The <code>Iterator</code>.
-     */
-    protected Iterator<AIColony> getAIColonyIterator() {
-        ArrayList<AIColony> ac = new ArrayList<AIColony>();
-        for (Colony colony : getPlayer().getColonies()) {
-            AIColony a = getAIColony(colony);
-            if (a != null) {
-                ac.add(a);
-            } else {
-                logger.warning("Could not find the AIColony for: " + colony);
-            }
-        }
-        return ac.iterator();
-    }
-
-    /**
-     * Gets the AI colony corresponding to a given colony, if any.
-     *
-     * @param colony The <code>Colony</code> to look up.
-     * @return The corresponding AI colony or null if not found.
-     */
-    protected AIColony getAIColony(Colony colony) {
-        return getAIMain().getAIColony(colony);
-    }
-
-    /**
-     * Gets the AI unit corresponding to a given unit, if any.
-     *
-     * @param unit The <code>Unit</code> to look up.
-     * @return The corresponding AI unit or null if not found.
-     */
-    protected AIUnit getAIUnit(Unit unit) {
-        return getAIMain().getAIUnit(unit);
-    }
-
-
-    /**
-     * Helper function for server communication - Ask the server
-     * to train a unit in Europe on behalf of the AIPlayer.
-     *
-     * TODO: Move this to a specialized Handler class (AIEurope?)
-     * TODO: Give protected access?
-     *
-     * @return the new AIUnit created by this action. May be null.
-     */
-    public AIUnit trainAIUnitInEurope(UnitType unitType) {
-        if (unitType==null) {
-            throw new IllegalArgumentException("Invalid UnitType.");
-        }
-
-        AIUnit aiUnit = null;
-        Europe europe = player.getEurope();
-        int n = europe.getUnitCount();
-
-        if (AIMessage.askTrainUnitInEurope(getConnection(), unitType)
-            && europe.getUnitCount() == n+1) {
-            aiUnit = getAIUnit(europe.getUnitList().get(n));
-        }
-        return aiUnit;
-    }
-
-    /**
-     * Helper function for server communication - Ask the server
-     * to recruit a unit in Europe on behalf of the AIPlayer.
-     *
-     * TODO: Move this to a specialized Handler class (AIEurope?)
-     * TODO: Give protected access?
-     *
-     * @param index The index of the unit to recruit in the recruitables list.
-     * @return the new AIUnit created by this action. May be null.
-     */
-    public AIUnit recruitAIUnitInEurope(int index) {
-        AIUnit aiUnit = null;
-        Europe europe = player.getEurope();
-        int n = europe.getUnitCount();
-
-        // CHEAT: give the AI a selection ability
-        final String selectAbility = "model.ability.selectRecruit";
-        boolean canSelect = player.hasAbility(selectAbility);
-        Ability ability = null;
-        if (!canSelect) {
-            ability = new Ability(selectAbility);
-            player.getFeatureContainer().addAbility(ability);
-        }
-        if (AIMessage.askEmigrate(getConnection(), index+1)
-            && europe.getUnitCount() == n+1) {
-            aiUnit = getAIUnit(europe.getUnitList().get(n));
-        }
-        if (ability != null) {
-            player.getFeatureContainer().removeAbility(ability);
-        }
-        return aiUnit;
-    }
 
     /**
      * Standard stance change determination.  If a change occurs,
@@ -315,80 +290,26 @@ public abstract class AIPlayer extends AIObject {
     public abstract void startWorking();
 
     /**
-     * Returns an <code>Iterator</code> over all the
-     * <code>TileImprovement</code>s needed by all of this player's colonies.
+     * Resolves a native demand.
+     * One of goods/gold is significant.
+     * Overridden by the European player.
      *
-     * @return The <code>Iterator</code>.
-     * @see net.sf.freecol.common.model.TileImprovement
+     * @param unit The native <code>Unit</code> making the demand.
+     * @param colony The <code>Colony</code> being demanded of.
+     * @param goods The <code>Goods</code> demanded (may be null).
+     * @param gold The gold demanded (invalid if goods non-null).
+     * @return The response of the player.
      */
-    public abstract Iterator<TileImprovementPlan> getTileImprovementPlanIterator();
-
-    /**
-     * Remove a <code>TileImprovementPlan</code> from the list
-     */
-    public abstract void removeTileImprovementPlan(TileImprovementPlan plan);
-
-    /**
-     * This is a temporary method which are used for forcing the computer
-     * players into building more colonies. The method will be removed after the
-     * proper code for deciding whether a colony should be built or not has been
-     * implemented.
-     *
-     * @return <code>true</code> if the AI should build more colonies.
-     */
-    public abstract boolean hasFewColonies();
-
-    /**
-     * Returns an <code>Iterator</code> for all the wishes. The items are
-     * sorted by the {@link Wish#getValue value}, with the item having the
-     * highest value appearing first in the <code>Iterator</code>.
-     *
-     * @return The <code>Iterator</code>.
-     * @see Wish
-     */
-    public abstract Iterator<Wish> getWishIterator();
-
-    /**
-     * Selects the most useful founding father offered.
-     *
-     * @param foundingFathers The founding fathers on offer.
-     * @return The founding father selected.
-     */
-    public abstract FoundingFather selectFoundingFather(List<FoundingFather> foundingFathers);
-
-    /**
-     * Decides whether to accept the monarch's tax raise or not.
-     *
-     * @param tax The new tax rate to be considered.
-     * @return <code>true</code> if the tax raise should be accepted.
-     */
-    public abstract boolean acceptTax(int tax);
-
-    /**
-     * Decides whether to accept an Indian demand, or not.
-     *
-     * @param unit The unit making demands.
-     * @param colony The colony where demands are being made.
-     * @param goods The goods demanded.
-     * @param gold The amount of gold demanded.
-     * @return <code>true</code> if this <code>AIPlayer</code> accepts the
-     *         indian demand and <code>false</code> otherwise.
-     */
-    public abstract boolean acceptIndianDemand(Unit unit, Colony colony, Goods goods, int gold);
-
-    /**
-     * Decides whether to accept a mercenary offer, or not.
-     *
-     * @return <code>true</code> if this <code>AIPlayer</code> accepts the
-     *         offer and <code>false</code> otherwise.
-     */
-    public abstract boolean acceptMercenaryOffer();
+    public boolean indianDemand(Unit unit, Colony colony,
+                                Goods goods, int gold) {
+        return false;
+    }
 
     public abstract boolean acceptDiplomaticTrade(DiplomaticTrade agreement);
 
     /**
-     * Called after another <code>Player</code> sends a <code>trade</code> message
-     *
+     * Called after another <code>Player</code> sends a
+     * <code>trade</code> message
      *
      * @param goods The goods which we are going to offer
      */
@@ -422,6 +343,189 @@ public abstract class AIPlayer extends AIObject {
      */
     public abstract int sellProposition(Unit unit, Settlement settlement, Goods goods, int gold);
 
+    /**
+     * Decides to accept a tax raise or not.
+     * Overridden by the European player.
+     *
+     * @param tax The tax raise.
+     * @return True if the raise is accepted.
+     */
+    public boolean acceptTax(int tax) {
+        return false;
+    }
+
+    /**
+     * Decides to accept an offer of mercenaries or not.
+     * Overridden by the European player.
+     *
+     * @return True if the mercenaries are accepted.
+     */
+    public boolean acceptMercenaries() {
+        return false;
+    }
+
+    /**
+     * Determines the stances towards each player.
+     * That is: should we declare war?
+     * TODO: something better, that includes peacemaking.
+     */
+    protected void determineStances() {
+        logger.finest("Entering method determineStances");
+        Player player = getPlayer();
+        for (Player p : getGame().getPlayers()) {
+            if (p != player && !p.isDead()) determineStance(p);
+        }
+    }
+
+    /**
+     * Aborts the mission for a unit, but tries to recover the unit onto
+     * a neighbouring carrier.
+     *
+     * @param aiU The <code>AIUnit</code> whose mission is ended.
+     * @param why A reason for aborting the mission.
+     */
+    private void abortUnitMission(AIUnit aiU, String why) {
+        aiU.abortMission(why);
+        // There is a common stuffup where the AIs converge on the
+        // same location on a small island.  First mover gets the
+        // colony, the rest get stuck there when their mission is aborted.
+        //
+        // TODO: drop this when a more general `marooned unit rescue'
+        // mission is written.
+        Unit unit = aiU.getUnit();
+        Tile tile = unit.getTile();
+        if (!unit.isCarrier() && !unit.isOnCarrier() && tile != null
+            && tile.getColony() == null) {
+            for (Tile t : tile.getSurroundingTiles(1)) {
+                for (Unit u : t.getUnitList()) {
+                    if (u.getOwner() == unit.getOwner()
+                        && u.isCarrier() && u.canCarryUnits()
+                        && u.getSpaceLeft() >= unit.getSpaceTaken()
+                        && unit.getMovesLeft() > 0) {
+                        AIMessage.askEmbark(getAIMain().getAIUnit(u), unit,
+                            tile.getDirection(t));
+                        return; // Let the carrier update its transport list.
+                    }
+                }
+            }
+        }
+    }
+
+    /**
+     * Aborts all the missions which are no longer valid.
+     */
+    protected void abortInvalidMissions() {
+        for (AIUnit au : getAIUnits()) {
+            Mission mission = au.getMission();
+            if (mission != null && !mission.isValid()) {
+                abortUnitMission(au, "invalid");
+            }
+        }
+    }
+
+    /**
+     * Aborts all the missions which are no longer valid.
+     */
+    protected void abortInvalidAndOneTimeMissions() {
+        for (AIUnit au : getAIUnits()) {
+            Mission mission = au.getMission();
+            if (mission == null) continue;
+            if (!mission.isValid()) {
+                abortUnitMission(au, "invalid");
+            } else if (mission.isOneTime()) {
+                abortUnitMission(au, "one-time");
+            }
+        }
+    }
+
+    /**
+     * Makes every unit perform their mission.
+     */
+    protected void doMissions() {
+        logger.finest("Entering method doMissions");
+        for (AIUnit au : getAIUnits()) {
+            if (au.hasMission() && au.getMission().isValid()
+                && !au.getUnit().isOnCarrier()) {
+                try {
+                    au.doMission(getConnection());
+                } catch (Exception e) {
+                    logger.log(Level.WARNING, "doMissions failed", e);
+                }
+            }
+        }
+    }
+
+    /**
+     * Find out if a tile contains a suitable target for seek-and-destroy.
+     * TODO: Package for access by a test only - necessary?
+     *
+     * @param attacker The attacking <code>Unit</code>.
+     * @param tile The <code>Tile</code> to attack into.
+     * @return True if an attack can be launched.
+     */
+    public boolean isTargetValidForSeekAndDestroy(Unit attacker, Tile tile) {
+        Player attackerPlayer = attacker.getOwner();
+
+        // Insist the attacker exists.
+        if (attacker == null) return false;
+
+        // Determine the defending player.
+        Settlement settlement = tile.getSettlement();
+        Unit defender = tile.getDefendingUnit(attacker);
+        Player defenderPlayer = (settlement != null) ? settlement.getOwner()
+            : (defender != null) ? defender.getOwner()
+            : null;
+
+        // Insist there be a defending player.
+        if (defenderPlayer == null) return false;
+
+        // Can not attack our own units.
+        if (attackerPlayer == defenderPlayer) return false;
+
+        // If European, do not attack if not at war.
+        // If native, do not attack if not at war and at least content.
+        // Otherwise some attacks are allowed even if not at war.
+        boolean atWar = attackerPlayer.atWarWith(defenderPlayer);
+        if (attackerPlayer.isEuropean()) {
+            if (!atWar) return false;
+        } else if (attackerPlayer.isIndian()) {
+            if (!atWar && attackerPlayer.getTension(defenderPlayer)
+                .getLevel().compareTo(Tension.Level.CONTENT) <= 0) {
+                return false;
+            }
+        }
+
+        // A naval unit can never attack a land unit or settlement,
+        // but a land unit *can* attack a naval unit if it is on land.
+        // Otherwise naval units can only fight at sea, land units
+        // only on land.
+        if (attacker.isNaval()) {
+            if (settlement != null
+                || !defender.isNaval() || defender.getTile().isLand()) {
+                return false;
+            }
+        } else {
+            if (defender != null && !defender.getTile().isLand()) {
+                return false;
+            }
+        }
+
+        // Otherwise, attack.
+        return true;
+    }
+
+    /**
+     * Selects the most useful founding father offered.
+     * Overridden by EuropeanAIPlayers.
+     *
+     * @param ffs The founding fathers on offer.
+     * @return The founding father selected.
+     */
+    public FoundingFather selectFoundingFather(List<FoundingFather> ffs) {
+        return null;
+    }
+
+
     /**
      * Writes this object to an XML stream.
      *
@@ -430,7 +534,11 @@ public abstract class AIPlayer extends AIObject {
      *             stream.
      */
     @Override
-    protected abstract void toXMLImpl(XMLStreamWriter out) throws XMLStreamException;
+    protected void toXMLImpl(XMLStreamWriter out) throws XMLStreamException {
+        out.writeStartElement(getXMLElementTagName());
+        out.writeAttribute(ID_ATTRIBUTE, getId());
+        out.writeEndElement();
+    }
 
     /**
      * Reads information for this object from an XML stream.
@@ -440,7 +548,12 @@ public abstract class AIPlayer extends AIObject {
      *             stream.
      */
     @Override
-    protected abstract void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException;
+    protected void readFromXMLImpl(XMLStreamReader in)
+        throws XMLStreamException {
+        setPlayer((ServerPlayer) getAIMain()
+            .getFreeColGameObject(in.getAttributeValue(null, ID_ATTRIBUTE)));
+        in.nextTag();
+    }
 
     /**
      * Returns the tag name of the root element representing this object.
@@ -450,5 +563,4 @@ public abstract class AIPlayer extends AIObject {
     public static String getXMLElementTagName() {
         return "aiPlayer";
     }
-
 }
