@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -19,6 +19,7 @@
 
 package net.sf.freecol.server.ai;
 
+import java.util.Random;
 import java.util.logging.Logger;
 
 import javax.xml.stream.XMLStreamConstants;
@@ -26,9 +27,16 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
+import net.sf.freecol.common.model.AbstractGoods;
+import net.sf.freecol.common.model.EquipmentType;
+import net.sf.freecol.common.model.Europe;
 import net.sf.freecol.common.model.Locatable;
 import net.sf.freecol.common.model.Location;
+import net.sf.freecol.common.model.Player;
+import net.sf.freecol.common.model.Settlement;
+import net.sf.freecol.common.model.Specification;
 import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.Unit.Role;
 import net.sf.freecol.common.networking.Connection;
 import net.sf.freecol.server.ai.goal.Goal;
 import net.sf.freecol.server.ai.mission.BuildColonyMission;
@@ -50,6 +58,7 @@ import net.sf.freecol.server.ai.mission.WorkInsideColonyMission;
 
 import org.w3c.dom.Element;
 
+
 /**
  * Objects of this class contains AI-information for a single {@link Unit}.
  *
@@ -64,11 +73,9 @@ import org.w3c.dom.Element;
  * @see Mission
  */
 public class AIUnit extends AIObject implements Transportable {
+
     private static final Logger logger = Logger.getLogger(AIUnit.class.getName());
 
-
-
-
     /**
      * The FreeColGameObject this AIObject contains AI-information for.
      */
@@ -120,7 +127,7 @@ public class AIUnit extends AIObject implements Transportable {
      *            of this object.
      */
     public AIUnit(AIMain aiMain, Element element) {
-        super(aiMain, element.getAttribute("ID"));
+        super(aiMain, element.getAttribute(ID_ATTRIBUTE));
         readFromXMLElement(element);
     }
 
@@ -133,7 +140,7 @@ public class AIUnit extends AIObject implements Transportable {
      * @see AIObject#readFromXML
      */
     public AIUnit(AIMain aiMain, XMLStreamReader in) throws XMLStreamException {
-        super(aiMain, in.getAttributeValue(null, "ID"));
+        super(aiMain, in.getAttributeValue(null, ID_ATTRIBUTE));
         readFromXML(in);
     }
 
@@ -161,6 +168,15 @@ public class AIUnit extends AIObject implements Transportable {
         return unit;
     }
 
+    /**
+     * Gets the PRNG to use with this unit.
+     *
+     * @return A <code>Random</code> instance.
+     */
+    public Random getAIRandom() {
+        return getAIMain().getAIPlayer(unit.getOwner()).getAIRandom();
+    }
+
     /**
      * Aborts the given <code>Wish</code>.
      *
@@ -193,7 +209,8 @@ public class AIUnit extends AIObject implements Transportable {
      * @return The source for this <code>Transportable</code>.
      */
     public Location getTransportSource() {
-        return getUnit().getLocation();
+        return (getUnit() == null || getUnit().isDisposed()) ? null
+            : getUnit().getLocation();
     }
 
     /**
@@ -207,11 +224,9 @@ public class AIUnit extends AIObject implements Transportable {
      * @return The destination for this <code>Transportable</code>.
      */
     public Location getTransportDestination() {
-        if (hasMission()) {
-            return mission.getTransportDestination();
-        } else {
-            return null;
-        }
+        return (getUnit() == null || getUnit().isDisposed() || !hasMission())
+            ? null
+            : mission.getTransportDestination();
     }
 
     /**
@@ -264,6 +279,7 @@ public class AIUnit extends AIObject implements Transportable {
      *
      */
     public void setTransport(AIUnit transport) {
+        if (this.transport == transport) return;
         AIUnit oldTransport = this.transport;
         this.transport = transport;
 
@@ -299,17 +315,39 @@ public class AIUnit extends AIObject implements Transportable {
      * @return <code>true</code> if this unit has a mission.
      */
     public boolean hasMission() {
-        return (mission != null);
+        return mission != null;
     }
 
     /**
-     * Assignes a mission to unit. The dynamic priority is reset.
+     * Aborts a mission.  Always use this instead of setMission(null),
+     * and provide a useful reason so that AI mission thrashing can be
+     * tracked down.
+     *
+     * @param why A string describing why the mission is to be aborted
+     *     (e.g. "invalid").
+     */
+    public void abortMission(String why) {
+        if (mission != null) {
+            logger.finest("Aborting old " + why
+                + " " + mission.getClass().getName()
+                + " for unit " + getUnit());
+            mission.dispose();
+            this.mission = null;
+        }
+    }
+
+    /**
+     * Assigns a mission to unit. The dynamic priority is reset.
+     * Do not call setMission(null), use abortMission above.
      *
      * @param mission The new <code>Mission</code>.
      */
     public void setMission(Mission mission) {
         final Mission oldMission = this.mission;
         if (oldMission != null) {
+            logger.finest("Replacing old " + oldMission.getClass().getName()
+                + " with " + mission.getClass().getName()
+                + " for unit " + getUnit());
             oldMission.dispose();
         }
         this.mission = mission;
@@ -323,16 +361,14 @@ public class AIUnit extends AIObject implements Transportable {
      *            with the server.
      */
     public void doMission(Connection connection) {
-        if (getMission() != null && getMission().isValid()) {
-            getMission().doMission(connection);
-        }
+        if (mission != null && mission.isValid()) mission.doMission(connection);
     }
 
     /**
      * Disposes this object and any attached mission.
      */
     public void dispose() {
-        setMission(null);
+        abortMission("disposed");
         setTransport(null);
         super.dispose();
     }
@@ -369,10 +405,71 @@ public class AIUnit extends AIObject implements Transportable {
         return getAIMain().getAIPlayer(unit.getOwner());
     }
 
+    /**
+     * Convenience accessor for the owning player connection.
+     *
+     * @return The connection.
+     */
     public Connection getConnection() {
         return getAIOwner().getConnection();
     }
 
+    /**
+     * Equips this AI unit for a particular role.
+     *
+     * The unit must be at a location where the required goods are available
+     * (possibly requiring a purchase, which may fail due to lack of gold
+     * or boycotts in effect).
+     *
+     * When multiple equipment types are needed, try them all --- so for
+     * example, if a request is made to equip a unit in Europe as a dragoon
+     * but muskets are boycotted, it may still acquire the horses and end
+     * up as a scout.
+     *
+     * TODO: remove cheat.
+     *
+     * @param r The <code>Role</code> to adopt.
+     * @param cheat Cheat goods purchase in Europe (but *not* boycotts).
+     * @return True if the role change was successful.
+     */
+    public boolean equipForRole(Role r, boolean cheat) {
+        final Specification spec = getSpecification();
+        final Player player = unit.getOwner();
+        Location loc = unit.getLocation();
+        Europe europe = (loc instanceof Europe) ? (Europe)loc : null;
+        Settlement settlement = loc.getSettlement();
+        if (settlement == null && europe == null) return false;
+
+        eq: for (EquipmentType e : r.getRoleEquipment(spec)) {
+            if (!unit.canBeEquippedWith(e)) {
+                // Weed out native/colonial-specific equipment types.
+                continue;
+            }
+            // Check that this should succeed before querying server.
+            if (europe != null) {
+                for (AbstractGoods ag : e.getGoodsRequired()) {
+                    if (player.getMarket().getArrears(ag.getType()) > 0) {
+                        continue eq; // Boycott prevents purchase.
+                    }
+                    int cost = player.getMarket().getBidPrice(ag.getType(),
+                                                              ag.getAmount());
+                    if (!player.checkGold(cost)) {
+                        if (cheat) {
+                            player.modifyGold(cost);
+                        } else {
+                            continue eq;
+                        }
+                    }
+                }
+            } else {
+                if (!settlement.canBuildEquipment(e)) continue eq;
+            }
+            // Should now only fail due to comms lossage.
+            AIMessage.askEquipUnit(this, e, 1);
+        }
+        return unit.getRole() == r;
+    }
+
     /**
      * Writes this object to an XML stream.
      *
@@ -381,9 +478,14 @@ public class AIUnit extends AIObject implements Transportable {
      *             stream.
      */
     protected void toXMLImpl(XMLStreamWriter out) throws XMLStreamException {
+        if (unit == null || unit.isDisposed()) {
+            logger.warning("Dead AIUnit: " + unit);
+            return;
+        }
+
         out.writeStartElement(getXMLElementTagName());
 
-        out.writeAttribute("ID", getId());
+        out.writeAttribute(ID_ATTRIBUTE, getId());
         if (transport != null) {
             if (transport.getUnit() == null) {
                 logger.warning("transport.getUnit() == null");
@@ -397,6 +499,9 @@ public class AIUnit extends AIObject implements Transportable {
             }
         }
         if (mission != null) {
+            if (!mission.isValid()) {
+                logger.warning("Writing invalid mission: " + mission);
+            }
             mission.toXML(out);
         }
 
@@ -410,8 +515,9 @@ public class AIUnit extends AIObject implements Transportable {
      * @throws XMLStreamException if there are any problems reading from the
      *             stream.
      */
-    protected void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException {
-        final String inID = in.getAttributeValue(null, "ID");
+    protected void readFromXMLImpl(XMLStreamReader in)
+        throws XMLStreamException {
+        final String inID = in.getAttributeValue(null, ID_ATTRIBUTE);
         unit = (Unit) getAIMain().getFreeColGameObject(inID);
 
         if (unit == null) {
@@ -445,7 +551,10 @@ public class AIUnit extends AIObject implements Transportable {
                 mission = new WishRealizationMission(getAIMain(), in);
             } else if (in.getLocalName().equals(UnitSeekAndDestroyMission.getXMLElementTagName())) {
                 mission = new UnitSeekAndDestroyMission(getAIMain(), in);
-            } else if (in.getLocalName().equals(PioneeringMission.getXMLElementTagName())) {
+            } else if (in.getLocalName().equals(PioneeringMission.getXMLElementTagName())
+                // @compat 0.10.3
+                || in.getLocalName().equals("tileImprovementPlanMission")) {
+                // @end compatibility code                
                 mission = new PioneeringMission(getAIMain(), in);
             } else if (in.getLocalName().equals(DefendSettlementMission.getXMLElementTagName())) {
                 mission = new DefendSettlementMission(getAIMain(), in);
@@ -469,6 +578,17 @@ public class AIUnit extends AIObject implements Transportable {
         }
     }
 
+    /**
+     * Returns a <code>String</code>-representation of this object.
+     *
+     * @return A <code>String</code> representing this object for
+     *     debugging purposes.
+     */
+    @Override
+    public String toString() {
+        return "AIUnit@" + hashCode() + "[ " + unit + " ]";
+    }
+
     /**
      * Returns the tag name of the root element representing this object.
      *
