@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -17,7 +17,6 @@
  *  along with FreeCol.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
 package net.sf.freecol.server.ai.mission;
 
 import java.util.logging.Logger;
@@ -26,6 +25,7 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
+import net.sf.freecol.common.model.Location;
 import net.sf.freecol.common.model.PathNode;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.Unit;
@@ -38,20 +38,21 @@ import org.w3c.dom.Element;
 
 
 /**
-* Mission for wandering in random directions.
-*/
+ * Mission for idling in colony.
+ * TODO: this has now been generalized to settlements, rename one day.
+ */
 public class IdleAtColonyMission extends Mission {
+
     private static final Logger logger = Logger.getLogger(IdleAtColonyMission.class.getName());
 
 
-
     /**
-    * Creates a mission for the given <code>AIUnit</code>.
-    *
-    * @param aiMain The main AI-object.
-    * @param aiUnit The <code>AIUnit</code> this mission
-    *        is created for.
-    */
+     * Creates a mission for the given <code>AIUnit</code>.
+     *
+     * @param aiMain The main AI-object.
+     * @param aiUnit The <code>AIUnit</code> this mission
+     *        is created for.
+     */
     public IdleAtColonyMission(AIMain aiMain, AIUnit aiUnit) {
         super(aiMain, aiUnit);
     }
@@ -70,7 +71,8 @@ public class IdleAtColonyMission extends Mission {
     }
 
     /**
-     * Creates a new <code>IdleAtColonyMission</code> and reads the given element.
+     * Creates a new <code>IdleAtColonyMission</code> and reads the
+     * given element.
      *
      * @param aiMain The main AI-object.
      * @param in The input stream containing the XML.
@@ -78,51 +80,67 @@ public class IdleAtColonyMission extends Mission {
      *      during parsing.
      * @see net.sf.freecol.server.ai.AIObject#readFromXML
      */
-    public IdleAtColonyMission(AIMain aiMain, XMLStreamReader in) throws XMLStreamException {
+    public IdleAtColonyMission(AIMain aiMain, XMLStreamReader in)
+        throws XMLStreamException {
         super(aiMain);
         readFromXML(in);
     }
 
+
     /**
-    * Performs the mission. This is done by moving in a random direction
-    * until the move points are zero or the unit gets stuck.
-    *
-    * @param connection The <code>Connection</code> to the server.
-    */
+     * Gets the transport destination for units with this mission.
+     *
+     * @return The destination for this <code>Transportable</code>.
+     */
+    public Location getTransportDestination() {
+        final Unit unit = getUnit();
+        if (unit.getTile() == null
+            || unit.getTile().getSettlement() != null) return null;
+        PathNode path = findNearestOtherSettlement(unit);
+        Tile target = (path == null) ? null : path.getLastNode().getTile();
+        return (target != null && shouldTakeTransportToTile(target))
+            ? target
+            : null;
+    }
+
+    /**
+     * Should this Mission only be carried out once?
+     *
+     * @return True.
+     */
+    public boolean isOneTime() {
+        return true;
+    }
+
+    /**
+     * Performs the mission. This is done by moving in a random direction
+     * until the move points are zero or the unit gets stuck.
+     *
+     * @param connection The <code>Connection</code> to the server.
+     */
     public void doMission(Connection connection) {
-        Tile thisTile = getUnit().getTile();
-        Unit unit = getUnit();
+        final Unit unit = getUnit();
+        final Tile tile = unit.getTile();
+        if (tile == null) return;
 
-        //Only deal with units on the map
-        if (thisTile != null) {
+        // If our tile contains a settlement, idle
+        if (tile.getSettlement() != null) {
+            logger.finest("Unit " + unit.getId()
+                + " idle at settlement: " + tile.getSettlement().getId());
+            return;
+        }
 
-            //if our tile contains a settlement, idle
-            if (thisTile.getSettlement()!=null) {
-                logger.info("Unit "+unit.getId()+" idle at settlement: "+thisTile.getSettlement().getId());
-                return;
-            }
-
-            //still here, so we're somewhere on the map; find some colony
-            PathNode pathToTarget = findNearestColony(unit);
-
-            if (pathToTarget != null) {
-                Direction r = moveTowards(pathToTarget);
-                if (r == null || !moveButDontAttack(r)) return;
-            } else {
-                // Just make a random move if no target can be found.
-                moveRandomly(connection);
-            }
+        PathNode path = findNearestOtherSettlement(unit);
+        if (path != null) {
+            Direction r = moveTowards(path);
+            if (r == null || !moveButDontAttack(r)) return;
+        } else {
+            // Just make a random move if no target can be found.
+            moveRandomly();
         }
     }
 
-    /**
-     * Checks if this mission is still valid to perform.
-     *
-     * @return True if this mission is still valid to perform.
-     */
-    public boolean isValid() {
-        return super.isValid();
-    }
+    // Serialization
 
     /**
      * Writes all of the <code>AIObject</code>s and other AI-related
@@ -133,27 +151,14 @@ public class IdleAtColonyMission extends Mission {
      *      to the stream.
      */
     protected void toXMLImpl(XMLStreamWriter out) throws XMLStreamException {
-        out.writeStartElement(getXMLElementTagName());
-
-        out.writeAttribute("unit", getUnit().getId());
-
-        out.writeEndElement();
+        toXML(out, getXMLElementTagName());
     }
 
     /**
-     * Reads all the <code>AIObject</code>s and other AI-related information
-     * from XML data.
-     * @param in The input stream with the XML.
+     * Returns the tag name of the root element representing this object.
+     *
+     * @return "idleAtColonyMission".
      */
-    protected void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException {
-        setAIUnit((AIUnit) getAIMain().getAIObject(in.getAttributeValue(null, "unit")));
-        in.nextTag();
-    }
-
-    /**
-    * Returns the tag name of the root element representing this object.
-    * @return The <code>String</code> "unitWanderMission".
-    */
     public static String getXMLElementTagName() {
         return "idleAtColonyMission";
     }
