@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -33,7 +33,6 @@ import net.sf.freecol.common.model.HistoryEvent;
 import net.sf.freecol.common.model.LastSale;
 import net.sf.freecol.common.model.Location;
 import net.sf.freecol.common.model.ModelMessage;
-import net.sf.freecol.common.model.ModelMessage.MessageType;
 import net.sf.freecol.common.model.Ownable;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Player.Stance;
@@ -43,7 +42,7 @@ import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.TradeRoute;
 import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.model.WorkLocation;
-import net.sf.freecol.common.networking.Message;
+import net.sf.freecol.common.networking.DOMMessage;
 import net.sf.freecol.server.model.ServerPlayer;
 
 import org.w3c.dom.Document;
@@ -371,7 +370,7 @@ public class ChangeSet {
      */
     private static class MessageChange extends Change {
         private ChangePriority priority;
-        private Message message;
+        private DOMMessage message;
 
         /**
          * Build a new MessageChange.
@@ -380,7 +379,7 @@ public class ChangeSet {
          * @param priority The priority of the change.
          * @param message The <code>Message</code> to add.
          */
-        MessageChange(See see, ChangePriority priority, Message message) {
+        MessageChange(See see, ChangePriority priority, DOMMessage message) {
             super(see);
             this.priority = priority;
             this.message = message;
@@ -476,10 +475,11 @@ public class ChangeSet {
         public List<Change> consequences(ServerPlayer serverPlayer) {
             if (seeOld(serverPlayer) && !seeNew(serverPlayer)) {
                 List<Change> changes = new ArrayList<Change>();
-                List<FreeColGameObject> objects = new ArrayList<FreeColGameObject>();
+                List<FreeColGameObject> objects
+                    = new ArrayList<FreeColGameObject>();
                 objects.add(unit);
                 changes.add(new RemoveChange(See.only(serverPlayer),
-                                             unit.getLocation(), objects));
+                        unit.getLocation(), objects));
                 return changes;
             }
             return Collections.emptyList();
@@ -800,7 +800,6 @@ public class ChangeSet {
         private Player first;
         private Stance stance;
         private Player second;
-        private boolean war;
 
         /**
          * Build a new StanceChange.
@@ -809,15 +808,12 @@ public class ChangeSet {
          * @param first The <code>Player</code> changing stance.
          * @param stance The <code>Stance</code> to change to.
          * @param second The <code>Player</code> wrt with to change.
-         * @param war The war status.
          */
-        StanceChange(See see, Player first, Stance stance, Player second,
-                     boolean war) {
+        StanceChange(See see, Player first, Stance stance, Player second) {
             super(see);
             this.first = first;
             this.stance = stance;
             this.second = second;
-            this.war = war;
         }
 
         /**
@@ -829,55 +825,6 @@ public class ChangeSet {
             return ChangePriority.CHANGE_STANCE.getPriority();
         }
 
-        /**
-         * Should a player perhaps be notified of this stance change?
-         * Yes, if they are the player that initiated the change, the
-         * player the stance change applies to, or it is a war, or
-         * they have enhaced diplomacy reporting.
-         *
-         * @param serverPlayer The <code>ServerPlayer</code> to notify.
-         * @return True if the player should be notified.
-         */
-        @Override
-        public boolean isPerhapsNotifiable(ServerPlayer serverPlayer) {
-            return (ServerPlayer) first == serverPlayer
-                || (ServerPlayer) second == serverPlayer
-                || (war && serverPlayer.hasContacted(first)
-                    && serverPlayer.hasContacted(second))
-                || serverPlayer.hasAbility("model.ability.betterForeignAffairsReport");
-        }
-
-        /**
-         * There are consequences to a stance change.  If it is
-         * visible to a player they should see a message about it,
-         * unless they initiated the change and already know.
-         *
-         * @param serverPlayer The <code>ServerPlayer</code> to notify.
-         * @return A list of changes if there are messages to send.
-         */
-        @Override
-        public List<Change> consequences(ServerPlayer serverPlayer) {
-            List<Change> changes = new ArrayList<Change>();
-            String sta = stance.toString();
-            ModelMessage m = ((ServerPlayer) first == serverPlayer)
-                ? new ModelMessage(MessageType.FOREIGN_DIPLOMACY,
-                                   "model.diplomacy." + sta + ".declared",
-                                   second)
-                    .addStringTemplate("%nation%", second.getNationName())
-                : ((ServerPlayer) second == serverPlayer)
-                ? new ModelMessage(MessageType.FOREIGN_DIPLOMACY,
-                                   "model.diplomacy." + sta + ".declared",
-                                   first)
-                    .addStringTemplate("%nation%", first.getNationName())
-                : new ModelMessage(MessageType.FOREIGN_DIPLOMACY,
-                                   "model.diplomacy." + sta + ".others",
-                                   first)
-                    .addStringTemplate("%attacker%", first.getNationName())
-                    .addStringTemplate("%defender%", second.getNationName());
-            changes.add(new OwnedChange(See.only(serverPlayer), m));
-            return changes;
-        }
-
         /**
          * Specialize a StanceChange to a particular player.
          *
@@ -1001,7 +948,7 @@ public class ChangeSet {
      * @param message The <code>Message</code> to add.
      * @return The updated <code>ChangeSet</code>.
      */
-    public ChangeSet add(See see, ChangePriority cp, Message message) {
+    public ChangeSet add(See see, ChangePriority cp, DOMMessage message) {
         changes.add(new MessageChange(see, cp, message));
         return this;
     }
@@ -1050,15 +997,13 @@ public class ChangeSet {
     /**
      * Helper function to add a removal for a disposal list to a ChangeSet.
      *
-     * @param owner The <code>ServerPlayer</code> that owns this object.
+     * @param see The visibility of this change.
      * @param loc The <code>Location</code> where the object was.
      * @param obj The <code>FreeColGameObject</code> to remove.
      * @return The updated <code>ChangeSet</code>.
      */
-    public ChangeSet addDispose(ServerPlayer owner, Location loc,
-                                FreeColGameObject obj) {
-        changes.add(new RemoveChange(See.perhaps().always(owner), loc,
-                                     obj.disposeList()));
+    public ChangeSet addDispose(See see, Location loc, FreeColGameObject obj) {
+        changes.add(new RemoveChange(see, loc, obj.disposeList()));
         return this;
     }
 
@@ -1223,12 +1168,11 @@ public class ChangeSet {
      * @param first The <code>Player</code> changing stance.
      * @param stance The <code>Stance</code> to change to.
      * @param second The <code>Player</code> wrt with to change.
-     * @param war Was this change to/from a war stance.
      * @return The updated <code>ChangeSet</code>.
      */
     public ChangeSet addStance(See see, Player first, Stance stance,
-                               Player second, boolean war) {
-        changes.add(new StanceChange(see, first, stance, second, war));
+                               Player second) {
+        changes.add(new StanceChange(see, first, stance, second));
         return this;
     }
 
@@ -1342,7 +1286,7 @@ public class ChangeSet {
         List<Change> c = new ArrayList<Change>(changes);
         List<Element> elements = new ArrayList<Element>();
         List<Change> diverted = new ArrayList<Change>();
-        Document doc = Message.createNewDocument();
+        Document doc = DOMMessage.createNewDocument();
 
         // For all sorted changes, if it is notifiable to the target
         // player then convert it to an Element, or divert for later
