@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -19,6 +19,9 @@
 
 package net.sf.freecol.server.control;
 
+import java.util.ArrayList;
+import java.util.List;
+import java.util.logging.Level;
 import java.util.logging.Logger;
 
 import net.sf.freecol.common.model.Player;
@@ -46,6 +49,7 @@ import net.sf.freecol.common.networking.DemandTributeMessage;
 import net.sf.freecol.common.networking.DiplomacyMessage;
 import net.sf.freecol.common.networking.DisbandUnitMessage;
 import net.sf.freecol.common.networking.DisembarkMessage;
+import net.sf.freecol.common.networking.DOMMessage;
 import net.sf.freecol.common.networking.EmbarkMessage;
 import net.sf.freecol.common.networking.EmigrateUnitMessage;
 import net.sf.freecol.common.networking.EquipUnitMessage;
@@ -60,8 +64,7 @@ import net.sf.freecol.common.networking.LoadCargoMessage;
 import net.sf.freecol.common.networking.LootCargoMessage;
 import net.sf.freecol.common.networking.MissionaryMessage;
 import net.sf.freecol.common.networking.MoveMessage;
-import net.sf.freecol.common.networking.MoveToAmericaMessage;
-import net.sf.freecol.common.networking.MoveToEuropeMessage;
+import net.sf.freecol.common.networking.MoveToMessage;
 import net.sf.freecol.common.networking.NetworkConstants;
 import net.sf.freecol.common.networking.NewLandNameMessage;
 import net.sf.freecol.common.networking.NewRegionNameMessage;
@@ -79,7 +82,6 @@ import net.sf.freecol.common.networking.SetDestinationMessage;
 import net.sf.freecol.common.networking.SetGoodsLevelsMessage;
 import net.sf.freecol.common.networking.SetTradeRoutesMessage;
 import net.sf.freecol.common.networking.SpySettlementMessage;
-import net.sf.freecol.common.networking.StatisticsMessage;
 import net.sf.freecol.common.networking.TrainUnitInEuropeMessage;
 import net.sf.freecol.common.networking.UnloadCargoMessage;
 import net.sf.freecol.common.networking.UpdateCurrentStopMessage;
@@ -89,6 +91,7 @@ import net.sf.freecol.server.FreeColServer;
 import net.sf.freecol.server.model.ServerPlayer;
 
 import org.w3c.dom.Element;
+import org.w3c.dom.NodeList;
 
 
 /**
@@ -209,15 +212,6 @@ public final class InGameInputHandler extends InputHandler
                 return new ChangeWorkTypeMessage(getGame(), element)
                     .handle(freeColServer, player, connection);
             }});
-        register("chooseFoundingFather",
-                 new CurrentPlayerNetworkRequestHandler() {
-            @Override
-            public Element handle(Player player, Connection connection,
-                                  Element element) {
-                return freeColServer.getInGameController()
-                    .chooseFoundingFather(freeColServer.getPlayer(connection),
-                        element.getAttribute("foundingFather"));
-            }});
         register(ClaimLandMessage.getXMLElementTagName(),
                  new CurrentPlayerNetworkRequestHandler() {
             @Override
@@ -409,20 +403,12 @@ public final class InGameInputHandler extends InputHandler
                 return new MoveMessage(getGame(), element)
                     .handle(freeColServer, player, connection);
             }});
-        register(MoveToAmericaMessage.getXMLElementTagName(),
+        register(MoveToMessage.getXMLElementTagName(),
                  new CurrentPlayerNetworkRequestHandler() {
             @Override
             public Element handle(Player player, Connection connection,
                                   Element element) {
-                return new MoveToAmericaMessage(getGame(), element)
-                    .handle(freeColServer, player, connection);
-            }});
-        register(MoveToEuropeMessage.getXMLElementTagName(),
-                 new CurrentPlayerNetworkRequestHandler() {
-            @Override
-            public Element handle(Player player, Connection connection,
-                                  Element element) {
-                return new MoveToEuropeMessage(getGame(), element)
+                return new MoveToMessage(getGame(), element)
                     .handle(freeColServer, player, connection);
             }});
         register(NewLandNameMessage.getXMLElementTagName(),
@@ -546,6 +532,27 @@ public final class InGameInputHandler extends InputHandler
                     .handle(freeColServer, player, connection);
             }});
 
+        register("multiple",
+                 new CurrentPlayerNetworkRequestHandler() {
+            @Override
+            public Element handle(Player player, Connection connection,
+                                  Element element) {
+                NodeList nodes = element.getChildNodes();
+                List<Element> results = new ArrayList<Element>();
+
+                for (int i = 0; i < nodes.getLength(); i++) {
+                    try {
+                        Element reply = super.handle(connection,
+                            (Element) nodes.item(i));
+                        if (reply != null) results.add(reply);
+                    } catch (Exception e) {
+                        logger.log(Level.WARNING, "Crash in multiple, item " + i
+                            + ", continuing.", e);
+                    }
+                }
+                return DOMMessage.collapseElements(results);
+            }});
+
         // NetworkRequestHandlers
         register(AssignTradeRouteMessage.getXMLElementTagName(),
                  new NetworkRequestHandler() {
@@ -554,6 +561,13 @@ public final class InGameInputHandler extends InputHandler
                 return new AssignTradeRouteMessage(getGame(), element)
                     .handle(freeColServer, connection);
             }});
+        register("checkHighScore",
+                 new NetworkRequestHandler() {
+            @Override
+            public Element handle(Connection connection, Element element) {
+                return freeColServer.getInGameController()
+                    .checkHighScore(freeColServer.getPlayer(connection));
+            }});
         register("continuePlaying",
                  new NetworkRequestHandler() {
             @Override
@@ -568,6 +582,13 @@ public final class InGameInputHandler extends InputHandler
                 return new DiplomacyMessage(getGame(), element)
                     .handle(freeColServer, connection);
             }});
+        register("enterRevengeMode",
+                 new NetworkRequestHandler() {
+            @Override
+            public Element handle(Connection connection, Element element) {
+                return freeColServer.getInGameController()
+                    .enterRevengeMode(freeColServer.getPlayer(connection));
+            }});
         register("getHighScores",
                  new NetworkRequestHandler() {
             @Override
@@ -579,7 +600,7 @@ public final class InGameInputHandler extends InputHandler
                  new NetworkRequestHandler() {
             @Override
             public Element handle(Connection connection, Element element) {
-                return new GetNationSummaryMessage(getGame(), element)
+                return new GetNationSummaryMessage(element)
                     .handle(freeColServer, connection);
             }});
         register("getNewTradeRoute",
@@ -618,13 +639,12 @@ public final class InGameInputHandler extends InputHandler
                 return new SpySettlementMessage(getGame(), element)
                     .handle(freeColServer, connection);
             }});
-        register(StatisticsMessage.getXMLElementTagName(),
+        register("getStatistics",
                  new NetworkRequestHandler() {
             @Override
             public Element handle(Connection connection, Element element) {
-                return new StatisticsMessage(getGame(),
-                                             freeColServer.getAIMain())
-                    .toXMLElement();
+                return freeColServer.getInGameController()
+                    .getStatistics(freeColServer.getPlayer(connection));
             }});
         register(UpdateCurrentStopMessage.getXMLElementTagName(),
                  new NetworkRequestHandler() {
