@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -20,8 +20,7 @@
 package net.sf.freecol.server.generator;
 
 import java.io.File;
-import java.io.PrintWriter;
-import java.io.StringWriter;
+import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Comparator;
@@ -32,15 +31,11 @@ import java.util.List;
 import java.util.Random;
 import java.util.logging.Logger;
 
-import javax.xml.stream.XMLStreamConstants;
-import javax.xml.stream.XMLStreamException;
-import javax.xml.stream.XMLStreamReader;
-
 import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.common.FreeColException;
 import net.sf.freecol.common.io.FreeColSavegameFile;
-import net.sf.freecol.common.io.FreeColTcFile;
+import net.sf.freecol.common.model.Ability;
 import net.sf.freecol.common.model.AbstractUnit;
 import net.sf.freecol.common.model.Building;
 import net.sf.freecol.common.model.BuildingType;
@@ -48,6 +43,7 @@ import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.ColonyTile;
 import net.sf.freecol.common.model.EuropeanNationType;
 import net.sf.freecol.common.model.Game;
+import net.sf.freecol.common.model.GameOptions;
 import net.sf.freecol.common.model.Goods;
 import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.IndianNationType;
@@ -56,6 +52,7 @@ import net.sf.freecol.common.model.LostCityRumour;
 import net.sf.freecol.common.model.Map;
 import net.sf.freecol.common.model.Map.Direction;
 import net.sf.freecol.common.model.Map.Position;
+import net.sf.freecol.common.model.Nation;
 import net.sf.freecol.common.model.NationType;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Specification;
@@ -65,16 +62,14 @@ import net.sf.freecol.common.model.TileImprovementType;
 import net.sf.freecol.common.model.TileItemContainer;
 import net.sf.freecol.common.model.TileType;
 import net.sf.freecol.common.model.Unit;
-import net.sf.freecol.common.model.Unit.UnitState;
 import net.sf.freecol.common.model.UnitType;
 import net.sf.freecol.common.option.FileOption;
+import net.sf.freecol.common.option.MapGeneratorOptions;
 import net.sf.freecol.common.option.OptionGroup;
 import net.sf.freecol.common.util.RandomChoice;
-import net.sf.freecol.common.util.XMLStream;
 import net.sf.freecol.server.FreeColServer;
 import net.sf.freecol.server.model.ServerBuilding;
 import net.sf.freecol.server.model.ServerColony;
-import net.sf.freecol.server.model.ServerGame;
 import net.sf.freecol.server.model.ServerIndianSettlement;
 import net.sf.freecol.server.model.ServerRegion;
 import net.sf.freecol.server.model.ServerUnit;
@@ -124,19 +119,29 @@ public class SimpleMapGenerator implements MapGenerator {
             / 100;
     }
 
-    /* (non-Javadoc)
+    /**
+     * Creates a map given for a game.
+     *
+     * @param game The <code>Game</code> to use.
      * @see net.sf.freecol.server.generator.IMapGenerator#createMap(net.sf.freecol.common.model.Game)
      * @see net.sf.freecol.server.generator.IMapGenerator#createMap(net.sf.freecol.common.model.Game)
      */
     public void createMap(Game game) throws FreeColException {
-
         // Prepare imports:
         final File importFile = ((FileOption) getMapGeneratorOptions()
-                                 .getOption(MapGeneratorOptions.IMPORT_FILE)).getValue();
+            .getOption(MapGeneratorOptions.IMPORT_FILE)).getValue();
         final Game importGame;
         if (importFile != null) {
-            logger.info("Importing file " + importFile.getPath());
-            importGame = loadSaveGame(importFile);
+            Game g = null;
+            try {
+                logger.info("Importing file " + importFile.getPath());
+                g = FreeColServer
+                    .readGame(new FreeColSavegameFile(importFile),
+                        game.getSpecification(), null);
+            } catch (IOException ioe) {
+                g = null;
+            }
+            importGame = g;
         } else {
             importGame = null;
         }
@@ -181,84 +186,6 @@ public class SimpleMapGenerator implements MapGenerator {
         terrainGenerator.createMap(game, null, landMap);
     }
 
-
-    /**
-     * Loads a <code>Game</code> from the given <code>File</code>.
-     *
-     * @param importFile The <code>File</code> to be loading the
-     *      <code>Game</code> from.
-     * @return The <code>Game</code>.
-     */
-    private Game loadSaveGame(File importFile) throws FreeColException {
-        /*
-         * TODO-LATER: We are using same method in FreeColServer.
-         *       Create a framework for loading games/maps.
-         */
-        XMLStream xs = null;
-        Game game = null;
-        try {
-            final FreeColSavegameFile fis = new FreeColSavegameFile(importFile);
-            xs = FreeColServer.createXMLStreamReader(fis);
-            final XMLStreamReader xsr = xs.getXMLStreamReader();
-            xsr.nextTag();
-
-            int savegameVersion = FreeColServer.getSavegameVersion(xsr);
-            logger.info("Found savegame version " + savegameVersion);
-
-            ArrayList<String> serverObjects = null;
-            while (xsr.nextTag() != XMLStreamConstants.END_ELEMENT) {
-                if (xsr.getLocalName().equals("serverObjects")) {
-                    serverObjects = new ArrayList<String>();
-                    while (xsr.nextTag() != XMLStreamConstants.END_ELEMENT) {
-                        serverObjects.add(xsr.getLocalName());
-                        serverObjects.add(xsr.getAttributeValue(null, "ID"));
-                        xsr.nextTag();
-                    }
-                } else if (xsr.getLocalName().equals(Game.getXMLElementTagName())) {
-                    // Read the game model:
-                    Specification specification = null;
-                    if (savegameVersion < 9) {
-                        // compatibility code
-                        logger.info("Compatibility code: providing fresh specification.");
-                        specification = new FreeColTcFile("freecol").getSpecification();
-                    }
-                    game = new ServerGame(null, xsr, serverObjects, specification);
-                    if (savegameVersion < 9) {
-                        logger.info("Compatibility code: applying difficulty level.");
-                        // Apply the difficulty level
-                        if (game.getDifficultyLevel() == null) {
-                            logger.fine("Difficulty level is null");
-                            game.getSpecification().applyDifficultyLevel("model.difficulty.medium");
-                        } else {
-                            logger.fine("Difficulty level is " + game.getDifficultyLevel().getId());
-                            //game.getSpecification().applyDifficultyLevel(game.getDifficultyLevel());
-                        }
-                    }
-                    game.setCurrentPlayer(null);
-                    game.checkIntegrity();
-                }
-            }
-        } catch (XMLStreamException e) {
-            StringWriter sw = new StringWriter();
-            e.printStackTrace(new PrintWriter(sw));
-            logger.warning(sw.toString());
-            throw new FreeColException(e.toString());
-        } catch (FreeColException fe) {
-            StringWriter sw = new StringWriter();
-            fe.printStackTrace(new PrintWriter(sw));
-            logger.warning(sw.toString());
-            throw new FreeColException(fe.toString());
-        } catch (Exception e) {
-            StringWriter sw = new StringWriter();
-            e.printStackTrace(new PrintWriter(sw));
-            logger.warning(sw.toString());
-            throw new FreeColException(e.toString());
-        } finally {
-            if (xs != null) xs.close();
-        }
-        return game;
-    }
-
     public LandGenerator getLandGenerator() {
         return landGenerator;
     }
@@ -311,9 +238,9 @@ public class SimpleMapGenerator implements MapGenerator {
                 .getInteger("model.option.rumourDifficulty");
             for (int i = 0; i < number; i++) {
                 for (int tries=0; tries<100; tries++) {
-                    Position p = map.getRandomLandPosition(random);
+                    Position p = terrainGenerator.getRandomLandPosition(map, random);
                     Tile t = map.getTile(p);
-                    if (map.isPolar(t)) continue; // No polar lost cities
+                    if (t.isPolar()) continue; // No polar lost cities
                     if (t.isLand() && !t.hasLostCityRumour()
                         && t.getSettlement() == null && t.getUnitCount() == 0) {
                         LostCityRumour r = new LostCityRumour(t.getGame(), t);
@@ -371,14 +298,14 @@ public class SimpleMapGenerator implements MapGenerator {
                 = ((IndianNationType) player.getNationType()).getRegionNames();
             Territory territory = null;
             if (regionNames == null || regionNames.isEmpty()) {
-                territory = new Territory(player, map.getRandomLandPosition(random));
+                territory = new Territory(player, terrainGenerator.getRandomLandPosition(map, random));
                 territoryMap.put(player.getId(), territory);
             } else {
                 for (String name : regionNames) {
                     if (territoryMap.get(name) == null) {
                         ServerRegion region = (ServerRegion) map.getRegion(name);
                         if (region == null) {
-                            territory = new Territory(player, map.getRandomLandPosition(random));
+                            territory = new Territory(player, terrainGenerator.getRandomLandPosition(map, random));
                         } else {
                             territory = new Territory(player, region);
                         }
@@ -410,7 +337,7 @@ public class SimpleMapGenerator implements MapGenerator {
                     if (territory == null) {
                         logger.warning("Unable to find free region for "
                                        + player.getName());
-                        territory = new Territory(player, map.getRandomLandPosition(random));
+                        territory = new Territory(player, terrainGenerator.getRandomLandPosition(map, random));
                         territoryMap.put(player.getId(), territory);
                     }
                 }
@@ -426,7 +353,7 @@ public class SimpleMapGenerator implements MapGenerator {
             = spec.getRangeOption("model.option.settlementNumber").getValue();
         List<Tile> settlementTiles = new ArrayList<Tile>();
         tiles: for (Tile tile : map.getAllTiles()) {
-            if (!map.isPolar(tile) && tile.getType().canSettle()) {
+            if (!tile.isPolar() && suitableForNativeSettlement(tile)) {
                 for (Tile t : settlementTiles) {
                     if (tile.getDistanceTo(t) < minSettlementDistance) {
                         continue tiles;
@@ -553,8 +480,7 @@ public class SimpleMapGenerator implements MapGenerator {
                      i > 0; i--) {
                     Tile tile = findFreeNeighbouringTile(is, tiles, random);
                     if (tile == null) break;
-                    tile.setOwner(is.getOwner());
-                    tile.setOwningSettlement(is);
+                    tile.changeOwnership(is.getOwner(), is);
                     tiles.add(tile);
                 }
             }
@@ -643,10 +569,29 @@ public class SimpleMapGenerator implements MapGenerator {
                     + " Indian settlements of maximum " + settlementsToPlace);
     }
 
+    /**
+     * Is a tile suitable for a native settlement?
+     * Require the tile be settleable, and at least half its neighbours
+     * also be settleable.  TODO: degrade the second test to usability,
+     * but fix this when the natives-use-water situation is sorted.
+     *
+     * @param tile The <code>Tile</code> to examine.
+     * @return True if this tile is suitable.
+     */
+    private boolean suitableForNativeSettlement(Tile tile) {
+        if (!tile.getType().canSettle()) return false;
+        int good = 0, n = 0;
+        for (Tile t : tile.getSurroundingTiles(1)) {
+            if (t.getType().canSettle()) good++;
+            n++;
+        }
+        return good >= n / 2;
+    }
+
     private Tile findFreeNeighbouringTile(IndianSettlement is,
                                           List<Tile> tiles, Random random) {
         for (Tile tile : tiles) {
-            for (Direction d : Direction.getRandomDirectionArray(random)) {
+            for (Direction d : Direction.getRandomDirections("freeTile", random)) {
                 Tile t = tile.getNeighbourOrNull(d);
                 if ((t != null)
                     && (t.getOwningSettlement() == null)
@@ -711,6 +656,7 @@ public class SimpleMapGenerator implements MapGenerator {
             = new ServerIndianSettlement(map.getGame(), player, name, tile,
                                          capital, skill,
                                          new HashSet<Player>(), null);
+        player.addSettlement(settlement);
         logger.fine("Generated skill: " + settlement.getLearnableSkill());
 
         int low = settlement.getType().getMinimumSize();
@@ -719,8 +665,7 @@ public class SimpleMapGenerator implements MapGenerator {
         for (int i = 0; i < unitCount; i++) {
             UnitType unitType = map.getSpecification().getUnitType("model.unit.brave");
             Unit unit = new ServerUnit(map.getGame(), settlement, player,
-                                       unitType, UnitState.ACTIVE,
-                                       unitType.getDefaultEquipment());
+                unitType, unitType.getDefaultEquipment());
             unit.setIndianSettlement(settlement);
 
             if (i == 0) {
@@ -729,7 +674,7 @@ public class SimpleMapGenerator implements MapGenerator {
                 unit.setLocation(settlement);
             }
         }
-        settlement.placeSettlement();
+        settlement.placeSettlement(true);
 
         // START DEBUG:
         if (FreeCol.isInDebugMode()) {
@@ -773,7 +718,7 @@ public class SimpleMapGenerator implements MapGenerator {
                                                         random, scaledSkills);
         if (skill == null) {
             // Seasoned Scout
-            List<UnitType> unitList = map.getSpecification().getUnitTypesWithAbility("model.ability.expertScout");
+            List<UnitType> unitList = map.getSpecification().getUnitTypesWithAbility(Ability.EXPERT_SCOUT);
             return unitList.get(random.nextInt(unitList.size()));
         } else {
             return skill;
@@ -811,10 +756,13 @@ public class SimpleMapGenerator implements MapGenerator {
                 logger.finest("found European player " + player);
             }
         }
-        int startingPositions = europeanPlayers.size();
-        List<Integer> startingYPositions = new ArrayList<Integer>();
 
-        for (Player player : europeanPlayers) {
+        List<Position> positions = generateStartingPositions(map, europeanPlayers);
+        List<Tile> startingTiles = new ArrayList<Tile>();
+
+        for (int index = 0; index < europeanPlayers.size(); index++) {
+            Player player = europeanPlayers.get(index);
+            Position position = positions.get(index);
             logger.fine("generating units for player " + player);
 
             List<Unit> carriers = new ArrayList<Unit>();
@@ -824,12 +772,12 @@ public class SimpleMapGenerator implements MapGenerator {
             for (AbstractUnit startingUnit : unitList) {
                 Unit newUnit = new ServerUnit(game, null, player,
                                               startingUnit.getUnitType(spec),
-                                              UnitState.SENTRY,
                                               startingUnit.getEquipment(spec));
                 if (newUnit.canCarryUnits() && newUnit.isNaval()) {
-                    newUnit.setState(UnitState.ACTIVE);
+                    newUnit.setState(Unit.UnitState.ACTIVE);
                     carriers.add(newUnit);
                 } else {
+                    newUnit.setState(Unit.UnitState.SENTRY);
                     passengers.add(newUnit);
                 }
 
@@ -841,23 +789,27 @@ public class SimpleMapGenerator implements MapGenerator {
                 startAtSea = false;
             }
 
-            // find an appropriate starting latitude
-            int x = width - 1;  // eastern edge of map
-            int y;
-            do {
-                 y = random.nextInt(height - poleDistance*2) + poleDistance;
-            } while (map.getTile(x, y).isLand() == startAtSea ||
-                     isStartingPositionTooClose(map, y, startingPositions, startingYPositions));
-            startingYPositions.add(new Integer(y));
-
-            if (startAtSea) {
-                // move westward to find the limit between high seas and coastal waters
-                while (map.getTile(x - 1, y).canMoveToEurope()) {
-                    x--;
+            Tile startTile = null;
+            int x = position.getX();
+            int y = position.getY();
+            for (int i = 0; i < 2 * map.getHeight(); i++) {
+                int offset = (i % 2 == 0) ? i / 2 : -(1 + i / 2);
+                startTile = findTileFor(map, y + offset, x, startAtSea);
+                if (startTile != null) {
+                    if (startingTiles.contains(startTile)) {
+                        startTile = null;
+                    } else {
+                        startingTiles.add(startTile);
+                        break;
+                    }
                 }
             }
 
-            Tile startTile = map.getTile(x,y);
+            if (startTile == null) {
+                throw new RuntimeException("Failed to find start tile for player "
+                                           + player);
+            }
+
             startTile.setExploredBy(player, true);
             player.setEntryLocation(startTile);
 
@@ -881,197 +833,234 @@ public class SimpleMapGenerator implements MapGenerator {
                 }
             }
 
-            if (!FreeCol.isInFullDebugMode()) continue;
-            // In debug mode give each player a few more units and a colony.
-            UnitType unitType = spec.getUnitType("model.unit.galleon");
-            Unit unit4 = new ServerUnit(game, startTile, player, unitType,
-                                        UnitState.ACTIVE);
-
-            unitType = spec.getUnitType("model.unit.privateer");
-            @SuppressWarnings("unused")
-            Unit privateer = new ServerUnit(game, startTile, player,
-                                            unitType, UnitState.ACTIVE);
-
-            unitType = spec.getUnitType("model.unit.freeColonist");
-            @SuppressWarnings("unused")
-            Unit unit5 = new ServerUnit(game, unit4, player, unitType,
-                                        UnitState.SENTRY);
-            unitType = spec.getUnitType("model.unit.veteranSoldier");
-            @SuppressWarnings("unused")
-            Unit unit6 = new ServerUnit(game, unit4, player, unitType,
-                                        UnitState.SENTRY);
-            unitType = spec.getUnitType("model.unit.jesuitMissionary");
-            @SuppressWarnings("unused")
-            Unit unit7 = new ServerUnit(game, unit4, player, unitType,
-                                        UnitState.SENTRY);
-
-            Tile colonyTile = null;
-            Iterator<Position> cti
-                = map.getFloodFillIterator(new Position(x, y));
-            while (cti.hasNext()) {
-                Tile tempTile = map.getTile(cti.next());
-                if (map.isPolar(tempTile)) {
-                    // do not place the initial colony at the pole
-                    continue;
-                }
-                if (player.canClaimToFoundSettlement(tempTile)) {
-                    colonyTile = tempTile;
-                    break;
-                }
+            if (FreeCol.getDebugLevel() >= FreeCol.DEBUG_FULL) {
+                createDebugUnits(map, player, startTile);
             }
+        }
+    }
 
-            if (colonyTile == null) {
-                logger.warning("Could not find a debug colony site.");
+    private void createDebugUnits(Map map, Player player, Tile startTile) {
+
+        Game game = map.getGame();
+        Specification spec = game.getSpecification();
+
+        // In debug mode give each player a few more units and a colony.
+        UnitType unitType = spec.getUnitType("model.unit.galleon");
+        Unit unit4 = new ServerUnit(game, startTile, player, unitType);
+
+        unitType = spec.getUnitType("model.unit.privateer");
+        @SuppressWarnings("unused")
+        Unit privateer = new ServerUnit(game, startTile, player, unitType);
+
+        unitType = spec.getUnitType("model.unit.freeColonist");
+        @SuppressWarnings("unused")
+        Unit unit5 = new ServerUnit(game, unit4, player, unitType);
+        unitType = spec.getUnitType("model.unit.veteranSoldier");
+        @SuppressWarnings("unused")
+        Unit unit6 = new ServerUnit(game, unit4, player, unitType);
+        unitType = spec.getUnitType("model.unit.jesuitMissionary");
+        @SuppressWarnings("unused")
+        Unit unit7 = new ServerUnit(game, unit4, player, unitType);
+
+        Tile colonyTile = null;
+        Iterator<Position> cti
+            = map.getFloodFillIterator(startTile.getPosition());
+        while (cti.hasNext()) {
+            Tile tempTile = map.getTile(cti.next());
+            if (tempTile.isPolar()) {
+                // do not place the initial colony at the pole
                 continue;
             }
+            if (player.canClaimToFoundSettlement(tempTile)) {
+                colonyTile = tempTile;
+                break;
+            }
+        }
+
+        if (colonyTile == null) {
+            logger.warning("Could not find a debug colony site.");
+            return;
+        }
+        for (TileType t : spec.getTileTypeList()) {
+            if (!t.isWater()) {
+                colonyTile.setType(t);
+                break;
+            }
+        }
+        unitType = spec.getUnitType("model.unit.expertFarmer");
+        Unit buildColonyUnit = new ServerUnit(game, colonyTile,
+                                              player, unitType);
+        String colonyName = Messages.message(player.getNationName())
+            + " Colony";
+        Colony colony = new ServerColony(game, player, colonyName, colonyTile);
+        player.addSettlement(colony);
+        colony.placeSettlement(true);
+        for (Tile tile : colonyTile.getSurroundingTiles(1)) {
+            if (tile.getSettlement() == null
+                && (tile.getOwner() == null
+                    || !tile.getOwner().isEuropean())) {
+                tile.changeOwnership(player, colony);
+                if (tile.hasLostCityRumour()) {
+                    tile.removeLostCityRumour();
+                }
+            }
+        }
+        buildColonyUnit.setLocation(colony);
+        if (buildColonyUnit.getLocation() instanceof ColonyTile) {
+            Tile ct = ((ColonyTile) buildColonyUnit.getLocation()).getWorkTile();
             for (TileType t : spec.getTileTypeList()) {
                 if (!t.isWater()) {
-                    colonyTile.setType(t);
+                    ct.setType(t);
+                    TileImprovementType plowType = map.getSpecification()
+                        .getTileImprovementType("model.improvement.plow");
+                    TileImprovementType roadType = map.getSpecification()
+                        .getTileImprovementType("model.improvement.road");
+                    TileImprovement road = new TileImprovement(game, ct, roadType);
+                    road.setTurnsToComplete(0);
+                    TileImprovement plow = new TileImprovement(game, ct, plowType);
+                    plow.setTurnsToComplete(0);
+                    ct.setTileItemContainer(new TileItemContainer(game, ct));
+                    ct.getTileItemContainer().addTileItem(road);
+                    ct.getTileItemContainer().addTileItem(plow);
                     break;
                 }
             }
-            unitType = spec.getUnitType("model.unit.expertFarmer");
-            Unit buildColonyUnit = new ServerUnit(game, colonyTile,
-                                                  player, unitType,
-                                                  UnitState.ACTIVE);
-            String colonyName = Messages.message(player.getNationName())
-                + " Colony";
-            Colony colony = new ServerColony(game, player, colonyName, colonyTile);
-            colony.placeSettlement();
-            for (Tile tile : colonyTile.getSurroundingTiles(1)) {
-                if (tile.getSettlement() == null
-                    && (tile.getOwner() == null
-                        || !tile.getOwner().isEuropean())) {
-                    tile.setOwner(player);
-                    tile.setOwningSettlement(colony);
-                    if (tile.hasLostCityRumour()) {
-                        tile.removeLostCityRumour();
-                    }
-                }
-            }
-            buildColonyUnit.setState(UnitState.IN_COLONY);
-            buildColonyUnit.setLocation(colony);
-            if (buildColonyUnit.getLocation() instanceof ColonyTile) {
-                Tile ct = ((ColonyTile) buildColonyUnit.getLocation()).getWorkTile();
-                for (TileType t : spec.getTileTypeList()) {
-                    if (!t.isWater()) {
-                        ct.setType(t);
-                        TileImprovementType plowType = map.getSpecification()
-                            .getTileImprovementType("model.improvement.plow");
-                        TileImprovementType roadType = map.getSpecification()
-                            .getTileImprovementType("model.improvement.road");
-                        TileImprovement road = new TileImprovement(game, ct, roadType);
-                        road.setTurnsToComplete(0);
-                        TileImprovement plow = new TileImprovement(game, ct, plowType);
-                        plow.setTurnsToComplete(0);
-                        ct.setTileItemContainer(new TileItemContainer(game, ct));
-                        ct.getTileItemContainer().addTileItem(road);
-                        ct.getTileItemContainer().addTileItem(plow);
-                        break;
-                    }
-                }
-            }
-            BuildingType schoolType = spec.getBuildingType("model.building.schoolhouse");
-            Building schoolhouse = new ServerBuilding(game, colony, schoolType);
-            colony.addBuilding(schoolhouse);
-            unitType = spec.getUnitType("model.unit.masterCarpenter");
-            while (!schoolhouse.canAdd(unitType)) {
-                schoolhouse.upgrade();
-            }
-            Unit carpenter = new ServerUnit(game, colonyTile, player,
-unitType, UnitState.ACTIVE);
-            carpenter.setLocation(colony.getBuildingForProducing(unitType.getExpertProduction()));
-
-            unitType = spec.getUnitType("model.unit.elderStatesman");
-            Unit statesman = new ServerUnit(game, colonyTile, player,
-                                            unitType, UnitState.ACTIVE);
-            statesman.setLocation(colony.getBuildingForProducing(unitType.getExpertProduction()));
-
-            unitType = spec.getUnitType("model.unit.expertLumberJack");
-            Unit lumberjack = new ServerUnit(game, colony, player,
-                                             unitType, UnitState.ACTIVE);
-            if (lumberjack.getLocation() instanceof ColonyTile) {
-                Tile lt = ((ColonyTile) lumberjack.getLocation()).getWorkTile();
-                for (TileType t : spec.getTileTypeList()) {
-                    if (t.isForested()) {
-                        lt.setType(t);
-                        break;
-                    }
-                }
-                lumberjack.setWorkType(lumberjack.getType().getExpertProduction());
-            }
-
-            unitType = spec.getUnitType("model.unit.seasonedScout");
-            @SuppressWarnings("unused")
-                Unit scout = new ServerUnit(game, colonyTile, player,
-                                            unitType, UnitState.ACTIVE);
-
-            unitType = spec.getUnitType("model.unit.veteranSoldier");
-            @SuppressWarnings("unused")
-                Unit unit8 = new ServerUnit(game, colonyTile, player,
-                                            unitType, UnitState.ACTIVE);
-
-            @SuppressWarnings("unused")
-                Unit unit9 = new ServerUnit(game, colonyTile, player,
-                                            unitType, UnitState.ACTIVE);
-
-            unitType = spec.getUnitType("model.unit.artillery");
-            @SuppressWarnings("unused")
-                Unit unit10 = new ServerUnit(game, colonyTile, player,
-                                             unitType, UnitState.ACTIVE);
-
-            @SuppressWarnings("unused")
-                Unit unit11 = new ServerUnit(game, colonyTile, player,
-                                             unitType, UnitState.ACTIVE);
-
-            @SuppressWarnings("unused")
-                Unit unit12 = new ServerUnit(game, colonyTile, player,
-                                             unitType, UnitState.ACTIVE);
-
-            unitType = spec.getUnitType("model.unit.treasureTrain");
-            Unit unit13 = new ServerUnit(game, colonyTile, player,
-                                         unitType, UnitState.ACTIVE);
-            unit13.setTreasureAmount(10000);
-
-            unitType = spec.getUnitType("model.unit.wagonTrain");
-            Unit unit14 = new ServerUnit(game, colonyTile, player,
-                                         unitType, UnitState.ACTIVE);
-            GoodsType cigarsType = spec.getGoodsType("model.goods.cigars");
-            Goods cigards = new Goods(game, unit14, cigarsType, 5);
-            unit14.add(cigards);
-
-            unitType = spec.getUnitType("model.unit.jesuitMissionary");
-            @SuppressWarnings("unused")
-                Unit unit15 = new ServerUnit(game, colonyTile, player,
-                                             unitType, UnitState.ACTIVE);
-            @SuppressWarnings("unused")
-                Unit unit16 = new ServerUnit(game, colonyTile, player,
-                                             unitType, UnitState.ACTIVE);
-            // END DEBUG
         }
+        BuildingType schoolType = spec.getBuildingType("model.building.schoolhouse");
+        Building schoolhouse = new ServerBuilding(game, colony, schoolType);
+        colony.addBuilding(schoolhouse);
+        unitType = spec.getUnitType("model.unit.masterCarpenter");
+        Unit carpenter = new ServerUnit(game, colonyTile, player, unitType);
+        carpenter.setLocation(colony.getBuildingForProducing(unitType.getExpertProduction()));
+
+        unitType = spec.getUnitType("model.unit.elderStatesman");
+        Unit statesman = new ServerUnit(game, colonyTile, player, unitType);
+        statesman.setLocation(colony.getBuildingForProducing(unitType.getExpertProduction()));
+
+        unitType = spec.getUnitType("model.unit.expertLumberJack");
+        Unit lumberjack = new ServerUnit(game, colony, player, unitType);
+        if (lumberjack.getLocation() instanceof ColonyTile) {
+            Tile lt = ((ColonyTile) lumberjack.getLocation()).getWorkTile();
+            for (TileType t : spec.getTileTypeList()) {
+                if (t.isForested()) {
+                    lt.setType(t);
+                    break;
+                }
+            }
+            lumberjack.setWorkType(lumberjack.getType().getExpertProduction());
+        }
+
+        unitType = spec.getUnitType("model.unit.seasonedScout");
+        @SuppressWarnings("unused")
+        Unit scout = new ServerUnit(game, colonyTile, player, unitType);
+
+        unitType = spec.getUnitType("model.unit.veteranSoldier");
+        @SuppressWarnings("unused")
+        Unit unit8 = new ServerUnit(game, colonyTile, player, unitType);
+
+        @SuppressWarnings("unused")
+        Unit unit9 = new ServerUnit(game, colonyTile, player, unitType);
+
+        unitType = spec.getUnitType("model.unit.artillery");
+        @SuppressWarnings("unused")
+        Unit unit10 = new ServerUnit(game, colonyTile, player, unitType);
+
+        @SuppressWarnings("unused")
+        Unit unit11 = new ServerUnit(game, colonyTile, player, unitType);
+
+        @SuppressWarnings("unused")
+        Unit unit12 = new ServerUnit(game, colonyTile, player, unitType);
+
+        unitType = spec.getUnitType("model.unit.treasureTrain");
+        Unit unit13 = new ServerUnit(game, colonyTile, player, unitType);
+        unit13.setTreasureAmount(10000);
+
+        unitType = spec.getUnitType("model.unit.wagonTrain");
+        Unit unit14 = new ServerUnit(game, colonyTile, player, unitType);
+        GoodsType cigarsType = spec.getGoodsType("model.goods.cigars");
+        Goods cigards = new Goods(game, unit14, cigarsType, 5);
+        unit14.add(cigards);
+
+        unitType = spec.getUnitType("model.unit.jesuitMissionary");
+        @SuppressWarnings("unused")
+        Unit unit15 = new ServerUnit(game, colonyTile, player, unitType);
+        @SuppressWarnings("unused")
+        Unit unit16 = new ServerUnit(game, colonyTile, player, unitType);
+        // END DEBUG
     }
 
-    /**
-     * Determine whether a proposed ship starting Y position is "too close"
-     * to those already used.
-     * @param map a <code>Map</code> value
-     * @param proposedY Proposed ship starting Y position
-     * @param startingPositions The number of starting positions
-     * @return True if the proposed position is too close
-     */
-    private boolean isStartingPositionTooClose(Map map, int proposedY, int startingPositions,
-                                                 List<Integer> usedYPositions) {
-        final int poleDistance = (int)(MIN_DISTANCE_FROM_POLE*map.getHeight()/2);
-        final int spawnableRange = map.getHeight() - poleDistance*2;
-        final int minimumDistance = spawnableRange / (startingPositions * 2);
-        for (Integer yPosition : usedYPositions) {
-            if (Math.abs(yPosition.intValue() - proposedY) < minimumDistance) {
-                return true;
+    private List<Position> generateStartingPositions(Map map, List<Player> players) {
+        int number = players.size();
+        List<Position> positions = new ArrayList<Position>(number);
+        if (number > 0) {
+            int west = 0;
+            int east = map.getWidth() - 1;
+            switch(map.getSpecification().getInteger(GameOptions.STARTING_POSITIONS)) {
+            case GameOptions.STARTING_POSITIONS_CLASSIC:
+                int distance = map.getHeight() / number;
+                int row = distance/2;
+                for (int index = 0; index < number; index++) {
+                    positions.add(new Position(east, row));
+                    row += distance;
+                }
+                Collections.shuffle(positions);
+                break;
+            case GameOptions.STARTING_POSITIONS_RANDOM:
+                distance = 2 * map.getHeight() / number;
+                row = distance/2;
+                for (int index = 0; index < number; index++) {
+                    if (index % 2 == 0) {
+                        positions.add(new Position(east, row));
+                    } else {
+                        positions.add(new Position(west, row));
+                        row += distance;
+                    }
+                }
+                Collections.shuffle(positions);
+                break;
+            case GameOptions.STARTING_POSITIONS_HISTORICAL:
+                for (Player player : players) {
+                    Nation nation = player.getNation();
+                    positions.add(new Position(nation.startsOnEastCoast() ? east : west,
+                                               map.getRow(nation.getPreferredLatitude())));
+                }
+                break;
             }
         }
-        return false;
+        return positions;
     }
 
+    private Tile findTileFor(Map map, int row, int start, boolean startAtSea) {
+        Tile tile = null;
+        if (0 <= row && row < map.getHeight()) {
+            Tile firstLand = null;
+            int offset = (start == 0) ? 1 : -1;
+            for (int x = start; 0 <= x && x < map.getWidth(); x += offset) {
+                tile = map.getTile(x, row);
+                if (tile.isLand()) {
+                    if (startAtSea) {
+                        firstLand = tile;
+                        start = x - offset;
+                        break;
+                    } else {
+                        return tile;
+                    }
+                }
+            }
+            if (firstLand != null) {
+                offset = -offset;
+                for (int x = start; 0 <= x && x < map.getWidth(); x += offset) {
+                    tile = map.getTile(x, row);
+                    if (tile.canMoveToEurope()) {
+                        return tile;
+                    }
+                }
+            }
+        }
+        return null;
+    }
+
+
     private class Territory {
         public ServerRegion region;
         public Position position;
