@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -22,6 +22,7 @@ package net.sf.freecol.server.generator;
 import java.awt.Rectangle;
 
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.EnumMap;
 import java.util.HashMap;
 import java.util.Iterator;
@@ -45,6 +46,7 @@ import net.sf.freecol.common.model.TileImprovement;
 import net.sf.freecol.common.model.TileImprovementType;
 import net.sf.freecol.common.model.TileItemContainer;
 import net.sf.freecol.common.model.TileType;
+import net.sf.freecol.common.option.MapGeneratorOptions;
 import net.sf.freecol.common.option.OptionGroup;
 import net.sf.freecol.server.model.ServerRegion;
 import net.sf.freecol.common.util.RandomChoice;
@@ -60,23 +62,20 @@ public class TerrainGenerator {
     public static final int LAND_REGIONS_SCORE_VALUE = 1000;
     public static final int LAND_REGION_MIN_SCORE = 5;
     public static final int PACIFIC_SCORE_VALUE = 100;
-
     public static final int LAND_REGION_MAX_SIZE = 75;
 
-    public static final Direction[] corners = new Direction[] {
-        Direction.N, Direction.E, Direction.S, Direction.W
-    };
-
     private final OptionGroup mapGeneratorOptions;
     private final Random random;
 
-    private TileType ocean;
     private TileType lake;
     private TileImprovementType riverType;
     private TileImprovementType fishBonusLandType;
     private TileImprovementType fishBonusRiverType;
 
-    private ArrayList<TileType> terrainTileTypes = null;
+    private ArrayList<TileType> terrainTileTypes;
+    private ArrayList<TileType> oceanTileTypes;
+
+    // TODO dynamic lakes, mountains and hills
 
     /**
      * Creates a new <code>TerrainGenerator</code>.
@@ -90,6 +89,36 @@ public class TerrainGenerator {
         this.random = random;
     }
 
+    // TODO: this might be useful elsewhere, too
+    private int limitToRange(int value, int lower, int upper) {
+        return Math.max(lower, Math.min(value, upper));
+    }
+
+    /**
+     * Select a random land position on the map.
+     *
+     * <b>Warning:</b> This method should not be used by any model
+     * object unless we have completed restructuring the model
+     * (making all model changes at the server). The reason is
+     * the use of random numbers in this method.
+     *
+     * @param map The <code>Map</code> to search in.
+     * @param random A <code>Random</code> number source.
+     * @return Position selected
+     */
+    public Position getRandomLandPosition(Map map, Random random) {
+        int x = (map.getWidth() < 10) ? random.nextInt(map.getWidth())
+            : random.nextInt(map.getWidth() - 10) + 5;
+        int y = (map.getHeight() < 10) ? random.nextInt(map.getHeight())
+            : random.nextInt(map.getHeight() - 10) + 5;
+        Position centerPosition = new Position(x, y);
+        Iterator<Position> it = map.getFloodFillIterator(centerPosition);
+        while (it.hasNext()) {
+            Position p = it.next();
+            if (map.getTile(p).isLand()) return p;
+        }
+        return null;
+    }
 
     /**
      * Creates a <code>Map</code> for the given <code>Game</code>.
@@ -125,7 +154,6 @@ public class TerrainGenerator {
      */
     public void createMap(Game game, Game importGame, boolean[][] landMap) {
         Specification spec = game.getSpecification();
-        ocean = spec.getTileType("model.tile.ocean");
         lake = spec.getTileType("model.tile.lake");
         riverType = spec.getTileImprovementType("model.improvement.river");
         fishBonusLandType = spec.getTileImprovementType("model.improvement.fishBonusLand");
@@ -141,8 +169,19 @@ public class TerrainGenerator {
 
         boolean mapHasLand = false;
         Tile[][] tiles = new Tile[width][height];
+        Map map = new Map(game, tiles);
+        int minimumLatitude = getMapGeneratorOptions()
+            .getInteger(MapGeneratorOptions.MINIMUM_LATITUDE);
+        int maximumLatitude = getMapGeneratorOptions()
+            .getInteger(MapGeneratorOptions.MAXIMUM_LATITUDE);
+        // make sure the values are in range
+        minimumLatitude = limitToRange(minimumLatitude, -90, 90);
+        maximumLatitude = limitToRange(maximumLatitude, -90, 90);
+        map.setMinimumLatitude(Math.min(minimumLatitude, maximumLatitude));
+        map.setMaximumLatitude(Math.max(minimumLatitude, maximumLatitude));
+
         for (int y = 0; y < height; y++) {
-            int latitude = (Math.min(y, (height-1) - y) * 200) / height; // lat=0 for poles; lat=100 for equator
+            int latitude = map.getLatitude(y);
             for (int x = 0; x < width; x++) {
                 if (landMap[x][y]) {
                     mapHasLand = true;
@@ -173,7 +212,6 @@ public class TerrainGenerator {
             }
         }
 
-        Map map = new Map(game, tiles);
         game.setMap(map);
 
         if (!importTerrain) {
@@ -205,7 +243,7 @@ public class TerrainGenerator {
             new EnumMap<Direction, Boolean>(Direction.class);
 
         // corners
-        for (Direction d : corners) {
+        for (Direction d : Direction.corners) {
             Tile tile = ocean.getNeighbourOrNull(d);
             connections.put(d, (tile != null && tile.isLand()));
         }
@@ -223,7 +261,7 @@ public class TerrainGenerator {
         }
         int result = 0;
         int index = 0;
-        for (Direction d : corners) {
+        for (Direction d : Direction.corners) {
             if (connections.get(d)) {
                 result += (int) Math.pow(2, index);
             }
@@ -243,7 +281,7 @@ public class TerrainGenerator {
         if (landMap[x][y]) {
             t = new Tile(game, getRandomLandTileType(game, latitude), x, y);
         } else {
-            t = new Tile(game, ocean, x, y);
+            t = new Tile(game, getRandomOceanTileType(game, latitude), x, y);
         }
 
         return t;
@@ -331,28 +369,44 @@ public class TerrainGenerator {
         return mapGeneratorOptions;
     }
 
+    /**
+     *
+     * @param game
+     * @param latitude
+     * @return
+     */
+    private TileType getRandomOceanTileType(Game game, int latitude) {
+    	// create the main list of TileTypes the first time, and reuse it afterwards
+        if (oceanTileTypes==null) {
+        	oceanTileTypes = new ArrayList<TileType>();
+            for (TileType tileType : game.getSpecification().getTileTypeList()) {
+                if (tileType.isWater() && tileType.isConnected()
+                	&& !tileType.hasAbility("model.ability.moveToEurope")) {
+                	oceanTileTypes.add(tileType);
+                }
+
+            }
+        }
+
+        return getRandomTileType(game, oceanTileTypes, latitude);
+    }
 
     /**
      * Gets a random land tile type based on the given percentage.
      *
      * @param game the Game
-     * @param latitudePercent The location of the tile relative to the north/south poles and equator,
-     *        100% is the mid-section of the map (equator)
-     *        0% is on the top/bottom of the map (poles).
+     * @param latitude The location of the tile relative to the north/south
+     *        poles and equator:
+     *          0 is the mid-section of the map (equator)
+     *          +/-90 is on the bottom/top of the map (poles).
      */
-    private TileType getRandomLandTileType(Game game, int latitudePercent) {
-        // decode options
-        final int forestChance = getMapGeneratorOptions().getInteger("model.option.forestNumber");
-        final int temperaturePreference = getMapGeneratorOptions().getInteger("model.option.temperature");
-
+    private TileType getRandomLandTileType(Game game, int latitude) {
         // create the main list of TileTypes the first time, and reuse it afterwards
         if (terrainTileTypes==null) {
             terrainTileTypes = new ArrayList<TileType>();
             for (TileType tileType : game.getSpecification().getTileTypeList()) {
-                if (tileType.getId().equals("model.tile.hills") ||
-                    tileType.getId().equals("model.tile.mountains") ||
-                    tileType.isWater()) {
-                    // do not generate hills or mountains at this time
+                if (tileType.isElevation() || tileType.isWater()) {
+                    // do not generate elevated and water tiles at this time
                     // they are created separately
                     continue;
                 }
@@ -360,46 +414,73 @@ public class TerrainGenerator {
             }
         }
 
-        // temperature calculation
+        return getRandomTileType(game, terrainTileTypes, latitude);
+    }
+
+    /**
+     * Returns a TileType, that fits to the regional requirements.
+     * <br/><br/>
+     * TODO: Can be used for mountains and rivers too.
+     *
+     * @param game The game.
+     * @param candidates A list of <tt>TileType</tt>s to use for calculations.
+     * @param latitude The latitude.
+     * @return A <tt>TileType</tt> that fits to the regional requirements.
+     */
+    private TileType getRandomTileType(Game game, List<TileType> candidates, int latitude) {
+    	// decode options
+        final int forestChance = getMapGeneratorOptions().getInteger("model.option.forestNumber");
+        final int temperaturePreference = getMapGeneratorOptions().getInteger("model.option.temperature");
+
+    	// temperature calculation
         int poleTemperature = -20;
         int equatorTemperature= 40;
-        if (temperaturePreference==MapGeneratorOptions.TEMPERATURE_COLD) {
+        switch(temperaturePreference) {
+        case MapGeneratorOptions.TEMPERATURE_COLD:
             poleTemperature = -20;
             equatorTemperature = 25;
-        } else if (temperaturePreference==MapGeneratorOptions.TEMPERATURE_CHILLY) {
+            break;
+        case MapGeneratorOptions.TEMPERATURE_CHILLY:
             poleTemperature = -20;
             equatorTemperature = 30;
-        } else if (temperaturePreference==MapGeneratorOptions.TEMPERATURE_TEMPERATE) {
+            break;
+        case MapGeneratorOptions.TEMPERATURE_TEMPERATE:
             poleTemperature = -10;
             equatorTemperature = 35;
-        } else if (temperaturePreference==MapGeneratorOptions.TEMPERATURE_WARM) {
+            break;
+        case MapGeneratorOptions.TEMPERATURE_WARM:
             poleTemperature = -5;
             equatorTemperature = 40;
-        } else if (temperaturePreference==MapGeneratorOptions.TEMPERATURE_HOT) {
+            break;
+        case MapGeneratorOptions.TEMPERATURE_HOT:
             poleTemperature = 0;
             equatorTemperature = 40;
+            break;
         }
+
         int temperatureRange = equatorTemperature-poleTemperature;
-        int localeTemperature = poleTemperature + latitudePercent*temperatureRange/100;
+        int localeTemperature = poleTemperature + (90 - Math.abs(latitude))
+            * temperatureRange/90;
         int temperatureDeviation = 7; // +/- 7 degrees randomization
         localeTemperature += random.nextInt(temperatureDeviation*2)-temperatureDeviation;
-        if (localeTemperature>40)
-            localeTemperature = 40;
-        if (localeTemperature<-20)
-            localeTemperature = -20;
+        localeTemperature = limitToRange(localeTemperature, -20, 40);
 
         // humidity calculation
         int localeHumidity = game.getSpecification().getRangeOption(MapGeneratorOptions.HUMIDITY).getValue();
         int humidityDeviation = 20; // +/- 20% randomization
         localeHumidity += random.nextInt(humidityDeviation*2) - humidityDeviation;
-        if (localeHumidity<0)
-            localeHumidity = 0;
-        if (localeHumidity>100)
-            localeHumidity = 100;
+        localeHumidity = limitToRange(localeHumidity, 0, 100);
 
-        // initialize list of candidates with all terrain TileTypes
-        ArrayList<TileType> candidateTileTypes = new ArrayList<TileType>();
-        candidateTileTypes.addAll(terrainTileTypes);
+        /*
+         * Make and use a backup of the specified list, because we are modifying the list here.
+         */
+        List<TileType> candidateTileTypes = new ArrayList<TileType>(candidates);
+        /*
+         * There will be a check for forested later, but there might be tiles like
+         * oceans/mountains that do not support forests. Therefore shuffle now,
+         * because in this case, we will just use the last one standing.
+         */
+        Collections.shuffle(candidateTileTypes, this.random);
 
         // remove those that do not match the current temperature
         Iterator<TileType> it = candidateTileTypes.iterator();
@@ -430,13 +511,19 @@ public class TerrainGenerator {
                     +" and humidity==" + localeHumidity);
         }
 
-        // Choose based on forested/unforested
+        // Choose based on forested/unforested if possible
         boolean forested = random.nextInt(100) < forestChance;
         it = candidateTileTypes.iterator();
         while (it.hasNext()) {
             TileType t = it.next();
             if (t.isForested() != forested) {
-                it.remove();
+            	/*
+            	 * Fix for land/ocean/mountain tiles that do not support forests. In this case
+            	 * return the last one standing. (shuffled before to get random tiles here)
+            	 */
+            	if (it.hasNext()) {
+            		it.remove();
+            	}
             }
         }
         if (candidateTileTypes.size() == 1) {
@@ -446,8 +533,11 @@ public class TerrainGenerator {
                     +" and humidity==" + localeHumidity + " and forested=="+forested);
         }
 
-        // All scoped, if none have been selected by elimination, randomly choose one
-        return candidateTileTypes.get(random.nextInt(candidateTileTypes.size()));
+        /*
+         * Return the first one, if more than one is left after termination.
+		 * Note, that we shuffled the list already.
+         */
+        return candidateTileTypes.get(0);
     }
 
 
@@ -805,7 +895,7 @@ public class TerrainGenerator {
                     break;
                 }
             }
-            logger.info("Created land region " + sr.getNameKey()
+            logger.fine("Created land region " + sr.getNameKey()
                         + " (size " + sr.getSize()
                         + ", score " + sr.getScoreValue()
                         + ", parent " + ((sr.getParent() == null) ? "(null)"
@@ -814,7 +904,7 @@ public class TerrainGenerator {
         }
 
         for (ServerRegion gr : geographicRegions) {
-            logger.info("Geographic region " + gr.getNameKey()
+            logger.fine("Geographic region " + gr.getNameKey()
                         + " (size " + gr.getSize()
                         + ", children " + gr.getChildren().size()
                         + ")");
@@ -977,7 +1067,7 @@ public class TerrainGenerator {
         int counter = 0;
         nextTry: for (int tries = 0; tries < 100; tries++) {
             if (counter < number) {
-                Position p = map.getRandomLandPosition(random);
+                Position p = getRandomLandPosition(map, random);
                 if (p == null) {
                     // this can only happen if the map contains no land
                     return;
@@ -1010,7 +1100,7 @@ public class TerrainGenerator {
                 mountainRegion.setDiscoverable(true);
                 mountainRegion.setClaimable(true);
                 map.setRegion(mountainRegion);
-                Direction direction = Direction.getRandomDirection(random);
+                Direction direction = Direction.getRandomDirection("getLand", random);
                 int length = maximumLength - random.nextInt(maximumLength/2);
                 for (int index = 0; index < length; index++) {
                     p = p.getAdjacent(direction);
@@ -1038,7 +1128,7 @@ public class TerrainGenerator {
                 }
                 int scoreValue = 2 * mountainRegion.getSize();
                 mountainRegion.setScoreValue(scoreValue);
-                logger.info("Created mountain region (direction " + direction +
+                logger.fine("Created mountain region (direction " + direction +
                             ", length " + length + ", size " + mountainRegion.getSize() +
                             ", score value " + scoreValue + ").");
             }
@@ -1051,7 +1141,7 @@ public class TerrainGenerator {
         counter = 0;
         nextTry: for (int tries = 0; tries < 1000; tries++) {
             if (counter < number) {
-                Position p = map.getRandomLandPosition(random);
+                Position p = getRandomLandPosition(map, random);
                 Tile t = map.getTile(p);
                 if (t.getType() == hills || t.getType() == mountains) {
                     // already a high ground
@@ -1102,7 +1192,7 @@ public class TerrainGenerator {
 
         for (int i = 0; i < number; i++) {
             nextTry: for (int tries = 0; tries < 100; tries++) {
-                Position position = map.getRandomLandPosition(random);
+                Position position = getRandomLandPosition(map, random);
                 if (!map.getTile(position).getType().canHaveImprovement(riverType)) {
                     continue;
                 }
@@ -1145,7 +1235,7 @@ public class TerrainGenerator {
             }
             scoreValue *= 2;
             region.setScoreValue(scoreValue);
-            logger.info("Created river region (length " + river.getLength() +
+            logger.fine("Created river region (length " + river.getLength() +
                         ", score value " + scoreValue + ").");
         }
     }
@@ -1225,7 +1315,9 @@ public class TerrainGenerator {
         do {
             for (Direction direction : Direction.values()) {
                 Position n = p.getAdjacent(direction);
-                if (Map.isValid(n,boolmap.length,boolmap[0].length) && boolmap[n.getX()][n.getY()] && !visited[n.getX()][n.getY()] && limit > 0) {
+                if (Map.isValid(n,boolmap.length,boolmap[0].length)
+                    && boolmap[n.getX()][n.getY()]
+                    && !visited[n.getX()][n.getY()] && limit > 0) {
                     visited[n.getX()][n.getY()] = true;
                     limit--;
                     q.add(n);
