@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -22,10 +22,10 @@ package net.sf.freecol.server.model;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
-import java.util.Map.Entry;
 import java.util.Random;
 import java.util.logging.Logger;
 
+import net.sf.freecol.common.model.Ability;
 import net.sf.freecol.common.model.AbstractGoods;
 import net.sf.freecol.common.model.BuildQueue;
 import net.sf.freecol.common.model.BuildableType;
@@ -34,7 +34,6 @@ import net.sf.freecol.common.model.BuildingType;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.ColonyTile;
 import net.sf.freecol.common.model.ExportData;
-import net.sf.freecol.common.model.FreeColGameObject;
 import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.Goods;
 import net.sf.freecol.common.model.GoodsContainer;
@@ -48,7 +47,6 @@ import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.TileImprovement;
 import net.sf.freecol.common.model.TypeCountMap;
 import net.sf.freecol.common.model.Unit;
-import net.sf.freecol.common.model.Unit.UnitState;
 import net.sf.freecol.common.model.UnitType;
 import net.sf.freecol.common.model.UnitTypeChange.ChangeType;
 import net.sf.freecol.common.model.WorkLocation;
@@ -84,7 +82,7 @@ public class ServerColony extends Colony implements ServerModelObject {
         super(game, owner, name, tile);
         Specification spec = getSpecification();
 
-        goodsContainer = new GoodsContainer(game, this);
+        setGoodsContainer(new GoodsContainer(game, this));
         sonsOfLiberty = 0;
         oldSonsOfLiberty = 0;
         established = game.getTurn();
@@ -128,10 +126,31 @@ public class ServerColony extends Colony implements ServerModelObject {
                 addBuilding(building);
             }
         }
-
     }
 
 
+    /**
+     * Is a goods type needed for a buildable that this colony could
+     * be building.
+     *
+     * @param goodsType The <code>GoodsType</code> to check.
+     * @return True if the goods could be used to build something.
+     */
+    private boolean neededForBuildableType(GoodsType goodsType) {
+        final Specification spec = getSpecification();
+        List<BuildableType> buildables = new ArrayList<BuildableType>();
+        buildables.addAll(spec.getBuildingTypeList());
+        buildables.addAll(spec.getUnitTypesWithoutAbility("model.ability.person"));
+        for (BuildableType bt : buildables) {
+            if (canBuild(bt)) {
+                for (AbstractGoods ag : bt.getGoodsRequired()) {
+                    if (ag.getType() == goodsType) return true;
+                }
+            }
+        }
+        return false;
+    }
+
     /**
      * New turn for this colony.
      * Try to find out if the colony is going to survive (last colonist does
@@ -146,16 +165,26 @@ public class ServerColony extends Colony implements ServerModelObject {
         ServerPlayer owner = (ServerPlayer) getOwner();
         Specification spec = getSpecification();
 
+        // The AI is prone to removing all units from a colony.
+        // Clean up such cases, to avoid other players seeing the
+        // nonsensical 0-unit colony.
+        if (getUnitCount() <= 0) {
+            logger.warning("Cleaning up 0-unit colony: " + getName());
+            cs.addDispose(See.perhaps().always(owner), getTile(), this);
+            return;
+        }
+
         boolean tileDirty = false;
         boolean newUnitBorn = false;
         GoodsContainer container = getGoodsContainer();
         container.saveState();
 
         // Check for learning by experience
-        for (WorkLocation workLocation : getWorkLocations()) {
-            ProductionInfo productionInfo = getProductionInfo(workLocation);
+        for (WorkLocation workLocation : getCurrentWorkLocations()) {
             ((ServerModelObject) workLocation).csNewTurn(random, cs);
-            if (workLocation.getUnitCount() > 0) {
+            ProductionInfo productionInfo = getProductionInfo(workLocation);
+            if (productionInfo == null) continue;
+            if (!workLocation.isEmpty()) {
                 for (AbstractGoods goods : productionInfo.getProduction()) {
                     UnitType expert = spec.getExpertForProducing(goods.getType());
                     int experience = goods.getAmount() / workLocation.getUnitCount();
@@ -176,25 +205,48 @@ public class ServerColony extends Colony implements ServerModelObject {
 
         // Check the build queues and build new stuff.  If a queue
         // does a build add it to the built list, so that we can
-        // remove the item built from it *after* updating applying the
+        // remove the item built from it *after* applying the
         // production changes.
-        List<BuildQueue> built = new ArrayList<BuildQueue>();
-        for (BuildQueue queue
-                 : new BuildQueue[] { buildQueue, populationQueue }) {
-            if (!getProductionInfo(queue).getConsumption().isEmpty()) {
+        List<BuildQueue<? extends BuildableType>> built
+            = new ArrayList<BuildQueue<? extends BuildableType>>();
+        for (BuildQueue<?> queue : new BuildQueue<?>[] { buildQueue,
+                                                         populationQueue }) {
+            ProductionInfo info = getProductionInfo(queue);
+            if (info == null) continue;
+            if (info.getConsumption().isEmpty()) {
+                BuildableType build = queue.getCurrentlyBuilding();
+                if (build != null) {
+                    AbstractGoods needed = new AbstractGoods();
+                    int complete = getTurnsToComplete(build, needed);
+                    // Warn if about to fail, or if no useful progress
+                    // towards completion is possible.
+                    if (complete == -1 && needed.getType() != null) {
+                        cs.addMessage(See.only(owner),
+                            new ModelMessage(ModelMessage.MessageType.MISSING_GOODS,
+                                "model.colony.buildableNeedsGoods",
+                                this, build)
+                            .addName("%colony%", getName())
+                            .add("%buildable%", build.getNameKey())
+                            .addAmount("%amount%", needed.getAmount())
+                            .add("%goodsType%", needed.getType().getNameKey()));
+                    }
+                }
+            } else {
                 // Ready to build something.  TODO: OO!
                 BuildableType buildable = csNextBuildable(queue, random, cs);
                 if (buildable == null) {
                     ; // It was invalid, ignore.
                 } else if (buildable instanceof UnitType) {
                     Unit newUnit = csBuildUnit(queue, random, cs);
-                    if (newUnit.hasAbility("model.ability.bornInColony")) {
+                    if (newUnit.hasAbility(Ability.BORN_IN_COLONY)) {
                         newUnitBorn = true;
                     }
                     built.add(queue);
                 } else if (buildable instanceof BuildingType) {
                     if (csBuildBuilding(queue, cs)) {
                         built.add(queue);
+                        // Building *might* have ejected units.
+                        tileDirty = true;
                     }
                 } else {
                     throw new IllegalStateException("Bogus buildable: "
@@ -210,17 +262,14 @@ public class ServerColony extends Colony implements ServerModelObject {
         // recalculated one will see the build as incomplete due to
         // missing the goods just updated.
         // Hence the need for a copy of the current production map.
-        int turnsToStarvation = INFINITY;
         TypeCountMap<GoodsType> productionMap = getProductionMap();
         for (GoodsType goodsType : productionMap.keySet()) {
             int net = productionMap.getCount(goodsType);
             int stored = getGoodsCount(goodsType);
             if (net + stored <= 0) {
-                goodsContainer.removeGoods(goodsType, stored);
-                modifySpecialGoods(goodsType, -stored);
+                removeGoods(goodsType, stored);
             } else {
-                goodsContainer.addGoods(goodsType, net);
-                modifySpecialGoods(goodsType, net);
+                addGoods(goodsType, net);
             }
 
             // Handle the food situation
@@ -230,7 +279,7 @@ public class ServerColony extends Colony implements ServerModelObject {
                     if (getUnitCount() > 1) {
                         Unit victim = Utils.getRandomMember(logger,
                             "Choose starver", getUnitList(), random);
-                        cs.addDispose(owner, this, victim);
+                        cs.addDispose(See.only(owner), null, victim);
                         cs.addMessage(See.only(owner),
                             new ModelMessage(ModelMessage.MessageType.UNIT_LOST,
                                              "model.colony.colonistStarved",
@@ -242,7 +291,8 @@ public class ServerColony extends Colony implements ServerModelObject {
                                              "model.colony.colonyStarved",
                                              this)
                                 .addName("%colony%", getName()));
-                        cs.addDispose(owner, getTile(), this);
+                        cs.addDispose(See.perhaps().always(owner),
+                            getTile(), this);
                         return;
                     }
                 } else if (net < 0) {
@@ -267,7 +317,7 @@ public class ServerColony extends Colony implements ServerModelObject {
         // Now that the goods have been updated it is safe to remove the
         // built item from its build queue.
         if (!built.isEmpty()) {
-            for (BuildQueue queue : built) {
+            for (BuildQueue<? extends BuildableType> queue : built) {
                 switch(queue.getCompletionAction()) {
                 case SHUFFLE:
                     if (queue.size() > 1) {
@@ -292,7 +342,7 @@ public class ServerColony extends Colony implements ServerModelObject {
         // Export goods if custom house is built.
         // Do not flush price changes yet, as any price change may change
         // yet again in csYearlyGoodsAdjust.
-        if (hasAbility("model.ability.export")) {
+        if (hasAbility(Ability.EXPORT)) {
             boolean gold = false;
             for (Goods goods : container.getCompactGoods()) {
                 GoodsType type = goods.getType();
@@ -369,7 +419,7 @@ public class ServerColony extends Colony implements ServerModelObject {
 
             // No problem this turn, but what about the next?
             if (!(exportData.isExported()
-                  && hasAbility("model.ability.export")
+                  && hasAbility(Ability.EXPORT)
                   && owner.canTrade(type, Market.Access.CUSTOM_HOUSE))
                 && amount <= limit) {
                 int loss = amount + getNetProductionOf(type) - limit;
@@ -392,6 +442,31 @@ public class ServerColony extends Colony implements ServerModelObject {
             }
         }
 
+        // If the build queue is empty, check that we are not
+        // producing any goods types useful for BuildableTypes, except
+        // if that type is the input to some other form of production.
+        // (Note: isBuildingMaterial is also true for goods used to
+        // produce EquipmentTypes, hence neededForBuildableType).
+        // Such production probably means we forgot to reset the build
+        // queue.  Thus, if hammers are being produced it is worth
+        // warning about, but not if producing tools.
+        if (buildQueue.isEmpty()) {
+            for (GoodsType g : spec.getGoodsTypeList()) {
+                if (g.isBuildingMaterial()
+                    && !g.isRawMaterial()
+                    && getAdjustedNetProductionOf(g) > 0
+                    && neededForBuildableType(g)) {
+                    cs.addMessage(See.only((ServerPlayer) owner),
+                        new ModelMessage(ModelMessage.MessageType.BUILDING_COMPLETED,
+                                         "model.colony.notBuildingAnything",
+                                         this)
+                            .addName("%colony%", getName()));
+                    break;
+                }
+            }
+        } else {
+        }
+
         // Update SoL.
         updateSoL();
         if (sonsOfLiberty / 10 != oldSonsOfLiberty / 10) {
@@ -431,7 +506,7 @@ public class ServerColony extends Colony implements ServerModelObject {
                                      ChangeSet cs) {
         // Can not happen if the building needs no input or no one is
         // working there.
-        if (building.canAutoProduce() || building.getUnitCount() <= 0) return;
+        if (building.canAutoProduce() || building.isEmpty()) return;
 
         // Check all goods types produced by this building.  If the
         // output for a type is zero and the maximum production is
@@ -467,12 +542,11 @@ public class ServerColony extends Colony implements ServerModelObject {
      * @param cs A <code>ChangeSet</code> to update.
      * @return The unit that was built.
      */
-    private Unit csBuildUnit(BuildQueue buildQueue, Random random,
-                             ChangeSet cs) {
+    private Unit csBuildUnit(BuildQueue<? extends BuildableType> buildQueue,
+                             Random random, ChangeSet cs) {
         Unit unit = new ServerUnit(getGame(), getTile(), owner,
-                                   (UnitType) buildQueue.getCurrentlyBuilding(),
-                                   UnitState.ACTIVE);
-        if (unit.hasAbility("model.ability.bornInColony")) {
+            (UnitType) buildQueue.getCurrentlyBuilding());
+        if (unit.hasAbility(Ability.BORN_IN_COLONY)) {
             cs.addMessage(See.only((ServerPlayer) owner),
                           new ModelMessage(ModelMessage.MessageType.UNIT_ADDED,
                                            "model.colony.newColonist",
@@ -498,7 +572,8 @@ public class ServerColony extends Colony implements ServerModelObject {
      * @param cs A <code>ChangeSet</code> to update.
      * @return True if the build was successful.
      */
-    private boolean csBuildBuilding(BuildQueue buildQueue, ChangeSet cs) {
+    private boolean csBuildBuilding(BuildQueue<? extends BuildableType> buildQueue,
+                                    ChangeSet cs) {
         BuildingType type = (BuildingType) buildQueue.getCurrentlyBuilding();
         BuildingType from = type.getUpgradesFrom();
         boolean success;
@@ -507,16 +582,14 @@ public class ServerColony extends Colony implements ServerModelObject {
             success = true;
         } else {
             Building building = getBuilding(from);
-            if (building.upgrade()) {
-                success = true;
-            } else {
+            success = building.upgrade();
+            if (!success) {
                 cs.addMessage(See.only((ServerPlayer) owner),
                     new ModelMessage(ModelMessage.MessageType.BUILDING_COMPLETED,
                                      "colonyPanel.unbuildable",
                                      this)
                         .addName("%colony%", getName())
                         .add("%object%", type.getNameKey()));
-                success = false;
             }
         }
         if (success) {
@@ -527,13 +600,8 @@ public class ServerColony extends Colony implements ServerModelObject {
                                  this)
                     .addName("%colony%", getName())
                     .add("%building%", type.getNameKey()));
-            if (buildQueue.size() == 1) {
-                cs.addMessage(See.only((ServerPlayer) owner),
-                    new ModelMessage(ModelMessage.MessageType.BUILDING_COMPLETED,
-                                     "model.colony.notBuildingAnything",
-                                     this)
-                        .addName("%colony%", getName())
-                        .add("%building%", type.getNameKey()));
+            if (owner.isAI()) {
+                firePropertyChange(REARRANGE_WORKERS, true, false);
             }
         }
         return success;
@@ -548,8 +616,8 @@ public class ServerColony extends Colony implements ServerModelObject {
      * @param cs A <code>ChangeSet</code> to update.
      * @return The next buildable that can be built, or null if nothing.
      */
-    private BuildableType csNextBuildable(BuildQueue queue, Random random,
-                                          ChangeSet cs) {
+    private BuildableType csNextBuildable(BuildQueue<? extends BuildableType> queue,
+                                          Random random, ChangeSet cs) {
         Specification spec = getSpecification();
         ServerPlayer owner = (ServerPlayer) getOwner();
         BuildableType buildable;
@@ -605,22 +673,21 @@ public class ServerColony extends Colony implements ServerModelObject {
         ServerPlayer serverPlayer = (ServerPlayer) getOwner();
         Tile tile = enemyUnit.getTile();
         ServerColonyTile ct = (ServerColonyTile) getColonyTile(tile);
-        Unit unit;
-        if (ct == null || (unit = ct.getUnit()) == null) return;
+        if (ct == null || ct.isEmpty()) return;
         Tile centerTile = getTile();
 
-        unit.setState(UnitState.ACTIVE);
-        unit.setLocation(centerTile);
+        for (Unit unit : ct.getUnitList()) {
+            unit.setLocation(centerTile);
+            cs.addMessage(See.only(serverPlayer),
+                new ModelMessage(ModelMessage.MessageType.WARNING,
+                    "model.colony.workerEvicted", this, this)
+                    .addName("%colony%", getName())
+                    .addStringTemplate("%unit%", unit.getLabel())
+                    .addStringTemplate("%location%", tile.getLocationName())
+                    .addStringTemplate("%enemyUnit%", enemyUnit.getLabel()));
+        }
         cs.add(See.only(serverPlayer), ct);
         cs.add(See.perhaps(), centerTile);
-        cs.addMessage(See.only(serverPlayer),
-                      new ModelMessage(ModelMessage.MessageType.WARNING,
-                                       "model.colony.workerEvicted",
-                                       this, this)
-                      .addName("%colony%", getName())
-                      .addStringTemplate("%unit%", unit.getLabel())
-                      .addStringTemplate("%location%", tile.getLocationName())
-                      .addStringTemplate("%enemyUnit%", enemyUnit.getLabel()));
     }
 
 
