@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -24,8 +24,12 @@ import java.util.Locale;
 
 import net.sf.freecol.util.test.FreeColTestCase;
 import net.sf.freecol.common.model.Colony;
+import net.sf.freecol.common.model.EquipmentType;
 import net.sf.freecol.common.model.Game;
+import net.sf.freecol.common.model.Player;
+import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.model.StringTemplate;
+import net.sf.freecol.server.model.ServerUnit;
 
 public class MessagesTest extends FreeColTestCase {
 
@@ -171,6 +175,31 @@ public class MessagesTest extends FreeColTestCase {
 
     }
 
+    public void testReplaceArbitraryTag() {
+        StringTemplate template = StringTemplate.template("tutorial.startGame")
+            .add("%direction%", "east");
+        String expected = "After months at sea, you have finally arrived off the "
+            + "coast of an unknown continent. Sail eastward in order to discover "
+            + "the New World and to claim it for the Crown.";
+        assertEquals(expected, Messages.message(template));
+
+        template = StringTemplate.template("tutorial.startGame")
+            .add("%direction%", "west");
+        expected = "After months at sea, you have finally arrived off the "
+            + "coast of an unknown continent. Sail westward in order to discover "
+            + "the New World and to claim it for the Crown.";
+        assertEquals(expected, Messages.message(template));
+
+        template = StringTemplate.template("tutorial.startGame")
+            .add("%direction%", "whatever");
+        expected = "After months at sea, you have finally arrived off the "
+            + "coast of an unknown continent. Sail into the wind in order to discover "
+            + "the New World and to claim it for the Crown.";
+        assertEquals(expected, Messages.message(template));
+
+    }
+
+
     public void testReplaceChoicesPlural() {
 
         String mapping = "some.key=This is {{plural:%number%|one=a test|other=one of several tests"
@@ -252,4 +281,97 @@ public class MessagesTest extends FreeColTestCase {
     }
 
 
+    public void testLabels() {
+        Game game = getStandardGame();
+        Player dutch = game.getPlayer("model.nation.dutch");
+
+        EquipmentType muskets = spec().getEquipmentType("model.equipment.muskets");
+        EquipmentType horses = spec().getEquipmentType("model.equipment.horses");
+        EquipmentType tools = spec().getEquipmentType("model.equipment.tools");
+        EquipmentType bible = spec().getEquipmentType("model.equipment.missionary");
+
+        // King's regulars
+        Unit unit = new ServerUnit(game, null, dutch, spec().getUnitType("model.unit.kingsRegular"));
+        assertEquals("King's Regular", Messages.message(Messages.getLabel(unit)));
+
+        unit.changeEquipment(muskets, 1);
+        assertEquals("Infantry", Messages.message(Messages.getLabel(unit)));
+
+        unit.changeEquipment(horses, 1);
+        assertEquals("Cavalry", Messages.message(Messages.getLabel(unit)));
+
+        // Colonial regulars
+        unit = new ServerUnit(game, null, dutch, spec().getUnitType("model.unit.colonialRegular"));
+        assertEquals("Colonial Regular", Messages.message(Messages.getLabel(unit)));
+
+        unit.changeEquipment(muskets, 1);
+        assertEquals("Continental Army", Messages.message(Messages.getLabel(unit)));
+
+        unit.changeEquipment(horses, 1);
+        assertEquals("Continental Cavalry", Messages.message(Messages.getLabel(unit)));
+
+        // Veteran Soldiers
+        unit = new ServerUnit(game, null, dutch, spec().getUnitType("model.unit.veteranSoldier"));
+        assertEquals(1, unit.getEquipment().getCount(muskets));
+        assertEquals("Veteran Soldier", Messages.message(Messages.getLabel(unit)));
+
+        unit.changeEquipment(muskets, -1);
+        assertEquals("Veteran Soldier (no muskets)", Messages.message(Messages.getLabel(unit)));
+
+        unit.changeEquipment(horses, 1);
+        unit.changeEquipment(muskets, 1);
+        assertEquals("Veteran Dragoon", Messages.message(Messages.getLabel(unit)));
+
+        // Indian Braves
+        unit = new ServerUnit(game, null, dutch, spec().getUnitType("model.unit.brave"));
+        assertEquals(0, unit.getEquipment().getCount(muskets));
+        assertEquals("Brave", Messages.message(Messages.getLabel(unit)));
+
+        unit.changeEquipment(muskets, 1);
+        assertEquals("Armed Brave", Messages.message(Messages.getLabel(unit)));
+
+        unit.changeEquipment(horses, 1);
+        assertEquals("Indian Dragoon", Messages.message(Messages.getLabel(unit)));
+
+        // Hardy Pioneers
+        unit = new ServerUnit(game, null, dutch, spec().getUnitType("model.unit.hardyPioneer"));
+        assertEquals(5, unit.getEquipment().getCount(tools));
+        assertEquals("Hardy Pioneer", Messages.message(Messages.getLabel(unit)));
+
+        unit.changeEquipment(tools, -5);
+        assertEquals("Hardy Pioneer (no tools)", Messages.message(Messages.getLabel(unit)));
+
+        // Jesuit Missionaries
+        unit = new ServerUnit(game, null, dutch, spec().getUnitType("model.unit.jesuitMissionary"));
+        assertEquals(1, unit.getEquipment().getCount(bible));
+        assertEquals("Jesuit Missionary", Messages.message(Messages.getLabel(unit)));
+
+        unit.changeEquipment(bible, -1);
+        assertEquals("Jesuit Missionary (not commissioned)", Messages.message(Messages.getLabel(unit)));
+
+        // REF addition message
+        StringTemplate template = StringTemplate.template("model.monarch.action.ADD_TO_REF")
+            .addAmount("%number%", 1)
+            .add("%unit%", spec().getUnitType("model.unit.kingsRegular").getNameKey());
+        String expected = "The Crown has added 1 King's Regular to the Royal Expeditionary Force."
+            + " Colonial leaders express concern.";
+        assertEquals(expected, Messages.message(template));
+
+        template = StringTemplate.template("model.monarch.action.ADD_TO_REF")
+            .addAmount("%number%", 2)
+            .add("%unit%", spec().getUnitType("model.unit.artillery").getNameKey());
+        expected = "The Crown has added 2 Pieces of Artillery to the Royal Expeditionary Force."
+            + " Colonial leaders express concern.";
+        assertEquals(expected, Messages.message(template));
+
+        template = StringTemplate.template("model.monarch.action.ADD_TO_REF")
+            .addAmount("%number%", 3)
+            .add("%unit%", spec().getUnitType("model.unit.manOWar").getNameKey());
+        expected = "The Crown has added 3 Men of War to the Royal Expeditionary Force."
+            + " Colonial leaders express concern.";
+        assertEquals(expected, Messages.message(template));
+
+    }
+
+
 }
