@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -29,16 +29,18 @@ public class AllTests {
         suite.addTestSuite(SpecificationTest.class);
 	//$JUnit-BEGIN$
 	suite.addTestSuite(AbstractUnitTest.class);
+	suite.addTestSuite(BaseCostDeciderTest.class);
 	suite.addTestSuite(BuildingTest.class);
 	suite.addTestSuite(ColonyTest.class);
 	suite.addTestSuite(ColonyProductionTest.class);
 	suite.addTestSuite(CombatTest.class);
-	suite.addTestSuite(BaseCostDeciderTest.class);
+	suite.addTestSuite(EuropeTest.class);
 	suite.addTestSuite(FoundingFatherTest.class);
 	suite.addTestSuite(GameTest.class);
 	suite.addTestSuite(GoodsTest.class);
 	suite.addTestSuite(GoodsContainerTest.class);
 	suite.addTestSuite(IndianSettlementTest.class);
+	suite.addTestSuite(IndividualFatherTest.class);
 	suite.addTestSuite(LimitTest.class);
 	suite.addTestSuite(MapTest.class);
 	suite.addTestSuite(MarketTest.class);
