@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -21,7 +21,6 @@ package net.sf.freecol.common.model;
 
 import java.util.List;
 
-import net.sf.freecol.common.model.Unit.UnitState;
 import net.sf.freecol.server.model.ServerBuilding;
 import net.sf.freecol.server.model.ServerColony;
 import net.sf.freecol.server.model.ServerUnit;
@@ -30,34 +29,49 @@ import net.sf.freecol.util.test.FreeColTestCase;
 
 public class ColonyProductionTest extends FreeColTestCase {
 
+    private static final BuildingType countryType
+        = spec().getBuildingType("model.building.country");
+    private static final BuildingType depotType
+        = spec().getBuildingType("model.building.depot");
+    private static final BuildingType townHallType
+        = spec().getBuildingType("model.building.townHall");
+
+    private static final GoodsType bellsType
+        = spec().getGoodsType("model.goods.bells");
+    private static final GoodsType clothType
+        = spec().getGoodsType("model.goods.cloth");
+    private static final GoodsType cottonType
+        = spec().getGoodsType("model.goods.cotton");
+    private static final GoodsType crossesType
+        = spec().getGoodsType("model.goods.crosses");
+    private static final GoodsType foodType
+        = spec().getGoodsType("model.goods.food");
+    private static final GoodsType grainType
+        = spec().getGoodsType("model.goods.grain");
+    private static final GoodsType horsesType
+        = spec().getGoodsType("model.goods.horses");
+
+    private static final TileType plainsType
+        = spec().getTileType("model.tile.plains");
+
+
     public void testProductionSoldier() {
 
         Game game = getStandardGame();
-
+        Map map = getTestMap();
+        game.setMap(map);
         Player dutch = game.getPlayer("model.nation.dutch");
 
-        Tile[][] tiles = new Tile[10][15];
+        Tile tile = map.getTile(5, 8);
+        Resource grain = new Resource(game, tile, spec().getResourceType("model.resource.grain"));
+        tile.addResource(grain);
 
-        for (int x = 0; x < 10; x++) {
-            for (int y = 0; y < 15; y++) {
-                tiles[x][y] = new Tile(game, spec().getTileType("model.tile.plains"), x, y);
-            }
-        }
-
-        Map map = new Map(game, tiles);
-
-        Resource grain = new Resource(game, map.getTile(5, 8),
-                                      spec().getResourceType("model.resource.grain"));
-        map.getTile(5, 8).addResource(grain);
-        map.getTile(5, 8).setExploredBy(dutch, true);
-        map.getTile(6, 8).setExploredBy(dutch, true);
-
-        game.setMap(map);
         UnitType veteran = spec().getUnitType("model.unit.veteranSoldier");
-        Unit soldier = new ServerUnit(game, map.getTile(6, 8), dutch, veteran, UnitState.ACTIVE, veteran.getDefaultEquipment());
+        Unit soldier = new ServerUnit(game, map.getTile(6, 8), dutch, veteran);
 
         Colony colony = new ServerColony(game, dutch, "New Amsterdam", soldier.getTile());
-        GoodsType foodType = spec().getGoodsType("model.goods.grain");
+        dutch.addSettlement(colony);
+        GoodsType foodType = grainType;
         soldier.setWorkType(foodType);
         nonServerBuildColony(soldier, colony);
 
@@ -88,7 +102,7 @@ public class ColonyProductionTest extends FreeColTestCase {
 
         assertEquals(foodType, soldier.getWorkType());
 
-        assertEquals(colony.getColonyTile(map.getTile(5,8)).getTile(), soldier.getLocation().getTile());
+        assertEquals(colony.getColonyTile(tile).getTile(), soldier.getLocation().getTile());
 
         assertEquals(0, soldier.getMovesLeft());
 
@@ -98,33 +112,20 @@ public class ColonyProductionTest extends FreeColTestCase {
     public void testProductionPioneer() {
 
         Game game = getStandardGame();
-
+        Map map = getTestMap();
+        game.setMap(map);
         Player dutch = game.getPlayer("model.nation.dutch");
 
-        Tile[][] tiles = new Tile[10][15];
+        Tile tile = map.getTile(5, 8);
+        Resource grain = new Resource(game, tile, spec().getResourceType("model.resource.grain"));
+        tile.addResource(grain);
 
-        for (int x = 0; x < 10; x++) {
-            for (int y = 0; y < 15; y++) {
-                tiles[x][y] = new Tile(game, spec().getTileType("model.tile.plains"), x, y);
-            }
-        }
-
-        Map map = new Map(game, tiles);
-
-        Resource grain = new Resource(game, map.getTile(5, 8),
-                                      spec().getResourceType("model.resource.grain"));
-        map.getTile(5, 8).addResource(grain);
-        map.getTile(5, 8).setExploredBy(dutch, true);
-        map.getTile(6, 8).setExploredBy(dutch, true);
-
-        game.setMap(map);
         UnitType pioneerType = spec().getUnitType("model.unit.hardyPioneer");
-        GoodsType foodType = spec().getGoodsType("model.goods.grain");
-        Unit pioneer = new ServerUnit(game, map.getTile(6, 8), dutch,
-                                      pioneerType, UnitState.ACTIVE,
-                                      pioneerType.getDefaultEquipment());
+        GoodsType foodType = grainType;
+        Unit pioneer = new ServerUnit(game, map.getTile(6, 8), dutch, pioneerType);
 
         Colony colony = new ServerColony(game, dutch, "New Amsterdam", pioneer.getTile());
+        dutch.addSettlement(colony);
         pioneer.setWorkType(foodType);
         nonServerBuildColony(pioneer, colony);
 
@@ -154,7 +155,7 @@ public class ColonyProductionTest extends FreeColTestCase {
         // Pioneer should be working on the field with the bonus
         assertEquals(foodType, pioneer.getWorkType());
 
-        assertEquals(colony.getColonyTile(map.getTile(5,8)).getTile(), pioneer.getLocation().getTile());
+        assertEquals(colony.getColonyTile(tile).getTile(), pioneer.getLocation().getTile());
 
         assertEquals(0, pioneer.getMovesLeft());
 
@@ -162,8 +163,6 @@ public class ColonyProductionTest extends FreeColTestCase {
     }
 
     public void testBellNetProduction(){
-    	GoodsType bellsType = spec().getGoodsType("model.goods.bells");
-
     	Game game = getStandardGame();
 
     	game.setMap(getTestMap());
@@ -193,10 +192,8 @@ public class ColonyProductionTest extends FreeColTestCase {
         game.setMap(getTestMap());
 
         Colony colony = getStandardColony(1);
-        GoodsType foodType = spec().getGoodsType("model.goods.food");
-        GoodsType horsesType = spec().getGoodsType("model.goods.horses");
 
-        Building pasture = colony.getBuilding(spec().getBuildingType("model.building.country"));
+        Building pasture = colony.getBuilding(countryType);
         assertEquals(horsesType, pasture.getGoodsOutputType());
         assertEquals("Wrong warehouse capacity in colony",100,colony.getWarehouseCapacity());
 
@@ -245,11 +242,9 @@ public class ColonyProductionTest extends FreeColTestCase {
                        consumers.get(index) instanceof BuildQueue);
         }
 
-        BuildingType countryType = spec().getBuildingType("model.building.country");
         Building country = colony.getBuilding(countryType);
         assertTrue(consumers.contains(country));
 
-        BuildingType depotType = spec().getBuildingType("model.building.depot");
         Building depot = colony.getBuilding(depotType);
         assertTrue(consumers.contains(depot));
 
@@ -292,11 +287,6 @@ public class ColonyProductionTest extends FreeColTestCase {
 
 
     public void testProductionMap() {
-
-        GoodsType cottonType = spec().getGoodsType("model.goods.cotton");
-        GoodsType foodType = spec().getGoodsType("model.goods.food");
-        GoodsType grainType = spec().getGoodsType("model.goods.grain");
-
         ProductionMap pm = new ProductionMap();
 
         pm.add(new AbstractGoods(cottonType, 33));
@@ -313,26 +303,16 @@ public class ColonyProductionTest extends FreeColTestCase {
         pm.remove(new AbstractGoods(foodType, 11));
         assertEquals(11, pm.get(grainType).getAmount());
         assertEquals(11, pm.get(foodType).getAmount());
-
     }
 
 
     public void testProduction() {
-
         Game game = getGame();
         game.setMap(getTestMap());
 
         Colony colony = getStandardColony(3);
         ColonyTile tile = colony.getColonyTile(colony.getTile());
 
-        GoodsType foodType = spec().getGoodsType("model.goods.food");
-        GoodsType grainType = spec().getGoodsType("model.goods.grain");
-        GoodsType clothType = spec().getGoodsType("model.goods.cloth");
-        GoodsType bellsType = spec().getGoodsType("model.goods.bells");
-        GoodsType cottonType = spec().getGoodsType("model.goods.cotton");
-        GoodsType horsesType = spec().getGoodsType("model.goods.horses");
-        GoodsType crossesType = spec().getGoodsType("model.goods.crosses");
-
         assertEquals(0, colony.getGoodsCount(foodType));
 
         assertEquals(grainType, tile.getProduction().get(0).getType());
@@ -439,4 +419,30 @@ public class ColonyProductionTest extends FreeColTestCase {
         */
     }
 
+    public void testGetPotentialProduction() {
+        Game game = getGame();
+        game.setMap(getTestMap());
+
+        Colony colony = getStandardColony(1);
+        ColonyTile colonyTile = colony.getColonyTile(colony.getTile());
+        assertNotNull(colonyTile);
+        assertEquals(plainsType, colony.getTile().getType());
+        Building townHall = colony.getBuilding(townHallType);
+        assertNotNull(townHall);
+        UnitType colonistType = spec().getDefaultUnitType();
+        assertNotNull(colonistType);
+
+        assertEquals("Zero potential production of cotton in town hall", 0,
+            townHall.getPotentialProduction(colonistType, cottonType));
+        assertEquals("Basic potential production of bells in town hall", 
+            (int) FeatureContainer.applyModifiers(townHallType.getBasicProduction(),
+                game.getTurn(), townHall.getProductionModifiers()),
+            townHall.getPotentialProduction(colonistType, bellsType));
+
+        assertEquals("Basic potential production of cotton in town hall",
+            plainsType.getProductionOf(cottonType, null),
+            colonyTile.getPotentialProduction(colonistType, cottonType));
+        assertEquals("Zero potential production of cotton in town hall", 0,
+            townHall.getPotentialProduction(colonistType, cottonType));
+    }
 }
