@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -20,10 +20,16 @@
 package net.sf.freecol.common.model;
 
 import java.util.Iterator;
+import java.util.List;
+import java.util.Random;
 import java.util.Set;
 
+import net.sf.freecol.common.model.CombatModel.CombatResult;
 import net.sf.freecol.common.model.Unit.MoveType;
-import net.sf.freecol.common.model.Unit.UnitState;
+import net.sf.freecol.server.ServerTestHelper;
+import net.sf.freecol.server.control.ChangeSet;
+import net.sf.freecol.server.control.InGameController;
+import net.sf.freecol.server.model.ServerPlayer;
 import net.sf.freecol.server.model.ServerUnit;
 import net.sf.freecol.util.test.FreeColTestCase;
 
@@ -58,8 +64,6 @@ public class CombatTest extends FreeColTestCase {
         = spec().getUnitType("model.unit.freeColonist");
     private static final UnitType veteranType
         = spec().getUnitType("model.unit.veteranSoldier");
-    private static final UnitType colonialType
-        = spec().getUnitType("model.unit.colonialRegular");
     private static final UnitType artilleryType
         = spec().getUnitType("model.unit.artillery");
     private static final UnitType damagedArtilleryType
@@ -95,8 +99,9 @@ public class CombatTest extends FreeColTestCase {
         tile2.setExploredBy(dutch, true);
         tile2.setExploredBy(french, true);
 
-        Unit colonist = new ServerUnit(game, tile1, dutch, colonistType, UnitState.FORTIFIED);
-        Unit soldier = new ServerUnit(game, tile2, french, veteranType, UnitState.ACTIVE, muskets, horses);
+        Unit colonist = new ServerUnit(game, tile1, dutch, colonistType);
+        colonist.setStateUnchecked(Unit.UnitState.FORTIFIED);
+        Unit soldier = new ServerUnit(game, tile2, french, veteranType, muskets, horses);
         soldier.setMovesLeft(1);
 
         Modifier bigMovementPenalty = spec().getModifiers(SimpleCombatModel.BIG_MOVEMENT_PENALTY)
@@ -164,8 +169,8 @@ public class CombatTest extends FreeColTestCase {
         tile2.setExploredBy(dutch, true);
         tile2.setExploredBy(french, true);
 
-        Unit galleon = new ServerUnit(game, tile1, dutch, galleonType, UnitState.ACTIVE);
-        Unit privateer = new ServerUnit(game, tile2, french, privateerType, UnitState.ACTIVE);
+        Unit galleon = new ServerUnit(game, tile1, dutch, galleonType);
+        Unit privateer = new ServerUnit(game, tile2, french, privateerType);
 
         /**
          * Only base modifiers should apply.
@@ -181,8 +186,7 @@ public class CombatTest extends FreeColTestCase {
         /**
          * Fortification should have no effect.
          */
-        galleon.setState(UnitState.FORTIFYING);
-        galleon.setState(UnitState.FORTIFIED);
+        galleon.setStateUnchecked(Unit.UnitState.FORTIFIED);
         defenceModifiers = combatModel.getDefensiveModifiers(privateer, galleon);
         assertEquals(1, defenceModifiers.size());
         assertEquals(Specification.BASE_DEFENCE_SOURCE, defenceModifiers.iterator().next().getSource());
@@ -238,7 +242,8 @@ public class CombatTest extends FreeColTestCase {
         assertEquals(-12.5f, goodsPenalty1.getValue());
 
         // Verify that the move is correctly interpreted
-        assertEquals("Wrong move type",MoveType.ATTACK, privateer.getMoveType(tile1));
+        assertEquals("Wrong move type", MoveType.ATTACK_UNIT,
+                     privateer.getMoveType(tile1));
     }
 
 
@@ -261,13 +266,13 @@ public class CombatTest extends FreeColTestCase {
 
         Unit colonist = colony.getUnitIterator().next();
         Unit attacker = new ServerUnit(getGame(), tile2, inca, braveType,
-                                       UnitState.ACTIVE, indianHorses, indianMuskets);
+                                       indianHorses, indianMuskets);
 
         assertEquals(colonist, colony.getDefendingUnit(attacker));
         assertEquals(colonist, colony.getTile().getDefendingUnit(attacker));
 
         Unit defender = new ServerUnit(getGame(), colony.getTile(), dutch,
-                                       colonistType, UnitState.ACTIVE);
+                                       colonistType);
         assertFalse("Colonist should not be defensive unit",defender.isDefensiveUnit());
         assertEquals(defender, colony.getTile().getDefendingUnit(attacker));
 
@@ -289,8 +294,7 @@ public class CombatTest extends FreeColTestCase {
         tile2.setExploredBy(inca, true);
 
         Unit colonist = colony.getUnitIterator().next();
-        Unit attacker = new ServerUnit(getGame(), tile2, inca, braveType,
-                                       UnitState.ACTIVE, indianHorses, indianMuskets);
+        Unit attacker = new ServerUnit(getGame(), tile2, inca, braveType, indianHorses, indianMuskets);
 
         assertEquals(colonist, colony.getDefendingUnit(attacker));
 
@@ -332,10 +336,8 @@ public class CombatTest extends FreeColTestCase {
         IndianSettlement settlement = builder.player(inca).settlementTile(tile1).skillToTeach(null).capital(true).build();
 
         //IndianSettlement settlement = new IndianSettlement(game, inca, tile1, true, null, false, null);
-        Unit defender = new ServerUnit(game, settlement, inca, braveType,
-                                       UnitState.ACTIVE);
-        Unit attacker = new ServerUnit(game, tile2, dutch, colonistType,
-                                       UnitState.ACTIVE, horses, muskets);
+        Unit defender = new ServerUnit(game, settlement, inca, braveType);
+        Unit attacker = new ServerUnit(game, tile2, dutch, colonistType, horses, muskets);
 
         for (EquipmentType equipment : dragoonEquipment) {
             for (AbstractGoods goods : equipment.getGoodsRequired()) {
@@ -367,17 +369,21 @@ public class CombatTest extends FreeColTestCase {
         dutch.setStance(french, Player.Stance.WAR);
         french.setStance(dutch, Player.Stance.WAR);
 
-        Unit colonist = new ServerUnit(game, tile1, dutch, colonistType,
-                                       UnitState.FORTIFIED);
+        Unit colonist = new ServerUnit(game, tile1, dutch, colonistType);
+        colonist.setStateUnchecked(Unit.UnitState.FORTIFIED);
         Unit soldier = new ServerUnit(game, tile2, french, veteranType,
-                                      UnitState.ACTIVE, muskets, horses);
+                                      muskets, horses);
+        soldier.setStateUnchecked(Unit.UnitState.FORTIFIED);
 
         assertEquals(tile1, colonist.getLocation());
         assertEquals(tile2, soldier.getLocation());
 
-        assertEquals(Unit.MoveType.ATTACK, soldier.getMoveType(tile2, tile1, 9, false));
-        assertEquals(Unit.MoveType.ATTACK, soldier.getMoveType(tile2, tile1, 1, false));
-        assertEquals(Unit.MoveType.MOVE_NO_MOVES, soldier.getMoveType(tile2, tile1, 0, false));
+        assertEquals(Unit.MoveType.ATTACK_UNIT,
+                     soldier.getMoveType(tile2, tile1, 9));
+        assertEquals(Unit.MoveType.ATTACK_UNIT,
+                     soldier.getMoveType(tile2, tile1, 1));
+        assertEquals(Unit.MoveType.MOVE_NO_MOVES,
+                     soldier.getMoveType(tile2, tile1, 0));
 
     }
 
@@ -400,9 +406,8 @@ public class CombatTest extends FreeColTestCase {
         tupi.setStance(spanish, Player.Stance.WAR);
 
         Unit soldier = new ServerUnit(game, tile1, spanish, colonistType,
-                                      UnitState.ACTIVE, muskets);
-        Unit brave = new ServerUnit(game, tile2, tupi, braveType,
-                                    UnitState.ACTIVE);
+                                      muskets);
+        Unit brave = new ServerUnit(game, tile2, tupi, braveType);
 
         assertEquals(tile1, soldier.getLocation());
         assertEquals(tile2, brave.getLocation());
@@ -420,4 +425,117 @@ public class CombatTest extends FreeColTestCase {
 
     }
 
+    public void testAttackShipWithLandUnit() {
+
+        Game game = getStandardGame();
+        Player spanish = game.getPlayer("model.nation.spanish");
+        Player tupi = game.getPlayer("model.nation.tupi");
+        Map map = getTestMap(plains, true);
+        game.setMap(map);
+
+        SimpleCombatModel combatModel = new SimpleCombatModel();
+
+        Tile tile1 = map.getTile(5, 8);
+        Tile tile2 = map.getTile(4, 8);
+
+        tile1.setType(hills);
+        assertEquals(hills, tile1.getType());
+        tile2.setType(ocean);
+        assertEquals(ocean, tile2.getType());
+
+        spanish.setStance(tupi, Player.Stance.WAR);
+        tupi.setStance(spanish, Player.Stance.WAR);
+
+        Unit galleon = new ServerUnit(game, tile2, spanish, galleonType);
+        Unit brave = new ServerUnit(game, tile1, tupi, braveType);
+
+        assertEquals(tile1, brave.getLocation());
+        assertEquals(tile2, galleon.getLocation());
+
+        assertEquals(Unit.MoveType.MOVE_NO_ACCESS_LAND,
+                     galleon.getMoveType(tile2, tile1, 3));
+
+        assertEquals(Unit.MoveType.MOVE_NO_ACCESS_EMBARK,
+                     brave.getMoveType(tile1, tile2, 3));
+
+    }
+
+    public void testRegulars() {
+        Random random = new Random(1);
+        // these are the first ten values the RNG will produce
+        float[] values = new float[] {
+            0.7308782f,
+            0.100473166f,
+            0.4100808f,
+            0.40743977f,
+            0.2077148f,
+            0.036235332f,
+            0.332717f,
+            0.6588672f,
+            0.96775585f,
+            0.7107396f
+        };
+
+        Map map = getTestMap(plains, true);
+        Game game = ServerTestHelper.startServerGame(map);
+        InGameController igc = ServerTestHelper.getInGameController();
+
+        ServerPlayer french = (ServerPlayer) game.getPlayer("model.nation.french");
+        french.getFeatureContainer().addAbility(new Ability("model.ability.independenceDeclared"));
+        ServerPlayer refPlayer = igc.createREFPlayer(french);
+
+        SimpleCombatModel combatModel = new SimpleCombatModel();
+
+        Tile tile1 = map.getTile(5, 8);
+        Tile tile2 = map.getTile(4, 8);
+
+        Unit colonial = new ServerUnit(game, tile1, french, colonialRegularType, muskets, horses);
+        Unit regular = new ServerUnit(game, tile1, french, kingsRegularType, muskets, horses);
+
+        // (regular + muskets + horses) * attack bonus
+        float offence = (4 + 2 + 1) * 1.5f;
+        assertEquals(offence, combatModel.getOffencePower(regular, colonial));
+        // colonial + muskets + horses + defence bonus
+        float defence = 3 + 1 + 1 + 1;
+        assertEquals(defence, combatModel.getDefencePower(regular, colonial));
+
+        List<CombatResult> result = combatModel.generateAttackResult(random, regular, colonial);
+        assertEquals(CombatResult.LOSE, result.get(0));
+        assertEquals(CombatResult.LOSE_EQUIP, result.get(1));
+
+        refPlayer.csCombat(regular, colonial, result, random, new ChangeSet());
+
+        // (regular + muskets) * attack bonus
+        offence = (4 + 2) * 1.5f;
+        assertEquals(offence, combatModel.getOffencePower(regular, colonial));
+
+        // slaughter King's Regular
+        result = combatModel.generateAttackResult(random, colonial, regular);
+        assertEquals(CombatResult.WIN, result.get(0));
+        assertEquals("King's Regular should be slaughtered upon losing all equipment.",
+                     CombatResult.SLAUGHTER_UNIT, result.get(1));
+
+        regular = new ServerUnit(game, tile1, french, kingsRegularType, muskets, horses);
+
+        result = combatModel.generateAttackResult(random, regular, colonial);
+        assertEquals(CombatResult.WIN, result.get(0));
+        assertEquals(CombatResult.LOSE_EQUIP, result.get(1));
+        refPlayer.csCombat(regular, colonial, result, random, new ChangeSet());
+
+        result = combatModel.generateAttackResult(random, regular, colonial);
+        assertEquals(CombatResult.WIN, result.get(0));
+        assertEquals(CombatResult.LOSE_EQUIP, result.get(1));
+        assertEquals(CombatResult.DEMOTE_UNIT, result.get(2));
+        refPlayer.csCombat(regular, colonial, result, random, new ChangeSet());
+        assertFalse(colonial.isArmed());
+        assertEquals(veteranType, colonial.getType());
+
+        result = combatModel.generateAttackResult(random, regular, colonial);
+        assertEquals(CombatResult.WIN, result.get(0));
+        assertEquals(CombatResult.CAPTURE_UNIT, result.get(1));
+        refPlayer.csCombat(regular, colonial, result, random, new ChangeSet());
+
+    }
+
+
 }
