@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -30,7 +30,6 @@ import net.sf.freecol.common.FreeColException;
 import net.sf.freecol.common.model.Map.Direction;
 import net.sf.freecol.common.model.Map.Position;
 import net.sf.freecol.common.model.Player.Stance;
-import net.sf.freecol.common.model.Unit.UnitState;
 import net.sf.freecol.common.model.pathfinding.CostDecider;
 import net.sf.freecol.common.model.pathfinding.CostDeciders;
 import net.sf.freecol.common.model.pathfinding.GoalDecider;
@@ -40,38 +39,49 @@ import net.sf.freecol.util.test.FreeColTestUtils;
 
 
 public class MapTest extends FreeColTestCase {
-    TileType oceanType = spec().getTileType("model.tile.ocean");
-    TileType plainsType = spec().getTileType("model.tile.plains");
 
-    UnitType colonistType = spec().getUnitType("model.unit.freeColonist");
-    
+    private final TileType oceanType
+        = spec().getTileType("model.tile.ocean");
+    private final TileType plainsType
+        = spec().getTileType("model.tile.plains");
+
+    private final UnitType artilleryType
+        = spec().getUnitType("model.unit.artillery");
+    private final UnitType colonistType
+        = spec().getUnitType("model.unit.freeColonist");
+    private final UnitType galleonType
+        = spec().getUnitType("model.unit.galleon");
+    private final UnitType pioneerType
+        = spec().getUnitType("model.unit.hardyPioneer");
+
+
     private Map getSingleLandPathMap(Game game){
         MapBuilder builder = new MapBuilder(game);
         builder.setBaseTileType(oceanType);
         // Land Stripe
         builder.setTile(1,11,plainsType);
         builder.setTile(2,10,plainsType);
-        builder.setTile(2,9,plainsType); 
+        builder.setTile(2,9,plainsType);
         builder.setTile(3,8,plainsType);
         builder.setTile(3,7,plainsType);
-       
+
         return builder.build();
     }
-    
+
     // (1,5)*
-    //          *            
+    //          *
     //      *        *     * F(3,7)
     //                  * C(3,8)
     //      *        *
-    //            
+    //
     //      *   *
-    //      
+    //
     //      *S(1,11)
     //
     private Map getShortLongPathMap(Game game){
         TileType oceanType = spec().getTileType("model.tile.ocean");
         TileType plainsType = spec().getTileType("model.tile.plains");
-        
+
         MapBuilder builder = new MapBuilder(game);
         builder.setBaseTileType(oceanType);
         //Start
@@ -89,14 +99,14 @@ public class MapTest extends FreeColTestCase {
         builder.setTile(3,8,plainsType);
         // Finish
         builder.setTile(3,7,plainsType);
-       
+
         return builder.build();
     }
 
     public void testMapGameInt() throws FreeColException {
         int expectedWidth = 20;
         int expectedHeigth = 15;
-        
+
         Game game = getStandardGame();
         MapBuilder builder = new MapBuilder(game);
         Map map = builder.setDimensions(expectedWidth, expectedHeigth).build();
@@ -172,10 +182,10 @@ public class MapTest extends FreeColTestCase {
 
         Set<Position> positions = new HashSet<Position>();
         Set<Tile> allTiles = new HashSet<Tile>();
-        
+
         for (int x = 0; x < 5; x++) {
             for (int y = 0; y < 6; y++) {
-                Tile tile = new Tile(game, spec().getTileType("model.tile.plains"), x, y);
+                Tile tile = new Tile(game, plainsType, x, y);
                 tiles[x][y] = tile;
                 allTiles.add(tile);
                 positions.add(new Position(x, y));
@@ -276,10 +286,10 @@ public class MapTest extends FreeColTestCase {
         Game game = getStandardGame();
         MapBuilder builder = new MapBuilder(game);
         builder.setDimensions(10, 15).build();
-        Direction[] dirs = Direction.getRandomDirectionArray(new Random(1));
+        Direction[] dirs = Direction.getRandomDirections("testRandomDirection", new Random(1));
         assertNotNull(dirs);
     }
-    
+
     /**
      * Tests path discoverability in a map with only one path available
      * That path is obstructed by a settlement, so is invalid
@@ -288,7 +298,7 @@ public class MapTest extends FreeColTestCase {
         Game game = getStandardGame();
         Map map = getSingleLandPathMap(game);
         game.setMap(map);
-        
+
         // set obstructing indian camp
         Tile settlementTile = map.getTile(2,10);
         FreeColTestCase.IndianSettlementBuilder builder = new FreeColTestCase.IndianSettlementBuilder(game);
@@ -299,13 +309,13 @@ public class MapTest extends FreeColTestCase {
         Tile unitTile = map.getTile(1, 11);
         Tile destinationTile = map.getTile(3,7);
         Unit colonist = new ServerUnit(game, unitTile, dutchPlayer,
-                                       colonistType, UnitState.ACTIVE);
+                                       colonistType);
         colonist.setDestination(destinationTile);
-        
-        PathNode path = map.findPath(colonist, colonist.getTile(), destinationTile);
+
+        PathNode path = colonist.findPath(destinationTile);
         assertNull("No path should be available",path);
     }
-    
+
     /**
      * Tests path discoverability in a map with only one path available
      * That path is obstructed by a settlement, so is invalid
@@ -314,7 +324,7 @@ public class MapTest extends FreeColTestCase {
         Game game = getStandardGame();
         Map map = getSingleLandPathMap(game);
         game.setMap(map);
-        
+
         // set obstructing french colony
         Player frenchPlayer = game.getPlayer("model.nation.french");
         Tile settlementTile = map.getTile(2,10);
@@ -327,39 +337,39 @@ public class MapTest extends FreeColTestCase {
         Tile unitTile = map.getTile(1, 11);
         Tile destinationTile = map.getTile(3,7);
         Unit colonist = new ServerUnit(game, unitTile, dutchPlayer,
-                                       colonistType, UnitState.ACTIVE);
+                                       colonistType);
         colonist.setDestination(destinationTile);
-        
-        PathNode path = map.findPath(colonist, colonist.getTile(), destinationTile);
+
+        PathNode path = colonist.findPath(destinationTile);
         assertNull("No path should be available",path);
     }
-    
+
     public void testMoveThroughTileWithEnemyUnit() {
-        final UnitType pioneerType = spec().getUnitType("model.unit.hardyPioneer");
         Game game = getStandardGame();
         Map map = getTestMap();
         game.setMap(map);
-        
+
         //Setup
         Tile enemyUnitTile = map.getTile(2,1);
         Player frenchPlayer = game.getPlayer("model.nation.french");
-        new ServerUnit(game, enemyUnitTile, frenchPlayer, pioneerType,
-                       UnitState.ACTIVE);
-        
+        new ServerUnit(game, enemyUnitTile, frenchPlayer, pioneerType);
+
         Tile unitTile = map.getTile(1, 1);
+        Tile otherTile = map.getTile(1, 2);
         Player dutchPlayer = game.getPlayer("model.nation.dutch");
-        Unit unit = new ServerUnit(game, unitTile, dutchPlayer, pioneerType,
-                                   UnitState.ACTIVE);
+        Unit unit = new ServerUnit(game, unitTile, dutchPlayer, pioneerType);
         // unit is going somewhere else
         Tile unitDestination = map.getTile(3, 1);
         unit.setDestination(unitDestination);
-        
+
         // Execute
-        final CostDecider decider = CostDeciders.avoidSettlementsAndBlockingUnits();
-        final int cost = decider.getCost(unit, unitTile, enemyUnitTile, 4, 0);
-        assertTrue("Move should be invalid", cost == CostDecider.ILLEGAL_MOVE);
-        final int cost2 = decider.getCost(unit, unitTile, enemyUnitTile, 4, 1);
-        assertTrue("Move should be valid", cost2 != CostDecider.ILLEGAL_MOVE);
+        CostDecider decider = CostDeciders.avoidSettlementsAndBlockingUnits();
+        assertTrue("No blocking unit, should be legal",
+                   decider.getCost(unit, unitTile, otherTile, 4)
+                   != CostDecider.ILLEGAL_MOVE);
+        assertTrue("Blocking unit, should be illegal",
+                   decider.getCost(unit, unitTile, enemyUnitTile, 4)
+                   == CostDecider.ILLEGAL_MOVE);
     }
 
     /**
@@ -370,26 +380,25 @@ public class MapTest extends FreeColTestCase {
         Game game = getStandardGame();
         Map map = getSingleLandPathMap(game);
         game.setMap(map);
-        
+
         // set obstructing unit
         Tile unitObstructionTile = map.getTile(2,10);
         Player frenchPlayer = game.getPlayer("model.nation.french");
-        new ServerUnit(game, unitObstructionTile, frenchPlayer, colonistType,
-                       UnitState.ACTIVE);
-        
+        new ServerUnit(game, unitObstructionTile, frenchPlayer, colonistType);
+
         // set unit
         Player dutchPlayer = game.getPlayer("model.nation.dutch");
         Tile unitTile = map.getTile(1, 11);
         Tile destinationTile = map.getTile(3,7);
         Unit colonist = new ServerUnit(game, unitTile, dutchPlayer,
-                                       colonistType, UnitState.ACTIVE);
+                                       colonistType);
         colonist.setDestination(destinationTile);
-        
+
         PathNode path = map.findPath(colonist, colonist.getTile(), destinationTile, null, CostDeciders.avoidSettlementsAndBlockingUnits());
         assertNull("No path should be available",path);
     }
-    
-    public void testShortestPathObstructed() {        
+
+    public void testShortestPathObstructed() {
         Game game = getStandardGame();
         Map map = getShortLongPathMap(getGame());
         game.setMap(map);
@@ -403,11 +412,11 @@ public class MapTest extends FreeColTestCase {
         Player dutchPlayer = game.getPlayer("model.nation.dutch");
         Tile unitTile = map.getTile(1, 11);
         Unit colonist = new ServerUnit(game, unitTile, dutchPlayer,
-                                       colonistType, UnitState.ACTIVE);
+                                       colonistType);
         Tile destinationTile = map.getTile(3,7);
         colonist.setDestination(destinationTile);
-        
-        PathNode path = map.findPath(colonist, colonist.getTile(), destinationTile);
+
+        PathNode path = colonist.findPath(destinationTile);
         assertNotNull("A path should be available",path);
     }
 
@@ -418,14 +427,10 @@ public class MapTest extends FreeColTestCase {
 
         Player dutchPlayer = game.getPlayer("model.nation.dutch");
         Player frenchPlayer = game.getPlayer("model.nation.french");
-        UnitType artilleryType = spec().getUnitType("model.unit.artillery");
-        UnitType galleonType = spec().getUnitType("model.unit.galleon");
         Tile unitTile = map.getTile(15, 5);
         Tile colonyTile = map.getTile(9, 9); // should be on coast
-        Unit galleon = new ServerUnit(game, unitTile, dutchPlayer, galleonType,
-                                      UnitState.ACTIVE);
-        Unit artillery = new ServerUnit(game, galleon, dutchPlayer,
-                                        artilleryType, UnitState.ACTIVE);
+        Unit galleon = new ServerUnit(game, unitTile, dutchPlayer, galleonType);
+        Unit artillery = new ServerUnit(game, galleon, dutchPlayer, artilleryType);
         FreeColTestUtils.getColonyBuilder()
             .player(frenchPlayer)
             .colonyTile(colonyTile)
@@ -488,4 +493,47 @@ public class MapTest extends FreeColTestCase {
                    path != null && path.getTransportDropNode() != null
                    && path.getLastNode().getTile() == colonyTile);
     }
+
+    public void testLatitude() {
+        Game game = getStandardGame();
+
+        MapBuilder builder = new MapBuilder(game);
+        Map map = builder.setDimensions(1, 181).build();
+
+        assertEquals(181, map.getHeight());
+        assertEquals(1f, map.getLatitudePerRow());
+        assertEquals(-90, map.getLatitude(0));
+        assertEquals(0, map.getRow(-90));
+        assertEquals(0, map.getLatitude(90));
+        assertEquals(90, map.getRow(0));
+        assertEquals(90, map.getLatitude(180));
+        assertEquals(180, map.getRow(90));
+
+        builder = new MapBuilder(game);
+        map = builder.setDimensions(1, 91).build();
+
+        assertEquals(91, map.getHeight());
+        assertEquals(2f, map.getLatitudePerRow());
+        assertEquals(-90, map.getLatitude(0));
+        assertEquals(0, map.getRow(-90));
+        assertEquals(0, map.getLatitude(45));
+        assertEquals(45, map.getRow(0));
+        assertEquals(90, map.getLatitude(90));
+        assertEquals(90, map.getRow(90));
+
+        builder = new MapBuilder(game);
+        map = builder.setDimensions(1, 91).build();
+        map.setMinimumLatitude(0);
+
+        assertEquals(91, map.getHeight());
+        assertEquals(1f, map.getLatitudePerRow());
+        assertEquals(0, map.getLatitude(0));
+        assertEquals(0, map.getRow(0));
+        assertEquals(45, map.getLatitude(45));
+        assertEquals(45, map.getRow(45));
+        assertEquals(90, map.getLatitude(90));
+        assertEquals(90, map.getRow(90));
+
+    }
+
 }
