@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -31,7 +31,6 @@ import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.GameOptions;
 import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.Unit;
-import net.sf.freecol.common.model.Unit.UnitState;
 import net.sf.freecol.common.model.UnitType;
 import net.sf.freecol.server.ServerTestHelper;
 import net.sf.freecol.util.test.FreeColTestCase;
@@ -143,7 +142,7 @@ public class ServerBuildingTest extends FreeColTestCase {
 
         Colony colony = getSchoolColony(4, SchoolLevel.SCHOOLHOUSE);
         Building school = colony.getBuilding(schoolType);
-        assertTrue(schoolType.hasAbility("model.ability.teach"));
+        assertTrue(school.canTeach());
         Iterator<Unit> units = colony.getUnitIterator();
 
         Unit colonist = units.next();
@@ -630,8 +629,7 @@ public class ServerBuildingTest extends FreeColTestCase {
         Building university = colony.getBuilding(universityType);
         Iterator<Unit> units = colony.getUnitIterator();
         Unit outsider = new ServerUnit(game, colony.getTile(),
-                                       colony.getOwner(), freeColonistType,
-                                       UnitState.ACTIVE);
+                                       colony.getOwner(), freeColonistType);
 
         Building townHall = colony.getBuilding(townHallType);
         Unit colonist = units.next();
@@ -707,19 +705,17 @@ public class ServerBuildingTest extends FreeColTestCase {
         assertEquals(1, getUnitList(colony, masterCarpenterType).size());
 
         // Now we want the colonist to be a carpenter. We just want to
-        // shuffle the teachers.
+        // shuffle the teachers.  Move them out of the school.
         teacher2.setLocation(colony.getVacantColonyTileFor(teacher2, true,
                                                            grainType));
-        // outside the colony is still considered OK (same Tile)
-        teacher1.putOutsideColony();
-
+        teacher1.setLocation(colony.getTile());
         assertNull(teacher1.getStudent());
         assertNull(teacher2.getStudent());
 
-        // Passing a turn outside school does not reset training at this time
+        // Moving outside school resets training
         ServerTestHelper.newTurn();
-        assertEquals(2, teacher1.getTurnsOfTraining());
-        assertEquals(2, teacher2.getTurnsOfTraining());
+        assertEquals(0, teacher1.getTurnsOfTraining());
+        assertEquals(0, teacher2.getTurnsOfTraining());
         assertEquals(1, getUnitList(colony, freeColonistType).size());
         assertEquals(1, getUnitList(colony, pettyCriminalType).size());
 
@@ -728,11 +724,13 @@ public class ServerBuildingTest extends FreeColTestCase {
         teacher2.setLocation(university);
 
         ServerTestHelper.newTurn();
-        assertEquals(3, teacher1.getTurnsOfTraining());
-        assertEquals(3, teacher2.getTurnsOfTraining());
+        assertEquals(1, teacher1.getTurnsOfTraining());
+        assertEquals(1, teacher2.getTurnsOfTraining());
         assertEquals(1, getUnitList(colony, freeColonistType).size());
         assertEquals(1, getUnitList(colony, pettyCriminalType).size());
 
+        ServerTestHelper.newTurn();
+        ServerTestHelper.newTurn();
         ServerTestHelper.newTurn();
         assertEquals(0, teacher1.getTurnsOfTraining());
         assertEquals(0, teacher2.getTurnsOfTraining());
@@ -777,16 +775,16 @@ public class ServerBuildingTest extends FreeColTestCase {
                                                            grainType));
         teacher2.setLocation(university);
         ServerTestHelper.newTurn();
-        assertEquals(3, teacher1.getTurnsOfTraining());
+        assertEquals(0, teacher1.getTurnsOfTraining());
         assertEquals(1, teacher2.getTurnsOfTraining());
 
-        // If we now also add teacher2 to the university, then
-        // Teacher1 will still be the teacher in charge
+        // If we now also add teacher1 to the university, but
+        // teacher2 should still be the teacher in charge
         teacher1.setLocation(university);
         assertNull(teacher1.getStudent());
         ServerTestHelper.newTurn();
 
-        assertEquals(3, teacher1.getTurnsOfTraining());
+        assertEquals(0, teacher1.getTurnsOfTraining());
         assertEquals(2, teacher2.getTurnsOfTraining());
     }
 
@@ -1129,7 +1127,7 @@ public class ServerBuildingTest extends FreeColTestCase {
         student.setType(pettyCriminalType);
         student.setLocation(colony.getBuilding(townHallType));
 
-        assertTrue(schoolType.hasAbility("model.ability.teach"));
+        assertTrue(school.canTeach());
         assertTrue(colony.canTrain(lumber));
         lumber.setLocation(school);
 
