@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -33,7 +33,6 @@ import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.TileType;
 import net.sf.freecol.common.model.Turn;
 import net.sf.freecol.common.model.Unit;
-import net.sf.freecol.common.model.Unit.UnitState;
 import net.sf.freecol.common.model.UnitType;
 import net.sf.freecol.server.ServerTestHelper;
 import net.sf.freecol.server.control.InGameController;
@@ -141,8 +140,8 @@ public class ServerPlayerTest extends FreeColTestCase {
         assertFalse("Setup error, tile2 should not be explored by dutch player",dutch.hasExplored(tile2));
         assertFalse("Setup error, tile2 should not be explored by french player",french.hasExplored(tile2));
 
-        new ServerUnit(game, tile1, dutch, colonistType, UnitState.SENTRY);
-        new ServerUnit(game, tile2, french, colonistType, UnitState.SENTRY);
+        new ServerUnit(game, tile1, dutch, colonistType);
+        new ServerUnit(game, tile2, french, colonistType);
         assertTrue("Tile1 should be explored by dutch player",dutch.hasExplored(tile1));
         assertFalse("Tile1 should not be explored by french player",french.hasExplored(tile1));
         assertFalse("Tile2 should not be explored by dutch player",dutch.hasExplored(tile2));
@@ -157,10 +156,9 @@ public class ServerPlayerTest extends FreeColTestCase {
         Colony colony = getStandardColony();
         ServerPlayer dutch = (ServerPlayer) game.getPlayer("model.nation.dutch");
         Unit wagonInColony = new ServerUnit(game, colony.getTile(), dutch,
-                                            wagonTrainType, UnitState.ACTIVE);
+                                            wagonTrainType);
         Unit wagonNotInColony = new ServerUnit(game, map.getTile(10, 10), dutch,
-                                               wagonTrainType,
-                                               UnitState.ACTIVE);
+                                               wagonTrainType);
         Goods cotton = new Goods(game, null, cottonType, 75);
 
         // Check if location null
@@ -216,10 +214,9 @@ public class ServerPlayerTest extends FreeColTestCase {
         ServerPlayer dutch = (ServerPlayer) game.getPlayer("model.nation.dutch");
         Goods cotton = new Goods(game, null, cottonType, 75);
         Europe europe = dutch.getEurope();
-        Unit privateer1 = new ServerUnit(game, europe, dutch,
-                                         privateerType, UnitState.ACTIVE);
-        Unit privateer2 = new ServerUnit(game, europe, dutch,
-                                         privateerType, UnitState.ACTIVE);
+        Map america = game.getMap();
+        Unit privateer1 = new ServerUnit(game, europe, dutch, privateerType);
+        Unit privateer2 = new ServerUnit(game, europe, dutch, privateerType);
 
         // While source in Europe, target in Europe
         cotton.setLocation(privateer1);
@@ -236,7 +233,7 @@ public class ServerPlayerTest extends FreeColTestCase {
         // While source moving from America, target in Europe
         cotton.setLocation(privateer1);
         assertEquals(europe, privateer1.getLocation());
-        igc.moveToAmerica(dutch, privateer1);
+        igc.moveTo(dutch, privateer1, america);
         try {
             igc.moveGoods(cotton, privateer2);
             fail();
@@ -245,7 +242,7 @@ public class ServerPlayerTest extends FreeColTestCase {
 
         // While source moving to America, target in Europe
         cotton.setLocation(privateer1);
-        igc.moveToEurope(dutch, privateer1);
+        igc.moveTo(dutch, privateer1, europe);
         try {
             igc.moveGoods(cotton, privateer2);
             fail();
@@ -254,7 +251,7 @@ public class ServerPlayerTest extends FreeColTestCase {
 
         // While source in Europe, target moving to America
         privateer1.setLocation(europe);
-        igc.moveToAmerica(dutch, privateer2);
+        igc.moveTo(dutch, privateer2, america);
         cotton.setLocation(privateer1);
         try {
             igc.moveGoods(cotton, privateer2);
@@ -264,7 +261,7 @@ public class ServerPlayerTest extends FreeColTestCase {
 
         // While source moving to America, target moving to America
         cotton.setLocation(privateer1);
-        igc.moveToAmerica(dutch, privateer1);
+        igc.moveTo(dutch, privateer1, america);
         try {
             igc.moveGoods(cotton, privateer2);
             fail();
@@ -273,7 +270,7 @@ public class ServerPlayerTest extends FreeColTestCase {
 
         // While source moving from America, target moving to America
         cotton.setLocation(privateer1);
-        igc.moveToEurope(dutch, privateer1);
+        igc.moveTo(dutch, privateer1, europe);
         try {
             igc.moveGoods(cotton, privateer2);
             fail();
@@ -282,7 +279,7 @@ public class ServerPlayerTest extends FreeColTestCase {
 
         // While source in Europe, target moving from America
         privateer1.setLocation(europe);
-        igc.moveToEurope(dutch, privateer2);
+        igc.moveTo(dutch, privateer2, europe);
 
         cotton.setLocation(privateer1);
         try {
@@ -293,7 +290,7 @@ public class ServerPlayerTest extends FreeColTestCase {
 
         // While source moving to America, target moving from America
         cotton.setLocation(privateer1);
-        igc.moveToAmerica(dutch, privateer1);
+        igc.moveTo(dutch, privateer1, america);
         try {
             igc.moveGoods(cotton, privateer2);
             fail();
@@ -302,7 +299,7 @@ public class ServerPlayerTest extends FreeColTestCase {
 
         // While source moving from America, target moving from America
         cotton.setLocation(privateer1);
-        igc.moveToEurope(dutch, privateer1);
+        igc.moveTo(dutch, privateer1, europe);
         try {
             igc.moveGoods(cotton, privateer2);
             fail();
@@ -335,7 +332,7 @@ public class ServerPlayerTest extends FreeColTestCase {
 
         ServerPlayer dutch = (ServerPlayer) game.getPlayer("model.nation.dutch");
 
-        new ServerUnit(game, map.getTile(4, 7), dutch, colonistType, UnitState.ACTIVE);
+        new ServerUnit(game, map.getTile(4, 7), dutch, colonistType);
 
         assertFalse("Should not be game over, has units",
                     dutch.checkForDeath());
@@ -347,10 +344,8 @@ public class ServerPlayerTest extends FreeColTestCase {
 
         ServerPlayer dutch = (ServerPlayer) game.getPlayer("model.nation.dutch");
 
-        new ServerUnit(game, dutch.getEurope(), dutch, colonistType,
-                       UnitState.SENTRY);
-        new ServerUnit(game, dutch.getEurope(), dutch, galleonType,
-                       UnitState.SENTRY);
+        new ServerUnit(game, dutch.getEurope(), dutch, colonistType);
+        new ServerUnit(game, dutch.getEurope(), dutch, galleonType);
         assertFalse("Should not be game over, not 1600 yet",
                     dutch.checkForDeath());
 
@@ -366,15 +361,13 @@ public class ServerPlayerTest extends FreeColTestCase {
 
         ServerPlayer dutch = (ServerPlayer) game.getPlayer("model.nation.dutch");
 
-        Unit galleon = new ServerUnit(game, map.getTile(6, 8), dutch,
-                                      galleonType, UnitState.ACTIVE);
-        Unit colonist = new ServerUnit(game, galleon, dutch, colonistType,
-                                       UnitState.SENTRY);
+        Unit galleon = new ServerUnit(game, map.getTile(6, 8), dutch, galleonType);
+        Unit colonist = new ServerUnit(game, galleon, dutch, colonistType);
         assertTrue("Colonist should be aboard the galleon",
                    colonist.getLocation() == galleon);
         assertEquals("Galleon should have a colonist onboard",
                      1, galleon.getUnitCount());
-        igc.moveToEurope(dutch, galleon);
+        igc.moveTo(dutch, galleon, dutch.getEurope());
 
         assertFalse("Should not be game over, units between new world and europe",
                     dutch.checkForDeath());
@@ -391,11 +384,11 @@ public class ServerPlayerTest extends FreeColTestCase {
 
         ServerPlayer dutch = (ServerPlayer) game.getPlayer("model.nation.dutch");
 
-        Unit galleon = new ServerUnit(game,dutch.getEurope() , dutch, galleonType, UnitState.ACTIVE);
-        Unit colonist = new ServerUnit(game, galleon, dutch, colonistType, UnitState.SENTRY);
+        Unit galleon = new ServerUnit(game,dutch.getEurope(), dutch, galleonType);
+        Unit colonist = new ServerUnit(game, galleon, dutch, colonistType);
         assertTrue("Colonist should be aboard the galleon",colonist.getLocation() == galleon);
         assertEquals("Galleon should have a colonist onboard",1,galleon.getUnitCount());
-        igc.moveToAmerica(dutch, galleon);
+        igc.moveTo(dutch, galleon, map);
 
         assertFalse("Should not be game over, units between new world and europe",
                     dutch.checkForDeath());
