@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2011  The FreeCol Team
+ *  Copyright (C) 2002-2012  The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -47,7 +47,6 @@ import net.sf.freecol.common.model.Specification;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.TileType;
 import net.sf.freecol.common.model.Unit;
-import net.sf.freecol.common.model.Unit.UnitState;
 import net.sf.freecol.common.model.UnitType;
 import net.sf.freecol.server.model.ServerGame;
 import net.sf.freecol.server.model.ServerIndianSettlement;
@@ -165,8 +164,9 @@ public class FreeColTestCase extends TestCase {
         for (Nation n : specification.getNations()) {
             Player p = new ServerPlayer(game, n.getRulerNameKey(), false, n,
                                         null, null);
-            p.setAI(!n.getType().isEuropean() || n.getType().isREF());
-            game.addPlayer(p);
+            boolean ai = !n.getType().isEuropean() || n.getType().isREF();
+            p.setAI(ai);
+            if (ai || game.canAddNewPlayer()) game.addPlayer(p);
         }
         return game;
     }
@@ -367,6 +367,7 @@ public class FreeColTestCase extends TestCase {
             }
             this.width = width;
             this.height = heigth;
+            this.tiles = new TileType[width][height];
             return this;
         }
 
@@ -566,15 +567,15 @@ public class FreeColTestCase extends TestCase {
                                              settlementTile, isCapital,
                                              skillToTeach, isVisited,
                                              residentMissionary);
+            indianPlayer.addSettlement(camp);
 
             // Add braves
             for(int i=0; i < initialBravesInCamp; i++){
                 Unit brave = new ServerUnit(game, camp, indianPlayer,
-                                            indianBraveType, UnitState.ACTIVE,
-                                            indianBraveType.getDefaultEquipment());
+                                            indianBraveType);
                 camp.addOwnedUnit(brave);
             }
-            camp.placeSettlement();
+            camp.placeSettlement(true);
             return camp;
         }
 
@@ -598,6 +599,7 @@ public class FreeColTestCase extends TestCase {
             Field productionBonus = Colony.class.getDeclaredField("productionBonus");
             productionBonus.setAccessible(true);
             productionBonus.setInt(colony, value);
+            colony.invalidateCache();
         } catch (Exception e) {
             // do nothing
         }
@@ -607,7 +609,7 @@ public class FreeColTestCase extends TestCase {
      * Build/place a colony with a unit, without requiring the server.
      */
     public void nonServerBuildColony(Unit builder, Colony colony) {
-        colony.placeSettlement();
+        colony.placeSettlement(true);
         nonServerJoinColony(builder, colony);
     }
 
@@ -615,7 +617,6 @@ public class FreeColTestCase extends TestCase {
      * Join a colony with a unit, without requiring the server.
      */
     public void nonServerJoinColony(Unit builder, Colony colony) {
-        builder.setState(UnitState.IN_COLONY);
         builder.setLocation(colony);
         builder.setMovesLeft(0);
     }
